#!/bin/bash
xdxgpu_device=()
busid=0

drv_num=`ls /dev/dri/card* |wc -l`
vga_num=`lspci -v|grep -i "VGA "|wc -l`
xdx_vga_num=`lspci -v| grep -i "VGA "|grep XDX|wc -l`
if [ $xdx_vga_num == "0" ];then
	xdx_vga_num=`lspci -v| grep -i "VGA "|grep 1eed|wc -l`
fi

if [ $drv_num -eq $vga_num ]&&[ $xdx_vga_num -lt $vga_num ];then
	onboard_flag=1
	search_string=""
else
	onboard_flag=0
	search_string="1eed"
fi

if [[ $onboard_flag == "0" ]];then
	while read line; do
	if [ ! -z "$line" ]; then
		busid=$(echo "$line" | awk '{print $1}')
		busid=$(echo "$busid" | sed 's/\./:/g')
		IFS=":" read -ra pci_fields <<< "$busid"
		decimal_pci_fields=()
		for field in "${pci_fields[@]}"; do
			decimal_pci_fields+=("$((16#$field))")
		done
		if [ ${#decimal_pci_fields[@]} == "4" ];then
			busid="${decimal_pci_fields[1]}@${decimal_pci_fields[0]}:${decimal_pci_fields[2]}:${decimal_pci_fields[3]}"
			#busid="${decimal_pci_fields[1]}:${decimal_pci_fields[2]}:${decimal_pci_fields[3]}"
		else
			busid="${decimal_pci_fields[0]}:${decimal_pci_fields[1]}:${decimal_pci_fields[2]}"
		fi
		if [[ $busid == *":1" ]]; then
			continue
		fi
		xdxgpu_device+=("$busid")
		xdxgpu_count=${#xdxgpu_device[@]}
	fi
	done < <(lspci -n | grep "$search_string")
else
	while read line; do
	if [ ! -z "$line" ]; then
		busid=$(echo "$line" | awk '{print $1}')
		busid=$(echo "$busid" | sed 's/\./:/g')
		IFS=":" read -ra pci_fields <<< "$busid"
		decimal_pci_fields=()
		for field in "${pci_fields[@]}"; do
			decimal_pci_fields+=("$((16#$field))")
		done
		if [ ${#decimal_pci_fields[@]} == "4" ];then
			busid="${decimal_pci_fields[1]}@${decimal_pci_fields[0]}:${decimal_pci_fields[2]}:${decimal_pci_fields[3]}"
			#busid="${decimal_pci_fields[1]}:${decimal_pci_fields[2]}:${decimal_pci_fields[3]}"
		else
			busid="${decimal_pci_fields[0]}:${decimal_pci_fields[1]}:${decimal_pci_fields[2]}"
		fi
		if [[ $busid == *":1" ]]; then
			continue
		fi
		xdxgpu_device+=("$busid")
		xdxgpu_count=${#xdxgpu_device[@]}
	fi
	done < <(lspci -v | grep "VGA "|grep -i "$search_string")
fi

xdxgpu_count=${#xdxgpu_device[@]}
for busid in "${xdxgpu_device[@]}"; do
	echo "Generated XConf for VGA busid: $busid"
done

if [ -e "xorg.conf" ]; then
	rm xorg.conf
	touch xorg.conf
fi

serverlayout='Section "ServerLayout"\n
	\tIdentifier     "X.org Configured"\n
	\tScreen      0  "Screen0" 0 0\n'
if  [ "$xdxgpu_count" -gt 1 ]; then
	for ((i=1; i<$xdxgpu_count; i++)); do
		serverlayout+="\tScreen      $i  \"Screen"$i"\" RightOf \"Screen"$(($i-1))"\"\n"
	done
fi
serverlayout+="EndSection\n"
echo -e $serverlayout >> xorg.conf

files='Section "Files"\n
	\tModulePath "/opt/xdxgpu/lib/xorg/modules"\n
	\tModulePath "/opt/xdxgpu/lib/x86_64-linux-gnu/xorg/modules"\n
	\tModulePath "/usr/lib/xorg/modules"\n
	\tModulePath "/usr/lib64/xorg/modules"\n
	\tModulePath "/usr/local/lib/xorg/modules"\n'
files+="EndSection\n"
echo -e $files >> xorg.conf

monitor='Section "Monitor"\n
	\tIdentifier   "Monitor0"\n
	\tVendorName   "Monitor Vendor"\n
	\tModelName     "Monitor Model"\n'
monitor+="EndSection\n\n"

if [ "$xdxgpu_count" -gt 1 ]; then
	for ((i=1; i<$xdxgpu_count; i++)); do
		monitor+="Section \"Monitor\"\n"
		monitor+="\tIdentifier   \"Monitor"$i"\"\n"
		monitor+="\tVendorName   \"Monitor Vendor\"\n"
		monitor+="\tModelName     \"Monitor Model\"\n"
		monitor+="EndSection\n\n"
	done
fi
echo -e $monitor >> xorg.conf
if [[ $onboard_flag == "1" ]];then
	device='Section "Device"\n
		\tIdentifier  "Card0"\n
		\tDriver      "modesetting"\n
		\tBusID       "PCI:'${xdxgpu_device[0]}'"\n'
else
	device='Section "Device"\n
		\tIdentifier  "Card0"\n
		\tDriver      "xdxgpu"\n
		\tBusID       "PCI:'${xdxgpu_device[0]}'"\n'
fi
device+="EndSection\n\n"

if [ "$xdxgpu_count" -gt 1 ]; then
	for ((i=1; i<$xdxgpu_count; i++)); do
		device+="Section \"Device\"\n"
		device+="\tIdentifier  \"Card"$i"\"\n"
		device+="\tDriver      \"xdxgpu\"\n"
		device+="\tBusID       \"PCI:"${xdxgpu_device["$i"]}"\"\n"
		device+="EndSection\n\n"
	done
fi
echo -e $device >> xorg.conf

screen='Section "Screen"\n
	\tIdentifier "Screen0"\n
	\tDevice     "Card0"\n
	\tMonitor    "Monitor0"\n'
screen+="EndSection\n\n"

if [ "$xdxgpu_count" -gt 1 ]; then
	for ((i=1; i<$xdxgpu_count; i++)); do
		screen+="Section \"Screen\"\n"
		screen+="\tIdentifier \"Screen"$i"\"\n"
		screen+="\tDevice     \"Card"$i"\"\n"
		screen+="\tMonitor    \"Monitor"$i"\"\n"
		screen+="EndSection\n\n"
	done
fi
echo -e $screen >> xorg.conf

xdx_compute_card_num=`lspci -v| grep -i "VGA "| grep 1eed | grep 15a4 |wc -l`
if [ "$xdx_compute_card_num" -gt 0 ]; then
	mv xorg.conf /usr/share/X11/xorg.conf.d/10-xdxgpu.conf
fi
