# -*- coding: utf-8 -*-

# from pathlib import Path
import re, glob
import subprocess
import sys

from xdxsmi.api.pyxdxml import *


def is_host():
    try:
        with open("/proc/cpuinfo", "r") as file:
            cpuinfo_contents = file.read()
    except FileNotFoundError:
        print("/proc/cpuinfo file is not exist.")
        sys.exit()

    if "hypervisor" in cpuinfo_contents:
        return 0
    else:
        return 1

class XdxGpuInfo():
    #  (self, index, user_index, handle) -> None
    def __init__(self, index, user_index, handle):
        # queryInfo = xdxml_device_query_by_out_of_band(handle)
        self.drm_index = index
        self.user_index = user_index
        self.handle = handle
        self.query_info()
        #Yufei think index(id) doesn't match with handle(device address)
        # self.drm_index = index
        # #GPU realted attributes xdx_vbios
    def query_info(self):
        queryInfo = xdxml_device_query_by_out_of_band(self.handle)
        self.is_host = is_host()
        self.name = queryInfo.name.decode('utf-8') if queryInfo.name!=None else 'N/A'
        # self.uuid = '0x' + str(queryInfo.uuid.decode('utf-8')) if queryInfo.uuid!=None else 'N/A'
        # self.minor_num = index

        try:
            self.vbios = queryInfo.vbios_version.decode('utf-8').split(' ',1)[0] if queryInfo.vbios_version!=None else 'N/A'
        except:
            self.vbios = 'N/A'

        try:
            self.fm_ver = queryInfo.fw_ver.decode('utf-8') if queryInfo.fw_ver!=None else 'N/A'
        except:
            self.fm_ver = 'N/A'
        
        self.pci_bdf = format(queryInfo.domain, '04x') + ':' + format(queryInfo.bus, '02x') + ':' + format(queryInfo.device, '02x') + '.' + format(queryInfo.func, '01x')
        self.domain = '0x' + format(queryInfo.domain, '04x') if self.pci_bdf!='N/A' else 'N/A'
        self.bus = '0x' + format(queryInfo.bus, '02x') if self.pci_bdf!='N/A' else 'N/A'
        self.dev = '0x' + format(queryInfo.device, '02x') if self.pci_bdf!='N/A' else 'N/A'

        #fix PANGUA0POS-2141/2129,device id and others showed something like 0x1eedL
        self.device_id = hex(queryInfo.device_id).rstrip("lL")        
        self.device_ID = hex(queryInfo.device_ID).rstrip("lL") 
        self.vendor_ID = hex(queryInfo.vendor_ID).rstrip("lL")
        self.subsystem_vendor_ID = hex(queryInfo.subsystem_vendor_ID).rstrip("lL")
        self.subsystem_device_ID = hex(queryInfo.subsystem_device_ID).rstrip("lL")

        # self.subsystem_device = self.device['subsystem_device'] if self.device['subsystem_device']!=None else 'N/A'
        # self.primary_core_id = queryInfo.primary_core_id.decode('utf-8') if queryInfo.primary_id!=None else 'N/A'
        # self.pcie_event = queryInfo.pcie_event if queryInfo.pcie_event!=None else 'N/A'
        # hide fan speed in VM
        if queryInfo.support_fan_info >= 1:
            self.fan_spd = queryInfo.fan_speed if queryInfo.fan_speed!=None and self.is_host else 'N/A'
        else:
            self.fan_spd = 'not support'
        # self.pwm1 = int(self.hwmon['pwm1']) if self.hwmon['pwm1']!=None else 'N/A'
        # self.perform_state = 'N/A'
        # self.clk_throt_reasons = 'N/A'


        # #GPU memory info
        # self.fb_free = 'N/A' if queryInfo.fb_free =='N/A' else int(int(queryInfo.fb_free) / (1024*1024))

        self.fb_total = 'N/A' if queryInfo.fb_total =='N/A' else int(int(queryInfo.fb_total))
        # self.fb_used = 'N/A' if self.fb_total=='N/A' or self.fb_free=='N/A' else int(self.fb_total - self.fb_free)

        # hide temp and gfx_clk in VM
        self.temp = str(queryInfo.temp) + str("°C") if queryInfo.temp is not None and self.is_host else 'N/A'
        self.gfx_clk = str(int(queryInfo.xdx_gfx_freq)) +'MHZ' if queryInfo.xdx_gfx_freq!=None and self.is_host else 'N/A'
        if queryInfo.support_power_info == 1:
            # hide power info in VM
            self.volt = str(queryInfo.volt/1e3) + "V" if queryInfo.volt!=None and self.is_host else 'N/A'
            self.current = str(queryInfo.current/1e3) + "A" if queryInfo.current!=None and self.is_host else 'N/A'
            self.pwr = str(queryInfo.pwr/1e3) + "W" if queryInfo.pwr!=None and self.is_host else 'N/A'

            # hide soc power if volt/current/pwr is 0, and only show the total power
            is_hide_soc_power = 0 if queryInfo.volt and queryInfo.current and queryInfo.pwr else 1
            self.volt = self.volt if is_hide_soc_power == 0 else 'N/A'
            self.current = self.current if is_hide_soc_power == 0 else 'N/A'
            self.pwr = self.pwr if is_hide_soc_power == 0 else 'N/A'

            pwr_total_tmp = 0 if queryInfo.pwr_total < 0 else queryInfo.pwr_total
            self.pwr_total = str(pwr_total_tmp) + "W" if queryInfo.pwr_total!=None and self.is_host else 'N/A'
        else:
            self.volt = 'not support'
            self.current = 'not support'
            self.pwr = 'not support'
            self.pwr_total = 'not support'

        self.chip_model = queryInfo.chip_model.decode('utf-8') if queryInfo.chip_model!=None else 'N/A'

    # (self)->None
    def update_dynamic_info(self):
        self.query_info()

    def __setitem__(self, name, val):
        return setattr(self, name, val)

    def __getitem__(self, name):
        return getattr(self, name)
