# -*- coding: utf-8 -*-

# from typing import Any, Callable, Dict, List, Sequence, Tuple
# from pathlib import Path
import sys
import time, re
import xdxsmi_config
from xdxsmi.api.xdxgpu import XdxGpu
from xdxsmi.api.out_of_band_gpu_info import XdxGpuInfo
from xdxsmi.api.pyxdxml import *
# from enum import Enum, auto


#XDX_VENDOR_ID = 0x10DE
# 1B36 is vendor id for qxl virtual GPU in KVM testing environment
#XDX_VENDOR_ID = 0x1B36

XDX_VENDOR_ID = 0x1EED




#Read version of ddk, umd and kmd
def read_versions():
    try:
        with open("/sys/kernel/debug/xdxgpu/gpu00/debug_dump") as fo:
            res = re.match( r'[\s\S]*rogueddk (.*)@[\s\S]*UM info: (.*) @.*[\s\S]*KM info: (.*) @.*[\s\S]*', fo.read())
    except Exception as e:
        #sys.exit("Exception: {0} \nPlease try another way or contact the software provider.".format(e)) 
        res = ['N/A', 'N/A', 'N/A', 'N/A']
    return res

#Read version of ddk, umd and kmd
# (handle: Any)
def C_read_versions(handle):
    res = ['N/A', 'N/A', 'N/A', 'N/A']
    ver = xdxml_device_get_driver_version(handle)
    res[1] = ver.DDK.decode('utf-8')
    res[2] = ver.KMD.decode('utf-8')
    res[3] = ver.UMD.decode('utf-8')
    return res

class XdxSmi():
    # (self, is_out_of_band = False) -> None
    def __init__(self, is_out_of_band = False):
        #print("enter XdxSmi")
        self.is_out_of_band = is_out_of_band
        self.is_init = 0
        if is_out_of_band:
            xdxml_out_of_band_init()
        else:
            xdxml_init()
        self.is_init = 1
        # under debugging
        self.device_num = xdxml_device_get_count()

        if self.device_num == 0:
            print("No xdxgpu devices found!")
            sys.exit()
        #print("device_num is " + str(self.device_num))
        self.handle = []
        for i in range(self.device_num):
            self.handle.append(xdxml_device_get_handle_by_index(i))
        if is_out_of_band:
            # print("process out of manager")
            self.xdxgpus = self.get_out_of_band_xdxgpu(self.handle)
        else:
            self.xdxgpus = self.detect_xdxgpus2(self.handle)
            self.vers = C_read_versions(self.handle[0])
            self.driver_ver = xdxml_device_get_driver_release_version(self.handle)

        #self.xdxgpus = self.detect_xdxgpus(self.handle)
        self.timestp = time.asctime(time.localtime(time.time()))
        self.smi_ver = xdxsmi_config.GlobalConfig.__version__
        self.gpuNums = len(self.xdxgpus)
        #check if we can query VPU
        self.isVpufunctionValid = False
        if not(is_out_of_band):
            self.isVpufunctionValid = xdxml_get_vpu_functions_valid()

    # under debugging
    # (self, handle) -> List[XdxGpu]
    def detect_xdxgpus2(self, handle):
        xdxgpus = []
        for i in range(self.device_num):
            # xdxgpus.append(XdxGpu(i,handle[i]))
            # get the true card index of device
            index = xdxml_device_get_card_index_by_handle(handle[i])
            xdxgpus.append(XdxGpu(index, i, handle[i]))
        return xdxgpus

    # (self, handle) -> List[XdxGpuInfo]
    def get_out_of_band_xdxgpu(self, handle):
        xdxgpus = []
        for i in range(self.device_num):
            # get the true card index of device
            index = xdxml_device_get_card_index_by_handle(handle[i])
            xdxgpus.append(XdxGpuInfo(index, i, handle[i]))
        return xdxgpus

    #Filter target gpus
    # (self, func: Callable[[XdxGpu, Tuple, Dict], List[str]],idx_list=(), *args, **kwargs)-> List[List[str]]
    def apply_on_gpus(self, func, idx_list=(), *args, **kwargs):
        #If idx_list is empty, will apply on all gpus by default
        target_gpus = self.xdxgpus
        if idx_list:
            target_gpus = [gpu for gpu in target_gpus if gpu.drm_index in idx_list]
        return [func(gpu, *args, **kwargs) for gpu in target_gpus]

    #Get target attribute values of GPU
    # get_attributes(nodes: list, attrs: Sequence[str], gpu: XdxGpu)
    def get_attributes(nodes, attrs, gpu):
        #List to record attribute Values
        res = []

        #Attributes belong to clock module
        unitConver1 = ["xdx_acpu_freq_mhz", "xdx_gddr_phys_freq_mhz", \
                       "xdx_gfx_freq_mhz", "xdx_ndf_freq_mhz", \
                       "xdx_host_freq_mhz", "xdx_gfx_freq_mhz", "xdx_cmcu_freq_mhz"]

        #Atrributes belong to power module
        unitConver2 = ["in0_input", "curr1_input", "power1_average", \
            "power1_cap_max", "power1_cap_min", "power1_cap"]

        #Atrributes belong to thermal module
        unitConver3 = ["temp1_input", "temp1_highest", "temp1_crit", "temp1_min", "temp1_max"]

        #Atrributes belong to fan module
        unitConver4 = ["fan1_input", "fan1_target", "pwm1"]

        #Add index for GPU
        res.insert(0, str(gpu.drm_index))

        #Unit conversion
        for node in nodes:
            for attr in attrs:
                if gpu[node][attr] == None:
                    continue
                
                #frequency in Hz or MHz or GHz
                if (attr in unitConver1):
                    val = int(gpu[node][attr])/1e9
                    res.append(str(val) + "GHz") \
                        if (val>=1) else res.append(str(val*1000)+"MHz")
                
                #Voltage in V or mV, current in A or mA, power in W or mW 
                elif (attr in unitConver2):
                    val = int(gpu[node][attr])
                    if attr == "in0_input":
                        res.append(str(val/1e3) + "V")
                    elif attr == "curr1_input":
                        res.append(str(val/1e3) + "A")
                    else:
                        res.append(str(val/1e3) + "W")

                #temperature in °C
                elif (attr in unitConver3):
                    res.append(gpu[node][attr] + str("°C"))

                #fan speed in RPM(Revolutions Per Minute)
                elif (attr in unitConver4):
                    res.append(gpu[node][attr]) # + "RPM"

                #no unit conversion needed, e.g. gpu, name
                else:
                    res.append(gpu[node][attr])

        return res

    #Update the value of the target attribute and return the values of attributes
    # (nodes: list, attrs: Sequence[str], newVal: Any)->Callable[[XdxGpu, Tuple, Dict], List[str]]
    @staticmethod
    def generate_attr_writer(nodes, attrs, newVal):
        # (gpu: XdxGpu, *args, **kwargs)->str
        def inner(gpu, *args, **kwargs):
            for node in nodes:
                for attr in attrs:
                    if gpu[node][attr] == None:
                        continue
                    gpu[node][attr] = newVal
            return None
        return inner

    # (nodes: list, attrs: Sequence[str])->Callable[[XdxGpu, Tuple, Dict], List[str]]
    #Return the values of attributes
    @staticmethod
    def generate_attr_reader(nodes, attrs):
        # (gpu: XdxGpu, *args, **kwargs)->str
        def inner(gpu, *args, **kwargs):
            return XdxSmi.get_attributes(nodes, attrs, gpu)
        return inner
