# -*- coding: utf-8 -*-

from xdxsmi.api.node import Node
from xdxsmi.api.pyxdxml import *
# from pathlib import Path
import sys
import re, glob
import subprocess
import math
import os.path

#Acquire frame buffer and system memory info
def read_mem_info():
    try:
        with open("/sys/kernel/debug/xdxgpu/gpu00/debug_dump") as fo:
            #[\s\S]*GPU_LOCAL, Type: LMA, CPU PA Base: 0x[0-9]*, GPU PA Base: 0x[0-9]*, Usage Flags: 0x[0-9]*, Refs: 10, Free Size: ([0-9]*), Total Size: ([0-9]*)[\s\S]*
            #[\s\S]*SYSMEM, Type: UMA, Usage Flags: 0x[0-9]*, Refs: 1, Free Size: ([0-9]*), Total Size: ([0-9]*)[\s\S]*     
            res = re.match( r'[\s\S]*GPU_LOCAL[\s\S]*10, Free Size: ([0-9]*), Total Size: ([0-9]*)[\s\S]*SYSMEM, Type: UMA, Usage Flags: 0x[0-9]*, Refs: 1, Free Size: ([0-9]*), Total Size: ([0-9]*)[\s\S]*', fo.read())
            if res == None:
                res = ['N/A', 'N/A', 'N/A', 'N/A', 'N/A']                
    except:
        res = ['N/A', 'N/A', 'N/A', 'N/A', 'N/A']
    return res

#Acquire statistics of pcie errors
# (file: str)
def read_err_corr(file):
    res = re.match(r'[\s\S]*TOTAL_ERR_COR ([0-9]*)[\s\S]*', file)
    return res[1] if res!=None else 'N/A'

# (file: str)
def read_err_fatal(file):
    res = re.match(r'[\s\S]*TOTAL_ERR_FATAL ([0-9]*)[\s\S]*', file)
    return res[1] if res!=None else 'N/A'

# (file: str)
def read_err_nonfat(file):
    res = re.match(r'[\s\S]*TOTAL_ERR_NONFATAL ([0-9]*)[\s\S]*', file)
    return res[1] if res!=None else 'N/A'

#Match PCIE link speed with PCIE link generation
# (file: str)
def speed_2_gen(speed):
    speed = float(speed)
    if speed == 64:
        return "6.0"
    elif speed == 32 or speed == 25:
        return "5.0"
    elif speed == 16:
        return "4.0"
    elif speed == 8:
        return "3.0"
    elif speed == 5:
        return "2.0"
    else:
        return "1.0"

#Match PCIE link generation with PCIE throughput
# (gen:str):
def gen_2_throughput(gen):
    if gen == "6":
        return "0.250 GB/s"
    elif gen == "5":
        return "0.500 GB/s"
    elif gen == "4":
        return "0.985 GB/s"
    elif gen == "3":
        return "1.969 GB/s"
    elif gen == "2":
        return "3.938 GB/s"
    elif gen == "1":
        return "7.563 GB/s"
    else:
        return "N/A"

# (type:int)
def GDDRType_2_str(type):
    if type == 0xDD5A1001:
        return "Hynix-H56G32CS4DX005-1GB"
    if type == 0xDD5A1002:
        return "Hynix-H56G42AS4DX014-2GB"
    if type == 0xDD5A2001:
        return "Micron-MT61K512M32KPA_14C_16C-2GB"
    if type == 0xDD5A0001:
        return "Samsung-K4Z80325BC_HC14-1GB"
    if type == 0xDD5A0002:
        return "Samsung-K4ZAF32FBC_SC16-2GB"
    if type == 0xDD5A0003:
        return "Samsung-K4ZAF32FBC_SC20-2GB"
    else:
        return "N/A"

def is_host():
    try:
        with open("/proc/cpuinfo", "r") as file:
            cpuinfo_contents = file.read()
    except FileNotFoundError:
        print("/proc/cpuinfo file is not exist")
        sys.exit()

    if "hypervisor" in cpuinfo_contents:
        return 0
    else:
        return 1

class XdxGpu():
    # (self, index, user_index, handle) -> None
    def __init__(self, index, user_index, handle):
        self.drm_index = index         # card slot
        self.user_index = user_index   # for user
        self.handle = handle
        # # init static info
        # self.queryInfo = xdxmlqueryInfo_t()
        # self.queryInfo = xdxml_device_query_static_info(self.handle, self.queryInfo)
        self.file_head = "/sys/class/drm/card"
        self.query_info()
        #VPU default 
        self.vpu_vendor = 'N/A'
        self.enc_cmd_rev_fps = 'N/A'
        self.dec_cmd_rev_fps = 'N/A'
    
    # (self) -> None
    def query_info(self):
        #print("enter XdxGpu.query_info()")
        #xdxml_init()
        
        index = self.drm_index
        # queryInfo = xdxml_device_query(self.handle)
        queryInfo = xdxml_device_query_two_parts(self.handle)
        # 1: this machine is host, 0: guest
        self.is_host = is_host()
        # print("is_host:", self.is_host)
        # update dynamic info
        # queryInfo = xdxml_device_query_dynamic_info(self.handle, self.queryInfo)
        # self.queryInfo = queryInfo 

        #Yufei think index(id) doesn't match with handle(device address)
        # 2023-8-30: index has been updated to the true index of card 
        # self.drm_index = index
        # self.user_index = user_index      # for user
        #cur_card_path = "{0}{1}".format(self.file_head, index)
        #self.card = Node(cur_card_path)
        #self.debugfs = Node(f"/sys/kernel/debug/dri/{index}")
        #self.debugfs = Node(f"/sys/kernel/debug/xdxgpu")
        cur_device_path = "{0}{1}{2}".format(self.file_head, index, "/device")
        self.device = Node(cur_device_path)
        self.dsp_type = ''
        self.dsp_mode = 'disconnected'
        
        try:
            cur_status_path = "{0}{1}{2}".format(self.file_head, index, "/card*/status")
            display_status_paths = glob.glob(cur_status_path)
            for path in display_status_paths:
                with open(path, 'r') as fo:
                    status = fo.read()
                    # do not use queryInfo.dsp_mode
                    # which is the wrong result when DP/HDMI is more than 1
                    if status == 'connected\n':
                        self.dsp_mode = 'connected'
                        if self.dsp_type == '':
                            # the content in file node to store HDMI and DP status has a '\n' in the tail
                            if (path.find('DP') == -1):
                                self.dsp_type = 'HDMI'
                            elif (path.find('HDMI') == -1):
                                self.dsp_type = 'DP'
                            else:
                                self.dsp_type = ''                        
        except:
            self.dsp_mode = 'N/A'
        try:
            cur_hwmon_path = "{0}{1}{2}{3}".format(self.file_head, index, "/device/hwmon/", "hwmon*")
            hwmon_paths = []
            for path in glob.glob(cur_hwmon_path):
                if os.path.isdir(path):
                    hwmon_paths.append(path)
            
            self.hwmon = Node(hwmon_paths[0])
        except StopIteration:
            self.hwmon = None

        #GPU realted attributes xdx_vbios
        self.name = queryInfo.name.decode('utf-8') if queryInfo.name!=None else 'N/A'
        self.core = queryInfo.core if queryInfo.core != None else 'N/A'
        self.primary_id = '0x' + queryInfo.primary_id.decode('utf-8') if queryInfo.primary_id!=None else 'N/A'
        self.gddr_count = '0x' + str(queryInfo.gddr_count) if queryInfo.gddr_count!=None else 'N/A'
        self.gddr_speed = str(queryInfo.gddr_speed/1e9) + 'Gb/s' if queryInfo.gddr_speed!=0 else 'N/A'
        self.gddr_size = str(queryInfo.gddr_size) + 'GB' if queryInfo.gddr_size != None else 'N/A' 
        self.gddr_width = str(queryInfo.gddr_width) + 'bits' if queryInfo.gddr_width != None else 'N/A'
        self.gddr_effec = str(queryInfo.gddr_speed/1e6) + 'MT/s' if queryInfo.gddr_speed!=0 else 'N/A'
        self.gddr_type = queryInfo.gddr_type.decode('utf-8') if queryInfo.gddr_type!=None else 'N/A'
        self.gddr_package_model = str(GDDRType_2_str(queryInfo.gddr_package_model)) if queryInfo.gddr_package_model!=None else 'N/A'
        # the file node of pcie_mmbar_type has been discarded, the content may be empty
        self.pcie_mmbar_type = queryInfo.mmbar_type.decode('utf-8') if queryInfo.mmbar_type else 'N/A'
        self.asci_type = queryInfo.asci_type.decode('utf-8') if queryInfo.asci_type!=None else 'N/A'
        #self.dsp_mode = self.dsp_mode if self.dsp_mode!=None else 'N/A'
        self.acct_mode = 'N/A'
        self.uuid = '0x' + str(queryInfo.uuid.decode('utf-8')) if queryInfo.uuid!=None else 'N/A'
        self.minor_num = index

        try:
            self.vbios = queryInfo.vbios_version.decode('utf-8').split(' ',1)[0] if queryInfo.vbios_version!=None else 'N/A'
            #self.vbios = self.hwmon['xdx_vbios'].split(' ',1)[0] if self.hwmon['xdx_vbios']!=None else 'N/A'
        except:
            self.vbios = 'N/A'

        self.fm_ver = queryInfo.fw_ver.decode('utf-8') if queryInfo.fw_ver!=None else 'N/A'
        
        self.pci_bdf = queryInfo.pci_dbdf.decode('utf-8') if queryInfo.pci_dbdf!=None else 'N/A'
        self.domain = '0x' + self.pci_bdf.split(':',3)[0] if self.pci_bdf!='N/A' else 'N/A'
        self.bus = '0x' + self.pci_bdf.split(':',3)[1] if self.pci_bdf!='N/A' else 'N/A'
        self.dev = '0x' + self.pci_bdf.split(':',3)[2].split('.',1)[0] if self.pci_bdf!='N/A' else 'N/A'

        #fix PANGUA0POS-2141/2129,device id and others showed something like 0x1eedL
        self.device_ID = hex(queryInfo.device_ID).rstrip("lL") if queryInfo.device_ID!=None else 'N/A'
        self.vendor_ID = hex(queryInfo.vendor_ID).rstrip("lL") if queryInfo.vendor_ID!=None else 'N/A'
        self.subsystem_vendor_ID = hex(queryInfo.subsystem_vendor_ID).rstrip("lL") if queryInfo.subsystem_vendor_ID!=None else 'N/A'
        self.subsystem_device_ID = hex(queryInfo.subsystem_device_ID).rstrip("lL") if queryInfo.subsystem_device_ID!=None else 'N/A'    
       
        self.primary_core_id = queryInfo.primary_core_id.decode('utf-8') if queryInfo.primary_id!=None else 'N/A'

        self.pcie_event = queryInfo.pcie_event if queryInfo.pcie_event!=None else 'N/A'
        self.perstAssert = True if(self.pcie_event != "N/A" and queryInfo.perst_assert) else False
        self.perstDeAssert = True if(self.pcie_event != "N/A" and queryInfo.perst_de_assert) else False
        self.linkClkStable = True if(self.pcie_event != "N/A" and queryInfo.link_clk_stable) else False
        self.linkDown = True if(self.pcie_event != "N/A" and queryInfo.link_down) else False
        self.perstAssertAcpuAck = True if(self.pcie_event != "N/A" and queryInfo.perst_assert_acpu_ack) else False
        self.perstDeAssertAcpuAck = True if(self.pcie_event != "N/A" and queryInfo.perst_de_assert_acpu_ack) else False

        #PCIE Generation and Width  
        gen_max = self.device['max_link_speed']
        self.pcie_gen_max = queryInfo.max_link_gen if queryInfo.max_link_gen!=None else 'N/A'
        gen_curr = self.device['current_link_speed']
        self.pcie_gen_cur = str(queryInfo.curr_link_gen) if queryInfo.curr_link_gen!=None else 'N/A'

        self.throughput = gen_2_throughput(self.pcie_gen_cur)

        self.width_max = 'x' + str(round(queryInfo.max_pcie_link_width, 1)) if queryInfo.max_pcie_link_width!=None else 'N/A'
        self.width_cur = 'x' + str(queryInfo.curr_pcie_link_width) if queryInfo.curr_pcie_link_width!=None else 'N/A'

        #Acquire values for pice error related attributes
        if self.device['aer_dev_correctable']!=None:
            self.pcie_err_corr = queryInfo.err_corr.decode('utf-8')
        else:
            self.pcie_err_corr = 'N/A'

        if self.device['aer_dev_fatal']!=None:
            self.pcie_err_fatal = queryInfo.err_fatal.decode('utf-8')
        else:
            self.pcie_err_fatal = 'N/A'

        if self.device['aer_dev_nonfatal']!=None:
            self.pcie_err_nonfat = queryInfo.non_fatal.decode('utf-8')
        else:
            self.pcie_err_nonfat = 'N/A'
        self.support_fan_info = int(queryInfo.support_fan_info)
        if self.support_fan_info:
            # the file node of pwm1 has been discarded, cause it is redundant with fan_spd
            self.fan_spd = self.hwmon['fan1_input'] if self.hwmon['fan1_input']!=None else 'N/A'
        else :
            self.fan_spd = 'N/A'

        # hide fan info in VM
        self.fan_spd = self.fan_spd if self.is_host == 1 else 'N/A' 

        self.perform_state = 'N/A'
        self.clk_throt_reasons = 'N/A'
        #self.mem = read_mem_info()

        #GPU memory info
        self.fb_free = 'N/A' if queryInfo.fb_free =='N/A' else int(int(queryInfo.fb_free) / (1024*1024))
        self.fb_total = 'N/A' if queryInfo.fb_total =='N/A' else int(int(queryInfo.fb_total) / (1024*1024))
        self.fb_used = 'N/A' if self.fb_total=='N/A' or self.fb_free=='N/A' else int(int(int(queryInfo.fb_total) - int(queryInfo.fb_free)) / (1024*1024)) 
        
        #Host memory info
        self.sys_free = 'N/A' if queryInfo.sys_free =='N/A' else int(int(queryInfo.sys_free) / (1024*1024))
        self.sys_total = 'N/A' if queryInfo.sys_total =='N/A' else int(int(queryInfo.sys_total) / (1024*1024))
        self.sys_used = 'N/A' if self.sys_total=='N/A' or self.sys_free=='N/A' else int(self.sys_total - self.sys_free)

        self.mem_util = 'N/A'
        self.ecc_curr = 'N/A'
        self.ecc_pend = 'N/A'
        self.ecc_error = 'N/A'
        self.temp = str(queryInfo.temp) + str("°C") if queryInfo.temp is not None else 'N/A'
        # hide temp in VM
        self.temp = self.temp if self.is_host == 1 else 'N/A'
        self.pwr_read = 'N/A'
        self.gfx_clk = str(int(queryInfo.xdx_gfx_freq/1e6)) +'MHz' if queryInfo.xdx_gfx_freq!=None else 'N/A'
        self.acpu_clk = str(int(queryInfo.xdx_acpu_freq/1e6)) +'MHz' if queryInfo.xdx_acpu_freq!=None else 'N/A'
        #self.host_clk = str(int(self.hwmon['xdx_host_freq_mhz'])/1000000)+'MHZ' if self.hwmon['xdx_host_freq_mhz']!=None else 'N/A'
        self.ndf_clk = str(int(queryInfo.xdx_ndf_freq/1e6)) +'MHz' if queryInfo.xdx_ndf_freq!=None else 'N/A'
        # gddr_clock_frequency = gddr_speed / 8 GHz = gddr_speed / 8 * 1000 MHz
        self.gddr_clk = str(int(queryInfo.gddr_speed/1e9 /8 * 1000)) +'MHz' if queryInfo.gddr_speed!=None else 'N/A'
        # hide gfx clock and gddr clock in VM
        self.gfx_clk = self.gfx_clk if self.is_host == 1 else 'N/A' 
        self.gddr_clk = self.gddr_clk if self.is_host == 1 else 'N/A'

        self.vid_clk = 'N/A'
        self.app_clk = 'N/A'
        self.def_app_clk = 'N/A'
        self.sup_clk = 'N/A'

        self.chip_model = queryInfo.chip_model.decode('utf-8')

        #wcj

        self.bValid = True if(queryInfo.bValid !=0 ) else False

        self.ui64GpuStatActive = queryInfo.ui64GpuStatActive
        self.ui64GpuStatBlocked = queryInfo.ui64GpuStatBlocked
        self.ui64GpuStatIdle = queryInfo.ui64GpuStatIdle
        self.ui64GpuStatCumulative = queryInfo.ui64GpuStatCumulative

        self.aaui64DMOSStatActive_0 = queryInfo.aaui64DMOSStatActive_0
        self.aaui64DMOSStatBlocked_0 = queryInfo.aaui64DMOSStatBlocked_0
        self.aaui64DMOSStatIdle_0 = queryInfo.aaui64DMOSStatIdle_0
        self.aaui64DMOSStatCumulative_0 = queryInfo.aaui64DMOSStatCumulative_0

        self.aaui64DMOSStatActive_1 = queryInfo.aaui64DMOSStatActive_1
        self.aaui64DMOSStatBlocked_1 = queryInfo.aaui64DMOSStatBlocked_1
        self.aaui64DMOSStatIdle_1 = queryInfo.aaui64DMOSStatIdle_1
        self.aaui64DMOSStatCumulative_1 = queryInfo.aaui64DMOSStatCumulative_1
        # self.twoDTDMusage = 'N/A' if  queryInfo.aaui64DMOSStatCumulative_1 == 0 else format(queryInfo.aaui64DMOSStatActive_1*100/queryInfo.aaui64DMOSStatCumulative_1,'.3f')+'%'
        self.twoDTDMusage = '0' if queryInfo.util_tdm == 0 else str(math.ceil(queryInfo.util_tdm)) + '%'

        self.aaui64DMOSStatActive_2 = queryInfo.aaui64DMOSStatActive_2
        self.aaui64DMOSStatBlocked_2 = queryInfo.aaui64DMOSStatBlocked_2
        self.aaui64DMOSStatIdle_2 = queryInfo.aaui64DMOSStatIdle_2
        self.aaui64DMOSStatCumulative_2 = queryInfo.aaui64DMOSStatCumulative_2
        # self.GEOMMusage = 'N/A' if  queryInfo.aaui64DMOSStatCumulative_2 == 0 else format(queryInfo.aaui64DMOSStatActive_2*100/queryInfo.aaui64DMOSStatCumulative_2,'.3f')+'%'
        self.GEOMMusage = '0' if queryInfo.util_geom == 0 else str(math.ceil(queryInfo.util_geom)) + '%'

        self.aaui64DMOSStatActive_3 = queryInfo.aaui64DMOSStatActive_3
        self.aaui64DMOSStatBlocked_3 = queryInfo.aaui64DMOSStatBlocked_3
        self.aaui64DMOSStatIdle_3 = queryInfo.aaui64DMOSStatIdle_3
        self.aaui64DMOSStatCumulative_3 = queryInfo.aaui64DMOSStatCumulative_3
        # self.threeDMusage = 'N/A' if  queryInfo.aaui64DMOSStatCumulative_3 == 0 else format(queryInfo.aaui64DMOSStatActive_3*100/queryInfo.aaui64DMOSStatCumulative_3,'.3f')+'%'
        self.threeDMusage = '0' if queryInfo.util_thrd == 0 else str(math.ceil(queryInfo.util_thrd)) + '%'

        self.aaui64DMOSStatActive_4 = queryInfo.aaui64DMOSStatActive_4
        self.aaui64DMOSStatBlocked_4 = queryInfo.aaui64DMOSStatBlocked_4
        self.aaui64DMOSStatIdle_4 = queryInfo.aaui64DMOSStatIdle_4
        self.aaui64DMOSStatCumulative_4 = queryInfo.aaui64DMOSStatCumulative_4
   
        self.aaui64DMOSStatActive_5 = queryInfo.aaui64DMOSStatActive_5
        self.aaui64DMOSStatBlocked_5 = queryInfo.aaui64DMOSStatBlocked_5
        self.aaui64DMOSStatIdle_5 = queryInfo.aaui64DMOSStatIdle_5
        self.aaui64DMOSStatCumulative_5 = queryInfo.aaui64DMOSStatCumulative_5
   
        self.aaui64DMOSStatActive_6 = queryInfo.aaui64DMOSStatActive_6
        self.aaui64DMOSStatBlocked_6 = queryInfo.aaui64DMOSStatBlocked_6
        self.aaui64DMOSStatIdle_6 = queryInfo.aaui64DMOSStatIdle_6
        self.aaui64DMOSStatCumulative_6 = queryInfo.aaui64DMOSStatCumulative_6
    
        self.aaui64DMOSStatActive_7 = queryInfo.aaui64DMOSStatActive_7
        self.aaui64DMOSStatBlocked_7 = queryInfo.aaui64DMOSStatBlocked_7
        self.aaui64DMOSStatIdle_7 = queryInfo.aaui64DMOSStatIdle_7
        self.aaui64DMOSStatCumulative_7 = queryInfo.aaui64DMOSStatCumulative_7

        self.aaui64DMOSStatActive_8 = queryInfo.aaui64DMOSStatActive_8
        self.aaui64DMOSStatBlocked_8 = queryInfo.aaui64DMOSStatBlocked_8
        self.aaui64DMOSStatIdle_8 = queryInfo.aaui64DMOSStatIdle_8
        self.aaui64DMOSStatCumulative_8 = queryInfo.aaui64DMOSStatCumulative_8

        self.ui64TimeStamp = queryInfo.ui64TimeStamp

        self.pronum = queryInfo.pronum

        self.pro1PID = queryInfo.process_1        
        self.pro1LMA = format(float(queryInfo.pro1LMA)/(1024*1024),'.3f')
        self.pro1Total = format(float(queryInfo.pro1Total)/(1024*1024),'.3f')

        self.pro2PID = queryInfo.process_2        
        self.pro2LMA = format(float(queryInfo.pro2LMA)/(1024*1024),'.3f')
        self.pro2Total = format(float(queryInfo.pro2Total)/(1024*1024),'.3f')

        self.pro3PID = queryInfo.process_3        
        self.pro3LMA = format(float(queryInfo.pro3LMA)/(1024*1024),'.3f')
        self.pro3Total = format(float(queryInfo.pro3Total)/(1024*1024),'.3f')

        self.pro4PID = queryInfo.process_4        
        self.pro4LMA = format(float(queryInfo.pro4LMA)/(1024*1024),'.3f')
        self.pro4Total = format(float(queryInfo.pro4Total)/(1024*1024),'.3f')

        self.pro5PID = queryInfo.process_5        
        self.pro5LMA = format(float(queryInfo.pro5LMA)/(1024*1024),'.3f')
        self.pro5Total = format(float(queryInfo.pro5Total)/(1024*1024),'.3f')

        self.pro1NAME = queryInfo.pro1NAME.decode('utf-8')
        self.pro2NAME = queryInfo.pro2NAME.decode('utf-8')
        self.pro3NAME = queryInfo.pro3NAME.decode('utf-8')
        self.pro4NAME = queryInfo.pro4NAME.decode('utf-8')
        self.pro5NAME = queryInfo.pro5NAME.decode('utf-8')

        self.support_power_info = int(queryInfo.support_power_info)

        # gpu_util: docker need
        self.util_gpu = str(math.ceil(queryInfo.util_gpu)) + '%' if queryInfo.util_gpu != None else 'N/A'
        # sum the data shown in front end: used_fb is the sum of LMA
        # self.used_GPU_memory = round(float(self.pro1LMA) + float(self.pro2LMA) + float(self.pro3LMA) + float(self.pro4LMA) + float(self.pro5LMA))
        # self.used_GPU_memory = self.fb_used if int(self.pronum) >= 5 else self.used_GPU_memory
        self.used_GPU_memory = self.fb_used
        
        self.free_GPU_memory = round(float(self.fb_total) - self.used_GPU_memory)

        self.device_id_num = queryInfo.device_id if queryInfo.device_id != None else 'N/A'

        # gddr crc info
        self.crc_write_cnt = queryInfo.crc_write_cnt if queryInfo.crc_write_cnt != None else 'N/A'
        self.crc_read_cnt = queryInfo.crc_read_cnt if queryInfo.crc_read_cnt != None else 'N/A'
        self.crc_write_retry_cnt = queryInfo.crc_write_retry_cnt if queryInfo.crc_write_retry_cnt != None else 'N/A'
        self.crc_read_retry_cnt = queryInfo.crc_read_retry_cnt if queryInfo.crc_read_retry_cnt != None else 'N/A'

    # (self)->None
    def update_dynamic_info(self):
        self.query_info()

    # (self)->None
    def update_vpu_static_info(self):
        # 获取vpu信息，在xdxml_init已经初始化了vpu信息了，这里直接拷贝，因为是静态信息，所以在构造函数里面获取
        vpu_codec_info = xdxml_device_get_vpu_info(self.handle)
        self.vpu_vendor = vpu_codec_info.vender_name.decode('utf-8') if vpu_codec_info.vender_name!=None else 'N/A'

    # (self)->None
    def update_vpu_ultilization_dynamic(self):
        enc_cmd_rev_fps, enc_cmd_exc_fps, dec_cmd_rev_fps, dec_cmd_exc_fps = xdxml_device_get_vpu_utlz(self.handle)
        self.enc_cmd_rev_fps = enc_cmd_rev_fps if enc_cmd_rev_fps != -1 else 'N/A'
        self.dec_cmd_rev_fps = dec_cmd_rev_fps if dec_cmd_rev_fps != -1 else 'N/A'

    def __setitem__(self, name, val):
        return setattr(self, name, val)

    def __getitem__(self, name):
        return getattr(self, name)
