# -*- coding: utf-8 -*-

from __future__ import print_function

# from typing import Dict
from sys import exit
import time
import sys
import traceback
import os
import atexit

#insert the path (if not exist) first instead of doing it after a failed import, as the cached module might cause failure again
#another way is re-org the files to make the path insertion unnecessary
from os.path import abspath, join, dirname

current_file = os.path.abspath(__file__)
project_root = os.path.dirname(os.path.dirname(os.path.dirname(current_file)))
if project_root not in [os.path.abspath(p) for p in sys.path]:
    sys.path.insert(0, project_root)    

try:
    import xdxsmi_config
    from xdxsmi_render import Render
    from xdxsmi.api.smi import XdxSmi
    from xdxsmi.api.xdxgpu import XdxGpu
    from xdxsmi.api.out_of_band_gpu_info import XdxGpuInfo
    from xdxsmi.api.third_party_docopt import docopt
    from xdxsmi.api.pyxdxml import xdxml_shutdown    
except ImportError:
    sys.path.insert(0, project_root)

    import xdxsmi_config
    from xdxsmi_render import Render
    from xdxsmi.api.smi import XdxSmi
    from xdxsmi.api.xdxgpu import XdxGpu
    from xdxsmi.api.out_of_band_gpu_info import XdxGpuInfo
    from xdxsmi.api.third_party_docopt import docopt
    from xdxsmi.api.pyxdxml import xdxml_shutdown
    

__doc__ = """XDXSMI - XDX System Management Interface
Version: {version}

XDXSMI provides monitoring information for XDX devices.
Its output is presented in plain text, via stdout or a file.
XDXSMI also provides several operations for changing device states.

Note that XDXSMI usually works when XDXGPU driver is loaded.
Without XDXGPU driver, only 'xdxsmi out' is accessible with a root authority. 

usage:
    xdxsmi [<command>] [<args>...] [options]
    
commands:
   clock          Control and query clock information of GPUs
   temp           Control and query temperature information of GPUs
   power          Control and query power information of GPUs
   fan            Control and query fan information of GPUs
   thermal        Control and query thermal information of GPUs
   dvfs           Control and query DVFS infomation of GPUs
   list           List all GPUs with their PCI info
   query          Query GPUs' attributes
   stats          show statistic information about GPUs
   out            Show GPU info by out of band manager way
   vpuinfo        show the VPU info   
   query_gpu      show some of GPU info
   pmon           monitor the GPU resources consumed by processes 
   vpumon         monitor the performace data related to VPU

options:
    --version                print version of this tool
    -h, --help               print this help message
    -f, --filename=<file>    redirect output to a specified file
    -l, --loop=<interval>    set query interval in seconds
    -i, --index=<idx>        index number of target GPU, starting from 0
    


Run 'xdxsmi <command> -h' for more information on a specific command.
Run 'xdxsmi' without any options or commands will print a general GPU status.
""".format(version=xdxsmi_config.GlobalConfig.__version__)


def loop_argument_parse(args):
    loop_para = args.get("--loop");
    loop_interval = -1
    try:
        if(loop_para != None):
            if(float(args['--loop']) > 0):
                loop_interval = float(args["--loop"])
            else:
                Render.render_content("Invalid input for --loop: '{}'. A value > 0 is expected. This parameter will be ignored.\n".format(args['--loop']), args) 
    except Exception as e:
        Render.render_content("Invalid input for --loop: '{}'. A value > 0 is expected. This parameter will be ignored.\n".format(args['--loop']), args) 
#        sys.exit()
#    if(loop_para!=None):
#        if isinstance(args['--loop'],float) and args(float(args['--loop']) > 0):
#            loop_interval = float(args["--loop"])
#        else:
#            Render.render_content("Invalid input for --loop: '{}'. A value > 0 is expected. This parameter will be ignored.\n".format(args['--loop']), args) 
    return loop_interval

def loop_argument_check(loop_support_flag, args):
    loop_para = args.get("--loop");

    if((not loop_support_flag) and (loop_para != None)):
        Render.render_content("parameter '-l|--loop' will be ignored.\n", args)

def index_argument_parse(xdx_smi, args):
    index_num = -1
    try:
        num_xdxgpus = len(xdx_smi.xdxgpus)
        num_xdxgpus2 = len(xdx_smi.handle)
        
        if(num_xdxgpus == 0 or num_xdxgpus2 == 0):
            raise Exception("No devices were found")
            
        if(args['--index'] == None):
            index_num = -1                    # default situation
        elif(args['--index'] != None and args['--index'].isdigit() and int(args['--index']) >= 0 and int(args['--index']) < num_xdxgpus):
            index_num = int(args['--index'])  # assign index
        else:
            Render.render_content("Invalid input: '--index: {}'. A value between [0, {}) is expected.\nAll detected GPUs will be used.\n".format(args['--index'], num_xdxgpus), args) 
            args['--index'] = -1
    except Exception as e:
        sys.exit("Exception: {0} \nPlease try another way or contact the software provider.".format(e))        
                
    return index_num


if __name__ == '__main__': 
    
    args = docopt(__doc__, version=xdxsmi_config.GlobalConfig.__version__, options_first=True)
    processing_doc = __doc__
    
    # for user, bus invisible
    try:
        # e.g. xdxsmi -f=a.txt clock get
        if(args['--filename'] != None):
            args['<args>'] += ['--filename=' + args['--filename']]

        # e.g. xdxsmi -l=5 clock get
        if(args['--loop'] != None):
            args['<args>'] += ['--loop=' + args['--loop']]

        # e.g. xdxsmi -i=0 clock get
        if(args['--index'] != None):
            args['<args>'] += ['--index=' + args['--index']]

        #"<command>" is out
        if args['<command>'] == 'out':
            import xdxsmi_out
            processing_doc = xdxsmi_out.__doc__
            
            xdx_smi = XdxSmi(True)

            argv = [args['<command>']] + args['<args>']
            args = docopt(xdxsmi_out.__doc__, argv=argv)

            if xdx_smi.is_init == 1:
                atexit.register(xdxml_shutdown)

            xdxsmi_out.handle(xdx_smi, args)
        else:
            xdx_smi =  XdxSmi()

            loop_support_flag = 1       #default not support loop
            loop_in_handle_flag = 0     #each module handle loop itself
            loop_func = None            #keep handle func for loop case
            loop_parameter = xdx_smi    #keep parameter for loop case        
            loop_interval = -1

            #"<command>" is blank
            if not args["<command>"]: 
                argv = args['<args>']
            else:
                argv = [args['<command>']] + args['<args>']
            
            if xdx_smi.is_init == 1:
                atexit.register(xdxml_shutdown)

            #"<command>" is blank
            if not args["<command>"]:
                import xdxsmi_blank
                processing_doc = xdxsmi_blank.__doc__
                args = docopt(xdxsmi_blank.__doc__, argv=argv)
                loop_interval = loop_argument_parse(args)
                loop_func = xdxsmi_blank.handle                
                xdxsmi_blank.handle(xdx_smi, args)
                
            #"<command>" is clock
            elif args['<command>'] == 'clock':
                import xdxsmi_clock
                processing_doc = xdxsmi_clock.__doc__
                args = docopt(xdxsmi_clock.__doc__, argv=argv)
                index_argument_parse(xdx_smi, args)
                loop_interval = loop_argument_parse(args)
                loop_func = xdxsmi_clock.handle
                xdxsmi_clock.handle(xdx_smi, args)
                
            #"<command>" is power        
            elif args['<command>'] == 'power':
                import xdxsmi_power
                processing_doc = xdxsmi_power.__doc__
                args = docopt(xdxsmi_power.__doc__, argv=argv)
                index_argument_parse(xdx_smi, args)
                loop_interval = loop_argument_parse(args)
                loop_func = xdxsmi_power.handle
                if args["set"]:
                    loop_support_flag = 0
                    loop_argument_check(loop_support_flag, args)
                               
                xdxsmi_power.handle(xdx_smi, args)
               
            #"<command>" is temp
            elif args['<command>'] == 'temp':
                import xdxsmi_temp
                processing_doc = xdxsmi_temp.__doc__
                args = docopt(xdxsmi_temp.__doc__, argv=argv)
                index_argument_parse(xdx_smi, args)
                loop_interval = loop_argument_parse(args)
                loop_func = xdxsmi_temp.handle
                xdxsmi_temp.handle(xdx_smi, args) 
                
            #"<command>" is util
            elif args['<command>'] == 'util':
                import xdxsmi_utility
                processing_doc = xdxsmi_utility.__doc__
                args = docopt(xdxsmi_utility.__doc__, argv=argv)
                index_argument_parse(xdx_smi, args)
                loop_interval = loop_argument_parse(args)
                loop_func = xdxsmi_utility.handle
                xdxsmi_utility.handle(xdx_smi, args)
               
            #"<command>" is fan    
            elif args['<command>'] == 'fan':
                import xdxsmi_fan
                processing_doc = xdxsmi_fan.__doc__
                args = docopt(xdxsmi_fan.__doc__, argv=argv)
                index_argument_parse(xdx_smi, args)
                loop_interval = loop_argument_parse(args)
                loop_func = xdxsmi_fan.handle
                if args["set"]:
                    loop_support_flag = 0
                    loop_argument_check(loop_support_flag, args)

                xdxsmi_fan.handle(xdx_smi, args)
  
            #"<command>" is list        
            elif args['<command>'] == 'list':
                import xdxsmi_list
                processing_doc = xdxsmi_list.__doc__
                args = docopt(xdxsmi_list.__doc__, argv=argv)
                loop_support_flag = 0
                loop_argument_check(loop_support_flag, args)
                xdxsmi_list.handle(xdx_smi, args)
                
            #"<command>" is set   
            elif args['<command>'] == 'set':
                import xdxsmi_set
                processing_doc = xdxsmi_set.__doc__
                args = docopt(xdxsmi_set.__doc__, argv=argv)
                index_argument_parse(xdx_smi, args)
                loop_support_flag = 0
                loop_argument_check(loop_support_flag, args)
                xdxsmi_set.handle(xdx_smi, args)
                
            #"<command>" is query    
            elif args['<command>'] == 'query':
                import xdxsmi_query
                processing_doc = xdxsmi_query.__doc__
                args = docopt(xdxsmi_query.__doc__, argv=argv)
                index_argument_parse(xdx_smi, args)
                loop_interval = loop_argument_parse(args)
                loop_func = xdxsmi_query.handle
                xdxsmi_query.handle(xdx_smi, args)
                
            #"<command>" is pmon
            elif args['<command>'] == 'pmon':
                import xdxsmi_pmon
                processing_doc = xdxsmi_pmon.__doc__
                args = docopt(xdxsmi_pmon.__doc__, argv=argv)
                index_argument_parse(xdx_smi, args)
                #since subcommand itself will keep querying process information every x seconds, not to loop here in main
                #"pmon" will use the --loop parameter
                loop_support_flag = 0             
                xdxsmi_pmon.handle(xdx_smi, args)
                
            #"<command>" is stats    
            elif args['<command>'] == 'stats':
                import xdxsmi_stats
                processing_doc = xdxsmi_stats.__doc__
                args = docopt(xdxsmi_stats.__doc__, argv=argv)
                index_argument_parse(xdx_smi, args)
                loop_interval = loop_argument_parse(args)
                loop_func = xdxsmi_stats.handle
                xdxsmi_stats.handle(xdx_smi, args)
                
            #"<command>" is vpuinfo   
            elif args['<command>'] == 'vpuinfo':
                if not xdx_smi.isVpufunctionValid:  # If there is an exception in the libva library, it will not be executed.
                    loop_support_flag = 0
                    loop_argument_check(loop_support_flag, args)
                    Render.render_content("This function is invalid because the xdxgpu driver is not installed correctly!\n", args)                     
                else:
                    import xdxsmi_vpu_info
                    processing_doc = xdxsmi_vpu_info.__doc__
                    args = docopt(xdxsmi_vpu_info.__doc__, argv=argv)
                    index_argument_parse(xdx_smi, args)
                    loop_interval = loop_argument_parse(args)
                    loop_func = xdxsmi_vpu_info.handle 
                    xdxsmi_vpu_info.handle(xdx_smi, args)
                    
            #"<command>" is vpumon         
            elif args['<command>'] == 'vpumon':
                if not xdx_smi.isVpufunctionValid:  # If there is an exception in the libva library, it will not be executed.
                    loop_support_flag = 0 
                    loop_argument_check(loop_support_flag, args)
                    Render.render_content("This function is invalid because the xdxgpu driver is not installed correctly!\n", args)  
                else:                    
                    import xdxsmi_vpumon
                    processing_doc = xdxsmi_vpumon.__doc__
                    args = docopt(xdxsmi_vpumon.__doc__, argv=argv)
                    index_argument_parse(xdx_smi, args)
                    
                    loop_support_flag = 0  
                    xdxsmi_vpumon.handle(xdx_smi, args)
                    
            #"<command>" is thermal        
            elif args['<command>'] == 'thermal':
                import xdxsmi_thermal
                processing_doc = xdxsmi_thermal.__doc__
                args = docopt(xdxsmi_thermal.__doc__, argv=argv)
                index_argument_parse(xdx_smi, args)
                loop_interval = loop_argument_parse(args)
                loop_func = xdxsmi_thermal.handle
                if args["set"]:
                    loop_support_flag = 0
                    loop_argument_check(loop_support_flag, args)
                
                xdxsmi_thermal.handle(xdx_smi, args)             
               
            #"<command>" is dvfs        
            elif args['<command>'] == 'dvfs':
                import xdxsmi_dvfs
                processing_doc = xdxsmi_dvfs.__doc__
                args = docopt(xdxsmi_dvfs.__doc__, argv=argv)
                index_argument_parse(xdx_smi, args)
                loop_interval = loop_argument_parse(args)
                loop_func = xdxsmi_dvfs.handle
                if args["set"] or args["show"]:
                    loop_support_flag = 0
                    loop_argument_check(loop_support_flag, args)
                
                xdxsmi_dvfs.handle(xdx_smi, args)
                           
            #"<command>" is query_gpu
            elif args['<command>'] == 'query_gpu':
                import xdxsmi_query_gpu_info
                processing_doc = xdxsmi_query_gpu_info.__doc__
                args = docopt(xdxsmi_query_gpu_info.__doc__, argv=argv)
                index_argument_parse(xdx_smi, args)
                loop_interval = loop_argument_parse(args)
                loop_func = xdxsmi_query_gpu_info.handle
                xdxsmi_query_gpu_info.handle(xdx_smi, args)            
                
            else:
                sys.exit("ERROR: Bad command format. Please run 'xdxsmi -h' for help.\n")

            try:
                while loop_support_flag:
                    if(loop_interval <= 0):
                        sys.exit()
                    time.sleep(loop_interval)
                    loop_func(xdx_smi, args)
            except KeyboardInterrupt:            
                sys.exit()
    except AttributeError:
        sys.exit("Exception: Bad command format. Please refer to the help document below.\n\n{doc}"\
                .format(doc=processing_doc))

    except Exception as e:         
        with open('/tmp/smiError.log', 'w+') as f:
            f.write(traceback.format_exc())
        os.chmod('/tmp/smiError.log', 0o666)
        sys.exit("Exception: {0} \nPlease try another way or contact the software provider.".format(e)) 
        #sys.exit("Exception: {0} \nPlease try another way or contact the software provider.".format(traceback.format_exc())) 

  
