# -*- coding: utf-8 -*-
"""
usage:
    xdxsmi [options]
    
options:
    -f, --filename=<file>           redirect output to a specified file    
    -l, --loop=<interval>           set query interval in seconds
"""
import sys
from xdxsmi_render import Render
from xdxsmi.api.smi import XdxSmi
from xdxsmi.api.xdxgpu import XdxGpu
from xdxsmi.api.out_of_band_gpu_info import XdxGpuInfo
from xdxsmi.api.third_party_docopt import docopt
from xdxsmi.api.pyxdxml import xdxml_shutdown


# (xdx_smi: XdxSmi, args: Dict)
def show_xdxgpu_status_table_header(xdx_smi, args):
    smi_ver = xdx_smi.smi_ver
    ver = xdx_smi.vers
    xdxgpu_status_table_header = \
"""
+--------------------------------------------------------------------------------------------------+
| XDXSMI {smi_ver:12}                  KMD Version {ver[2]:11}           UMD Version {ver[3]:11}   |
+-------------------------------------+---------------------------------+--------------------------+
| GPU      Name       CHIP-INFO       | Bus-Id                   Disp.A |    Volatile Uncorr. ECC  |
| Fan      Temp       Pwr:Usage/Cap   |                    Memory-Usage |                 GPU-Util |
|                     Vpu-Driver-Name |           Vpu-Encoder-Util(fps) |    Vpu-Decoder-Util(fps) |
|                                     |                                 |                          |
|=====================================+=================================+==========================|""".format(smi_ver=smi_ver, ver=ver)

    Render.render_content(xdxgpu_status_table_header, args, end='')    

# (xdxgpu: XdxGpu, args: Dict)
def show_xdxgpu_status_table_body(xdxgpu, args):
    idx = xdxgpu.user_index
    x0_info = xdxgpu.asci_type.split("_")[-1] if len(xdxgpu.asci_type) > 1 else "X0"
    name = xdxgpu.name + " " + x0_info  #PANGU A0/B0
    bsid = xdxgpu.pci_bdf    
    chip_info = xdxgpu.chip_model 
    
    # xdxgpu.dsp_mode #connected or disconnected or N\A
    # xdxgpu.dsp_type # HDMI or DP
    dsp_info = xdxgpu.dsp_type + " " + xdxgpu.dsp_mode
    dsp_info_len = len(str(dsp_info))
    
    if xdxgpu.fan_spd=="0":        
        fan = 'N/A'
    else:
        fan = xdxgpu.fan_spd        
    fan = str(fan) + "RPM"
    
    temp = xdxgpu.temp
    temp_l = len(str(temp))
    temp_space_len = 11 - temp_l if temp_l < 11 else 1
    #temp = tmp_temp + str("°C")    
    if (sys.version_info[0] == 2): # for python2
        temp_space_len += 1

    # processes in multi-card environments only exists on card 0
    used_GPU_memory = xdxgpu.used_GPU_memory if xdxgpu.user_index == 0 else xdxgpu.fb_used
    used_GPU_memory_len = len(str(used_GPU_memory))
    fb_total = xdxgpu.fb_total
    fb_total_len = len(str(xdxgpu.fb_total))
    mem_use_space_len = 25 - used_GPU_memory_len - fb_total_len
    util_gpu = xdxgpu.util_gpu
    vpu_vendor = '{:>35}'.format(xdxgpu.vpu_vendor) if len(xdxgpu.vpu_vendor) <= 35 else xdxgpu.vpu_vendor[:32] + "..."
    
    enc_cmd_rev_fps = xdxgpu.enc_cmd_rev_fps
    dec_cmd_rev_fps = xdxgpu.dec_cmd_rev_fps

    NA = 'N/A'
    #ellipsis = '...'
    space = ' '
    bsid_l = 12    #for bus id:0000:01:00.0    
    
    dsp_space_len = 31 - bsid_l - dsp_info_len if 31 > (bsid_l + dsp_info_len) else 1
    
    xdxgpu_status_table_body = ""    

    xdxgpu_status_table_body = \
"""
| {idx:<2}{space:7}{name:11}{chip_info:16}| {bsid:{bsid_l}}{space:{dsp_space_len}}{dsp_info:{dsp_info_len}} | {space:21}{NA} |
| {fan:9}{temp:{temp_l}}{space:{temp_space_len}}{NA:16}|{space:{mem_use_space_len}}{used_GPU_memory:{used_GPU_memory_len}}MiB/{fb_total:{fb_total_len}}MiB |{util_gpu: >25} |
| {vpu_vendor:35} |{enc_cmd_rev_fps:>32} |{dec_cmd_rev_fps:>25} |
|                                     |                                 |                          |
+-------------------------------------+---------------------------------+--------------------------+\
""".format(
            idx = idx, name = name, NA = NA, chip_info = chip_info, bsid = bsid, bsid_l = bsid_l, dsp_info=dsp_info, dsp_space_len=dsp_space_len,
            dsp_info_len=dsp_info_len, space = space, fan = fan, temp = temp, temp_l =temp_l, temp_space_len=temp_space_len, mem_use_space_len=mem_use_space_len,
            used_GPU_memory=used_GPU_memory, used_GPU_memory_len=used_GPU_memory_len, fb_total=fb_total, fb_total_len=fb_total_len, 
            util_gpu = util_gpu, vpu_vendor = vpu_vendor, enc_cmd_rev_fps = enc_cmd_rev_fps, dec_cmd_rev_fps = dec_cmd_rev_fps
          )

    Render.render_content(xdxgpu_status_table_body, args, end=' ')  


# (xdx_smi: XdxSmi, args: Dict)
def show_xdxgpu_status_table_process_header(xdx_smi, args):
    xdxgpu_status_table_process_header = \
"""
+--------------------------------------------------------------------------------------------------+
| top-5 Processes: [Wait to finish]                                                                |
| GPU      PID        Type        Process name                                    GPU Memory Usage |
|==================================================================================================|"""
    Render.render_content(xdxgpu_status_table_process_header, args) 
    
    
# (xdxgpu: XdxGpu, args: Dict)
def show_xdxgpu_status_table_process_body(xdxgpu, args):
    NA = 'N/A'
    space = ' '
    blank =''

    process_number = xdxgpu.pronum + 1
    # default_body = f"""|   {xdxgpu.user_index: >2}{' ': <3}{'N/A': <5}{'N/A': <5}{'': <4}{'N/A': <8}{'N/A': <3}{'': <3}{'N/A': <11}{' ': <30}{'N/A': >8}{' ': <4}|"""
    default_body = \
"""| {user_index: <2}{space: <7}{NA: <11}{NA: <12}{NA: <38}{NA: >26}{space:1}|
""".format(user_index = xdxgpu.user_index, space = space, NA = NA, blank = blank)

    xdxgpu_status_table_process_body = default_body if process_number == 1 else ''

    # Count the number of processes with non-zero fb memory
    non_zero_process = 0
    for i in range(1, process_number):
        attr_name = "pro{}LMA".format(i)
        if(float(getattr(xdxgpu, attr_name)) != 0):
            non_zero_process += 1
    
    for i in range(1, process_number):
        # gpu_id = xdxgpu.user_index if xdxgpu.user_index != None else 'N/A'
        gpu_id = 0
        attr_name = "pro{}PID".format(i)
        pid = getattr(xdxgpu, attr_name) if getattr(xdxgpu, attr_name) != None else 'N/A'
        
        type = 'N/A'
        attr_name = "pro{}NAME".format(i)
        p_name = getattr(xdxgpu, attr_name) if getattr(xdxgpu, attr_name) != None else 'N/A'
        
        attr_name = "pro{}LMA".format(i)
        g_usage = getattr(xdxgpu, attr_name) if getattr(xdxgpu, attr_name) != None else 'N/A'

        if(float(g_usage) == 0):
            continue
        else:
            g_usage = str(g_usage) + "MiB"

        line = \
"""| {gpu_id: <2}{space: <7}{pid: <11}{NA: <12}{p_name: <38}{g_usage: >26}{space:1}|
""".format(gpu_id = gpu_id, space = space, NA = NA, blank = blank, pid = pid, p_name = p_name, g_usage = g_usage)
        if(non_zero_process != 1):
            line += ''
        xdxgpu_status_table_process_body += line
        non_zero_process -= 1

    # option scheme: seperate processes by gpu number, if so, do not use table foot
    # xdxgpu_status_table_process_body += "\n+-----------------------------------------------------------------------------------------+"
    # only show processes of gpu 0
    xdxgpu_status_table_process_body = default_body if xdxgpu.user_index != 0 else xdxgpu_status_table_process_body
    Render.render_content(xdxgpu_status_table_process_body, args) 
    

# (xdx_smi: XdxSmi, args: Dict)
def show_xdxgpu_status_table_foot(xdx_smi, args):
    xdxgpu_status_table_foot = "+--------------------------------------------------------------------------------------------------+"
    Render.render_content(xdxgpu_status_table_foot, args) 


# (xdx_smi: XdxSmi, args: Dict)
def handle(xdx_smi, args):
    try:
        target_gpus = xdx_smi.xdxgpus
        
        # The static information of the VPU can be obtained first.
        # Moreover, the video team hasn't modified the UMD code yet, which will print redundant information and affect the display effect,
        # especially when multiple cards are in use.
        if not xdx_smi.isVpufunctionValid:
            outStr = "Vpu query are not allowed because the xdxgpu driver is not installed correctly!\n"
            Render.render_content(outStr, args, end="")                        
        else:
            for xdxgpu in target_gpus:
                xdxgpu.update_vpu_static_info()

        
        show_xdxgpu_status_table_header(xdx_smi, args)
        for xdxgpu in target_gpus:
            if xdx_smi.isVpufunctionValid:  #VPU utilization depends on libva library
                xdxgpu.update_vpu_ultilization_dynamic()
            show_xdxgpu_status_table_body(xdxgpu, args)

        # running processes get by ioctl are the whole processes
        # and cannot disginguished by gpu, so here show process once
        target_gpus = [target_gpus[0]]
        show_xdxgpu_status_table_process_header(xdx_smi, args)
        for xdxgpu in target_gpus:
            # each addition run time of thie function, the memory of process python3 adds 0.001MiB
            # till here, the process_memory has been computed, if update info again, the sum of the process fb memory is incorrect
            # xdxgpu.update_dynamic_info()
            show_xdxgpu_status_table_process_body(xdxgpu, args)

        show_xdxgpu_status_table_foot(xdx_smi, args)
    except Exception as e:        
        sys.exit("Exception: {0} \nPlease try another way or contact the software provider.".format(e)) 
  


if __name__ == "__main__":
    print(docopt(__doc__))
  
