"""
usage:
    xdxsmi out [options]

options:
    -h, --help                    print this help message
    -f, --filename=<file>         redirect output to a specified file
"""

import os
# from typing import Dict

from xdxsmi_render import Render
from xdxsmi_attrs_ops import setAttrs, getAttrs
from xdxsmi.api.third_party_docopt import docopt
from xdxsmi.api.xdxgpu import XdxGpu
from xdxsmi.api.out_of_band_gpu_info import XdxGpuInfo
from xdxsmi.api.smi import XdxSmi
from xdxsmi.api.pyxdxml import is_out_of_band

# (gpu: XdxGpuInfo, args: Dict)
def printAttributes(gpu, args):    
    gpuAttrs = """\
    GPU {user_index}
        Product Name                : {name}
        Chip Model                  : {chip_model}
        Temperature                 : {temp}
        VBIOS Version               : {vbios}
        FW Version                  : {fm_ver}
        PCI
            Bus                     : {bus}
            Device                  : {dev}
            Domain                  : {domain}
            Bus Id                  : {pci_bdf}
            Vendor Id               : {vendor_ID}
            Device Id               : {device_ID}
            Subsystem Vendor ID     : {subsystem_vendor_ID}
            Subsystem ID            : {subsystem_device_ID}

        FB Memory Usage(MiB)
            Total                   : {fb_total}
        Power
            Volt                    : {volt}
            Current                 : {current}
            Pwr                     : {pwr}
            Pwr Total               : {pwr_total}
        Fan Speed[RPM]              : {fan_spd}
        Clocks
            Graphics                : {gfx_clk}\n""".format(user_index=gpu.user_index, name=gpu.name, chip_model=gpu.chip_model, temp=gpu.temp, vbios=gpu.vbios,
                                                                fm_ver=gpu.fm_ver, bus=gpu.bus, dev=gpu.dev, domain=gpu.domain, pci_bdf=gpu.pci_bdf, 
                                                                vendor_ID=gpu.vendor_ID, device_ID=gpu.device_ID, subsystem_vendor_ID=gpu.subsystem_vendor_ID,
                                                                subsystem_device_ID=gpu.subsystem_device_ID, fb_total=gpu.fb_total, volt=gpu.volt, current=gpu.current,
                                                                pwr=gpu.pwr, pwr_total=gpu.pwr_total, fan_spd=gpu.fan_spd, gfx_clk=gpu.gfx_clk,
                                                                )

    Render.render_content(gpuAttrs, args)

# (xdx_smi: XdxSmi, args: Dict)
def handle(xdx_smi, args):
    header = """\
==========================XDXSMI LOG==========================
    Version                         : {smi_ver}
    Timestamp                       : {timestp}
    Attached GPUs                   : {gpuNums}\n""".format(smi_ver=xdx_smi.smi_ver, timestp=xdx_smi.timestp, gpuNums=xdx_smi.gpuNums)

    # if driver is loaded, out can get info without root authority
    # 'xdxsmi out' runs with root authority only if driver is not loaded.
    if(os.geteuid() != 0 and is_out_of_band() == 1):
        sys.exit()

    Render.render_content(header, args)

    for xdxgpu in xdx_smi.xdxgpus:
        printAttributes(xdxgpu, args)


if __name__ == "__main__":
    print(docopt(__doc__))