# -*- coding: utf-8 -*-

"""
usage:
   xdxsmi temp [options] get [<ip>...]

options:
   -h, --help              print this help message
   -f, --filename=<file>   redirect output to a specified file
   -l, --loop=<interval>   set query interval in seconds
   -i, --index=<idx>       index number of target GPU, starting from 0

ip:
   gddr_temp             current GDDR temperature
   temp                  current GPU temperature
   temp_highest          the recorded highest GPU temperature
   temp_min              GPU temperature to speed up the fan
   temp_max              GPU temperature to lower the frequency
   temp_crit             GPU temperature to cause a restart or blue screen
"""

#xdxsmi temp [options] set temp_crit <mhz>
# from typing import Dict
import time
import sys
from xdxsmi_render import Render
from xdxsmi_attrs_ops import setAttrs, getAttrs
from xdxsmi.api.third_party_docopt import docopt
from xdxsmi.api.smi import XdxSmi
from xdxsmi.api.pyxdxml import *

temp_map = {
    "gpu" : "xdx_dbdf",
    "name": "xdx_product_name",
    "temp": "temp1_input",
    "temp_highest": "temp1_highest",
    "temp_min": "temp1_min",
    "temp_max": "temp1_max",
    "temp_crit": "temp1_crit",
    "gddr_temp": "xdx_gddr_temp"
}
temp_unit_map = {
    "temp": "°C",
    "temp_highest": "°C",
    "temp_min": "°C",
    "temp_max": "°C",
    "temp_crit": "°C",
    "gddr_temp": "°C"
}

# (xdx_smi: XdxSmi, args: Dict)
def handle(xdx_smi, args):
    #Collect target gpus
    #target_gpus = [xdx_smi.handle[0]]
    index_num = -1
    if(args['--index'] != None):
        index_num = int(args['--index'])
    
    target_gpus = xdx_smi.xdxgpus
    if(index_num >= 0):
        target_gpus = [xdx_smi.xdxgpus[index_num]]
        
    #Get values of target attributes   
    if args["get"]:
        target_ip = ["gddr_temp", "temp", "temp_highest", "temp_min", "temp_max", "temp_crit"]
        wrong_ip=[]
        right_ip=[]
        if args["<ip>"]:
            args_ip_lower = [ip.lower() for ip in args["<ip>"]]
#            target_ip = [ip for ip in target_ip if ip.lower() in args_ip_lower]            
            for ip in args_ip_lower:
                if ip not in target_ip:
                    wrong_ip.append(ip)
                else:
                    right_ip.append(ip)
                
        # li: judge the input target_ip is legal
#        if len(target_ip) == 0:
#            element_ip = next(iter(args["<ip>"]))
#            sys.exit("ERROR: unknown '{element_ip}'. Please refer to the help document below.\n{doc}"\
#                .format(element_ip=element_ip, doc=__doc__))
    
        if len(right_ip)==0:
	    if len(wrong_ip)!=0:
                sys.exit("ERROR: unknown '{element_ip}'. Please refer to the help document below.\n{doc}".format(element_ip=wrong_ip[0], doc=__doc__))
	    else:
		right_ip=target_ip  #default to output all ip info if no input for <ip>
        elif len(wrong_ip)!=0:
            print("Warning: Invalid temp ip '{}'".format(wrong_ip[0]))


        
        data = []
        # i = 0
#        header = ["index", "gpu", "name"] + target_ip
        header = ["index", "gpu", "name"] + right_ip
        data.insert(0, header)
        for gpu in target_gpus:
            xdev = gpu.handle
            tempInfo = xdxml_device_get_temper_info(xdev)
            
            #print(clkInfo)
            row = [
                str(gpu.user_index),
                xdxml_device_get_pci_info(xdev).pci_dbdf.decode("utf8"),
                xdxml_device_get_device_model(xdev).decode('utf-8')]
            # modify row to show content in target_ip
#            for ip in target_ip:
            for ip in right_ip:
                if gpu.is_host:
                    cur_value = str(getattr(tempInfo, ip)) + temp_unit_map[ip] + "     "
                    row.append(cur_value)
                else: 
                    row.append('N/A')
            data.append(row)
            # i += 1
        if sys.version_info[0] == 2:
            Render.render_table_py2(data, args)
        else:
            Render.render_table(data, args)

    else:
        sys.exit("ERROR: Bad command format.  Please run 'xdxsmi temp -h' for help.\n")

if __name__ == "__main__":
    print(docopt(__doc__))
