"""
usage:
   xdxsmi util [options] get [<ip>...]

options:
   -h, --help             print this help message
   -f, --filename=<file>  redirect output to a specified file
   -l, --loop=<interval>  set query interval in seconds
   -i, --index=<idx>      index number of target GPU, starting from 0

ip:
   gpu                  gpu utilization
   tdm                  tdm pipeline utilization
   geom                 geom pipeline utilization
   3d                   3d pipeline utilization
   cdm                  cdm pipeline utilization
"""

# from typing import Dict
from xdxsmi_render import Render
from xdxsmi_attrs_ops import setAttrs, getAttrs
from xdxsmi.api.third_party_docopt import docopt
from xdxsmi.api.smi import XdxSmi
from xdxsmi.api.pyxdxml import *
from xdxsmi.api.xdxgpu import XdxGpu


# (xdx_smi: XdxSmi, args: Dict)
def handle(xdx_smi, args):
    #Collect target gpus
    #target_gpus = [xdx_smi.handle[0]]

    index_num = -1
    if(args['--index'] != None):
        index_num = int(args['--index'])
    
    target_gpus = xdx_smi.xdxgpus
    if(index_num >= 0):
        target_gpus = [xdx_smi.xdxgpus[index_num]]

    if args["get"]:
        target_ip = ["gpu", "tdm", "geom", "3d", "cdm"]
        
        #ignore the case
        if args["<ip>"]:
            args_ip_lower = [ip.lower() for ip in args["<ip>"]]
            target_ip = [ip for ip in target_ip if ip.lower() in args_ip_lower]

        if len(target_ip) == 0:
            element_ip = next(iter(args["<ip>"]))
            sys.exit("ERROR: unknown '{element_ip}'. Please refer to the help document below.\n{doc}"\
                .format(element_ip=element_ip, doc=__doc__))

        data = []
        i = 0
        header = ["INDEX", "GPU", "NAME"] + target_ip
        data.insert(0, header)
        
        for gpu in target_gpus:
            xdev = gpu.handle
            utilInfo = xdxml_device_get_util_info(xdev)

            util_info_dict = {
            "gpu": str(utilInfo.gpu) + "%",
            "tdm": str(utilInfo.tdm) + "%",
            "geom": str(utilInfo.geom) + "%",
            "3d": str(utilInfo.thrd) + "%",
            "cdm": str(utilInfo.cdm) + "%"
            }

            row = [ str(i),
                    xdxml_device_get_pci_info(xdev).pci_dbdf.decode("utf8"),
                    xdxml_device_get_device_model(xdev).decode('utf-8')]
            
            for ip in target_ip:
                row.append(util_info_dict[ip])
            data.append(row)
            i += 1
            
        Render.render_table(data, args)

    else:
        sys.exit("ERROR: Bad command format.  Please run 'xdxsmi util -h' for help.\n")    

if __name__ == "__main__":
    print(docopt(__doc__))