# -*- coding: utf-8 -*-
"""
usage:
   xdxsmi vpuinfo [options]

options:
    -h, --help                    print this help message
    -f, --filename=<file>         redirect output to a specified file
    -i, --index=<idx>             index number of target GPU, starting from 0
"""

# from typing import Dict
from xdxsmi_render import Render
from xdxsmi.api.third_party_docopt import docopt
from xdxsmi.api.smi import XdxSmi
from xdxsmi.api.pyxdxml import *
from xdxsmi.api.xdxgpu import XdxGpu

# (vpuEnc: vpu_enc_info_t, theLast: bool) ->string
def printOneEncInfo(vpuEnc, theLast):
    outStr = str("")
    outStr += "  |\n"
    tempStr = vpuEnc.enc_name.decode('utf-8') if vpuEnc.enc_name!=None else 'N/A'
    outStr += "  |__" + tempStr + "\n"
    brCtrlMtdList = [] #先打印支持的码率控制方法
    for brCtrlMtd in vpuEnc.enc_br_crtl_mtd_List:
        if(brCtrlMtd.is_valid == 1):
            brCtrlMtdList.append(brCtrlMtd)
    count = len(brCtrlMtdList)
    if(count > 0):
        if(theLast == False):
            if(count == 1):
                outStr += "  |    |__supported br control method: "
            else:
                outStr += '  |    |__supported br control methods(%d): ' % count
        else:
            if(count == 1):
                outStr += "       |__supported br control method: "
            else:
                outStr += '       |__supported br control methods(%d): ' % count
        i =0
        for brCtrlMtd in brCtrlMtdList:
            tempStr = brCtrlMtd.br_ctrl_mth_name.decode('utf-8') if brCtrlMtd.br_ctrl_mth_name!=None else 'N/A'
            if(i != 0):
                outStr += ", "+tempStr
            else:
                outStr += tempStr
            i += 1
        outStr += "\n"  #最后换行
    rtImgFmtList = []   #再打印支持的原始图像格式
    for rtImgFmt in vpuEnc.img_fmt_list:
        if(rtImgFmt.is_valid == 1):
            rtImgFmtList.append(rtImgFmt)
    count = len(rtImgFmtList)
    if(count > 0):
        if(theLast == False):
            if(count == 1):
                outStr += "  |    |__supported src image format: "
            else:
                outStr += '  |    |__supported src image formats(%d): ' % count
        else:
            if(count == 1):
                outStr += "       |__supported src image format: "
            else:
                outStr += '       |__supported src image formats(%d): ' % count
        i =0
        for rtImgFmt in rtImgFmtList:
            tempStr = rtImgFmt.img_fmt_name.decode('utf-8') if rtImgFmt.img_fmt_name!=None else 'N/A'
            if(i != 0):
                outStr += ", "+tempStr
            else:
                outStr += tempStr
            i += 1
        outStr += "\n"
    return outStr

# (vpudec: vpu_dec_info_t, theLast: bool) ->string
def printOneDecInfo(vpudec, theLast):
    outStr = str("")
    outStr += "  |\n"
    tempStr = vpudec.dec_name.decode('utf-8') if vpudec.dec_name!=None else 'N/A'
    outStr += "  |__" + tempStr + "\n"
    i = 0
    count = 0
    rtImgFmtList = []
    for rtImgFmt in vpudec.img_fmt_list:
        if(rtImgFmt.is_valid == 1):
            rtImgFmtList.append(rtImgFmt)
    count = len(rtImgFmtList)
    if(count > 0):
        if(theLast == False):
            if(count == 1):
                outStr += "  |    |__supported output image format: "
            else:
                outStr += '  |    |__supported output image formats(%d): ' % count
        else:
            if(count == 1):
                outStr += "       |__supported output image format: "
            else:
                outStr += '       |__supported output image formats(%d): ' % count
        i =0
        for rtImgFmt in rtImgFmtList:
            tempStr = rtImgFmt.img_fmt_name.decode('utf-8') if rtImgFmt.img_fmt_name!=None else 'N/A'
            if(i != 0):
                outStr += ", "+tempStr
            else:
                outStr += tempStr
            i += 1
        outStr += "\n"
    return outStr

#(vpuCodecInfo: vpu_dec_info_t, xdxgpu: XdxGpu) ->string
def printOneVpuCodecInfo(vpuCodecInfo, xdxgpu):
    decoder_ability, encoder_ability = xdxml_device_get_video_decoder_encoder(xdxgpu.handle)

    outStr = str("")
    if(vpuCodecInfo == None):
        outStr += "None\n"
        return outStr
    elif(vpuCodecInfo.is_valid != 1):
        outStr += "Invalid\n"
        return outStr
    driverName = vpuCodecInfo.vender_name.decode('utf-8') if vpuCodecInfo.vender_name!=None else 'N/A'
    outStr += "vendor: " + driverName + "\n"
    vpuEncoderList = []  #获得支持的编码器
    for vpuEncoder in vpuCodecInfo.encoders:
        if(vpuEncoder.is_valid == 1):
            vpuEncoderList.append(vpuEncoder)
    vpuDecoderList = []  #获得支持的解码器
    for vpuDecoder in vpuCodecInfo.decoders:
        if(vpuDecoder.is_valid == 1):
            vpuDecoderList.append(vpuDecoder)
    encoderCount = len(vpuEncoderList)  #支持的编码器数量
    decoderCount = len(vpuDecoderList)  #支持的解码器数量
    if(encoderCount == 1):  #只有一个编码器但不是jpeg图片编码器就能最大支持4k@30fps
        if(vpuEncoderList[0].profile_type != VP_JPEG_BASELINE):
            outStr += "max encoding ability: " + encoder_ability + "\n"
    elif(encoderCount > 1): #不只一个编码器，就算有一个是jpeg图片编码器但另一个肯定不是，就肯定最大支持4k@30fps
        outStr += "max encoding ability: " + encoder_ability + "\n"
    if(decoderCount == 1):  #只有一个解码器但不是jpeg图片解码器就能最大支持4k@120fps
        if(vpuDecoderList[0].profile_type != VP_JPEG_BASELINE):
            outStr += "max decoding ability: " + decoder_ability + "\n"
    elif(decoderCount > 1): #不只一个编码器，就算有一个是jpeg图片编码器但另一个肯定不是，就肯定最大支持4k@30fps
        outStr += "max decoding ability: " + decoder_ability + "\n"
    coutt = encoderCount
    if(coutt > 0):
        outStr += "supported encoder:" if coutt == 1 else 'supported encoders(%d):' % coutt
        outStr += "\n"
        i = 1
        for vpuEnc in vpuEncoderList:
            outStr += printOneEncInfo(vpuEnc, i == coutt)
            i += 1
    else:
        outStr += "no supported endcoder!\n"
    coutt = decoderCount
    if(coutt > 0):
        outStr += "supported decoder:" if coutt == 1 else 'supported decoders(%d):' % coutt
        outStr += "\n"
        i = 1
        for vpuDec in vpuDecoderList:
            outStr += printOneDecInfo(vpuDec, i == coutt)
            i += 1
    else:
        outStr += "no supported decoder!\n"
    #print("")
    return outStr

# (xdx_smi: XdxSmi, theIdx: int)->string
def makeOutStr(xdx_smi, theIdx):
    outStr = str("")
    #先获取所有vpu信息
    vpuInfoList = []
    if(theIdx >= 0):   #如果是指定的设备，就只取那个
        vpuInfoList.append(xdxml_device_get_vpu_info(xdx_smi.handle[theIdx]))
    else:
        theIdx = 0
        for devHd in xdx_smi.handle:
            vpuInfoList.append(xdxml_device_get_vpu_info(devHd))
    #再打印
    i = 0
    for vpuInfo in vpuInfoList:
        if(i != 0):  #不是第一行就换一行
            outStr += '\n'
        outStr += '=====================VPU Info of GPU %02d=====================\n' % (theIdx)
        target_gpu = xdx_smi.xdxgpus[theIdx]
        outStr += printOneVpuCodecInfo(vpuInfo, target_gpu)
        theIdx += 1
        i += 1
    return outStr
    

# (xdx_smi: XdxSmi, args: Dict)
def handle(xdx_smi, args):
    index_num = -1
    if(args['--index'] != None):
        index_num = int(args['--index'])  
        
    outStr = makeOutStr(xdx_smi, index_num)

    Render.render_content(outStr, args)
    


if __name__ == "__main__":
    print(docopt(__doc__))