# -*- coding: utf-8 -*-
"""
usage:
   xdxsmi vpumon [options]

options:
    -h, --help                  print this help message
    -f, --filename=<file>       redirect output to a specified file
    -i, --index=<idx>           index number of target GPU, starting from 0
    -l, --loop=<interval>       set query interval in seconds [default=2sec, max=10sec]
"""

# from typing import Dict
import sys
from xdxsmi_render import Render
from xdxsmi.api.third_party_docopt import docopt
from xdxsmi.api.smi import XdxSmi
from xdxsmi.api.pyxdxml import *

# (encRev: int, encExc: int, decRev: int, decExc: int)->string
def makeOneMsg(encRev, encExc, decRev, decExc):
    outStr = str("")
    if(encRev == -1 or encExc == -1 or decRev == -1 or decExc == -1):  #只有任意一个读不出来就认为没有
        outStr = "N/A\n"
    else:
        outStr = 'encode-vcmd reserved %dfps, excuted %dfps; decode-vcmd reserved %dfps, excuted %dfps\n' % (encRev, encExc, 
            decRev, decExc)
    return outStr

# (theIdx: int)->string
def makeHeader(theIdx):
    outStr = str("")
    if(theIdx >=0):
        outStr += '=====================VPU utilization of GPU %02d===========================\n' % theIdx
    else:
        outStr += "==============================VPU utilization==============================\n"
    return outStr

# (xdx_smi: XdxSmi, theIdx: int)->string
def makeSingleBody(xdx_smi, theIdx):
    outStr = str("")
    if(theIdx >= 0):
        encRev, encExc, decRev, decExc = xdxml_device_get_vpu_utlz(xdx_smi.handle[theIdx])
        outStr += makeOneMsg(encRev, encExc, decRev, decExc)
    else:  #没指定gpu则会在前面加上编号，哪怕只有一个
        i = 0
        for devHd in xdx_smi.handle:
            encRev, encExc, decRev, decExc = xdxml_device_get_vpu_utlz(xdx_smi.handle[theIdx])
            outStr +='VPU(%02d): '%i + makeOneMsg(encRev, encExc, decRev, decExc)
            i += 1
    return outStr

# (xdx_smi: XdxSmi, args: Dict)->None
def handle(xdx_smi, args):
    index_num = -1
    if(args['--index'] != None):
        index_num = int(args['--index'])

    loop_interval = 2.0
    try:
        if(args['--loop'] != None):
            loop_interval = float(args["--loop"])
    except Exception as e:
        Render.render_content("Invalid input for --loop: '{}'. A value > 0 is expected. This parameter will be ignored.\n".format(args['--loop']), args)
        loop_interval=2.0
    if(loop_interval > 10.0):
        loop_interval = 10.0

    Render.render_content('loop interval: {} seconds:\n'.format(loop_interval), args)

    #if only one time query: not supported 
#    if(loop_interval == -1): 

    if(loop_interval <= 0): 
        outStr = makeHeader(index_num) + makeSingleBody(xdx_smi, index_num)
        Render.render_content(outStr, args)
    else:
        Render.render_content(makeHeader(index_num), args)
        
        try:
            i = 0
            secs = 0
            while True:
                # xdx_smi =  XdxSmi()
                Render.render_content('loop %d in %d seconds:\n'%(i,secs) + makeSingleBody(xdx_smi, index_num), args)
                
                i += 1
                time.sleep(loop_interval)
                secs += loop_interval
        except KeyboardInterrupt:
            print("")   #最后再换一行，好看点


if __name__ == "__main__":
    print(docopt(__doc__))