#!/bin/bash

printUsage() {
    echo
    echo "Usage: $(basename "${BASH_SOURCE}") [options ...] [make options]"
    echo
    echo "Options:"
    echo "  -a,  --add              add the module name to the dkms tree"
    echo "  -b,  --build            dkms build the module"
    echo "  -i,  --install          dkms install the module"
    echo "  -u,  --uninstall        dkms uninstall the module"
    echo "  -r,  --remove           remove the module from the dkms tree for the current kernel"
    echo "  -c,  --clean            remove the module from the dkms tree for all kernels"
    echo "  -m,  --mktarball        dkms make tarball for the current kernel"
	return 0
}

KERNEL_VERSION=`uname -r`


#parse the arguments
#VALID_STR=`getopt -o hcraso: --long help,add,build,install,uninstall,remove,clean: -- "$@"`
#eval set -- "$VALID_STR"

TARGET="build"
MODULE_NAME="xdxgpu"
MODULE_VERSION="0.1"
XDXKMD_SRC="/usr/src/$MODULE_NAME-$MODULE_VERSION"
while true ;
do
    #echo "parocessing $1"
    case "$1" in
        (-h | --help)
            printUsage ; exit 0;;
        (-a | --add)
            echo "add";
	    TARGET="add"; 
            break;;    
        (-b | --build)
            TARGET="build"; 
  	    break;;
        (-i | --install)
            TARGET="install"; 
            break;;
        (-u | --uninstall)
            TARGET="uninstall";
            break;;     
        (-r | --remove)
            TARGET="remove"; 
            break;;   
        (-c | --clear)
            TARGET="clear" ;
	    break;;
		(-m | --mktarbal)
		    TARGET="mktarball";
			break;;
        (*)
            echo "please input {a|b|i|u|r|c|m}"; exit 20;;
    esac

done

do_dkms_add(){
	echo "do_dkms_add"
	dkms add -m $MODULE_NAME -v $MODULE_VERSION -k $KERNEL_VERSION
}

do_dkms_build(){
	echo "do_dkms_build"
	dkms build -m $MODULE_NAME -v $MODULE_VERSION -k $KERNEL_VERSION
}
do_dkms_install(){
	echo "do_dkms_install"
	dkms install -m $MODULE_NAME -v $MODULE_VERSION -k $KERNEL_VERSION
    modprobe $MODULE_NAME
	ls /dev/dri
	#rm /lib/modules/$KERNEL_VERSION/updates/dkms/xdxgpu.ko
	echo "Operation complete"
	dkms uninstall -m $MODULE_NAME -v $MODULE_VERSION -k $KERNEL_VERSION
}
do_dkms_uninstall(){
	echo "do_dkms_uninstall"
	dkms uninstall -m $MODULE_NAME -v $MODULE_VERSION -k $KERNEL_VERSION
}
do_dkms_remove(){
	echo "do_dkms_remove"
	dkms remove -m $MODULE_NAME -v $MODULE_VERSION -k $KERNEL_VERSION
}
do_dkms_clear(){
    echo "do_dkms_clear"
	dkms remove -m $MODULE_NAME -v $MODULE_VERSION --all  
}
do_dkms_mktarball(){
	echo "do_dkms_mktarball"
	dkms mktarball -m $MODULE_NAME -v $MODULE_VERSION -k $KERNEL_VERSION
}
case $TARGET in
	(add) 
		do_dkms_add ;;
	(build)
		do_dkms_build ;;
	(install)
		do_dkms_install ;;  
	(uninstall) 
		do_dkms_uninstall ;;
	(remove)
		do_dkms_remove ;;
	(clear)
		do_dkms_clear ;;
	(mktarball)
        do_dkms_mktarball ;; 
	(*) die "Invalid target $TARGET" ;;
esac




