/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef __XDXACPU_H__
#define __XDXACPU_H__
#include "pangu.h"
#include <linux/version.h>
#include <linux/mm_types.h>
#include <linux/io.h>
#include "lock_types.h"
#include "img_types.h"
#include "img_defs.h"
#include "pvrsrv_error.h"
struct xdx_device;

#define MAX_DOORBELL_POOL_SIZE      (32)
#define MAX_TINYBUFFER_POOL_SIZE    (32)
#define MAX_TINYBUFFER_MAX_SIZE     (32)
#define INVALID_DB_INDEX            (~0x0U)

#define  MAX_ACPU_MEMSET_SIZE     (16*1024*1024U - 4*1024U)
#define  MAX_ACPU_MEMSET_SIZE_4K  (16*1024*1024U - 4*1024U)
#define  MAX_ACPU_MEMSET_SIZE_128 (32*1024U)
#define MAX_ACPU_MEMCOPY_SIZE     (64*1024*1024)

typedef enum {
        MSIX = 0,
//    MSI,
        INTX,
        INTTYPEMAX
} int_type;

typedef union {
        u32 all;
        struct {
                u32 support_clr_gpuint:1;
                u32 resv:31;
        };
} fwcap_type;

typedef union {
	u32 all;
	struct {
		u32 pcie_int_type:2;
		u32 clr_gpu_int:1;
		u32 resv:29;
	};
} fwcntl_type;

typedef enum {
	acpu_dbcmd_debug = 0,
	acpu_dbcmd_memop,
	acpu_dbcmd_ddkmisc,
	acpu_dbcmd_max
} acpu_dbcmd_type;

typedef enum {
	acpu_dbcmd_memop_tiny = 0,
	acpu_dbcmd_memop_memmet4kaligned,
	acpu_dbcmd_memop_memset32baligned,
	acpu_dbcmd_memop_max
} acpu_dbcmd_memop_subtype;

typedef enum {
	acpu_dbcmd_memop_subtype_tiny_copy = 0,
	acpu_dbcmd_memop_subtype_tiny_set,
	acpu_dbcmd_memop_subtype_tiny_reg,
	acpu_dbcmd_memop_subtype_tiny_max
} acpu_dbcmd_memop_subtype_tiny_sbutype;

typedef union {
	u64 all;
	struct {
		u32 done:1;
		u32 cmd:3;
		u32 subCmd:4;
		u32 isFast:1;
		u32 resv:23;
		u32 index;
	};
} acpu_dbcmd_memop_tiny_cmd_type;

typedef enum {
	acpu_dbcmd_ddkmisc_handshake_getcap = 0,
	acpu_dbcmd_ddkmisc_handshake_cntl,
	acpu_dbcmd_ddkmisc_shramaddr = 7,
	acpu_dbcmd_ddkmisc_max
} acpu_dbcmd_ddkmisc_subtype;

typedef union {
	u64 all;
	struct {
		u32 done:1;
		u32 cmd:3;
		u32 subcmd:4;
		u32 resv:24;
		union {
			fwcntl_type fwcntl;
			fwcap_type fwcap;
		};
	};
} ddkhandshake_cmd_type;

typedef union {
	u64 all;
	struct {
		u64 done:1;
		u64 cmd:3;
		u64 subcmd:4;
		u64 address:40;
		u64 resv:16;
	};
} ddkshramaddr_cmd_type;

typedef union {
	u64 all;
	struct {
		u64 done:1;
		u64 cmd:3;
		u64 subCmd:4;
		u64 padding:56;
	};
	struct {
		u64 done:1;
		u64 cmd:3;
		u64 subCmd:4;
		u64 addr32B:35;	//address should be 32byte aligned, size should be 4byte aligned
		u64 num4B:13;	//Max support 32KB
		u64 pattern:8;
	} align32B;
	struct {
		u64 done:1;
		u64 cmd:3;
		u64 subCmd:4;
		u64 addr4K:28;	//address should be 4KB aligned, size should be 4KB aligned
		u64 num4K:20;	// Max Support 16MB
		u64 pattern:8;
	} align4K;
} acpu_dbcmd_memop_memset_cmd_type;

typedef enum {
	cCmcuHostDBCmdDDKPower = 0,
	cCmcuHostDBCmdMax
} CmcuHostDBCmdType_T;

typedef union {
	u64 all;
	struct {
		u32 low;
		u32 high;
	};
	struct {
		u64 done:1;	//----
		u64 cmd:3;	//   |---- Header
		u64 subCmd:4;	//----
		u64 type:4;	//----
		u64 channel:4;

		u64 power_evt:32;
		u64 power_flags:16;

	} power;
	struct {
		u64 done:1;	//----
		u64 cmd:3;	//   |---- Header
		u64 subCmd:4;	//----
		u64 type:4;
		u64 channel:4;

		u64 freq:32;
		u64 volt:16;

	} dvfs;
	struct {
		u64 done:1;	//----
		u64 cmd:3;	//   |---- Header
		u64 subCmd:4;	//----
		u64 type:4;
		u64 channel:4;

		u64 temp:32;
		u64 fan_rpm:16;

	} thermal;
	struct {
		u64 done:1;	//----
		u64 cmd:3;	//   |---- Header
		u64 subCmd:4;	//----
		u64 type:4;
		u64 channel:4;

		u64 pwmValue:32;
		u64 resv:16;

	} pwm;
	struct {
		u64 done:1;	//----
		u64 cmd:3;	//   |---- Header
		u64 subCmd:4;	//----
		u64 type:4;
		u64 channel:4;

		u64 tempValue:32;
		u64 resv:16;

	} temp;
	struct {
		u64 done:1;	//----
		u64 cmd:3;	//   |---- Header
		u64 subCmd:4;	//----
		u64 type:4;
		u64 channel:4;

		u64 fanValue:32;
		u64 resv:16;

	} fan;
	struct {
		u64 done:1;
		u64 cmd:3;
		u64 subcmd:4;
		u64 type:4;
		u64 channel:4;
		u64 workload:48;
	} structure;

} power_cmd_type;

typedef union
{
	u64 all;
	struct
	{
		u64 low0       :  1;
		s32 low31      : 31;
		u64 high_data :  32;
	};
	struct
	{
		s32 low32;
		u32 high32;
	};
	struct
	{
		u64 done       :  1;
		u64 cmd        :  3;
		u64 subcmd     :  4;
		u64 type       :  4;
		u64 channel    :  4;

		u64 data       : 32;
		u64 resv       : 16;
	};
} CMCUPowerCmd_T, CMCUThermalCmd_T;


/* For struct CMCUPowerCmd_T, doorbell45 */
typedef enum
{
	cH02CPowerDBCmdDDK = 0,
	cH02CPowerDBCmdDVFS,
	cH02CPowerDBCmdPMIC,
	cH02CPowerDBCmdMax
}H02CPowerDBCmdType_T;

typedef enum
{
	cH02CPowerDDKSubCmdStateChange = 0,
	cH02CPowerDDKSubCmdMax
}H02CPowerDDKSubCmdType_T;

typedef enum
{
	cH02CPowerDVFSSubCmdSetFreq = 0,
	cH02CPowerDVFSSubCmdGetFreq,
	cH02CPowerDVFSSubCmdSetVolt,
	cH02CPowerDVFSSubCmdMax
}H02CPowerDVFSSubCmdType_T;

typedef enum
{
	cH02CPowerPMICSubCmdSetVolt = 0,
	cH02CPowerPMICSubCmdMax
}H02CPowerPPMICSubCmdType_T;


/* For struct CMCUThermalCmd_T, doorbell55 */
typedef enum
{
	cH02CThermalCmdTemp = 0,
	cH02CThermalCmdFan,
	cH02CThermalCmdPWM,
	cH02CThermalDBCmdMax
}H02CThermalDBCmdType_T;

typedef enum
{
	FanSubCmdPercentSet = 1,
	FanSubCmdFanControlModeSet,
	FanSubCmdMax
}H02CThermalFanSubCmdType_T;

typedef enum
{
	FanControlModeAuto = 0,
	FanControlModeUser,
	FanControlModeMax
}FanControlModeType_T;

enum h02c_db_power_thm_type {
	THERMAL_DEBUG_ENA_GET = 0,
	THERMAL_DEBUG_ENA_SET,
	THERMAL_TEST_ENA_GET,
	THERMAL_TEST_ENA_SET,
	THERMAL_FILTER_ENA_GET,
	THERMAL_FILTER_ENA_SET,
	THERMAL_FILTER_TYPE_GET,
	THERMAL_FILTER_TYPE_SET,
	THERMAL_FILTER_TEST_ENA_GET,
	THERMAL_FILTER_TEST_ENA_SET,
};

enum h02c_db_power_fan_type {
	FAN_INFO_GET = 0,
	FAN_RPM_SET,
	FAN_RPM_GET,
	FAN_PERCENT_SET,
	FAN_DUTY_SET,
	FAN_RPM_GET_MIN,
	FAN_RPM_GET_MAX,
	FAN_CTRL_MODE_SET,
	FAN_CTRL_MODE_GET,
};

enum h02c_db_power_pwm_type {
	PWM_MIN_GET = 0,
	PWM_MAX_GET,
	PWM_GET,
	PWM_SET,
};

enum h02c_db_power_temp_type {
	TEMP_GET = 0,
	TEMP_HIGHEST_GET,
	TEMP_GET_MIN,
	TEMP_SET_MIN,
	TEMP_GET_MAX,
	TEMP_SET_MAX,
	TEMP_GET_CRITICAL,
	TEMP_SET_CRITICAL,
	TEMP_GET_EMERGENCY,
	TEMP_SET_EMERGENCY,
	TEMP_GET_CTRL_MODE,
	TEMP_SET_CTRL_MODE,
	TEMP_OVER_PROTECT,
	TEMP_OVER_Fatal,
};

enum h02c_db_power_dvfs_type {
	DVFS_SET_FREQ = 0,
	DVFS_SET_VOLT,
	DVFS_GET_FREQ,
};

enum h02c_db_power_volt_type {
	VOLTAGE_GET = 0,
	VOLTAGE_SET,
};

enum h02c_db_power_curr_type {
	CURRENT_GET = 0,
	CURRENT_GET_AVG,
};

enum h02c_db_power_power_type {
	POWER_GET = 0,
	POWER_GET_AVG,
};
#define WAIT_DB_MAX_TIMEOUT_US                   (10)
typedef void (*dma_async_tx_callback)(void *dma_async_param);
typedef struct _acpudb_pool_ {
	struct mutex pool_mutex;
	/* total max number of permitted entries in the pool */
	uint max_entries;
	/* currently number of pool entries created. these may be in the pool
	 * or in-use
	 */
	uint used_entries;
	/* number of not-in-use entries currently free in the pool */
	uint free_entries;

	uint32_t db_bitmap;	//Doorbell pool mask
	void *db_event_object;
	uint32_t db_submit;	//Dorrbell submit mask

	//void      *pvMISRData;
	//struct tasklet_struct task;
	struct workqueue_struct *wq;
	struct work_struct work;

	dma_async_tx_callback callback[MAX_DOORBELL_POOL_SIZE];
	void *callback_param[MAX_DOORBELL_POOL_SIZE];
	resource_size_t base;
} acpudb_pool_t;

/*only doorbell 0-31 used for command from host to acpu
*each doorbell use 8Byte(index << 3)
*fw team use bitmap to share between threads, here simplify it
*/
u32 DOORBELL_OFFSET(int index);
s32 H02CDoorBellPowerGet(struct xdx_device *xdev, u64 value);
int32_t H02CDoorBellPowerSet(struct xdx_device *xdev, u64 value);
int32_t H02CDoorBellThermalSet(struct xdx_device *xdev, u64 value);
int32_t acpu_doorbell_pool_create(uint32_t max_entries,
				    resource_size_t base,
				    acpudb_pool_t **db_pool);
void acpu_doorbell_pool_destroy(acpudb_pool_t *pool);
uint32_t acpu_doorbell_get_timeout(acpudb_pool_t *pool,
					  uint32_t timeout);
int32_t acpu_waitfor_value(volatile uint64_t __iomem addr,
				uint32_t val, uint32_t mask);
uint32_t acpu_doorbell_pool_get(acpudb_pool_t *pool);
void acpu_doorbell_pool_put(acpudb_pool_t *pool);
int acpu_wait_all_db_done(acpudb_pool_t *pool, int timeout_ms);

int acpu_dbcmd_memop_memset(struct xdx_device *xdev, uint64_t devPAddr, uint32_t size, uint8_t pattern, bool align4k);
int acpu_dbcmd_memop_memset_multiple(struct xdx_device *xdev, uint64_t devPAddr, uint32_t size, uint8_t pattern, bool align4k);
#endif /* __XDXACPU_H__ */
