/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#pragma once
/*****************************************************************************************/
/*                                Include Header Files                                   */
/*****************************************************************************************/
#include <sound/core.h>
#include <linux/pci.h>
#include <sound/pcm.h>
#include <linux/workqueue.h>
#include <sound/initval.h>
#include "../display/output/dp/xdx_dp_drv.h"
#include "../xdx_device.h"
#include "../xdx_irq.h"

/*****************************************************************************************/
/*                                Public Macros                                          */
/*****************************************************************************************/
#define XDX_VAUDIO_VENDOR                   (0x1eed)
#define XDX_VAUDIO_DEVICE                   (0x1101)

#define XDX_AUDIO_BAR4_16M					16*1024*1024
#define XDX_AUDIO_BAR4_1M					1*1024*1024
/*****************************************************************************************/
/*                                Data Structure Define                                  */
/*****************************************************************************************/
enum PANGU_INTERRUPT {
    PANGU_INTERRUPT_H1_START    = 0,
    PANGU_INTERRUPT_VD0MCU 	    = 0,
    PANGU_INTERRUPT_VD1MCU	    = 1,
    PANGU_INTERRUPT_CMCU        = 2,
    PANGU_INTERRUPT_ACPU   	    = 3,
    PANGU_INTERRUPT_RESERVE   	= 4,
    PANGU_INTERRUPT_MAX,
};

typedef enum {
	PCM_SAMPLEBRATE_32K	= 32000,
	PCM_SAMPLEBRATE_441K	= 44100,
	PCM_SAMPLEBRATE_48K	= 48000,
	PCM_SAMPLEBRATE_64K	= 64000,
	PCM_SAMPLEBRATE_882K	= 88200,
	PCM_SAMPLEBRATE_96K	= 96000,
	PCM_SAMPLEBRATE_128K	= 128000,
	PCM_SAMPLEBRATE_1764K	= 176400,
	PCM_SAMPLEBRATE_192K	= 192000,
	PCM_SAMPLEBRATE_256K	= 256000,
	PCM_SAMPLEBRATE_3528K	= 352800,
	PCM_SAMPLEBRATE_384K	= 384000,
	PCM_SAMPLEBRATE_MAX,
} PCM_SAMPLERATE_MODE;

struct pangu_interrupt_handler {
	bool enabled;
	void (*handler_function)(void *data);
	void *handler_data;
};

typedef enum {
	XDX_AUDIO_MODULE_NULL 		= 0,
	XDX_AUDIO_MODULE_HDMI 		= 1,
	XDX_AUDIO_MODULE_HDMI1 		= 2,
	XDX_AUDIO_MODULE_DP   		= 3,
	XDX_AUDIO_MODULE_DP1   		= 4,
	XDX_AUDIO_MODULE_DOUBLE     	= 5,
} PlayDrv;
typedef enum {
	XDX_AUDIO_MODULE_DISCONNECT = 0,
	XDX_AUDIO_MODULE_CONNECT 	= 1,
} PlayCon;

typedef enum {
        AUDIOIF_HDMI	= 0,
        AUDIOIF_DP	= 1,
        AUDIOIF_HDMI1	= 2,
        AUDIOIF_DP1	= 3,
} VIDEO_CONNECT;

enum {
	DMA_BYTES_16BIT2CHS = 1536,
	DMA_BYTES_16BIT4CHS = 3072,
	DMA_BYTES_16BIT6CHS = 2304,
	DMA_BYTES_16BIT8CHS = 3072,
	DMA_BYTES_24BIT2CHS = 4608,
	DMA_BYTES_24BIT4CHS = 4608,
	DMA_BYTES_24BIT6CHS = 3456,
	DMA_BYTES_24BIT8CHS = 4608,
};

enum {
	CHNL_HEAD_2CHS = 1,
	CHNL_HEAD_4CHS = 2,
	CHNL_HEAD_6CHS = 4,
	CHNL_HEAD_8CHS = 4,
};

enum {
	BUF_NUM_2CHS = 16,
	BUF_NUM_4CHS = 8,
	BUF_NUM_6CHS = 4,
	BUF_NUM_8CHS = 4,
};

enum {
	Off = 0,
	On = 1,
};
struct snd_xdx_hdmi_audio {
	struct xdx_device *xdev;
	struct snd_card *card;
	struct pci_dev *pci;
	struct snd_pcm *pcm;
	struct device* dev;
	struct hdmi_dev* hdmi;
	struct hdmi_dev* hdmi1;
	struct dp_dev *dp;
	struct dp_dev *dp1;
	int irq;
	struct xdx_irq_src *irq_src_audio;
	struct xdx_irq_src *irq_src_audio1;
	struct xdx_irq_src *irq_src_vdma1;

	spinlock_t    lock;
	struct mutex open_mutex;

	struct pangu_interrupt_handler interrupt_handlers[PANGU_INTERRUPT_MAX];
	struct workqueue_struct *pWorkQueue;
	struct work_struct audioIntWork;
	//AudioAcpuHostCmd_T * audioAcpuHostCmd;
	u32 *audioAcpuHostInt;
	u32 *audioAcpuHostClearInt;

	unsigned long iomem;
	unsigned long irqcount;
	unsigned long inIRQ;
	void __iomem *iobase;
	void __iomem *iobase_audio0;
	void __iomem *iobase_audio1;
	void __iomem *iobase_vdma1;
	void __iomem *iobase_dp0;
	void __iomem *iobase_dp1;
	void __iomem *iobase_acpu;

	u8 *pcmbufferData;
	u8 *periodBufPtr;
	u64 periodBufPhy;
	u8 *tempBufPtr;
	u8 *bitBufPtr;
	u8 *chnl6BufPtr;
	uint32_t periodBufSize;
	struct snd_dma_buffer dma_period;
	struct snd_dma_buffer tempBuf;
	struct snd_dma_buffer bitBuf;
	struct snd_dma_buffer chnl6Buf;

	size_t periodsize;
	int channels;
	PCM_SAMPLERATE_MODE rate;
	uint32_t video_type;
	uint32_t bit_width;
	uint32_t sample_rate;
	int currentBuffer;
	uint32_t dmaTag;
	volatile uint32_t pos;
	volatile uint32_t fifo_bytes;
	volatile uint32_t fifo_empty;
	volatile uint32_t status;
	volatile uint32_t dma_pending_cnt;
	volatile uint32_t dma_bytes;

	struct snd_pcm_substream *playback_substream;
	struct snd_pcm_substream *capture_substream;


	PlayDrv  audioif_video_connect;        //0:NULL; 1:hdmi; 2:dp
	uint32_t flag;
	struct mutex *transfer_mutex;
	struct mutex *hw_mutex;
	int transfer_flag;
	int blkSize;
	uint32_t last_bytes;
	uint32_t first_enable;
	uint32_t buf_ptr;
	uint32_t ptr;
	uint32_t ptr_bytes;
	VIDEO_CONNECT video_connect;
	uint32_t current_connect[4];
	uint32_t audioif_video_disconnect;
	uint32_t last_connect;
	uint32_t audioif_hdmi;
};

/*****************************************************************************************/
/*                                Public Function Statement                              */
/*****************************************************************************************/
int snd_xdx_hdmi_audio_create(struct snd_card *card, struct xdx_device *xdev, struct snd_xdx_hdmi_audio **rchip);
int xdx_hdmi_audio_setRate(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr);
int xdx_audio_drv_hotPlug(struct xdx_device *xdev, PlayDrv module, PlayCon status);
int xdx_audio_resume(struct xdx_device *xdev);
int xdxaudio_isr_install(struct xdx_device *xdev);
int xdxaudio_isr_uninstall(struct xdx_device *xdev);
void pangu_a0_audiof0_db_interrupt(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr);
