/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/types.h>
#include <linux/pm_runtime.h>
#include <linux/pci.h>
#include <linux/firmware.h>
#include <linux/component.h>
#include <linux/vmalloc.h>

#include <drm/drm_atomic.h>
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0)
#include <drm/drm_atomic_uapi.h>
#endif
#include <drm/drm_atomic_helper.h>
#if ((LINUX_VERSION_CODE >= KERNEL_VERSION(5, 18, 0)) && (LINUX_VERSION_CODE < KERNEL_VERSION(5, 19, 0)))
#include <drm/dp/drm_dp_mst_helper.h>
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 19, 0))
#include <drm/display/drm_dp_mst_helper.h>
#else
#include <drm/drm_dp_mst_helper.h>
#endif
#include <drm/drm_fb_helper.h>
#include <drm/drm_fourcc.h>
#include <drm/drm_edid.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 13, 0))
#include <drm/drm_vblank.h>
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 8, 0))
#include <drm/drm_irq.h>
#else
#include <drm/drmP.h>
#endif
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 15, 0))
#include <drm/drm_audio_component.h>
#endif
#if ((LINUX_VERSION_CODE >= KERNEL_VERSION(5, 1, 0)) || (defined(REDHAT_83))) || (defined(XDXGPU_CENTOS85_SUPPORT)) || ((defined(XDXGPU_CENTOS82_SUPPORT)) && (!defined(XDXGPU_CENTOS79_SUPPORT)))
#include <drm/drm_probe_helper.h>
#else
#include <drm/drm_crtc_helper.h>
#endif

#include "dm.h"
#include "xdx_device.h"
#include "dm_fb.h"
#include "dm_connector.h"
#include "dm_encoder.h"
#include "dc8200.h"
#include "output/dp/xdx_dp_drv.h"
#include "output/hdmi/xdx_hdmi_drv.h"
#include "xdx_gem.h"
#include "pm/xdx_pm.h"

static int dm_early_init(void *handle)
{
	return 0;
}

static int dm_late_init(void *handle)
{
	return 0;
}

static int dm_sw_init(void *handle)
{
	struct xdx_device *xdev = (struct xdx_device *)handle;
	struct drm_device *ddev = xdev_to_drm(xdev);
	struct xdx_dm *dm = &xdev->dm;
	int ret = -1;
	struct dm_connector *dm_connector;
	struct dm_encoder *dm_encoder;
	struct dc8200_dc *dc8200_dc[MAX_OUTPUT_NUM];
	int i,j = 0;
	drm_mode_config_init(ddev);

	switch (xdev->asic_type) {
	case CHIP_PANGU_A0:
		dc8200_early_init(&xdev->dm, 0);
		break;
	case CHIP_PANGU_B0:
		for(i = 0; i < dm->dc_num; i++ ) {
			dc8200_early_init(&xdev->dm, i);
		}
		break;
	default:
		break;
	}
	xdev->flags.virtual_output = false;

	for(i = 0; i < dm->dc_num; i++){
		dc8200_dc[i] = to_dc8200_dc(dm->dc[i]);
		if (xdev->flags.virtual_output) {
			for(j = 0; j < dc8200_dc[i]->info->panel_num; j++){
				dm_connector = kzalloc(sizeof(*dm_connector), GFP_KERNEL);
				if (!dm_connector)
					goto err_mode;

				dm_encoder = kzalloc(sizeof(*dm_encoder), GFP_KERNEL);
				if (!dm_encoder)
					goto err_mode;

				if (dm_encoder_init(dm, dm_encoder, i)) {
					kfree(dm_encoder);
					dev_err(xdev->dev, "failed to initialize encoder\n");
					goto err_mode;
				}

				if (dm_connector_init(dm, dm_connector, i, DRM_MODE_CONNECTOR_HDMIA,
							dm_encoder)) {
					kfree(dm_connector);
					dev_err(xdev->dev, "failed to initialize connector\n");
					goto err_mode;
				}
			}
		} else {
#if defined SUPPORT_PANGU_DC_HDMI
			xdx_hdmi_init(dm,i);
#endif

#if defined SUPPORT_PANGU_DC_DP
			xdx_dp_init(dm,i);
#endif
		}

	}

	xdx_dm_mode_config_init(ddev);

	ret = drm_vblank_init(ddev, ddev->mode_config.num_crtc);
	if (ret) {
		dev_err(xdev->dev, "failed to init vblank (%d)\n", ret);
		goto err_mode;
	}

	drm_mode_config_reset(ddev);

	drm_kms_helper_poll_init(ddev);

	return 0;

err_mode:
	drm_mode_config_cleanup(ddev);
	for(i = 0; i < dm->dc_num; i++){
		if (xdev->dm.dc[i])
			xdev->dm.dc[i]->funcs->destroy(xdev->dm.dc[i]);
	}
	return ret;
}

#if (LINUX_VERSION_CODE <= KERNEL_VERSION(4, 11, 0))
void xdx_drm_atomic_helper_shutdown(struct drm_device *dev)
{
        struct drm_modeset_acquire_ctx ctx;
        struct xdx_device *xdev = drm_to_xdev(dev);
        int ret;

        drm_modeset_acquire_init(&ctx, 0);
        while (1) {
                ret = drm_modeset_lock_all_ctx(dev, &ctx);
                if (!ret) {
                        ret = drm_atomic_helper_disable_all(dev, &ctx);
                        if (ret)
                                dev_err(xdev->dev, "Disabling all crtc's during unload failed with %i\n", ret);
                }
                if (ret == -EDEADLK)
                        break;
                drm_modeset_backoff(&ctx);
        }
        drm_modeset_drop_locks(&ctx);
        drm_modeset_acquire_fini(&ctx);
}
#endif

static int dm_sw_fini(void *handle)
{
	struct xdx_device *xdev = (struct xdx_device *)handle;
	struct xdx_dm *dm = &xdev->dm;
	struct drm_device *ddev = xdev_to_drm(xdev);
	struct dm_dc *dc_0 = xdev->dm.dc[0];
	struct dc8200_dc* dc8200_0 = container_of(dc_0, struct dc8200_dc, base);
	struct dm_dc *dc_1 = xdev->dm.dc[1];
	struct dc8200_dc* dc8200_1 = container_of(dc_1, struct dc8200_dc, base);
	int i;

#if (LINUX_VERSION_CODE  <= KERNEL_VERSION(4, 11, 0))
        xdx_drm_atomic_helper_shutdown(ddev);
#else
        drm_atomic_helper_shutdown(ddev);
#endif
	drm_kms_helper_poll_fini(ddev);

	mutex_destroy(&dc8200_0->cursor_mtx);

	if(dm->dc_num == 2) {
		mutex_destroy(&dc8200_1->cursor_mtx);
	}

	for(i = 0; i < dm->dc_num; i++){
		if (!xdev->flags.virtual_output) {
#if defined(SUPPORT_PANGU_DC_HDMI)
			xdx_hdmi_deinit(dm, i);
#endif
#if defined(SUPPORT_PANGU_DC_DP)
			xdx_dp_deinit(dm, i);
#endif
		}
		//drm_mode_config_cleanup(ddev);
		if (xdev->dm.dc[i]->funcs->destroy){
			xdev->dm.dc[i]->funcs->destroy(xdev->dm.dc[i]);
		}
	}

	return 0;
}

static int dm_early_fini(void *handle)
{
	return 0;
}

static int dm_hw_init(void *handle)
{
	struct xdx_device *xdev = (struct xdx_device *)handle;
	struct xdx_dm *dm = &xdev->dm;
	int i;

	for(i = 0; i < dm->dc_num; i++){
		if (xdev->dm.dc[i]->funcs->hw_init)
			xdev->dm.dc[i]->funcs->hw_init(xdev->dm.dc[i]);
	}

	return 0;
}

static int dm_hw_fini(void *handle)
{
	return 0;
}

#if (LINUX_VERSION_CODE <= KERNEL_VERSION(4, 11, 0)) && (!defined(XDXGPU_CENTOS79_SUPPORT))
#ifdef CONFIG_LOCKDEP
static struct lockdep_map connector_list_iter_dep_map = {
        .name = "drm_connector_list_iter"
};
#endif
/**
 * drm_connector_list_iter_begin - initialize a connector_list iterator
 * @dev: DRM device
 * @iter: connector_list iterator
 *
 * Sets @iter up to walk the &drm_mode_config.connector_list of @dev. @iter
 * must always be cleaned up again by calling drm_connector_list_iter_end().
 * Iteration itself happens using xdx_drm_connector_list_iter_next() or
 * drm_for_each_connector_iter().
 */
void drm_connector_list_iter_begin(struct drm_device *dev,
                                   struct drm_connector_list_iter *iter)
{
        iter->dev = dev;
        iter->conn = NULL;
        lock_acquire_shared_recursive(&connector_list_iter_dep_map, 0, 1, NULL, _RET_IP_);
}

void drm_connector_free(struct drm_connector *conn)
{
        //struct drm_device *dev = conn->dev;

        //drm_mode_object_unregister(dev, &conn->base);
        conn->funcs->destroy(conn);
}

/*
 * Extra-safe connector put function that works in any context. Should only be
 * used from the connector_iter functions, where we never really expect to
 * actually release the connector when dropping our final reference.
 */
static void
__drm_connector_put_safe(struct drm_connector *conn)
{
        if (!atomic_dec_and_test(&conn->base.refcount.refcount))
                return;

        drm_connector_free(conn);
}

/**
 * drm_connector_list_iter_next - return next connector
 * @iter: connector_list iterator
 *
 * Returns the next connector for @iter, or NULL when the list walk has
 * completed.
 */
struct drm_connector *
drm_connector_list_iter_next(struct drm_connector_list_iter *iter)
{
        struct drm_connector *old_conn = iter->conn;
        struct drm_mode_config *config = &iter->dev->mode_config;
        struct list_head *lhead;

        lhead = old_conn ? &old_conn->head : &config->connector_list;

        do {
                if (lhead->next == &config->connector_list) {
                        iter->conn = NULL;
                        break;
                }

                lhead = lhead->next;
                iter->conn = list_entry(lhead, struct drm_connector, head);

                /* loop until it's not a zombie connector */
        } while (!kref_get_unless_zero(&iter->conn->base.refcount));

        if (old_conn)
                __drm_connector_put_safe(old_conn);

        return iter->conn;
}

/**
 * drm_connector_list_iter_end - tear down a connector_list iterator
 * @iter: connector_list iterator
 *
 * Tears down @iter and releases any resources (like &drm_connector references)
 * acquired while walking the list. This must always be called, both when the
 * iteration completes fully or when it was aborted without walking the entire
 * list.
 */
void drm_connector_list_iter_end(struct drm_connector_list_iter *iter)
{
        iter->dev = NULL;
        if (iter->conn) {
                __drm_connector_put_safe(iter->conn);
        }
        lock_release(&connector_list_iter_dep_map, 0, _RET_IP_);
}
#endif

static int dm_suspend(void *handle)
{
	struct xdx_device *xdev = (struct xdx_device *)handle;
	struct drm_atomic_state *state = drm_atomic_helper_suspend(xdev_to_drm(xdev));
	int ret;
	struct drm_connector *connector;
	struct drm_connector_list_iter iter;

	ret = PTR_ERR_OR_ZERO(state);
	if (ret)
		dev_err(xdev->dev, "Suspending crtc's failed with %i\n",
			ret);
	else
		xdev->modeset_restore_state = state;

	/* stop hpd hw irq and drain hpd workqueue for all connectors */
	drm_connector_list_iter_begin(xdev_to_drm(xdev), &iter);
	drm_for_each_connector_iter(connector, &iter)
		if ((connector->connector_type == DRM_MODE_CONNECTOR_DisplayPort) ||
			(connector->connector_type == DRM_MODE_CONNECTOR_VGA))	{
#if defined(SUPPORT_PANGU_DC_DP)
			dptx_interrupt_disable(connector_to_dp(connector));
#endif
			drain_workqueue(connector_to_dp(connector)->workqueue);
		} else if ((connector->connector_type == DRM_MODE_CONNECTOR_HDMIA) ||
			(connector->connector_type == DRM_MODE_CONNECTOR_DVII))	{
#if defined(SUPPORT_PANGU_DC_HDMI)
			xdx_hdmi_hpd_irq_disable(connector_to_hdmi(connector));
#endif
			drain_workqueue(connector_to_hdmi(connector)->workqueue);
		}
	drm_connector_list_iter_end(&iter);



	return ret;
}

static int dm_resume(void *handle)
{
	struct xdx_device *xdev = (struct xdx_device *)handle;
	struct xdx_dm *dm = &xdev->dm;
	int i,ret;

	dm_hw_init(handle);

	for(i = 0; i < dm->dc_num; i++){
#if defined(SUPPORT_PANGU_DC_HDMI)
		xdx_hdmi_hw_init(dm, i);
#endif
#if defined(SUPPORT_PANGU_DC_DP)
		xdx_dp_hw_init(dm, i);
#endif
	}

	ret = drm_atomic_helper_resume(xdev_to_drm(xdev),
				       xdev->modeset_restore_state);
	if (ret) {
		dev_err(xdev->dev, "Resuming crtc's failed with %i\n",
			ret);
		return ret;
	}

	return ret;
}

static bool dm_is_idle(void *handle)
{
	return 0;
}

static int dm_wait_for_idle(void *handle)
{
	return 0;
}

int xdx_dm_init(void *handle)
{
	int ret;

	ret = dm_sw_init(handle);
	if (ret)
		return ret;

	ret = dm_hw_init(handle);

	return ret;
}

int xdx_dm_fini(void *handle)
{
	int ret;

	ret = dm_hw_fini(handle);
	if (ret)
		return ret;

	ret = dm_sw_fini(handle);	

	return ret;
}

static int enter_rpm(struct dm_rpm_data *data)
{
	struct drm_framebuffer *fb;
	uint64_t size = (uint64_t) data->normal_gobj->size;
	void *src_vaddr, *vaddr = NULL;
	u64 paddr = 0;
	struct xdx_bo *bo;
	struct xdx_bo_va *va;
	int ret;
	int gddr_cnt = data->xdev->fw_st_info.gddr_count;
	uint32_t chunk;
	uint64_t offset;
	uint32_t page_offset;

#ifdef CONFIG_MMU_1K_16M
	ret = xdx_bo_create_reserved(data->xdev,
				     round_up(size, 1 << 24) * gddr_cnt,
				     PAGE_SIZE << 12, XDXGPU_PL_VRAM,
				     &bo, &paddr, (void **)&vaddr);
#else
	ret = xdx_bo_create_reserved(data->xdev, size * gddr_cnt,
				     PAGE_SIZE, XDXGPU_PL_VRAM,
				     &bo, &paddr, (void **)&vaddr);
#endif
	if (ret || !bo || !vaddr || !paddr) {
		dev_err(data->xdev->dev,
			"failed to create rpm fb\n");
		return ret;
	}

	src_vaddr = vmalloc(size);
	xdx_bo_sparse_copy(to_xdx_gem(data->normal_gobj), src_vaddr);

	offset = rpm_valid_gddr_offset(data->xdev, bo->pages[0]);
	vaddr += offset;
	page_offset = offset / PAGE_SIZE;

	/* point bo's first page to the rpm active one, to serve dc mmu */
	data->pages_backup = bo->pages;
	data->num_pages_backup = bo->num_pages;
	bo->pages = &bo->pages[page_offset];
	bo->num_pages -= page_offset;

	for (chunk = 0; chunk < size / PAGE_SIZE; chunk++)
		memcpy(vaddr + chunk * PAGE_SIZE * gddr_cnt,
		       src_vaddr + chunk * PAGE_SIZE, PAGE_SIZE);
	vfree(src_vaddr);

	data->rpm_gobj = &bo->base;


	fb = data->plane->base.base.state->fb;
#if (LINUX_VERSION_CODE > KERNEL_VERSION(4, 13, 0)) || (defined(XDXGPU_CENTOS79_SUPPORT))
	fb->obj[0] = &bo->base;
	bo = to_xdx_gem(fb->obj[0]);
#else
	bo = to_xdx_gem(&bo->base);
#endif

	va = xdx_vm_bo_find(&data->xdev->vmm, &data->xdev->vmm.dc, bo);
	if (!va)
		va = xdx_vm_bo_add(&data->xdev->vmm, &data->xdev->vmm.dc, bo);
	WARN_ON(!va);
	data->plane->base.dma_addr[0] = va->addr.low + fb->offsets[0];

	dm_dc_plane_update(&data->plane->base);
	dc8200_hw_rpm_update(to_dc8200_dc(data->xdev->dm.dc[0]));

//	if (dm->dc_num == 2) {
		//todo, need to do power control for dc[1]
//	}
	return 0;
}

static int exit_rpm(struct dm_rpm_data *data)
{
	struct drm_framebuffer *fb;
	struct xdx_bo *bo;
#if (LINUX_VERSION_CODE <= KERNEL_VERSION(4, 13, 0)) && (!defined(XDXGPU_CENTOS79_SUPPORT))
	struct drm_xdxgpu_framebuffer *xdx_fb;
#endif

	fb = data->plane->base.base.state->fb;
#if (LINUX_VERSION_CODE <= KERNEL_VERSION(4, 13, 0)) && (!defined(XDXGPU_CENTOS79_SUPPORT))
	xdx_fb = to_xdx_fb(fb);
	xdx_fb->obj[0] = data->normal_gobj;
#else
	fb->obj[0] = data->normal_gobj;
#endif

	/* restore origin pages view for cleaning */
	bo = to_xdx_gem(data->rpm_gobj);
	bo->pages = data->pages_backup;
	bo->num_pages = data->num_pages_backup;
#if ((LINUX_VERSION_CODE >= KERNEL_VERSION(5, 1, 0)) || (defined(REDHAT_83))) || (defined(XDXGPU_CENTOS82_SUPPORT)) || (defined(XDXGPU_CENTOS85_SUPPORT))
	data->rpm_gobj->funcs->free(data->rpm_gobj);
#else
	data->xdev->ddev->driver->gem_free_object(data->rpm_gobj);
#endif

	return 0;
}

static void xdx_dm_rpm_data_init(struct xdx_dm *dm)
{
	struct dc8200_dc *hw = to_dc8200_dc(dm->dc[0]);
	struct dc8200_plane *plane;
	struct drm_gem_object *gobj;
	struct dm_rpm_data *rpm_data;
#if (LINUX_VERSION_CODE <= KERNEL_VERSION(4, 13, 0)) && (!defined(XDXGPU_CENTOS79_SUPPORT))
	struct drm_xdxgpu_framebuffer *xdx_fb;
#endif
	u8 id, layer_num = hw->info->layer_num;
	u32 i;

	if (dm->dc_num == 2) {
		//todo, need to do power control for dc[1]
	}

	for (i = 0; i < layer_num; i++) {
		id = hw->info->planes[i].id;
		if (id == PRIMARY_PLANE_0 || id == PRIMARY_PLANE_1) {
			plane = &hw->plane[i];
			if (!plane->base.base.state->fb)
				/* display not connected */
				continue;
#if (LINUX_VERSION_CODE <= KERNEL_VERSION(4, 13, 0)) && (!defined(XDXGPU_CENTOS79_SUPPORT))
			xdx_fb = to_xdx_fb(plane->base.base.state->fb);
			gobj = xdx_fb->obj[0];
#else
			gobj = plane->base.base.state->fb->obj[0];
#endif

			rpm_data = kmalloc(sizeof(*rpm_data), GFP_KERNEL);
			rpm_data->xdev = dm_to_xdev(dm);
			rpm_data->plane = plane;
			rpm_data->normal_gobj = gobj;
			dm->rpm_data[i] = rpm_data;
		}
	}
}

static int get_max_vrefresh_interval(struct xdx_dm *dm)
{
	int i;
	struct dm_rpm_data *rpm_data;
	struct drm_crtc *crtc;
	int result = 0;


	for (i = 0; i < XDX_MAX_CRTCS; i++) {
		rpm_data = dm->rpm_data[i];
		if (!rpm_data)
			return result + 1;
		crtc = rpm_data->plane->base.base.state->crtc;
		result = max(result,
			     1000 / drm_mode_vrefresh(&crtc->state->mode));
	}

	return result + 1;
}

int xdx_dm_rpm_suspend(struct xdx_dm *dm)
{
	int i;
	struct dm_rpm_data *rpm_data;

	/*
	 * display connectors might be hot-pluged
	 * so always scan current status
	 */
	xdx_dm_rpm_data_init(dm);

	for (i = 0; i < XDX_MAX_CRTCS; i++) {
		rpm_data = dm->rpm_data[i];
		if (!rpm_data)
			continue;
		enter_rpm(rpm_data);
	}

	/* wait for dc scanout the previous fb */
	msleep(get_max_vrefresh_interval(dm));
	return 0;
}

int xdx_dm_rpm_resume(struct xdx_dm *dm)
{
	int i;
	struct dm_rpm_data *rpm_data;

	for (i = 0; i < XDX_MAX_CRTCS; i++) {
		rpm_data = dm->rpm_data[i];
		if (!rpm_data)
			continue;
		exit_rpm(rpm_data);

		kfree(rpm_data);
		dm->rpm_data[i] = NULL;
	}

	check_and_handle_underflow(dm->dc[0]);

	if (dm->dc_num == 2) {
		//todo, need to do power control for dc[1]
	}
	return 0;
}

const struct xdx_ip_block_version dm_ip_block_version =
{
	.name = "dm",
	.type = XDX_IP_BLOCK_TYPE_DC,
	.major = 1,
	.minor = 0,
	.rev = 0,
	.funcs = {
		.early_init = dm_early_init,
		.late_init = dm_late_init,
		.sw_init = dm_sw_init,
		.sw_fini = dm_sw_fini,
		.early_fini = dm_early_fini,
		.hw_init = dm_hw_init,
		.hw_fini = dm_hw_fini,
		.suspend = dm_suspend,
		.resume = dm_resume,
		.is_idle = dm_is_idle,
		.wait_for_idle = dm_wait_for_idle,
	},
};
