/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef __DM_DC_H__
#define __DM_DC_H__

#include "dm_crtc.h"
#include "dm_plane.h"
#include "dm_dec.h"
#if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 0, 0)
#include <drm/drm_blend.h>
#endif

struct xdx_dm;
struct dm_dc;

#define XDX_MAX_CRTCS 6

enum dc_color_format {
	DC_FORMAT_X4R4G4B4,
	DC_FORMAT_A4R4G4B4,
	DC_FORMAT_X1R5G5B5,
	DC_FORMAT_A1R5G5B5,
	DC_FORMAT_R5G6B5,
	DC_FORMAT_X8R8G8B8,
	DC_FORMAT_A8R8G8B8,
	DC_FORMAT_YUY2,
	DC_FORMAT_UYVY,
	DC_FORMAT_INDEX8,
	DC_FORMAT_MONOCHROME,
	DC_FORMAT_YV12 = 0xf,
	DC_FORMAT_A8,
	DC_FORMAT_NV12,
	DC_FORMAT_NV16,
	DC_FORMAT_RG16,
	DC_FORMAT_R8,
	DC_FORMAT_NV12_10BIT,
	DC_FORMAT_A2R10G10B10,
	DC_FORMAT_NV16_10BIT,
	DC_FORMAT_INDEX1,
	DC_FORMAT_INDEX2,
	DC_FORMAT_INDEX4,
	DC_FORMAT_P010,
	DC_FORMAT_YUV444,
	DC_FORMAT_YUV444_10BIT,
};

enum dc_out {
	DC_OUT_DPI,
	DC_OUT_DP,
};

enum dc_cursor_size {
	DC_CURSOR_SIZE_32X32 = 0,
	DC_CURSOR_SIZE_64X64,
};

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0)) || (defined(XDXGPU_CENTOS82_SUPPORT))
enum dc_blend_mode {
	/* out.rgb = plane_alpha * fg.rgb +
     *      (1 - (plane_alpha * fg.alpha)) * bg.rgb
     */
	DC_BLEND_PREMULTI,
	/* out.rgb = plane_alpha * fg.alpha * fg.rgb +
     *      (1 - (plane_alpha * fg.alpha)) * bg.rgb
     */
	DC_BLEND_COVERAGE,
	/* out.rgb = plane_alpha * fg.rgb +
     *      (1 - plane_alpha) * bg.rgb
     */
	DC_BLEND_PIXEL_NONE,
};
#endif

enum dc_rotation {
	DC_ROT_0 = 0,
	DC_ROT_90 = 4,
	DC_ROT_180 = 5,
	DC_ROT_270 = 6,
	DC_FLIP_X = 1,
	DC_FLIP_Y = 2,
	DC_FLIP_XY = 3,
};

enum dc_swizzle {
	DC_SWIZZLE_ARGB = 0,
	DC_SWIZZLE_RGBA,
	DC_SWIZZLE_ABGR,
	DC_SWIZZLE_BGRA,
};

enum dc_yuv_color_space {
	DC_YUV_COLOR_SPACE_601 = 0,
	DC_YUV_COLOR_SPACE_709 = 1,
	DC_YUV_COLOR_SPACE_2020 = 3,
};

#define fourcc_mod_xdx_get_type(val) \
            (((val) & DRM_FORMAT_MOD_XDX_TYPE_MASK) >> 54)

struct dc_fb {
	u32 y_address;
	u32 u_address;
	u32 v_address;
	u32 clear_value;
	u32 water_mark;
	u16 y_stride;
	u16 u_stride;
	u16 v_stride;
	u16 width;
	u16 height;
	u8 format;
	u8 tile_mode;
	u8 rotation;
	u8 yuv_color_space;
	u8 swizzle;
	u8 uv_swizzle;
	u8 zpos;
	u8 display_id;
	bool clear_enable;
	bool dec_enable;
	bool enable;
};

struct dc_scale {
	u32 scale_factor_x;
	u32 scale_factor_y;
	bool enable;
};

struct dc_position {
	u16 start_x;
	u16 start_y;
	u16 end_x;
	u16 end_y;
};

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0)) || (defined(XDXGPU_CENTOS82_SUPPORT))
struct dc_blend {
	u8 alpha;
	u8 blend_mode;
};
#endif

struct dc_colorkey {
	u32 colorkey;
	u32 colorkey_high;
	bool transparency;
};

struct dc_roi {
	u16 x;
	u16 y;
	u16 width;
	u16 height;
	bool enable;
};

struct dc_cursor {
	u32 address;
	u16 x;
	u16 y;
	u16 hot_x;
	u16 hot_y;
	u8 size;
	u8 display_id;
	bool enable;
};

struct dc_display {
	u32 color_format;
	u32 bpc;
	u16 h_active;
	u16 h_total;
	u16 h_sync_start;
	u16 h_sync_end;
	u16 v_active;
	u16 v_total;
	u16 v_sync_start;
	u16 v_sync_end;
	u16 sync_mode;
	u32 bg_color;
	u8 id;
	bool h_sync_polarity;
	bool v_sync_polarity;
	bool enable;
	bool sync_enable;
	bool dither_enable;
};

struct dc_qos {
	u8 low_value;
	u8 high_value;
};

struct dc_hw_info {
	uint32_t pitch_alignment;
};

struct dm_dc_funcs {
	void (*destroy)(struct dm_dc *dc);
	void (*reg_write)(struct dm_dc *dc, u32 reg, u32 value);
	u32 (*reg_read)(struct dm_dc *dc, u32 reg);
	void (*enable_interrupt)(struct dm_dc *dc, bool enable);
	int (*hw_init)(struct dm_dc *dc);
	void (*update_roi)(struct dm_dc *dc, struct dm_plane *dm_plane,
			   struct dc_roi *roi);
	void (*update_degamma)(struct dm_dc *dc, struct dm_plane *dm_plane,
			       u32 mode);
	void (*update_colorkey)(struct dm_dc *dc, struct dm_plane *dm_plane,
				struct dc_colorkey *colorkey);
	#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 20, 0)) || (defined(XDXGPU_CENTOS82_SUPPORT))
	void (*update_plane)(struct dm_dc *dc, struct dm_plane *dm_plane,
			     struct dc_fb *fb, struct dc_scale *scale,
			     struct dc_position *pos, struct dc_blend *blend);
	#else
	void (*update_plane)(struct dm_dc *dc, struct dm_plane *dm_plane,
			     struct dc_fb *fb, struct dc_scale *scale,
			     struct dc_position *pos);
	#endif
	void (*dec_config)(struct dm_dc *dc, struct dm_plane *dm_plane,
			   struct dec_fb *dec_fb);
	void (*update_qos)(struct dm_dc *dc, struct dc_qos *qos);
	void (*update_cursor)(struct dm_dc *dc, u8 id, struct dc_cursor *cursor);
	void (*interrupt_handle)(struct dm_dc *dc);
	void (*mmu_flush)(struct dm_dc *dc);
	void (*query_hw_info)(struct dm_dc *dc, struct dc_hw_info *info);
};

#define DC_WREG32(reg, v) dc->funcs->reg_write(dc, (reg), (v))
#define DC_RREG32(reg) dc->funcs->reg_read(dc, (reg))

typedef struct __dc_irq_callback
{
	void (*func)(void *user_data);
	void *user_data;
	struct list_head list;
}dc_irq_callback;

/* display controler */
struct dm_dc {
	struct xdx_dm *dm;
	struct dm_crtc *crtcs[XDX_MAX_CRTCS];
	/* 0 means no gamma LUT */
	u16 gamma_size;
	u8 gamma_bits;
	u8 id;
	const struct dm_dc_funcs *funcs;
	struct xdx_irq_src *irq_src;
	struct list_head dc_irq_tbl;
};

int dm_dc_universal_init(struct xdx_dm *dm, struct dm_dc *dc,
			 const struct dm_dc_funcs *funcs, u32 id);
void dm_dc_universal_fini(struct dm_dc *dc);

int dm_dc_hw_init(struct dm_dc *dc);
void dm_dc_hw_fini(struct dm_dc *dc);

void dm_dc_plane_update(struct dm_plane *dm_plane);
void dm_dc_plane_disable(struct dm_plane *dm_plane,
				 struct drm_plane_state *old_state);

uint32_t dm_dc_get_crtc_mask(struct dm_dc *dc);

#endif /* __DM_DC_H__ */
