/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef __DM_PLANE_H__
#define __DM_PLANE_H__

#include <drm/drm_plane_helper.h>
#include <drm/drm_fourcc.h>

#include "dm_fb.h"

#define MAX_NUM_PLANES 3 /* colour format plane */

struct dm_plane;
struct dm_dc;

struct dm_plane_funcs {
	void (*destroy)(struct dm_plane *dm_plane);
	void (*update)(struct dm_plane *dm_plane);
	void (*disable)(struct dm_plane *dm_plane,
			struct drm_plane_state *old_state);
	int (*check)(struct dm_plane *dm_plane, struct drm_plane_state *state);
};

struct dm_plane_status {
	u32 tile_mode;
	struct drm_rect src;
	struct drm_rect dest;
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 14, 0) && LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0)
	struct drm_format_name_buf format_name;
#endif
};

struct dm_plane_state {
	struct drm_plane_state base;
	struct dm_plane_status status; /* for debugfs */

	struct drm_property_blob *watermark;
	struct drm_property_blob *color_mgmt;
	struct drm_property_blob *roi;

	u32 degamma;
	bool degamma_changed;
};

static inline struct dm_plane_state *
to_dm_plane_state(struct drm_plane_state *state)
{
	return container_of(state, struct dm_plane_state, base);
}

struct dm_plane {
	struct drm_plane base;
	struct xdx_dm *dm;
	struct dm_dc *dc;
	dma_addr_t dma_addr[MAX_NUM_PLANES];
	u8 id;
	u8 zpos;
	struct drm_property *degamma_mode;
	struct drm_property *watermark_prop;
	struct drm_property *color_mgmt_prop;
	struct drm_property *roi_prop;

	const struct dm_plane_funcs *funcs;
};

static inline struct dm_plane *to_dm_plane(struct drm_plane *plane)
{
	return container_of(plane, struct dm_plane, base);
}

static inline struct xdx_dm* dm_plane_to_dm(struct dm_plane *plane)
{
	return plane->dm;
}

int dm_universal_plane_init(struct xdx_dm *dm, struct dm_plane *dm_plane,
			    struct dm_dc *dc, uint32_t possible_crtcs,
			    struct dm_plane_funcs *funcs,
			    const uint32_t *formats, unsigned int format_count,
			    const uint64_t *format_modifiers, enum drm_plane_type type,
			    uint32_t id, u8 zpos, const char *name);
#endif /* __DM_PLANE_H__ */
