/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#ifndef __XDX_HDMI_DRV_H__
#define __XDX_HDMI_DRV_H__

#include <linux/version.h>
#include <linux/hdmi.h>
#include <drm/drm_of.h>
#include <drm/drm_edid.h>
#include <drm/drm_atomic_helper.h>
#include "dm.h"
#include "../dp/xdx_dp_drv.h"

#define PANGU_DEVICE_NAME_HDMI          "xdx_hdmi"
#define HDMI_DEBUG                      1
#define REFCLK_HZ                       300000000
//#define REFCLK_HZ                     80000000
#define PB0_OFFSET                      3
//#define IIC_HAVE_IRQ
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(6, 8, 0))
#define I2C_CLASS_DDC                   (1 << 3)
#endif
struct hdmi_tx_phy_ops {
    int (*init)(struct hdmi_dev *hdmi);
    int (*enable)(struct hdmi_dev *hdmi);
    int (*disable)(struct hdmi_dev *hdmi);
};

struct xdx_hdmi_i2c {
    struct  i2c_adapter adap;
    struct  mutex lock;
#ifdef IIC_HAVE_IRQ
    struct  completion comp;
#endif
    u32     stat;
    u8  slave_reg;
    bool    is_regaddr;
    bool    is_segment;
    u32 fast_speed;
    u32 slow_speed;
};

typedef struct {
    enum hdmi_colorspace colorspace;
    enum hdmi_quantization_range rgb_quant_range;
    enum hdmi_colorimetry colorimetry;
    enum hdmi_extended_colorimetry extended_colorimetry;
    enum hdmi_content_type content_type;
    bool itc;
    u32  bpc;
} videoParams_t;

struct hdmi_trans_para {
    int clock;
    int hdisplay;
    int hsync_start;
    int hsync_end;
    int htotal;
    int hskew;
    int vdisplay;
    int vsync_start;
    int vsync_end;
    int vtotal;
    int vscan;
    int vrefresh;
    u32 flag;

    u32 colordeep;
    u32 colorformat;
    u32 vic_code;
    u32 pixclock;
    u32 mode;
    u32 bpc;
    u32 multipixel;
    u32 mode_flag_pvsync;
    u32 lanes;
    u32 rate;
    u32 dc_idx;
    enum hdmi_picture_aspect picture_aspect_ratio;
    bool scrambling_support;
    u8 eq_vboost;
};

struct hdmi_dev {
    struct device *dev;
    struct drm_connector connector;
    struct drm_encoder encoder;
    struct xdx_hdmi_i2c *i2c;
    struct i2c_adapter *ddc;
    /* Memory Resources */
    //void __iomem      *mem[MAX_MEM_IDX];
    //void __iomem      *mb;

    struct xdx_output  *output;
    struct workqueue_struct *workqueue;
    struct delayed_work work;

    /* Video parameters */
    struct drm_display_info display_info;
    struct drm_display_mode hdmi_mode;
    struct edid *edid;
    videoParams_t   pVideo;

    struct {
        const struct hdmi_tx_phy_ops *ops;
        const char *name;
        bool enabled;
        int eq_value;
    } phy;

    struct mutex mutex;
    enum drm_connector_force force;
    bool disabled;
    bool hpd_state;
    bool sink_is_hdmi;
    bool sink_has_audio;
    int vic;
	u32 id;

    struct xdx_irq_src *irq_src;
    void (*core_write)(struct hdmi_dev *hdmi, u32 offset, u32 value);
    u32 (*core_read)(struct hdmi_dev *hdmi, u32 offset);
};

static inline struct xdx_output *hdmi_to_output(struct hdmi_dev *hdmi)
{
    return hdmi->output;
}

static inline struct hdmi_dev *connector_to_hdmi(struct drm_connector *connector) {
    return container_of(connector, struct hdmi_dev, connector);
}
static inline struct hdmi_dev *encoder_to_hdmi(struct drm_encoder *encoder) {
    return container_of(encoder, struct hdmi_dev, encoder);
}

int xdx_hdmi_init(struct xdx_dm *dm, u32 id);
int xdx_hdmi_deinit(struct xdx_dm *dm, u32 id);
void xdx_hdmi_hpd_irq_disable(struct hdmi_dev *hdmi);
int xdx_hdmi_hw_init(struct xdx_dm *dm, u32 id);
void xdx_hdmi_suspend(struct hdmi_dev *hdmi);
void xdx_hdmi_resume(struct xdx_dm *dm, u32 id);

#endif /* __XDX_HDMI_DRV_H__ */
