/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#include <linux/version.h>
#include <linux/platform_device.h>
#include <linux/dmaengine.h>
#include <linux/module.h>
#include <linux/dma-mapping.h>
#include <linux/types.h>
#include <linux/of_dma.h>
#include <linux/of_device.h>
#include <linux/kernel.h>
#include <linux/cdev.h>
#include <linux/fs.h>
#include <linux/slab.h>
#include <linux/uaccess.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0))
#include <linux/sched/signal.h>
#else
#include <linux/signal.h>
#endif
#include "dma_drv_common.h"
#include "xdx_dma.h"
#include "img_types.h"
#if LINUX_VERSION_CODE <= KERNEL_VERSION(3, 10, 0)
bool vdma_mem_to_mem_filter(struct dma_chan *chan, void *param)
{
	struct device *dev = param;
	if (strstr(dev_name(dev), "xdxVdma"))
		return true;
	return false;
}

bool tdma_mem_to_mem_filter(struct dma_chan *chan, void *param)
{
	struct device *dev = param;
	if (strstr(dev_name(dev), "xdxTdma"))
		return true;
	return false;
}
#endif
struct dma_chan *get_dma_channel(struct device *dma_dev)
{
	struct dma_chan *vchan;
#if LINUX_VERSION_CODE <= KERNEL_VERSION(3, 10, 0)
	dma_cap_mask_t mask;
#endif
	if (!dma_dev) {
		XDX_DMA_ERR(dma_dev, "[%s]dma_dev is NULL", __func__);
		return NULL;
	}
	XDX_DMA_INFO(dma_dev, "xdx_dma(%s) = 0x%llx\n", dev_name(dma_dev), (IMG_UINT64)(dma_dev));
#if LINUX_VERSION_CODE > KERNEL_VERSION(3, 10, 0)
	vchan = dma_request_chan(dma_dev, "xdx_dma");
#else
	dma_cap_zero(mask);
	dma_cap_set(DMA_PRIVATE, mask);
	dma_cap_set(DMA_SLAVE, mask);
	vchan = dma_request_channel(mask, vdma_mem_to_mem_filter, dma_dev);
#endif
	if(IS_ERR_OR_NULL(vchan)) {
		XDX_DMA_ERR(dma_dev,  "Get DMAchan failed %ld.\n", PTR_ERR(vchan));
		return NULL;
	}
	XDX_DMA_DBG(dma_dev, "Requested Dma chan id = %d\n", vchan->chan_id);
	return vchan;
}

struct dma_chan *get_tdma_channel(struct device *tdma_dev)
{
	struct dma_chan *vchan;
#if LINUX_VERSION_CODE <= KERNEL_VERSION(3, 10, 0)
	dma_cap_mask_t mask;
#endif

	if (!tdma_dev) {
		XDX_DMA_ERR(tdma_dev, "[%s]tdma_dev is NULL", __func__);
		return NULL;
	}
	XDX_DMA_INFO(tdma_dev, "xdx_tdma(%s) = 0x%llx\n", dev_name(tdma_dev), (IMG_UINT64)(tdma_dev));
#if LINUX_VERSION_CODE > KERNEL_VERSION(3, 10, 0)
	vchan = dma_request_chan(tdma_dev, "xdx_twiddle_dma");
#else
	dma_cap_zero(mask);
	dma_cap_set(DMA_PRIVATE, mask);
	dma_cap_set(DMA_SLAVE, mask);
	vchan = dma_request_channel(mask, tdma_mem_to_mem_filter, tdma_dev);
#endif
	if(vchan != NULL)
		XDX_DMA_INFO(tdma_dev, "Tdma chan id = %d\n", vchan->chan_id);
	else
		return NULL;
	return vchan;
}

void release_dma_channel(struct dma_chan *pchan)
{
	dma_release_channel(pchan);
	XDX_DMA_DBG(NULL, "release_dma_channel ...\n");
	return;
}
