/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

//#include "Debug.h"
//#include "HalReg.h"
//#include "TStamp.h"
//#include "MemIo.h"
#include "HalTDMA.h"
#include "utils.h"
#include <linux/delay.h>
#include "xdx_dma_dbglevel.h"

void InitTDMA(uint64_t gTdmaBaseAddr)
{
    TDMA_StreamId_T sSid;
    TDMA_NdfAxiCtrl_T rNdfAxiCtrl;
    TDMA_HostAxiCtrl_T rHostAxiCtrl;

    TDMA_SoftReset(gTdmaBaseAddr);

    sSid.RpaSid = 0;
    sSid.RvaSid = 1;
    sSid.WpaSid = 2;
    sSid.WvaSid = 3;
    TDMA_SetStreamId(gTdmaBaseAddr, &sSid);

    rNdfAxiCtrl.all = 0;
    rNdfAxiCtrl.ArLen = TDMA_COM_AXLEN_8;
    rNdfAxiCtrl.AwLen = TDMA_COM_AXLEN_8;
    rNdfAxiCtrl.AwCache = 0xf;
    TDMA_WrNdfAxiCtrl(gTdmaBaseAddr, rNdfAxiCtrl.all);

    rHostAxiCtrl.all = 0;
    rHostAxiCtrl.ArLen = TDMA_COM_AXLEN_8;
    TDMA_WrHostAxiCtrl(gTdmaBaseAddr, rHostAxiCtrl.all);

    TDMA_WrNdfOutStandingNum(gTdmaBaseAddr, 512);

//    TDMA_DumpCtrlRegs(gTdmaBaseAddr);
}

void TDMA_SoftReset(uint64_t tdmaBaseAddr)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;

    dmaRegs->SOFT_RST = 1;
 //   TStamp_DelayUS(10);
    dmaRegs->SOFT_RST = 0;
}

void TDMA_EnDisIntr(uint64_t tdmaBaseAddr, uint32_t enDis)
{
    TDMA_REGS_T *dmaRegs;
    TDMA_IntrCtrlReg_T intrCtrlReg;

    DBG_ASSERT(0 != tdmaBaseAddr);
   // DBG_ASSERT(enDis <= ENABLE);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;

    intrCtrlReg.all = dmaRegs->DMA_INTR_CTRL;
    intrCtrlReg.IntrEn = enDis;
    dmaRegs->DMA_INTR_CTRL = intrCtrlReg.all;
}

void TDMA_SetIntrDst(uint64_t tdmaBaseAddr, uint32_t intrDst)
{
    TDMA_REGS_T *dmaRegs;
    TDMA_IntrCtrlReg_T intrCtrlReg;

    DBG_ASSERT(0 != tdmaBaseAddr);
    DBG_ASSERT((intrDst <= TDMA_INTRDST_META1) && (intrDst != TDMA_INTRDST_RSVD));

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;

    intrCtrlReg.all = dmaRegs->DMA_INTR_CTRL;
    intrCtrlReg.IntrDst = intrDst;
    dmaRegs->DMA_INTR_CTRL = intrCtrlReg.all;
}

void TDMA_WrHostAxiCtrl(uint64_t tdmaBaseAddr, uint32_t regVal)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;
    dmaRegs->HOST_AXI_CTRL = regVal;
}

uint32_t TDMA_RdHostAxiCtrl(uint64_t tdmaBaseAddr)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;
    return dmaRegs->HOST_AXI_CTRL;
}

void TDMA_WrNdfAxiCtrl(uint64_t tdmaBaseAddr, uint32_t regVal)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;
    dmaRegs->NDF_AXI_CTRL = regVal;
}

uint32_t TDMA_RdNdfAxiCtrl(uint64_t tdmaBaseAddr)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;
    return dmaRegs->NDF_AXI_CTRL;
}

void TDMA_WrNdfOutStandingNum(uint64_t tdmaBaseAddr, uint32_t regVal)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;
    dmaRegs->NDF_WRITE_OUTSTANDING_NUM = TDMA_LEN_ADJUST(regVal);
}

void TDMA_SetStreamId(uint64_t tdmaBaseAddr, TDMA_StreamId_T *psSid)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;
    dmaRegs->RVA_STREAM_ID = psSid->RvaSid;
    dmaRegs->RPA_STREAM_ID = psSid->RpaSid;
    dmaRegs->WVA_STREAM_ID = psSid->WvaSid;
    dmaRegs->WPA_STREAM_ID = psSid->WpaSid;
}

uint32_t TDMA_GetCmdRemainSize(uint64_t tdmaBaseAddr)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;

    return dmaRegs->REMAIN_CMD_BUF_SIZE;
}

void TDMA_WrCmdBuf(uint64_t tdmaBaseAddr, uint64_t *pu64WrDataBuf, uint32_t WrDwlen)
{
    TDMA_REGS_T *dmaRegs;
    uint64_t* CmdBuf;
    uint32_t i;

    DBG_ASSERT(0 != tdmaBaseAddr);
    DBG_ASSERT(WrDwlen != 0);
    DBG_ASSERT(pu64WrDataBuf != NULL);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;

    CmdBuf = (uint64_t *)&(dmaRegs->CMD[0]);
    for (i = 0; i < WrDwlen; i++) {
        CmdBuf[i] = pu64WrDataBuf[i];
    }
}

uint64_t TDMA_RdCmdCmplIntSts(uint64_t tdmaBaseAddr)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;
    return dmaRegs->CMD_CMPL_INTR_STS;
}

uint64_t TDMA_RdBusErrIntSts(uint64_t tdmaBaseAddr)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;
    return dmaRegs->BUS_ERR_INTR_STS;
}

uint64_t TDMA_RdCmdCmplRawSts(uint64_t tdmaBaseAddr)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;
    return dmaRegs->CMD_CMPL_RAW_STS;
}

uint64_t TDMA_RdBusErrRawSts(uint64_t tdmaBaseAddr)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;
    return dmaRegs->BUS_ERR_RAW_STS;
}

void TDMA_WrCmdCmplClrReg(uint64_t tdmaBaseAddr, uint64_t clrRegVal)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;
    dmaRegs->CMD_CMPL_RAW_CLR = clrRegVal;
}

void TDMA_WrBusErrClrReg(uint64_t tdmaBaseAddr, uint64_t clrRegVal)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;
    dmaRegs->BUS_ERR_RAW_CLR = clrRegVal;
}

uint64_t TDMA_RdDbgCmdCmplRawSts(uint64_t tdmaBaseAddr)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;
    return dmaRegs->DBG_CMD_CMPL_RAW_STS;
}

void TDMA_WrDbgCmdCmplClrReg(uint64_t tdmaBaseAddr, uint64_t clrRegVal)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;
    dmaRegs->DBG_CMD_CMPL_CLR = clrRegVal;
}

void TDMA_WrGpu0GpioCfg(uint64_t tdmaBaseAddr, uint32_t gpioCfg)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;
    dmaRegs->GPU0_GPIO_CFG = gpioCfg;
}

void TDMA_WrGpu1GpioCfg(uint64_t tdmaBaseAddr, uint32_t gpioCfg)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;
    dmaRegs->GPU1_GPIO_CFG = gpioCfg;
}

void TDMA_WrScrReg(uint64_t tdmaBaseAddr, uint32_t regVal)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;
    dmaRegs->SCRATCH = regVal;
}

int32_t TDMA_WaitXferCompleted(uint64_t tdmaBaseAddr, uint32_t tag)
{
    uint64_t CmdCmplRawSts, BusErrRawSts;
    uint64_t ExpCmdCmplRawSts;
    int32_t Ret = 0;

    ExpCmdCmplRawSts = (0x1ll<<tag);

    XDX_DMA_ERR(NULL, "Wait for TDMA completed\n");
    while (1) {
        CmdCmplRawSts = TDMA_RdCmdCmplRawSts(tdmaBaseAddr);
        if (CmdCmplRawSts) {
            XDX_DMA_ERR(NULL, "CmdCmplRawSts=0x%llx\n", CmdCmplRawSts);
            if (CmdCmplRawSts != ExpCmdCmplRawSts) {
                XDX_DMA_ERR(NULL, "Mismatch! ExpCmdCmplRawSts=0x%llx\n", ExpCmdCmplRawSts);
                Ret = 1;
            }

            TDMA_WrCmdCmplClrReg(tdmaBaseAddr, CmdCmplRawSts);
            break;
        }

        BusErrRawSts = TDMA_RdBusErrRawSts(tdmaBaseAddr);
        if (BusErrRawSts) {
            XDX_DMA_ERR(NULL, "BusErrRawSts=0x%llx\n", BusErrRawSts);
            TDMA_WrBusErrClrReg(tdmaBaseAddr, BusErrRawSts);
            break;
        }

//        TStamp_DelayUS(1000);
    }

    return Ret;
}

int32_t TDMA_CheckXferCompleted(uint64_t tdmaBaseAddr, uint32_t tag)
{
    uint64_t CmdCmplRawSts, BusErrRawSts;
    uint64_t ExpCmdCmplRawSts;
    int32_t Ret = 0;

    ExpCmdCmplRawSts = (0x1ll<<tag);

    XDX_DMA_ERR(NULL, "Wait for TDMA completed\n");
    while (1) {
        CmdCmplRawSts = TDMA_RdCmdCmplRawSts(tdmaBaseAddr);
        if (CmdCmplRawSts) {
            XDX_DMA_ERR(NULL, "CmdCmplRawSts=0x%llx\n", CmdCmplRawSts);
            if (CmdCmplRawSts != ExpCmdCmplRawSts) {
                XDX_DMA_ERR(NULL, "Mismatch! ExpCmdCmplRawSts=0x%llx\n", ExpCmdCmplRawSts);
                Ret = 1;
            }

            //TDMA_WrCmdCmplClrReg(tdmaBaseAddr, CmdCmplRawSts);
            break;
        }

        BusErrRawSts = TDMA_RdBusErrRawSts(tdmaBaseAddr);
        if (BusErrRawSts) {
            XDX_DMA_ERR(NULL, "BusErrRawSts=0x%llx\n", BusErrRawSts);
            TDMA_WrBusErrClrReg(tdmaBaseAddr, BusErrRawSts);
            break;
        }

//        TStamp_DelayUS(1000);
    }

    return Ret;
}


void TDMA_GenCmd(TDMA_CmdCfg_T *psTdmaCfg, TDMA_CmdData_T *psTdmaCmd)
{
    DBG_ASSERT(0 != psTdmaCfg);
    DBG_ASSERT(0 != psTdmaCmd);
    DBG_ASSERT(psTdmaCfg->SrcAddrMode <= TDMA_SRCADDRMODE_GPU_PA);
    DBG_ASSERT(psTdmaCfg->DstAddrMode <= TDMA_DSTADDRMODE_GPU_PA);
    DBG_ASSERT(psTdmaCfg->IntrEn <= 1);
    DBG_ASSERT(psTdmaCfg->DontSnoop <= 1);
    DBG_ASSERT(psTdmaCfg->LenUnit <= TDMA_LENUNIT_16B);
    DBG_ASSERT(psTdmaCfg->Tag <= TDMA_COMMAND_TAG_LIMIT);
    DBG_ASSERT((TDMA_OPCODE_2DCOPY == psTdmaCfg->Opcode) || (TDMA_OPCODE_3DCOPY == psTdmaCfg->Opcode));

    // clear Cmd
   // MemFillU32(&psTdmaCmd->cmd32[0], 0, sizeof(TDMA_CmdData_T));
   	memset(&psTdmaCmd->cmd32[0], 0, sizeof(TDMA_CmdData_T));

    psTdmaCmd->dw0.SrcAddr = psTdmaCfg->SrcAddr;
    psTdmaCmd->dw0.SrcAddrMode = psTdmaCfg->SrcAddrMode;
    psTdmaCmd->dw0.LenUnit = psTdmaCfg->LenUnit;
    psTdmaCmd->dw0.Tag = psTdmaCfg->Tag;
    psTdmaCmd->dw0.Opcode = psTdmaCfg->Opcode;

    psTdmaCmd->dw1.DontSnoop = psTdmaCfg->DontSnoop;
    psTdmaCmd->dw1.IntrEn = psTdmaCfg->IntrEn;
    psTdmaCmd->dw1.DstAddr = psTdmaCfg->DstAddr;
    psTdmaCmd->dw1.DstAddrMode = psTdmaCfg->DstAddrMode;

    psTdmaCmd->dw2.Xstride = TDMA_LEN_ADJUST(psTdmaCfg->Xstride);
    psTdmaCmd->dw2.Ystep = TDMA_LEN_ADJUST(psTdmaCfg->Ystep);
    psTdmaCmd->dw2.Xstep = TDMA_LEN_ADJUST(psTdmaCfg->Xstep);

    psTdmaCmd->dw3.Ystride = TDMA_LEN_ADJUST(psTdmaCfg->Ystride);
    psTdmaCmd->dw3.Zstep = TDMA_LEN_ADJUST(psTdmaCfg->Zstep);
}

#define DBG_PRINT(var, ...) XDX_DMA_PRINT(var, ## __VA_ARGS__)
#define PRINT_DBG_VAR(var) XDX_DMA_INFO_VAR(var)
#define PRINT_DBG_VAR64(var) XDX_DMA_INFO_VAR(var)

void TDMA_DumpCmd(TDMA_CmdData_T *psTdmaCmd)
{
    uint32_t i;

    DBG_ASSERT(psTdmaCmd != NULL);

    DBG_PRINT("== Dump TDMA CMD Begin ==\n");

  //  PRINT_DBG_VAR64(psTdmaCmd->dw0.SrcAddr);
    PRINT_DBG_VAR(psTdmaCmd->dw0.SrcAddrMode);
    PRINT_DBG_VAR(psTdmaCmd->dw0.LenUnit);
    PRINT_DBG_VAR(psTdmaCmd->dw0.Tag);
    PRINT_DBG_VAR(psTdmaCmd->dw0.Opcode);

    PRINT_DBG_VAR(psTdmaCmd->dw1.DontSnoop);
    PRINT_DBG_VAR(psTdmaCmd->dw1.IntrEn);
    PRINT_DBG_VAR64((uint64_t)psTdmaCmd->dw1.DstAddr);
    PRINT_DBG_VAR(psTdmaCmd->dw1.DstAddrMode);

    PRINT_DBG_VAR(psTdmaCmd->dw2.Xstride);
    PRINT_DBG_VAR(psTdmaCmd->dw2.Ystep);
    PRINT_DBG_VAR(psTdmaCmd->dw2.Xstep);

    PRINT_DBG_VAR(psTdmaCmd->dw3.Ystride);
    PRINT_DBG_VAR(psTdmaCmd->dw3.Zstep);

    for (i = 0; i < 8; ++i) {
        DBG_PRINT("CmdData[%d] = 0x%x\n", i, psTdmaCmd->cmd32[i]);
    }
}

void TDMA_DumpRegs(uint64_t tdmaBaseAddr)
{
    TDMA_REGS_T *dmaRegs;

    DBG_ASSERT(0 != tdmaBaseAddr);

    dmaRegs =  (TDMA_REGS_T*)(uint64_t)tdmaBaseAddr;

    DBG_PRINT("== Dump TDMA REG Begin ==\n");

    PRINT_DBG_VAR(dmaRegs->SOFT_RST);
    PRINT_DBG_VAR(dmaRegs->DMA_IDLE);
    PRINT_DBG_VAR(dmaRegs->DMA_INTR_CTRL);
    PRINT_DBG_VAR(dmaRegs->HOST_AXI_CTRL);
    PRINT_DBG_VAR(dmaRegs->NDF_AXI_CTRL);
    PRINT_DBG_VAR(dmaRegs->NDF_WRITE_OUTSTANDING_NUM);
    PRINT_DBG_VAR(dmaRegs->RVA_STREAM_ID);
    PRINT_DBG_VAR(dmaRegs->RPA_STREAM_ID);
    PRINT_DBG_VAR(dmaRegs->WVA_STREAM_ID);
    PRINT_DBG_VAR(dmaRegs->WPA_STREAM_ID);
    PRINT_DBG_VAR(dmaRegs->REMAIN_CMD_BUF_SIZE);
    PRINT_DBG_VAR(dmaRegs->CMD_FIFO_STS);
    PRINT_DBG_VAR(dmaRegs->SCRATCH);
    PRINT_DBG_VAR(dmaRegs->GPU0_GPIO_CFG);
    PRINT_DBG_VAR(dmaRegs->GPU1_GPIO_CFG);
    PRINT_DBG_VAR64(dmaRegs->CMD_CMPL_INTR_STS);
    PRINT_DBG_VAR64(dmaRegs->BUS_ERR_INTR_STS);
    PRINT_DBG_VAR64(dmaRegs->CMD_CMPL_RAW_STS);
    PRINT_DBG_VAR64(dmaRegs->BUS_ERR_RAW_STS);
    PRINT_DBG_VAR64(dmaRegs->CMD_CMPL_RAW_CLR);
    PRINT_DBG_VAR64(dmaRegs->BUS_ERR_RAW_CLR);
    PRINT_DBG_VAR64(dmaRegs->DBG_CMD_CMPL_RAW_STS);
    PRINT_DBG_VAR64(dmaRegs->DBG_CMD_CMPL_CLR);
    PRINT_DBG_VAR(dmaRegs->DBG_PARSE_CNT0);
    PRINT_DBG_VAR(dmaRegs->DBG_PARSE_CNT1);
    PRINT_DBG_VAR(dmaRegs->DBG_PARSE_STATE);
    PRINT_DBG_VAR(dmaRegs->DBG_RD_CNT0);
    PRINT_DBG_VAR(dmaRegs->DBG_RD_CNT1);
    PRINT_DBG_VAR(dmaRegs->DBG_RD_STATE);
    PRINT_DBG_VAR(dmaRegs->DBG_WR_CNT0);
    PRINT_DBG_VAR(dmaRegs->DBG_WR_CNT1);
    PRINT_DBG_VAR(dmaRegs->DBG_WR_CNT2);
    PRINT_DBG_VAR(dmaRegs->DBG_WR_CNT3);
    PRINT_DBG_VAR(dmaRegs->DBG_WR_CNT4);
    PRINT_DBG_VAR(dmaRegs->DBG_WR_CNT5);
    PRINT_DBG_VAR(dmaRegs->DBG_WR_CNT6);
    PRINT_DBG_VAR(dmaRegs->DBG_WR_CNT7);
    PRINT_DBG_VAR(dmaRegs->DBG_WR_STATE);
    PRINT_DBG_VAR(dmaRegs->DBG_H_AXI_RD_REAT_CNT);
    PRINT_DBG_VAR(dmaRegs->DBG_H_AXI_RD_REQ_REAT_CNT);
    PRINT_DBG_VAR(dmaRegs->DBG_H_AXI_RD_STATE0);
    PRINT_DBG_VAR(dmaRegs->DBG_H_AXI_RD_STATE1);
    PRINT_DBG_VAR(dmaRegs->DBG_N_AXI_RD_REAT_CNT);
    PRINT_DBG_VAR(dmaRegs->DBG_N_AXI_RD_REQ_REAT_CNT);
    PRINT_DBG_VAR(dmaRegs->DBG_N_AXI_RD_STATE0);
    PRINT_DBG_VAR(dmaRegs->DBG_N_AXI_RD_STATE1);
    PRINT_DBG_VAR(dmaRegs->DBG_N_AXI_WR_REAT_CNT);
    PRINT_DBG_VAR(dmaRegs->DBG_N_AXI_WR_STATE0);
    PRINT_DBG_VAR(dmaRegs->DBG_N_AXI_WR_STATE1);
    PRINT_DBG_VAR(dmaRegs->DBG_AXI_OUTSTANDING_STATE0);
    PRINT_DBG_VAR(dmaRegs->DBG_AXI_OUTSTANDING_STATE1);
}

int32_t TDMA_Copy(uint64_t tdmaBaseAddr, TDMA_CmdCfg_T *psTdmaCfg, uint8_t useIntr)
{
    TDMA_CmdData_T tdmaCmd;
    int32_t Ret = 0;

    TDMA_GenCmd(psTdmaCfg, &tdmaCmd);
//    TDMA_DumpCmd(&tdmaCmd);
    TDMA_WrCmdBuf(tdmaBaseAddr, tdmaCmd.cmd64, 4);

    if (!useIntr) {
        Ret = TDMA_WaitXferCompleted(tdmaBaseAddr, psTdmaCfg->Tag);
    }

    return Ret;
}


