/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
//#define DEBUG
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/moduleparam.h>
#include <linux/version.h>
#include <linux/dma-mapping.h>

#include "sysinfo.h"
//#include "pangu_drv.h"
#include "xdx_dma.h"
#include "B0_HalVdma.h"

#include "pangu.h"
#include "virt-dma.h"

#include "pvrmodule.h"
//#include <linux/platform_device.h>
#include <linux/dma-mapping.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/mm.h>
#include <linux/device.h>
#include <linux/dmaengine.h>
#include <linux/hardirq.h>
#include <linux/spinlock.h>
#include <linux/percpu.h>
#include <linux/rcupdate.h>
#include <linux/mutex.h>
#include <linux/jiffies.h>
#include <linux/rculist.h>
#include <linux/idr.h>
#include <linux/slab.h>
#include <linux/acpi.h>
#include <linux/acpi_dma.h>
#include <linux/of_dma.h>
#include <linux/mempool.h>
#include <linux/numa.h>
#include <linux/mod_devicetable.h>
#include <asm/cacheflush.h>

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 11, 0))
#include <linux/dma-map-ops.h>
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 11, 0))
#include <linux/gfp.h>
#endif
#include <linux/highmem.h>
#include "osfunc.h"
#include "hash.h"
#include "pvr_debug.h"
#include "dma_drv_common.h"
#include "generated/volcanic/dma_bridge/common_dma_bridge.h"

#include <linux/dmaengine.h>
#include <linux/delay.h>

#include "dma_debug.h"
#include "xdx_device.h"
#include "HalTDMA.h"
#include "xdx_dma_drv.h"
#include "xdx_drv.h"
#include "syscommon.h"
#include "xdx_dma_fs.h"


#define VDMA_ADDR_OFFSET (0x200000)
xdx_dma_desc_blk_t* alloc_desc_block(struct xdx_dma_dev *sdev, dma_addr_t *p_desc_blk)
{
	struct page *pages;
	xdx_dma_desc_blk_t *v_desc_blk;

	pages = alloc_pages(GFP_ATOMIC | __GFP_HIGHMEM, 0);
	if (!pages) {
		pages = alloc_pages(GFP_KERNEL, 0);
		if (!pages) {
			XDX_DMA_ERR(sdev->dev, "Alloc pages Failed.");
			return NULL;
		}
	}
	*p_desc_blk = dma_map_page(sdev->dev->parent, pages, 0, PAGE_SIZE, DMA_TO_DEVICE);
	v_desc_blk = kmap(pages);
	if (!v_desc_blk) {
		XDX_DMA_ERR(sdev->dev, "Alloc desc block Failed.");
		return NULL;
	}
	v_desc_blk->desc_num = 0;
	v_desc_blk->desc_addr = *p_desc_blk;
	v_desc_blk->data_size = 0;
	v_desc_blk->desc_total =
		(PAGE_SIZE - offsetof(xdx_dma_desc_blk_t, desc_entry)) >> ((sdev->dcfg->desc_entry_size == VDMA_DESC_DATE_SIZE)	? 4 : 3);
	INIT_LIST_HEAD(&v_desc_blk->list);
	INIT_LIST_HEAD(&v_desc_blk->next_list);
	/* kmap_atomic maps pages as cached so it's more quick to access by host cpu. *
	 * here (also pages are always 8 bytes aligned anyway) */
//	XDX_DMA_ERR(sdev->dev, "[%s]v_desc_blk Alloc:%llx", __func__, v_desc_blk->desc_addr);
	return v_desc_blk;
}

void free_desc_block(struct xdx_dma_dev *sdev, xdx_dma_desc_blk_t *desc_blk)
{
	struct page* pages;
	if (!desc_blk)
		return;
	dma_unmap_page(sdev->dev->parent, desc_blk->desc_addr, PAGE_SIZE, DMA_TO_DEVICE);
	pages = kmap_to_page(desc_blk);
	if (!pages)
		return;
	kunmap(pages);
	__free_pages(pages, 0);
}

#ifdef DEBUG
static void xdx_dma_dump_desc_block(struct xdx_dma_dev *sdev, xdx_dma_desc_blk_t *desc_blk)
{
	int i;
	VDMA_B0_DescData_T *ddata;

	uint64_t NextDescPtr_47_3;
	uint64_t NextDescNum;
	uint64_t LinkEnd;
	XDX_DMA_DBG(sdev->dev, "DescDBG:Head desc_addr:%llx desc num:%d/%d",
			desc_blk->desc_addr, desc_blk->desc_num, desc_blk->desc_total);
	for (i = 0 ;i < desc_blk->desc_num; i++) {
		ddata = (VDMA_B0_DescData_T *)&desc_blk->desc_entry[i];
		if (i != desc_blk->desc_num -1) {
			NextDescPtr_47_3 = ddata->nonend.DataBlkPtr_47_3;
			NextDescNum = ddata->nonend.TransferLen;
			LinkEnd = ddata->nonend.LenUnit;
			XDX_DMA_DBG(sdev->dev, "DescEntry: %d-:%016llx , DataBlkPtr:%llx,TransferLen:%lld %lld", i,
					desc_blk->desc_entry[i].dword,
					NextDescPtr_47_3 << 3, NextDescNum, LinkEnd);
		} else {
			NextDescPtr_47_3 = ddata->end.NextDescPtr_47_3;
			NextDescNum = ddata->end.NextDescNum;
			LinkEnd = ddata->end.LinkEnd;
			XDX_DMA_DBG(sdev->dev, "DescEntry: %d-:%016llx NextDes:%llx NextDescNum:%llx LinkEnd:%lld", i,
					desc_blk->desc_entry[i].dword,
					NextDescPtr_47_3 << 3, NextDescNum, LinkEnd);
		}
	}
}
#else
static void xdx_dma_dump_desc_block(struct xdx_dma_dev *sdev, xdx_dma_desc_blk_t *desc_blk)
{

}
#endif
int xdx_dma_flush_desc_block(
	struct xdx_dma_dev *sdev, struct xdx_desc *txd, xdx_dma_desc_blk_t *desc_blk)
{
	xdx_dma_desc_blk_t *subdesc_blk;

	xdx_dma_dump_desc_block(sdev, desc_blk);
	if (0) {
		IMG_CPU_PHYADDR addr = {0};
		OSCPUCacheFlushRangeKM(NULL,
				(IMG_BYTE*)desc_blk,
				(IMG_BYTE*)desc_blk + PAGE_SIZE,
				addr, addr);
	} else {
		dma_sync_single_for_cpu(sdev->dev->parent, desc_blk->desc_addr, PAGE_SIZE, DMA_TO_DEVICE);
		dma_sync_single_for_device(sdev->dev->parent, desc_blk->desc_addr, PAGE_SIZE, DMA_TO_DEVICE);
//		flush_dcache_page(pfn_to_page(__phys_to_pfn(desc_blk->desc_addr)));
	}
	list_for_each_entry(subdesc_blk, &desc_blk->next_list, next_list) {
		xdx_dma_dump_desc_block(sdev, subdesc_blk);
		if (0) {
			IMG_CPU_PHYADDR addr = {0};
			OSCPUCacheFlushRangeKM(NULL,
					(IMG_BYTE*)subdesc_blk,
					(IMG_BYTE*)subdesc_blk + PAGE_SIZE,
					addr, addr);
		} else {
			dma_sync_single_for_cpu(sdev->dev->parent, subdesc_blk->desc_addr, PAGE_SIZE, DMA_TO_DEVICE);
			dma_sync_single_for_device(sdev->dev->parent, subdesc_blk->desc_addr, PAGE_SIZE, DMA_TO_DEVICE);
//			flush_dcache_page(pfn_to_page(__phys_to_pfn(subdesc_blk->desc_addr)));
		}
	}
	return 0;
}

struct xdx_desc *to_xdx_desc(struct dma_async_tx_descriptor *tx)
{
	return container_of(tx, struct xdx_desc, vd.tx);
}

struct xdx_desc *xdx_dma_alloc_txd(struct xdx_dma_dev *sdev)
{
	struct xdx_desc *txd;
	txd = kmem_cache_alloc(sdev->txpool, GFP_NOWAIT);
	if (!txd)
		return NULL;
	memset(txd, 0x00, sizeof(struct xdx_desc));
	txd->sdev = sdev;
	INIT_LIST_HEAD(&txd->desc_src_head);
	INIT_LIST_HEAD(&txd->desc_dest_head);
	INIT_LIST_HEAD(&txd->cmd_cfgs_head);
	return txd;
}

void xdx_dma_free_txd(struct xdx_dma_dev *sdev, struct xdx_desc *txd)
{
	kmem_cache_free(sdev->txpool, txd);
}

xdx_desc_cmd_cfg_t *xdx_dma_alloc_cmd_cfg(struct xdx_dma_dev *sdev)
{
	xdx_desc_cmd_cfg_t *cfg;
	cfg = kmem_cache_alloc(sdev->cmd_pool, GFP_NOWAIT);
	if (!cfg)
		return NULL;
	INIT_LIST_HEAD(&cfg->list);
	return cfg;
}

void xdx_dma_free_cmd_cfg(struct xdx_dma_dev *sdev, xdx_desc_cmd_cfg_t *cdm_cfg)
{
	kmem_cache_free(sdev->cmd_pool, cdm_cfg);
	return ;
}

void xdx_dma_free_desc(struct xdx_virt_dma_desc *vd)
{
	struct xdx_desc *txd = to_xdx_desc(&vd->tx);
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(vd->tx.chan->device);
	xdx_desc_cmd_cfg_t *cfg, *tcfg;
	xdx_dma_desc_blk_t *desc_blk,*tdesc_blk;
	xdx_dma_desc_blk_t *ndesc_blk,*tndesc_blk;

	list_for_each_entry_safe(cfg, tcfg, &txd->cmd_cfgs_head, list) {
		list_del(&cfg->list);
		xdx_dma_free_cmd_cfg(sdev, cfg);
	}
	list_for_each_entry_safe(desc_blk, tdesc_blk, &txd->desc_src_head, list) {
		list_del(&desc_blk->list);
		list_for_each_entry_safe(ndesc_blk, tndesc_blk, &desc_blk->next_list, next_list) {
			list_del(&ndesc_blk->next_list);
			free_desc_block(sdev, ndesc_blk);
		}
		free_desc_block(sdev, desc_blk);
	}
	list_for_each_entry_safe(desc_blk, tdesc_blk, &txd->desc_dest_head, list) {
		list_del(&desc_blk->list);
		list_for_each_entry_safe(ndesc_blk, tndesc_blk, &desc_blk->next_list, next_list) {
			list_del(&ndesc_blk->next_list);
			free_desc_block(sdev, ndesc_blk);
		}
		free_desc_block(sdev, desc_blk);
	}
	xdx_dma_free_txd(sdev, txd);
}

int xdx_dma_request_command_tag( struct xdx_vchan *vchan,
			struct xdx_dma_dev *sdev, struct xdx_pchan **ppchan)
{
	int cmd_tag = -EBUSY, i, ret;
	uint32_t max_free_chan = 0;
	struct xdx_pchan *pchan = *ppchan;

	if (pchan == NULL || pchan->free_cmd <=0 ) {
		for(i = 0; i < vchan->pchan_cnt; i++) {
			if(vchan->pchans[max_free_chan]->free_cmd < vchan->pchans[i]->free_cmd)
				max_free_chan = i;
		}
		if (pchan != NULL)
			XDX_DMA_DBG(sdev->dev, "[%s]switch pchan to:%d", __func__, max_free_chan);
		pchan = &(sdev->pchans[max_free_chan]);
	}
	if (pchan->free_cmd <= 0) {
		*ppchan = NULL;
		return -EBUSY;
	}
	ret = xdx_dma_cmd_buffer_remain(sdev, pchan);
	if (ret < 1 && ret != -ENOTSUPP) {
		*ppchan = NULL;
		XDX_DMA_DBG(sdev->dev,"cmd_remain_size lack");
		return -EBUSY;
	}
	spin_lock(&pchan->lock);
	if(pchan->DmaCmdPoolMask) {
		cmd_tag = GetLeadOnefromU64(pchan->DmaCmdPoolMask);
		BIT_UNSET(pchan->DmaCmdPoolMask, cmd_tag);
		pchan->free_cmd--;
		pchan->submit_cmd++;
		*ppchan = pchan;
	}
	spin_unlock(&pchan->lock);
	return cmd_tag;
}

void xdx_dma_release_command_tag(struct xdx_pchan *pchan, int cmd_tag)
{
	DBG_ASSERT(pchan->free_cmd < CMD_MAX);
	DBG_ASSERT(!BIT_ISSET(pchan->DmaCmdPoolMask, cmd_tag));
	DBG_ASSERT(pchan->submit_cmd > 0);
	BIT_SET(pchan->DmaCmdPoolMask, cmd_tag);
	pchan->cmdRef[cmd_tag].desc = NULL;
	pchan->free_cmd++;
	pchan->submit_cmd--;
#ifdef DEBUG_DMA
	XDX_DMA_DBG(pchan->dev,"ISR pchan %d free cmd %d, submit cmd %d, pool mask %llx...\n",
				pchan->idx, pchan->free_cmd, pchan->submit_cmd, pchan->DmaCmdPoolMask);
#endif
}

static int xdx_dma_vchan_free_cmds(struct xdx_vchan *vchan)
{
	uint32_t i, nTotalFree = 0;
	struct xdx_pchan *pchan;

	for(i = 0; i < vchan->pchan_cnt; i++) {
		pchan = vchan->pchans[i];
		nTotalFree += pchan->free_cmd;
	}
	return nTotalFree;
}

int xdx_dma_package_desc(struct dma_async_tx_descriptor *tx)
{
	int ret = 0;
	struct xdx_desc *txd =to_xdx_desc(tx);
	struct xdx_dma_dev *sdev = txd->sdev;

	switch(txd->xmit_type) {
		case XDX_XMIT_COPY:
			ret = xdx_dma_parse_sgls(sdev, txd);
			if (ret < 0) {
				XDX_DMA_ERR(sdev->dev, "[%s]%d:xdx_dma_parse_sgls Faild.(%d)", __func__, __LINE__, ret);
				return ret;
			}
			ret = xdx_dma_package_cfg(sdev, txd);
				if (ret < 0) {
					XDX_DMA_ERR(sdev->dev, "[%s]%d:xdx_dma_package_cfg Faild.(%d)", __func__, __LINE__, ret);
					return ret;
				}
		break;
		case XDX_XMIT_COPY_2D:
		case XDX_XMIT_TWD_2D3D:
		case XDX_XMIT_MEMSET:
			ret = xdx_dma_package_cfg(sdev, txd);
			if (ret < 0) {
				XDX_DMA_ERR(sdev->dev, "[%s]%d:xdx_dma_package_cfg Faild.(%d)", __func__, __LINE__, ret);
				return ret;
			}
		break;
		case XDX_XMIT_POLL:
		default:
			ret = -ENOTSUPP;
		break;
	}
	return ret;
}

static int xdx_dma_submit_cmd_cfgs(struct xdx_dma_dev *sdev, struct xdx_vchan *vchan, struct xdx_desc *txd)
{
	xdx_desc_cmd_cfg_t *cfg;
	int ret;

	list_for_each_entry(cfg, &txd->cmd_cfgs_head, list) {
		ret = xdx_dma_issue_chan(sdev, vchan, txd, cfg);
		if (ret < 0) {
			XDX_DMA_DBG(sdev->dev, "[%s]cookie:%d Issue Faield.%d", __func__, txd->vd.tx.cookie, ret);
			return ret;
		}
	}
	return 0;
}

static int xdx_dma_start_desc(struct xdx_dma_dev *sdev, struct xdx_vchan *vchan)
{
	struct xdx_virt_dma_desc *desc;
	struct xdx_desc *txd;
	int iRet = 0;

	do {
		desc = xdx_vchan_next_desc(&vchan->vc);
		if (!desc) {
			break;
		}

		txd = to_xdx_desc(&desc->tx);
		XDX_DMA_DBG(sdev->dev, "[%s]cookie:%d Submit.cmdcnt:%d", __func__,
				txd->vd.tx.cookie, atomic_read(&txd->cmd_cnt));
		if (atomic_read(&txd->cmd_cnt) > xdx_dma_vchan_free_cmds(vchan)) {
			XDX_DMA_DBG(sdev->dev, "free command %d less than %d\n",
				xdx_dma_vchan_free_cmds(vchan), atomic_read(&txd->cmd_cnt));
			iRet = -EBUSY;
			break;
		}
		list_del(&desc->node);
		iRet = xdx_dma_submit_cmd_cfgs(sdev, vchan, txd);
		if (iRet == -EBUSY) {
			XDX_DMA_DBG(sdev->dev, "free_cmd or cmd_remain_size lack\n");
			list_add_tail(&desc->node, &vchan->vc.desc_issued);
			break;
		}
		list_add_tail(&desc->node, &vchan->vc.desc_issued_pending);
	} while(1);
	return iRet;
}

static void xdx_dma_tasklet(unsigned long data)
{
	struct xdx_dma_dev *sdev = (struct xdx_dma_dev *)data;
	struct xdx_vchan *vchan;
	unsigned int vchan_idx, vcMask = sdev->pendingMask;
	unsigned long flags;

	while(vcMask) {
		vchan_idx = GetLeadOnefromU32(vcMask);
		BIT_UNSET(vcMask, vchan_idx);
		vchan = &sdev->vchans[vchan_idx];
		spin_lock_irqsave(&vchan->vc.lock, flags);
		if (xdx_dma_start_desc(sdev, vchan) == -EBUSY) {
			XDX_DMA_DBG(sdev->dev, "xdx_dma_tasklet ... vc %d is full!!!\n", vchan->tag);
		}
		spin_unlock_irqrestore(&vchan->vc.lock, flags);
	}
}

static inline s8 convert_buswidth(enum dma_slave_buswidth addr_width)
{
	return ilog2(addr_width);
}

static struct dma_async_tx_descriptor *xdx_dma_prep_dma_memcpy(
		struct dma_chan *chan, dma_addr_t dest, dma_addr_t src,
		size_t len, unsigned long flags)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	struct xdx_desc *txd;

	txd = xdx_dma_alloc_txd(sdev);
	if (!txd)
		return NULL;

	if (XDX_DMA_ADDR_HOST & XDX_DMA_ADDR_GET_FLAG(src)) {
		if (XDX_DMA_ADDR_HOST & XDX_DMA_ADDR_GET_FLAG(dest))
			txd->direct =  DMA_MEM_TO_MEM;
		else
			txd->direct =  DMA_MEM_TO_DEV;
	} else if (XDX_DMA_ADDR_HOST & XDX_DMA_ADDR_GET_FLAG(dest)) {
		txd->direct =  DMA_DEV_TO_MEM;
	} else {
		txd->direct =  DMA_DEV_TO_DEV;
	}

	txd->paras.src.addr = XDX_DMA_ADDR_CLEAR_FLAG(src);
	txd->paras.src.addr_type = XDX_DMA_ADDR_DMA;
	txd->paras.dest.addr = XDX_DMA_ADDR_CLEAR_FLAG(dest);
	txd->paras.dest.addr_type = XDX_DMA_ADDR_DMA;
	txd->paras.len = len;
	txd->xmit_type = XDX_XMIT_COPY;
	return xdx_vchan_tx_prep(&vchan->vc, &txd->vd, flags);
}

static struct dma_async_tx_descriptor *xdx_dma_prep_slave_sg(
		struct dma_chan *chan, struct scatterlist *sgl,
		unsigned int sg_len, enum dma_transfer_direction dir,
		unsigned long flags, void *context)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	struct xdx_desc *txd = (struct xdx_desc *)context;

	if (!sgl)
		return NULL;

	if (!txd)
		txd = xdx_dma_alloc_txd(sdev);
	if (!txd)
		return NULL;
	if (dir == DMA_MEM_TO_DEV || dir == DMA_MEM_TO_MEM) {
		txd->paras.src.sg_tab.sgl = sgl;
		txd->paras.src.sg_tab.sg_len = sg_len;
		txd->paras.src.addr_type = XDX_DMA_ADDR_SGL;
		txd->paras.dest.addr_type = XDX_DMA_ADDR_NULL;
	} else {
		txd->paras.dest.sg_tab.sgl = sgl;
		txd->paras.dest.sg_tab.sg_len = sg_len;
		txd->paras.dest.addr_type = XDX_DMA_ADDR_SGL;
		txd->paras.src.addr_type = XDX_DMA_ADDR_NULL;
	}
	{
		struct scatterlist *sg;
		int i;
		for_each_sg(sgl, sg, sg_len, i)
		{
			txd->paras.len += sg_dma_len(sg); //accumulate transfer_len
		}
	}
	txd->xmit_type = XDX_XMIT_COPY;
	txd->direct = dir;
	XDX_DMA_DBG(chan->device->dev, "[%s] FirstAddr:%llx +%x dmal:%x sg_len:%d src.type:%d dest.addr:%d txd->direct:%d",
		__func__, sg_dma_address(sgl), sgl->offset, sg_dma_len(sgl),sg_len, txd->paras.src.addr_type,
		txd->paras.dest.addr_type, txd->direct);
	return xdx_vchan_tx_prep(&vchan->vc, &txd->vd, flags);
}

static struct dma_async_tx_descriptor *xdx_dma_prep_dma_memset(
		struct dma_chan *chan, dma_addr_t dest, int value,
		size_t len, unsigned long flags)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	struct xdx_desc *txd;

	if (unlikely(!len)) {
		XDX_DMA_ERR(chan->device->dev, "%s: length is zero!\n", __func__);
		return NULL;
	}
	if (!is_dma_fill_aligned(chan->device, dest, 0, len)) {
		XDX_DMA_ERR(chan->device->dev, "%s: buffer is not aligned\n", __func__);
		return NULL;
	}

	txd = xdx_dma_alloc_txd(sdev);
	if (!txd)
		return NULL;

	if (XDX_DMA_ADDR_HOST & XDX_DMA_ADDR_GET_FLAG(dest))
		txd->direct =  DMA_DEV_TO_MEM;
	else
		txd->direct =  DMA_MEM_TO_DEV;

	txd->paras.dest.addr = XDX_DMA_ADDR_CLEAR_FLAG(dest);
	txd->paras.dest.addr_type = XDX_DMA_ADDR_DMA;
	txd->paras.para_memset.len = len;
	txd->paras.para_memset.Pattern_len = XDX_DMA_MEMSET_SUPPORT_VALUE_LEN;
	txd->paras.para_memset.Pattern[0] = value;
	txd->xmit_type = XDX_XMIT_MEMSET;
	XDX_DMA_DBG(chan->device->dev,"Pattern[0] = %x  len = %ld\n",txd->paras.para_memset.Pattern[0],len);
	return xdx_vchan_tx_prep(&vchan->vc, &txd->vd, flags);
}

struct dma_async_tx_descriptor *xdx_dma_prep_memset_sg(
		struct dma_chan *chan, struct scatterlist *sgl,
		unsigned int nents, int value, enum dma_transfer_direction dir,	unsigned long flags)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	struct xdx_desc *txd;

	if (!sgl)
		return NULL;
	txd = xdx_dma_alloc_txd(sdev);
	if (!txd)
		return NULL;
	txd->direct =  dir;
	txd->paras.dest.sg_tab.sgl = sgl;
	txd->paras.dest.sg_tab.sg_len = nents;
	txd->paras.dest.addr_type = XDX_DMA_ADDR_SGL;
	txd->paras.para_memset.Pattern_len = XDX_DMA_MEMSET_SUPPORT_VALUE_LEN;
	txd->paras.para_memset.Pattern[0] = value;
	txd->xmit_type = XDX_XMIT_MEMSET;
	{
		struct scatterlist *sg;
		int i;
		for_each_sg(sgl, sg, nents, i) {
			txd->paras.para_memset.len += sg_dma_len(sg); //accumulate transfer_len
		}
	}
	XDX_DMA_DBG(chan->device->dev, "Pattern[0] = %x  len = %lld\n", txd->paras.para_memset.Pattern[0], txd->paras.para_memset.len);
	return xdx_vchan_tx_prep(&vchan->vc, &txd->vd, flags);
}

struct dma_async_tx_descriptor *xdx_prep_2d_memcpy(
		struct dma_chan *chan,
		void *para_info,
		enum dma_transfer_direction dir,
		unsigned long flags)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	struct xdx_desc *txd;
	DMA_XMIT_2D_PARA *twiddle_2d = (DMA_XMIT_2D_PARA *)para_info;

	if (!xdx_dma_check_para(sdev, XDX_XMIT_COPY_2D, twiddle_2d))
		return ERR_PTR(-EINVAL);

	txd = xdx_dma_alloc_txd(sdev);
	if (!txd)
		return NULL;

	txd->xmit_type = XDX_XMIT_COPY_2D;
	txd->direct = dir;
	txd->paras.src.addr = (dma_addr_t)twiddle_2d->SrcAddr;
	txd->paras.src.addr_type = XDX_DMA_ADDR_DMA;
	txd->paras.src.addr_flag |=
			twiddle_2d->SrcAddrFlag & DMA_ADDR_FLAG_VA ?
			XDX_DMA_ADDR_SUPSMMU : 0;
	txd->paras.src.addr_flag |=
			twiddle_2d->SrcAddrFlag & DMA_ADDR_FLAG_HOST_PORT ?
			XDX_DMA_ADDR_HOST : 0;

	txd->paras.dest.addr = (dma_addr_t)twiddle_2d->DstAddr;
	txd->paras.dest.addr_type = XDX_DMA_ADDR_DMA;
	txd->paras.dest.addr_flag |=
			twiddle_2d->DstAddrFlag & DMA_ADDR_FLAG_VA ?
			XDX_DMA_ADDR_SUPSMMU : 0;
	txd->paras.dest.addr_flag |=
			twiddle_2d->DstAddrFlag & DMA_ADDR_FLAG_HOST_PORT ?
			XDX_DMA_ADDR_HOST : 0;

	txd->paras.para_2d.width = twiddle_2d->Width;
	txd->paras.para_2d.height = twiddle_2d->Height;
	txd->paras.para_2d.x_stride = twiddle_2d->XStride;
	txd->paras.para_2d.len_unit = twiddle_2d->LenUnit;

	XDX_DMA_DBG(sdev->dev,"width = %lld height = %lld\n",twiddle_2d->Width, twiddle_2d->Height);
	return xdx_vchan_tx_prep(&vchan->vc, &txd->vd, flags);
}

struct dma_async_tx_descriptor *xdx_prep_twd_2d3d_memcpy(
		struct dma_chan *chan, void *twd_info,
		enum dma_transfer_direction dir, unsigned long flags)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	struct xdx_desc *txd;
	DMA_TWD_2D3D_PARA *twiddle_2d3d = (DMA_TWD_2D3D_PARA *)twd_info;

	if (!xdx_dma_check_para(sdev, XDX_XMIT_TWD_2D3D, twiddle_2d3d))
		return ERR_PTR(-EINVAL);

	txd = xdx_dma_alloc_txd(sdev);
	if (!txd)
		return NULL;
	txd->paras.src.addr = (dma_addr_t)twiddle_2d3d->SrcAddr;
	txd->paras.src.addr_type = XDX_DMA_ADDR_DMA;
	txd->paras.src.addr_flag |=
			twiddle_2d3d->SrcAddrFlag & DMA_ADDR_FLAG_VA ?
			XDX_DMA_ADDR_SUPSMMU : 0;
	txd->paras.src.addr_flag |=
			twiddle_2d3d->SrcAddrFlag & DMA_ADDR_FLAG_HOST_PORT ?
			XDX_DMA_ADDR_HOST : 0;

	txd->paras.dest.addr = (dma_addr_t)twiddle_2d3d->DstAddr;
	txd->paras.dest.addr_type = XDX_DMA_ADDR_DMA;
	txd->paras.dest.addr_flag |=
			twiddle_2d3d->DstAddrFlag & DMA_ADDR_FLAG_VA ?
			XDX_DMA_ADDR_SUPSMMU : 0;
	txd->paras.dest.addr_flag |=
			twiddle_2d3d->DstAddrFlag & DMA_ADDR_FLAG_HOST_PORT ?
			XDX_DMA_ADDR_HOST : 0;

	txd->paras.twiddle_2d3d.width = twiddle_2d3d->Width;
	txd->paras.twiddle_2d3d.height = twiddle_2d3d->Height;
	txd->paras.twiddle_2d3d.depth = twiddle_2d3d->Depth;
	txd->paras.twiddle_2d3d.x_stride = twiddle_2d3d->XStride;
	txd->paras.twiddle_2d3d.y_stride = twiddle_2d3d->YStride;
	txd->paras.twiddle_2d3d.len_unit = twiddle_2d3d->LenUnit;
	txd->direct = dir;
	txd->xmit_type = XDX_XMIT_TWD_2D3D;

	XDX_DMA_DBG(sdev->dev,
		"SrcAddr = %llx dst_addr = %llx width = %lld height = %lld\n",
		twiddle_2d3d->SrcAddr, twiddle_2d3d->DstAddr,
		twiddle_2d3d->Width, twiddle_2d3d->Height);
	return xdx_vchan_tx_prep(&vchan->vc, &txd->vd, flags);
}
#define DIR_PRINT(dir) ((dir > 2) ? "DMA_DEV_TO_DEV" : ((dir > 1) ? "DMA_DEV_TO_MEM" : "DMA_MEM_TO_DEV"))
struct dma_async_tx_descriptor *xdx_dma_prep_slave_sg2sg(
		struct dma_chan *chan, struct scatterlist *sg_src,
		unsigned int sg_src_len, struct scatterlist *sg_dest,
		unsigned int sg_dest_len, enum dma_transfer_direction dir,
		unsigned long flags)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	struct xdx_desc *txd;
	uint64_t dest_len = 0;

	if (!sg_src || !sg_dest)
		return NULL;

	txd = xdx_dma_alloc_txd(sdev);
	if (!txd)
		return NULL;

	if(dir == DMA_MEM_TO_DEV || dir == DMA_DEV_TO_DEV) {
		txd->paras.src.sg_tab.sgl = sg_src;
		txd->paras.src.sg_tab.sg_len = sg_src_len;
		txd->paras.dest.sg_tab.sgl = sg_dest;
		txd->paras.dest.sg_tab.sg_len = sg_dest_len;
	} else {
		txd->paras.src.sg_tab.sgl = sg_dest;
		txd->paras.src.sg_tab.sg_len = sg_dest_len;
		txd->paras.dest.sg_tab.sgl = sg_src;
		txd->paras.dest.sg_tab.sg_len = sg_src_len;
	}
	txd->paras.src.addr_type = XDX_DMA_ADDR_SGL;
	txd->paras.dest.addr_type = XDX_DMA_ADDR_SGL;

	{
		struct scatterlist *sg;
		int i;
		for_each_sg(txd->paras.src.sg_tab.sgl, sg, txd->paras.src.sg_tab.sg_len, i)
		{
			txd->paras.len += sg_dma_len(sg); //accumulate transfer_len
		}
		for_each_sg(txd->paras.dest.sg_tab.sgl, sg, txd->paras.dest.sg_tab.sg_len, i)
		{
			dest_len += sg_dma_len(sg); //accumulate transfer_len
		}
	}

	txd->xmit_type = XDX_XMIT_COPY;
	txd->direct = dir;
	XDX_DMA_DBG(chan->device->dev, "[%s] Direct: %s  SrcAddr:%llx offset:%x len:%llx -- DestAddr:%llx offset:%x len:%llx\n",
		__func__, DIR_PRINT(dir), sg_dma_address(txd->paras.src.sg_tab.sgl), txd->paras.src.sg_tab.sgl->offset,
		txd->paras.len, sg_dma_address(txd->paras.dest.sg_tab.sgl), txd->paras.dest.sg_tab.sgl->offset, dest_len);
	return xdx_vchan_tx_prep(&vchan->vc, &txd->vd, flags);
}

static int xdx_dma_config(struct dma_chan *chan,
			    struct dma_slave_config *config)
{
	//struct xdx_vchan *vchan = to_xdx_vchan(chan);
	//pr_info("xdx_dma_config ... \n");
	//memcpy(&vchan->cfg, config, sizeof(*config));
	return 0;
}

static int xdx_dma_pause(struct dma_chan *chan)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);

	XDX_DMA_DBG(xdxdma_chan2dev(chan), "vchan %p: pause\n", &vchan->vc);
	{
		spin_lock_bh(&sdev->lock);
		BIT_UNSET(sdev->pendingMask, vchan->tag);
		spin_unlock_bh(&sdev->lock);
	}

	return 0;
}

static int xdx_dma_resume(struct dma_chan *chan)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	//struct xdx_pchan *pchan = vchan->phy;
	unsigned long flags;

	XDX_DMA_DBG(xdxdma_chan2dev(chan), "vchan %p: resume\n", &vchan->vc);

	spin_lock_irqsave(&vchan->vc.lock, flags);
	if (!list_empty(&vchan->vc.desc_issued)) {
		spin_lock(&sdev->lock);
		BIT_SET(sdev->pendingMask, vchan->tag);
		//list_add_tail(&vchan->node, &sdev->pending);
		spin_unlock(&sdev->lock);
	}
	spin_unlock_irqrestore(&vchan->vc.lock, flags);

	return 0;
}

static int xdx_dma_terminate_all(struct dma_chan *chan)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	//struct xdx_pchan *pchan = vchan->phy;
	unsigned long flags;
	LIST_HEAD(head);

	XDX_DMA_DBG(xdxdma_chan2dev(chan), "terminate all\n");
	spin_lock_bh(&sdev->lock);
	//list_del_init(&vchan->node);
	BIT_UNSET(sdev->pendingMask, vchan->tag);
	spin_unlock_bh(&sdev->lock);

	spin_lock_irqsave(&vchan->vc.lock, flags);
	if (vchan->cyclic) {
		vchan->cyclic = false;
	}
	xdx_vchan_get_all_descriptors(&vchan->vc, &head);
	spin_unlock_irqrestore(&vchan->vc.lock, flags);
	xdx_vchan_dma_desc_free_list(&vchan->vc, &head);
	return 0;
}


static enum dma_status xdx_dma_tx_status(struct dma_chan *chan,
		dma_cookie_t cookie,
		struct dma_tx_state *state)
{
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	struct xdx_virt_dma_desc *vd;
	struct xdx_desc *txd;
	enum dma_status ret;
	unsigned long flags;
	size_t bytes = 0;

	ret = dma_cookie_status(chan, cookie, state);
	if (ret == DMA_COMPLETE || !state)
		return ret;

	spin_lock_irqsave(&vchan->vc.lock, flags);
	vd = xdx_vchan_find_desc(&vchan->vc, cookie);
	txd = to_xdx_desc(&vd->tx);
	if (vd) {
	//	bytes = txd->transfer_len;
		bytes	=	1;
	}
	spin_unlock_irqrestore(&vchan->vc.lock, flags);
	dma_set_residue(state, bytes);
	return ret;
}

static void xdx_dma_issue_pending(struct dma_chan *chan)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	unsigned long flags;

	spin_lock_irqsave(&vchan->vc.lock, flags);
	if (xdx_vchan_issue_pending(&vchan->vc))
	{
		spin_lock(&sdev->lock);
		if(!BIT_ISSET(sdev->pendingMask, vchan->tag))
		{
			BIT_SET(sdev->pendingMask, vchan->tag);
		}
		spin_unlock(&sdev->lock);
		tasklet_schedule(&sdev->task);
	} else {
		XDX_DMA_DBG(xdxdma_chan2dev(chan), "vchan %p: nothing to issue\n",
			&vchan->vc);
	}

	spin_unlock_irqrestore(&vchan->vc.lock, flags);
}

static void xdx_dma_free_chan_resources(struct dma_chan *chan)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	struct xdx_vchan *vchan = to_xdx_vchan(chan);
	//unsigned long flags;

	spin_lock_bh(&sdev->lock);
	//list_del_init(&vchan->node);
	BIT_UNSET(sdev->pendingMask, vchan->tag);
	spin_unlock_bh(&sdev->lock);

	xdx_vchan_free_chan_resources(&vchan->vc);
}

static inline void xdx_kill_tasklet(struct xdx_dma_dev *sdev)
{
	/* Prevent spurious interrupts from scheduling the tasklet */
	atomic_inc(&sdev->tasklet_shutdown);

	/* Make sure we won't have any further interrupts */
	/* Actually prevent the tasklet from being scheduled */
	tasklet_kill(&sdev->task);
}

static inline void xdx_dma_free(struct xdx_dma_dev *sdev)
{
	int i;

	for (i = 0; i < sdev->num_vchans; i++) {
		struct xdx_vchan *vchan = &sdev->vchans[i];
		list_del(&vchan->vc.chan.device_node);
		tasklet_kill(&vchan->vc.task);
	}
}
#if (LINUX_VERSION_CODE > KERNEL_VERSION(3, 10, 0))
static bool xdx_dma_filter_fn(struct dma_chan *chan,void *param)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(chan->device);
	XDX_DMA_DBG(sdev->dev,"xdx_dma_filter_fn (%s) ...\n", (char *)param);
	return true;
}
#endif
static int xdx_dma_chans_bind(
	struct xdx_dma_dev *sdc,
	struct xdx_vchan *vchan,
	struct xdx_dma_vchan_cfg *vcfg
)
{
	int i, j;

	vchan->pchans = devm_kcalloc(sdc->slave.dev,
		vcfg->num_pchan, sizeof(struct xdx_pchan*), GFP_KERNEL);
	if (IS_ERR_OR_NULL( vchan->pchans))
		return -ENOMEM;
	memset(vchan->pchans, U32_MAX, vcfg->num_pchan * sizeof(struct xdx_pchan*));
	for (i = 0; i < vcfg->num_pchan; i++) {
		for (j = 0; j < sdc->num_pchans; j++) {
			if (sdc->pchans[j].idx == vcfg->pchans[i]) {
				vchan->pchans[i] = &sdc->pchans[j];
				XDX_DMA_DBG(sdc->dev, "[%s]Vchan %d Bind Pchan:%x", __func__, i, sdc->pchans[j].idx);
				break;
			}
		}
	}
	vchan->pchan_cnt = vcfg->num_pchan;
	return 0;
}

static int xdx_dma_pchan_init(
	struct xdx_dma_dev *sdc,
	struct xdx_pchan *pchan,
	struct xdx_dma_pchan_cfg *pcfg)
{
	pchan->dev = sdc->dev;
	pchan->idx = pcfg->chan_id;
	if ((sdc->dcfg->flags & XDXDMA_B0_TDMA_MASK) == XDXDMA_B0_TDMA_MASK) {
		pchan->base = sdc->base;
	} else {
		pchan->base = sdc->base + VDMA_CHANNEL_REG_OFFSET +
			DMA_DMA_NUM(pchan->idx) * VDMA_ADDR_OFFSET
			+ DMA_CHAN_NUM(pchan->idx) * VDMA_CHANNEL_REG_SEC_SIZE;
	}
	XDX_DMA_DBG(sdc->dev, "pchan[%x]base[%pK] Add!.", pchan->idx, pchan->base);
	pchan->DmaCmdPoolMask = ~0x0UL;
	pchan->submit_cmd = 0;
	spin_lock_init(&pchan->lock);
	pchan->free_cmd = CMD_MAX;

	pchan->inited = 1;
	return 0;
}

static int xdx_dma_ops_init(struct xdx_dma_dev *sdc)
{
	if ((sdc->dcfg->flags & XDXDMA_A0_VDMA_MASK) == XDXDMA_A0_VDMA_MASK) {
		sdc->ops = &vdma_a0_ops;
	} else if ((sdc->dcfg->flags & XDXDMA_B0_VDMA_MASK) == XDXDMA_B0_VDMA_MASK) {
		sdc->ops = &vdma_b0_ops;
	} else if ((sdc->dcfg->flags & XDXDMA_B0_TDMA_MASK) == XDXDMA_B0_TDMA_MASK) {
		sdc->ops = &tdma_b0_ops;
	} else {
		XDX_DMA_ERR(sdc->dev, "not support device type:%x", sdc->dcfg->flags & 0xf);
		sdc->ops = NULL;
		return -EINVAL;
	}
	if (sdc->ops->hw_init)
		sdc->ops->hw_init(sdc);
	return 0;
}

static int xdx_dma_slave_init(struct platform_device *pdev, struct xdx_dma_dev *sdc)
{
	INIT_LIST_HEAD(&sdc->slave.channels);
	sdc->slave.device_free_chan_resources = xdx_dma_free_chan_resources;
	sdc->slave.device_tx_status = xdx_dma_tx_status;
	sdc->slave.device_issue_pending = xdx_dma_issue_pending;
	sdc->slave.device_prep_slave_sg	= xdx_dma_prep_slave_sg;
	sdc->slave.device_prep_dma_memcpy = xdx_dma_prep_dma_memcpy;
	sdc->slave.device_prep_dma_memset = xdx_dma_prep_dma_memset;
//	sdc->slave.device_prep_dma_memset_sg = xdx_dma_prep_dma_memset_sg;
	//sdc->slave.device_prep_dma_cyclic = xdx_dma_prep_dma_cyclic;
#if (LINUX_VERSION_CODE > KERNEL_VERSION(3, 10, 0))
	sdc->slave.copy_align = DMAENGINE_ALIGN_4_BYTES;
#else
	sdc->slave.copy_align = 4;
#endif
	sdc->slave.device_config = xdx_dma_config;
	sdc->slave.device_pause = xdx_dma_pause;
	sdc->slave.device_resume = xdx_dma_resume;
	sdc->slave.device_terminate_all = xdx_dma_terminate_all;
	sdc->slave.directions = BIT(DMA_DEV_TO_MEM) | BIT(DMA_MEM_TO_DEV);
	sdc->slave.residue_granularity = DMA_RESIDUE_GRANULARITY_BURST;
	sdc->slave.dev = &pdev->dev;
	XDX_DMA_DBG(&pdev->dev, "sdc->slave.dev = 0x%llx\n",(IMG_UINT64)sdc->slave.dev);
	tasklet_init(&sdc->task, xdx_dma_tasklet, (unsigned long)sdc);
	return 0;
}

static int xdx_dma_vchan_init(struct xdx_dma_dev *sdc, struct xdx_vchan *vchan)
{
	vchan->vc.desc_free = xdx_dma_free_desc;
	return  0;
}

static int xdx_dma_probe(struct platform_device *pdev)
{
	struct xdx_dma_dev *sdc;
	int ret;
	struct resource *res;
	u32 i;
	const struct platform_device_id *pdev_id;
	void __iomem *reg_base = NULL;
	void __iomem *hostmisc_base = NULL;
	const struct xdx_dma_drv_cfg *dcfg;
#if LINUX_VERSION_CODE > KERNEL_VERSION(3, 10, 0)
	struct dma_slave_map *dma_maps;
#endif
	sdc = devm_kzalloc(&pdev->dev, sizeof(*sdc), GFP_KERNEL);
	if (!sdc)
		return -ENOMEM;

	XDX_DMA_DBG(&pdev->dev, "dma init start ...\n");
	res = platform_get_resource_byname(pdev, IORESOURCE_MEM, "base");
	if (!res) {
		XDX_DMA_ERR(&pdev->dev, "Get REG base resource faild.\n");
		return -EINVAL;
	}
	reg_base = devm_ioremap_resource(&pdev->dev, res);
	if (IS_ERR_OR_NULL(reg_base)) {
		XDX_DMA_ERR(&pdev->dev, "base registers resource map error.%ld\n", PTR_ERR(reg_base));
		return PTR_ERR(reg_base);
	}
	res = platform_get_resource_byname(pdev, IORESOURCE_MEM, "hostmisc-base");
	if (res) {
		XDX_DMA_DBG(&pdev->dev, "set hostmisc reg.");
		hostmisc_base = devm_ioremap_resource(&pdev->dev, res);
		if (IS_ERR_OR_NULL(hostmisc_base)) {
			XDX_DMA_ERR(&pdev->dev, "hostmisc map error.%ld\n", PTR_ERR(hostmisc_base));
			return PTR_ERR(hostmisc_base);
		}
		sdc->hostmisc_base = hostmisc_base;
	} else {
		sdc->hostmisc_base = NULL;
	}
	pdev_id = platform_get_device_id(pdev);
	if (!pdev_id || !pdev_id->driver_data)
		return -EINVAL;
	dcfg =  (struct xdx_dma_drv_cfg *)pdev_id->driver_data;
	sdc->dcfg = dcfg;
//	sdc->cmd_cfg = &xdx_dma_cmd_cfg[0];
	sdc->base = reg_base;
	sdc->dev = &pdev->dev;
	sdc->mcu_id = VDMA_HAL_COM_RdVdMcuId((uint64_t)reg_base);
	sdc->pdata = dev_get_platdata(&pdev->dev);
//	dev_info(&pdev->dev, "dma maped addr:%pK mcu_id:%x\n", reg_base, sdc->mcu_id);
	res = platform_get_resource_byname(pdev, IORESOURCE_REG, "irq-cid");
	if (!res) {
		XDX_DMA_ERR(&pdev->dev, "Get irq-cid faild.\n");
		return -EINVAL;
	}
	sdc->irq_cid = res->start;
	xdx_dma_ops_init(sdc);
	xdx_dma_irq_init(sdc);

	sdc->pool = dmam_pool_create(dev_name(&pdev->dev), &pdev->dev,
					PAGE_SIZE, 0, 0);
	if (!sdc->pool) {
		XDX_DMA_ERR(&pdev->dev, "No memory for descriptors dma pool\n");
		return -ENOMEM;
	}
	sdc->txpool = kmem_cache_create("pvr-tx", sizeof(struct xdx_desc), 0, 0, NULL);
	if (!sdc->txpool) {
		XDX_DMA_ERR(&pdev->dev, "mempool_create_slab_pool fail!\n");
		return -ENOMEM;
	}
	sdc->cmd_pool = kmem_cache_create("cmd-cfgs", sizeof(xdx_desc_cmd_cfg_t), 0, 0, NULL);
	if (!sdc->cmd_pool) {
		XDX_DMA_ERR(&pdev->dev, "mempool_create_slab_pool fail!\n");
		return -ENOMEM;
	}

	//INIT_LIST_HEAD(&sdc->pending);
	spin_lock_init(&sdc->lock);
	platform_set_drvdata(pdev, sdc);
	dma_set_mask_and_coherent(&pdev->dev, DMA_BIT_MASK(48));
	dma_cap_set(DMA_PRIVATE, sdc->slave.cap_mask);
	dma_cap_set(DMA_SLAVE, sdc->slave.cap_mask);
	sdc->slave.src_addr_widths = dcfg->src_addr_widths;
	sdc->slave.dst_addr_widths = dcfg->dst_addr_widths;
	xdx_dma_slave_init(pdev, sdc);

#if LINUX_VERSION_CODE > KERNEL_VERSION(3, 10, 0)
	dma_maps = devm_kcalloc(&pdev->dev, 1, sizeof(struct dma_slave_map), GFP_KERNEL);
	if (!dma_maps)
		return -ENOMEM;
	memcpy(dma_maps, sdc->dcfg->fn_map, sizeof(struct dma_slave_map));
	dma_maps->devname = (const char *)dev_name(&pdev->dev);
	sdc->slave.filter.map = dma_maps;
	sdc->slave.filter.fn = xdx_dma_filter_fn;
	sdc->slave.filter.mapcnt = 1;
	dev_info(&pdev->dev, "xdx_dma_map.devname = %s\n", sdc->slave.filter.map->devname);
#endif

	sdc->num_pchans = dcfg->num_pchan;
	sdc->pchans = devm_kcalloc(&pdev->dev, sdc->num_pchans,
				sizeof(struct xdx_pchan), GFP_KERNEL);
	if (!sdc->pchans)
		return -ENOMEM;

	sdc->num_vchans = dcfg->num_vchan;
	sdc->vchans = devm_kcalloc(&pdev->dev, sdc->num_vchans,
				sizeof(struct xdx_vchan), GFP_KERNEL);
	if (!sdc->vchans)
		return -ENOMEM;
	for (i = 0; i < sdc->num_pchans; i++) {
		ret = xdx_dma_pchan_init(sdc, &sdc->pchans[i], &dcfg->pchans[i]);
		if (ret < 0)
			return ret;
	}

	for (i = 0; i < sdc->num_vchans; i++) {
		struct xdx_vchan *vchan = &sdc->vchans[i];
		vchan->tag = i; //vc index
		vchan->dev = &pdev->dev;
		xdx_dma_vchan_init(sdc, vchan);
		xdx_vchan_init(&vchan->vc, &sdc->slave);
		ret = xdx_dma_chans_bind(sdc, vchan, &dcfg->vchans[i]);
		if (ret < 0)
			goto error_out;
	}
	ret = dma_async_device_register(&sdc->slave);
	if (ret) {
		XDX_DMA_WARN(&pdev->dev, "Failed to register DMA engine device\n");
		goto err_irq_disable;
	}
	xdx_vdma_dbglevel_init();
	xdx_dma_dev_fs_init(sdc);
	XDX_DMA_DBG(&pdev->dev, "dma init done ...\n");
	return 0;
err_irq_disable:
	dma_async_device_unregister(&sdc->slave);
error_out:
	xdx_kill_tasklet(sdc);
	xdx_dma_free(sdc);
	return ret;
}

static int xdx_dma_remove(struct platform_device *pdev)
{
	struct xdx_dma_dev *sdc = platform_get_drvdata(pdev);

	xdx_dma_dev_fs_exit(sdc);
	dma_async_device_unregister(&sdc->slave);
	XDX_DMA_DBG(NULL, "exit xdx_dma_remove ###\n");
	xdx_dma_irq_deinit(sdc);

	xdx_kill_tasklet(sdc);
	dmam_pool_destroy(sdc->pool);
	kmem_cache_destroy(sdc->txpool);
	kmem_cache_destroy(sdc->cmd_pool);
	devm_iounmap(&pdev->dev, sdc->base);
	if (sdc->hostmisc_base)
		devm_iounmap(&pdev->dev, sdc->hostmisc_base);
	xdx_dma_free(sdc);
	XDX_DMA_DBG(sdc->dev,"exit xdx_dma_remove done !!!\n");
	return 0;
}

static struct platform_device_id xdx_platform_device_id_table[] = {
		{XDX_VDMA_A0_NAME,(kernel_ulong_t)&xdxvdma_a0},
		{XDX_VDMA_B0_NAME,(kernel_ulong_t)&xdxvdma_b0},
		{XDX_TDMA_B0_NAME,(kernel_ulong_t)&xdxtdma_b0},
		{}
};


static struct platform_driver xdx_dma_driver = {
	.probe = xdx_dma_probe,
	.remove = xdx_dma_remove,
	.driver = {
		.name = "xdx-dma",
		.owner = THIS_MODULE,
	},
	.id_table = xdx_platform_device_id_table,
};

int xdx_dma_drv_init(void)
{
	int err;
	err = platform_driver_register(&xdx_dma_driver);
	if(err)
		return err;

	return 0;
}
//EXPORT_SYMBOL(xdx_dma_drv_init);

void xdx_dma_drv_exit(void)
{
	platform_driver_unregister(&xdx_dma_driver);
}
