/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
//#define DEBUG
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/moduleparam.h>
#include <linux/version.h>
#include <linux/dma-mapping.h>

#include "sysinfo.h"
#include "xdx_dma.h"
#include "pangu.h"
#include "virt-dma.h"

#include "pvrmodule.h"
#include <linux/dma-mapping.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/mm.h>
#include <linux/device.h>
#include <linux/dmaengine.h>
#include <linux/hardirq.h>
#include <linux/spinlock.h>
#include <linux/percpu.h>
#include <linux/rcupdate.h>
#include <linux/mutex.h>
#include <linux/jiffies.h>
#include <linux/rculist.h>
#include <linux/idr.h>
#include <linux/slab.h>
#include <linux/acpi.h>
#include <linux/acpi_dma.h>
#include <linux/of_dma.h>
#include <linux/mempool.h>
#include <linux/numa.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 11, 0))
#include <linux/dma-map-ops.h>
#endif
#include "osfunc.h"
#include "hash.h"
#include "pvr_debug.h"
#include "dma_drv_common.h"
#include "generated/volcanic/dma_bridge/common_dma_bridge.h"

#include <linux/dmaengine.h>
#include "dma_debug.h"
#include "xdx_device.h"
#include "xdx_drv.h"
#include "syscommon.h"
#include "xdx_dma_drv.h"
#include "vdma_drv.h"

struct xdx_dma_pchan_cfg vdma_a0_pchans[] = {
//	{.chan_id = DMA_CHAN_ID(0, 0), .prio = 0},
//	{.chan_id = DMA_CHAN_ID(0, 1), .prio = 0},
	{.chan_id = DMA_CHAN_ID(0, 2), .prio = 0},
//	{.chan_id = DMA_CHAN_ID(0, 3), .prio = 0},
//	{.chan_id = DMA_CHAN_ID(1, 0), .prio = 0},
//	{.chan_id = DMA_CHAN_ID(1, 1), .prio = 0},
	{.chan_id = DMA_CHAN_ID(1, 2), .prio = 0},
//	{.chan_id = DMA_CHAN_ID(1, 3), .prio = 0},
};

u32 vdma_a0_pchans1[] = {
//	DMA_CHAN_ID(0, 0),
//	DMA_CHAN_ID(0, 1),
	DMA_CHAN_ID(0, 2),
//	DMA_CHAN_ID(0, 3),
//	DMA_CHAN_ID(1, 0),
//	DMA_CHAN_ID(1, 1),
	DMA_CHAN_ID(1, 2),
//	DMA_CHAN_ID(1, 3),
};

struct xdx_dma_vchan_cfg vdma_a0_vchans[] = {
	{
		.dev_id = 0,
		.prio = -1,
		.num_pchan = ARRAY_SIZE(vdma_a0_pchans1),
		.pchans = vdma_a0_pchans1,
	},
};
#if LINUX_VERSION_CODE > KERNEL_VERSION(3, 10, 0)
static struct dma_slave_map xdx_vdmaa0_map[] = {
	{.devname = "xdx_vdma", .slave = "xdx_dma",.param = "xdx_vdma"},
};
#endif
static void xdx_set_burst_length(u32 *p_cfg, s8 src_burst, s8 dst_burst);

struct xdx_dma_drv_cfg xdxvdma_a0 = {
	.set_burst_length = xdx_set_burst_length,
	.src_addr_widths   = BIT(DMA_SLAVE_BUSWIDTH_1_BYTE) |
				BIT(DMA_SLAVE_BUSWIDTH_2_BYTES) |
				BIT(DMA_SLAVE_BUSWIDTH_4_BYTES),
	.dst_addr_widths   = BIT(DMA_SLAVE_BUSWIDTH_1_BYTE) |
				BIT(DMA_SLAVE_BUSWIDTH_2_BYTES) |
				BIT(DMA_SLAVE_BUSWIDTH_4_BYTES),
	.flags = XDXDMA_FLAG_VDMA | XDXDMA_FLAG_A0,
	.num_vchan = ARRAY_SIZE(vdma_a0_vchans),
	.vchans = vdma_a0_vchans,
	.num_pchan = ARRAY_SIZE(vdma_a0_pchans),
	.pchans = vdma_a0_pchans,
#if LINUX_VERSION_CODE > KERNEL_VERSION(3, 10, 0)
	.fn_map = xdx_vdmaa0_map,
#endif
	.desc_entry_size = VDMA_DESC_DATE_SIZE,
};

#define XDX_DMA_DESC_ENTYR_OFFSET(__desc_entry_size, _entry_num) \
	(((_entry_num) * (__desc_entry_size)) + offsetof(xdx_dma_desc_blk_t, desc_entry))

static void xdx_set_burst_length(u32 *p_cfg, s8 src_burst, s8 dst_burst)
{

}

static VDMA_DESC_Data_t *xdx_vdma_get_desc_entry_a0(struct xdx_dma_dev *sdev,
		struct list_head *list, uint64_t data_size)
{
	dma_addr_t p_desc_blk;
	xdx_dma_desc_blk_t *desc_blk;
	xdx_dma_desc_blk_t *desc_blk_temp;
	struct list_head *desc_nexthead;

	if (list_empty(list)) {
		desc_blk = alloc_desc_block(sdev, &p_desc_blk);
		if (!desc_blk)
			return ERR_PTR(-ENOMEM);
		list_add_tail(&desc_blk->list, list);
	}
	desc_blk = list_last_entry(list, xdx_dma_desc_blk_t, list);
	desc_nexthead = &desc_blk->next_list;
retry:
	desc_blk = list_last_entry(desc_nexthead, xdx_dma_desc_blk_t, next_list);

	if (desc_blk->desc_num < desc_blk->desc_total) {
		uint64_t NextDescAddr = 0;
		if (desc_blk->desc_num != 0) {
			NextDescAddr = desc_blk->desc_addr +
				XDX_DMA_DESC_ENTYR_OFFSET(sdev->dcfg->desc_entry_size, desc_blk->desc_num);
			if (desc_blk->desc_num == desc_blk->desc_total - 1)
				XDX_DMA_DBG(sdev->dev, "Pad Linkend to %d/%d.", desc_blk->desc_num, desc_blk->desc_total);
			VDMA_ADDVdmaNextDesc(NextDescAddr,
				(VDMA_DESC_Data_t *)xdx_dma_get_desc_entry(sdev, desc_blk, desc_blk->desc_num -1));
		}else if(!list_empty(desc_nexthead)) {
			desc_blk_temp = list_prev_entry(desc_blk, next_list);
			NextDescAddr = desc_blk->desc_addr +
				XDX_DMA_DESC_ENTYR_OFFSET(sdev->dcfg->desc_entry_size, desc_blk->desc_num);
			XDX_DMA_DBG(sdev->dev, "[%s]%d: NextDesc BLOCK:%llx",__func__, __LINE__, NextDescAddr);
			VDMA_ADDVdmaNextDesc(NextDescAddr,
				(VDMA_DESC_Data_t *)xdx_dma_get_desc_entry(sdev, desc_blk_temp, desc_blk_temp->desc_num -1));
		}
		desc_blk_temp = list_entry(desc_nexthead, xdx_dma_desc_blk_t, next_list);
		desc_blk_temp->data_size += data_size;
		return (VDMA_DESC_Data_t *)xdx_dma_get_desc_entry(sdev, desc_blk, desc_blk->desc_num++);
	}
	desc_blk_temp = alloc_desc_block(sdev, &p_desc_blk);
	if (!desc_blk_temp)
		return ERR_PTR(-ENOMEM);
	list_add_tail(&desc_blk_temp->next_list, desc_nexthead);
	goto retry;
}

static VDMA_DESC_Data_t *xdx_vdma_new_desc_list_a0(struct xdx_dma_dev *sdev,
	struct list_head *list, uint64_t align_addr, uint64_t data_size)
{
	dma_addr_t p_desc_blk;
	xdx_dma_desc_blk_t *desc_blk;

	desc_blk = alloc_desc_block(sdev, &p_desc_blk);
	if (!desc_blk)
		return ERR_PTR(-ENOMEM);
	desc_blk->align_addr = align_addr;
	list_add_tail(&desc_blk->list, list);
	return xdx_vdma_get_desc_entry_a0(sdev, list, data_size);
}

static int xdx_vdma_desc_entry_add_a0(
	struct xdx_dma_dev *sdev,
	struct list_head *list,
	dma_addr_t blk_addr,
	uint64_t lens,
	bool new_blk,
	bool is_host)
{
	VDMA_DESC_Data_t *desc_entry;
	uint64_t seg_lens;
	uint64_t remain_size = lens;
	uint64_t offset = 0;

	do {
		seg_lens = remain_size;
		if (remain_size < VDMA_MAX_SIZE_SL_8B) {
			seg_lens = remain_size;
		} else if (MOD_2(remain_size, VDMA_ADDR_BEST_ALIGN)) {
			seg_lens = MOD_2(remain_size, VDMA_ADDR_BEST_ALIGN);
		}
		seg_lens = MIN(seg_lens, VDMA_MAX_SIZE_SL_1K);

		if (!new_blk) {
			desc_entry = xdx_vdma_get_desc_entry_a0(sdev, list, seg_lens);
			if (IS_ERR_OR_NULL(desc_entry))
				return -ENOMEM;
		} else {
			desc_entry = xdx_vdma_new_desc_list_a0(sdev, list, blk_addr + offset, seg_lens);
			if (IS_ERR_OR_NULL(desc_entry))
				return -ENOMEM;
		}
		memset(desc_entry,0 ,sizeof(VDMA_DESC_Data_t));
		if (MOD_2(seg_lens, VDMA_ADDR_BEST_ALIGN)) {
			VDMA_GenVdmaDesc(blk_addr  + offset,
				seg_lens >> VDMA_DRV_LENUNIT_8B_SHIFT, VDMA_DRV_LENUNIT_8B, desc_entry, is_host);
		} else {
			VDMA_GenVdmaDesc(blk_addr + offset,
				seg_lens >> VDMA_DRV_LENUNIT_1K_SHIFT, VDMA_DRV_LENUNIT_1K, desc_entry, is_host);
		}
		offset += seg_lens;
		remain_size -= seg_lens;
		new_blk = 0;
	} while (offset < lens);
	return 0;
}

const static int unit_shift_tab[4] = {3, 6, 10, 20};
static inline int xdx_dma_calc_xmit_len(uint8_t unit, uint32_t unit_cnt)
{
	if  (unit > 4)
		return -EINVAL;
	return unit_cnt << unit_shift_tab[unit];
}

static inline int xdx_dma_cmd_cfg_init(
		struct xdx_desc *txd, VDMA_CMD_Config_t *VdmaCfg)
{
	VdmaCfg->IntrEn = 1;
	VdmaCfg->PcieFuncNum = 0;
#if defined(SUPPORT_DONT_SNOOP) || defined(__arm64__) || defined(__aarch64__)
	VdmaCfg->PcieDontSnoop = 1;
#endif
	switch (txd->direct)
	{
		case DMA_MEM_TO_DEV:
			VdmaCfg->CopyParam.SrcAddrMode = VDMA_DRV_ADDRMODE_HOST_SVA;
			VdmaCfg->CopyParam.DstAddrMode = VDMA_DRV_ADDRMODE_GPU_SVA;
			break;
		case DMA_DEV_TO_MEM:
			VdmaCfg->CopyParam.SrcAddrMode = VDMA_DRV_ADDRMODE_GPU_SVA;
			VdmaCfg->CopyParam.DstAddrMode = VDMA_DRV_ADDRMODE_HOST_SVA;
			break;
		case DMA_DEV_TO_DEV:
			VdmaCfg->CopyParam.SrcAddrMode = VDMA_DRV_ADDRMODE_HOST_SVA;
			VdmaCfg->CopyParam.DstAddrMode = VDMA_DRV_ADDRMODE_HOST_SVA;
			break;
		case DMA_MEM_TO_MEM:
			VdmaCfg->CopyParam.SrcAddrMode = VDMA_DRV_ADDRMODE_HOST_SVA;
			VdmaCfg->CopyParam.DstAddrMode = VDMA_DRV_ADDRMODE_HOST_SVA;
			break;
		default:
			return -EINVAL;
	}
	return 0;
}
static int  xdx_dma_package_d2d_copy_cfg(
	struct xdx_dma_dev *sdev, struct xdx_desc *txd,
	dma_addr_t src, dma_addr_t dest, uint8_t unit,
	uint32_t unit_cnt)
{
	xdx_desc_cmd_cfg_t *cfg;
	VDMA_CMD_Config_t *VdmaCfg;

	cfg = xdx_dma_alloc_cmd_cfg(sdev);
	if (!cfg)
		return -ENOMEM;
	VdmaCfg = &cfg->va0_cfg;
	memset(VdmaCfg, 0, sizeof(VDMA_CMD_Config_t));
	xdx_dma_cmd_cfg_init(txd, VdmaCfg);
	VdmaCfg->OpCode = VDMA_DRV_OPCODE_COPY_DSRC_DDST;
	VdmaCfg->CopyParam.DstAddr = dest;
	VdmaCfg->CopyParam.SrcAddr = src;

	VdmaCfg->CopyParam.LengthUnit = unit;
	VdmaCfg->CopyParam.TransferLen = unit_cnt;
	list_add_tail(&cfg->list, &txd->cmd_cfgs_head);
	atomic_inc(&txd->cmd_cnt);
	return xdx_dma_calc_xmit_len(unit, unit_cnt);
}

static void xdx_dma_dump_desc_a0(struct xdx_dma_dev *sdev, xdx_dma_desc_blk_t *desc_blk)
{
	VDMA_DESC_Data_t *entry;
	int i;
	xdx_dma_desc_blk_t *pdesc_blk = desc_blk;
	uint64_t v1,v2;

	for (i = 0; i < desc_blk->desc_num; i++) {
		entry = (VDMA_DESC_Data_t *)xdx_dma_get_desc_entry(sdev, desc_blk, i);
		if (i < 3 || i >= desc_blk->desc_num -3) {
			v1 = (uint64_t)entry->Dword1.DataBlkPtr_47_21 << 21;
			v1 |= (uint64_t)entry->Dword0.DataBlkPtr_20_5 << 5;
			v2 = entry->Dword0.NextDescPtr_47_4 << 4;
			XDX_DMA_DBG(sdev->dev, "Dump:%d/%d, DataBlkPtr:%llx NextDescPtr_47_4:%llx Ulit:%x TransferLen:%x End:%x",
				i +1 , desc_blk->desc_num, v1, v2,
				entry->Dword1.LengthUnit, entry->Dword1.TransferLen, entry->Dword1.LinkEnd);
		}
	}
	list_for_each_entry(pdesc_blk, &desc_blk->next_list, next_list) {
		for (i = 0; i < pdesc_blk->desc_num; i++) {
			entry = (VDMA_DESC_Data_t *)xdx_dma_get_desc_entry(sdev, pdesc_blk, i);
			if (i < 3 || i >= pdesc_blk->desc_num - 3) {
				v1 = (uint64_t)entry->Dword1.DataBlkPtr_47_21 << 21;
				v1 |= (uint64_t)entry->Dword0.DataBlkPtr_20_5 << 5;
				v2 = entry->Dword0.NextDescPtr_47_4 << 4;
				XDX_DMA_DBG(sdev->dev,"Dump:%d/%d, DataBlkPtr:%llx NextDescPtr_47_4:%llx Ulit:%x TransferLen:%x End:%x",
					i + 1 , pdesc_blk->desc_num, v1, v2,
					entry->Dword1.LengthUnit, entry->Dword1.TransferLen, entry->Dword1.LinkEnd);
			}
		}
	}
}

static int  xdx_dma_package_d2l_l2d_copy_cfg(
	struct xdx_dma_dev *sdev, struct xdx_desc *txd,
	dma_addr_t dma_addr, xdx_dma_desc_blk_t *desc_blk, bool src_dma)
{
	xdx_desc_cmd_cfg_t *cfg;
	VDMA_CMD_Config_t *VdmaCfg;

	cfg = xdx_dma_alloc_cmd_cfg(sdev);
	if (!cfg)
		return -ENOMEM;
	VdmaCfg = &cfg->va0_cfg;

	xdx_dma_flush_desc_block(sdev, txd, desc_blk);
	memset(VdmaCfg, 0, sizeof(VDMA_CMD_Config_t));
	xdx_dma_dump_desc_a0(sdev, desc_blk);
	xdx_dma_cmd_cfg_init(txd, VdmaCfg);
	if (src_dma) {
		VdmaCfg->OpCode = VDMA_DRV_OPCODE_COPY_DSRC_LDST;
		VdmaCfg->CopyParam.SrcAddr = dma_addr;
		VdmaCfg->CopyParam.DstAddr = desc_blk->desc_addr + offsetof(xdx_dma_desc_blk_t, desc_entry);
		if (desc_blk->desc_num == 1) {
			VdmaCfg->OpCode = VDMA_DRV_OPCODE_COPY_DSRC_DDST;
			VdmaCfg->CopyParam.DstAddr = desc_blk->align_addr;
		}
	}
	else {
		VdmaCfg->OpCode = VDMA_DRV_OPCODE_COPY_LSRC_DDST;
		VdmaCfg->CopyParam.SrcAddr = desc_blk->desc_addr + offsetof(xdx_dma_desc_blk_t, desc_entry);
		VdmaCfg->CopyParam.DstAddr = dma_addr;
		if (desc_blk->desc_num == 1) {
			VdmaCfg->OpCode = VDMA_DRV_OPCODE_COPY_DSRC_DDST;
			VdmaCfg->CopyParam.SrcAddr = desc_blk->align_addr;
		}
	}

	if ((desc_blk->data_size & 0X3FF)) {
		VdmaCfg->CopyParam.LengthUnit= VDMA_DRV_LENUNIT_8B;
		VdmaCfg->CopyParam.TransferLen = desc_blk->data_size >> VDMA_DRV_LENUNIT_8B_SHIFT;
	} else {
		VdmaCfg->CopyParam.LengthUnit = VDMA_DRV_LENUNIT_1K;
		VdmaCfg->CopyParam.TransferLen = desc_blk->data_size >> VDMA_DRV_LENUNIT_1K_SHIFT;
	}
	XDX_DMA_DBG(sdev->dev, "[%s] opcode:%x src:%llx dst:%llx LengthUnit:%d TransferLen:%x data_size:%llx", __func__,
		VdmaCfg->OpCode, VdmaCfg->CopyParam.SrcAddr, VdmaCfg->CopyParam.DstAddr, VdmaCfg->CopyParam.LengthUnit,
		VdmaCfg->CopyParam.TransferLen, desc_blk->data_size);
	list_add_tail(&cfg->list, &txd->cmd_cfgs_head);
	atomic_inc(&txd->cmd_cnt);
	return desc_blk->data_size;
}

static int  xdx_dma_package_l2l_copy_cfg(
	struct xdx_dma_dev *sdev, struct xdx_desc *txd,
	xdx_dma_desc_blk_t *desc_blk_src, xdx_dma_desc_blk_t *desc_blk_dest)
{
	xdx_desc_cmd_cfg_t *cfg;
	VDMA_CMD_Config_t *VdmaCfg;

	cfg = xdx_dma_alloc_cmd_cfg(sdev);
	if (!cfg)
		return -ENOMEM;
	xdx_dma_flush_desc_block(sdev, txd, desc_blk_src);
	xdx_dma_flush_desc_block(sdev, txd, desc_blk_dest);
	VdmaCfg = &cfg->va0_cfg;
	memset(VdmaCfg, 0, sizeof(VDMA_CMD_Config_t));
	xdx_dma_cmd_cfg_init(txd, VdmaCfg);
	VdmaCfg->OpCode = VDMA_DRV_OPCODE_COPY_LSRC_LDST;
	VdmaCfg->CopyParam.SrcAddr = desc_blk_src->desc_addr + offsetof(xdx_dma_desc_blk_t, desc_entry);
	VdmaCfg->CopyParam.DstAddr = desc_blk_dest->desc_addr + offsetof(xdx_dma_desc_blk_t, desc_entry);
	if ((desc_blk_src->data_size & 0X3FF)) {
		VdmaCfg->CopyParam.LengthUnit= VDMA_DRV_LENUNIT_8B;
	} else {
		VdmaCfg->CopyParam.LengthUnit = VDMA_DRV_LENUNIT_1K;
	}
	XDX_DMA_DBG(sdev->dev, "[%s] opcode:%x src:%llx dst:%llx", __func__,
		VdmaCfg->OpCode, VdmaCfg->CopyParam.SrcAddr, VdmaCfg->CopyParam.DstAddr);
	list_add_tail(&cfg->list, &txd->cmd_cfgs_head);
	atomic_inc(&txd->cmd_cnt);
	return txd->paras.len;
}

static int xdx_vdma_package_copy_cfg_a0(struct xdx_dma_dev *sdev, struct xdx_desc *txd)
{
	uint64_t remain_len;
	xdx_dma_desc_blk_t *desc_blk;
	uint64_t offset = 0;
	int copy_size;

	if (txd->paras.src.addr_type == XDX_DMA_ADDR_SGL &&
		txd->paras.dest.addr_type == XDX_DMA_ADDR_SGL) {
		copy_size = xdx_dma_package_l2l_copy_cfg(sdev, txd,
				list_first_entry(&txd->desc_src_head, xdx_dma_desc_blk_t, list),
				list_first_entry(&txd->desc_dest_head, xdx_dma_desc_blk_t, list));
		if (copy_size <= 0)
			return copy_size;
	} else if (txd->paras.src.addr_type == XDX_DMA_ADDR_SGL) {
		offset = 0;
		if (txd->paras.dest.addr_type == XDX_DMA_ADDR_NULL) {
			txd->paras.dest.addr = txd->vd.tx.phys;
			txd->paras.dest.addr_type = XDX_DMA_ADDR_DMA;
		}
		list_for_each_entry(desc_blk, &txd->desc_src_head, list) {
			copy_size = xdx_dma_package_d2l_l2d_copy_cfg(sdev, txd,
				txd->paras.dest.addr + offset, desc_blk, 0);
			if (copy_size <= 0)
				return copy_size;
			offset += copy_size;
		}
		XDX_DMA_DBG(sdev->dev, "L2D Copy Size:%llx --%llx", offset, txd->paras.len);
	} else if (txd->paras.dest.addr_type == XDX_DMA_ADDR_SGL) {
		offset = 0;
		if (txd->paras.src.addr_type == XDX_DMA_ADDR_NULL) {
			txd->paras.src.addr = txd->vd.tx.phys;
			txd->paras.src.addr_type = XDX_DMA_ADDR_DMA;
		}
		list_for_each_entry(desc_blk, &txd->desc_dest_head, list) {
			copy_size = xdx_dma_package_d2l_l2d_copy_cfg(sdev, txd,
				txd->paras.src.addr + offset, desc_blk, 1);
			if (copy_size <= 0)
				return copy_size;
			offset += copy_size;
		}
		XDX_DMA_DBG(sdev->dev, "D2L Copy Size:%llx --%llx", offset, txd->paras.len);
	} else {
		offset = 0;
		remain_len = txd->paras.len;
		if (remain_len <= VDMA_MAX_SIZE_SL_8B) {
			copy_size = xdx_dma_package_d2d_copy_cfg(sdev, txd, txd->paras.src.addr,
					txd->paras.dest.addr, VDMA_DRV_LENUNIT_8B, remain_len >> VDMA_DRV_LENUNIT_8B_SHIFT);
			if (copy_size <= 0)
				return copy_size;
		} else {
			copy_size = MOD_2(remain_len, VDMA_ADDR_BEST_ALIGN);
			if (copy_size) {
				copy_size = xdx_dma_package_d2d_copy_cfg(sdev, txd, txd->paras.src.addr,
					txd->paras.dest.addr, VDMA_DRV_LENUNIT_8B, copy_size >> VDMA_DRV_LENUNIT_8B_SHIFT);
				if (copy_size <= 0)
					return copy_size;;
				remain_len -= copy_size;
				offset += copy_size;
			}
			while(remain_len > 0) {
				copy_size = MIN(remain_len, VDMA_MAX_SIZE_SL_1K);
				copy_size = xdx_dma_package_d2d_copy_cfg(sdev, txd, txd->paras.src.addr + offset,
					txd->paras.dest.addr + offset, VDMA_DRV_LENUNIT_1K, copy_size >> VDMA_DRV_LENUNIT_1K_SHIFT);
				if (copy_size <= 0)
					return copy_size;
				remain_len -= copy_size;
				offset += copy_size;
			}
		}
	}
	XDX_DMA_DBG(sdev->dev, "[%s] Src type:%d,Dest type:%d Cnt:%d", __func__,
		txd->paras.src.addr_type, txd->paras.dest.addr_type, atomic_read(&txd->cmd_cnt));
	return 0;
}


static int  _xdx_dma_package_memset_cfg_a0(struct xdx_dma_dev *sdev, struct xdx_desc *txd,
		uint64_t remain_len, dma_addr_t dstAddr)
{
	xdx_desc_cmd_cfg_t *cfg;
	VDMA_CMD_Config_t *VdmaCfg;
	int copy_size = 0;
	uint64_t offset = 0;

	while (remain_len > 0) {
		copy_size = MIN(remain_len, VDMA_DRV_XFERLEN_LIMIT * txd->paras.para_memset.Pattern_len);
		cfg = xdx_dma_alloc_cmd_cfg(sdev);
		if (!cfg)
			return -ENOMEM;
		VdmaCfg = &cfg->va0_cfg;
		memset(VdmaCfg, 0, sizeof(VDMA_CMD_Config_t));
		VdmaCfg->IntrEn = 1;
		VdmaCfg->PcieFuncNum = 0;
#if defined(SUPPORT_DONT_SNOOP) || defined(__arm64__) || defined(__aarch64__)
		VdmaCfg->PcieDontSnoop = 1;
#endif
		VdmaCfg->OpCode = VDMA_DRV_OPCODE_WRITE_DIRECT;
		VdmaCfg->WriteDirectParam.DstAddr = dstAddr + offset;
		if (txd->direct == DMA_MEM_TO_DEV) {
			VdmaCfg->WriteDirectParam.DstAddrMode = VDMA_DRV_ADDRMODE_GPU_SVA;
		} else {
			VdmaCfg->WriteDirectParam.DstAddrMode = VDMA_DRV_ADDRMODE_HOST_SVA;
		}
		VdmaCfg->WriteDirectParam.TransferLen = copy_size >> VDMA_DRV_CMD_REFBUF_OFFSET;
		VdmaCfg->WriteDirectParam.PatternLen = txd->paras.para_memset.Pattern_len;
		VdmaCfg->WriteDirectParam.patternData = txd->paras.para_memset.Pattern[0];
		XDX_DMA_DBG(sdev->dev, "[%s] opcode:%x dst:%llx TransferLen:%x PatternLen:%d  cmd_cnt:%d", __func__,
			VdmaCfg->OpCode,  VdmaCfg->WriteDirectParam.DstAddr, VdmaCfg->WriteDirectParam.TransferLen,
			VdmaCfg->WriteDirectParam.PatternLen, atomic_read(&txd->cmd_cnt));
		list_add_tail(&cfg->list, &txd->cmd_cfgs_head);
		atomic_inc(&txd->cmd_cnt);

		remain_len -= copy_size;
		offset += copy_size;
	}
	return 0;
}

static int  xdx_dma_package_memset_cfg_a0(struct xdx_dma_dev *sdev, struct xdx_desc *txd)
{
	struct scatterlist *sg;
	int iRet = 0, i = 0;

	for_each_sg(txd->paras.dest.sg_tab.sgl, sg, txd->paras.dest.sg_tab.sg_len, i) {
		if (sg_dma_len(sg) == 0)
			continue;
		iRet = _xdx_dma_package_memset_cfg_a0(sdev, txd, sg_dma_len(sg), sg_dma_address(sg));
		if (iRet < 0)
			return iRet;
	}
	return iRet;
}


static int xdx_dma_package_cfg_a0(struct xdx_dma_dev *sdev, struct xdx_desc *txd)
{
	int ret = 0;

	switch(txd->xmit_type) {
		case XDX_XMIT_COPY:
			ret = xdx_vdma_package_copy_cfg_a0(sdev, txd);
		break;
		case XDX_XMIT_MEMSET:
			ret = xdx_dma_package_memset_cfg_a0(sdev, txd);
		break;
		case XDX_XMIT_COPY_2D:
		case XDX_XMIT_TWD_2D3D:
		case XDX_XMIT_POLL:
		default:
			ret = -ENOTSUPP;
		break;
	}
	return ret;
}

static int xdx_dma_parse_sgl_a0(struct xdx_dma_dev *sdev,
	xdxdma_xmit_sg_t *sgt, struct list_head *list, uint64_t data_size, bool unit_ignor, bool is_host)
{
	struct scatterlist *sg;
	int ret = 0;
	int i = 0;
	uint64_t remain_len;
	uint64_t offset = 0;
	uint64_t end = 0;
	uint64_t continous_len = 0;
	uint32_t blk_len;
	uint32_t seg_len;
	bool force_switch = 0;
	dma_addr_t blk_addr;

	if (!sgt->sgl || ! sgt->sg_len)
		return -EINVAL;
	remain_len = data_size;
	for_each_sg(sgt->sgl, sg, sgt->sg_len, i) {
		blk_len = sg_dma_len(sg);
		blk_addr = sg_dma_address(sg);
		if (blk_len == 0)
			continue;
		if (unit_ignor) {
			ret = xdx_vdma_desc_entry_add_a0(sdev, list, blk_addr, blk_len, 0, is_host);
			if (ret < 0)
				return ret;
		} else {
			while (blk_len && remain_len) {
				if (MOD_2(blk_addr, PAGE_SIZE)) {
					seg_len = MIN(PAGE_SIZE - MOD_2(blk_addr,PAGE_SIZE), blk_len);
					seg_len = MIN(seg_len, remain_len);
					continous_len = 0;
					force_switch = 1;
					XDX_DMA_DBG(sdev->dev,
						"[%s]0.DataSize:%llx to offset:%llx blk_len:%x seg_len:%x  blk_addr:%llx switch",
						__func__, data_size, offset, blk_len, seg_len, blk_addr);
				} else {
					if (force_switch) {
						continous_len = 0;
						force_switch = 0;
					}
					seg_len = MIN(remain_len, blk_len);
					seg_len = MIN(seg_len, VDMA_MAX_SIZE_SL_1K);
					end = continous_len + seg_len;
					if (end > VDMA_MAX_SIZE_SL_1K ||
							(end > VDMA_MAX_SIZE_SL_8B && MOD_2(end, VDMA_ADDR_BEST_ALIGN))) {
						if (seg_len > VDMA_MAX_SIZE_SL_8B && MOD_2(seg_len, VDMA_ADDR_BEST_ALIGN)) {
							seg_len -= MOD_2(seg_len, VDMA_ADDR_BEST_ALIGN);
						}
						continous_len = 0;
						XDX_DMA_DBG(sdev->dev,
							"[%s]1.DataSize:%llx to offset:%llx blk_len:%x seg_len:%x  blk_addr:%llx switch",
							__func__, data_size, offset, blk_len, seg_len, blk_addr);
					}
				}
				ret = xdx_vdma_desc_entry_add_a0(sdev, list, blk_addr, seg_len, !continous_len, is_host);
				if (ret < 0)
					return ret;
				continous_len += seg_len;
				blk_len -= seg_len;
				remain_len -= seg_len;
				offset += seg_len;
				blk_addr += seg_len;
			}
		}
	}
	return ret;
}

static int  xdx_dma_parse_sgls_a0(struct xdx_dma_dev *sdev, struct xdx_desc *txd)
{
	int ret;

	if (txd->paras.src.addr_type == XDX_DMA_ADDR_SGL && txd->paras.dest.addr_type == XDX_DMA_ADDR_SGL) {
		ret = xdx_dma_parse_sgl_a0(sdev, &txd->paras.src.sg_tab, &txd->desc_src_head, txd->paras.len, 1,
			(txd->direct == DMA_MEM_TO_MEM || txd->direct == DMA_MEM_TO_DEV));
		if (ret < 0)
			return ret;
		ret = xdx_dma_parse_sgl_a0(sdev, &txd->paras.dest.sg_tab, &txd->desc_dest_head, txd->paras.len, 1,
			(txd->direct == DMA_MEM_TO_MEM || txd->direct == DMA_DEV_TO_MEM));
		if (ret < 0)
			return ret;
	} else if (txd->paras.src.addr_type == XDX_DMA_ADDR_SGL) {
		ret = xdx_dma_parse_sgl_a0(sdev, &txd->paras.src.sg_tab, &txd->desc_src_head, txd->paras.len, 0,
			(txd->direct == DMA_MEM_TO_MEM || txd->direct == DMA_MEM_TO_DEV));
		if (ret < 0)
			return ret;
	} else if (txd->paras.dest.addr_type == XDX_DMA_ADDR_SGL) {
		ret = xdx_dma_parse_sgl_a0(sdev, &txd->paras.dest.sg_tab, &txd->desc_dest_head, txd->paras.len, 0,
			(txd->direct == DMA_MEM_TO_MEM || txd->direct == DMA_DEV_TO_MEM));
		if (ret < 0)
			return ret;
	}
	return 0;
}

static int xdx_dma_issue_vdma_a0(
		struct xdx_vchan *vchan, struct xdx_desc *txd, xdx_desc_cmd_cfg_t *cfg)
{
	struct xdx_dma_dev *sdev = to_xdx_dma_dev(vchan->vc.chan.device);
	struct xdx_pchan *pchan = NULL;
	VDMA_CMD_Data_t VdmaCmd;
	uint32_t cmd_tag;
	VDMA_CMD_Config_t *pVdmaCfg = &cfg->va0_cfg;

	//Get_freeCommand
	cmd_tag = xdx_dma_request_command_tag(vchan, sdev, &txd->pchan_used);
	if (cmd_tag < 0 || txd->pchan_used == NULL)
		return -EBUSY;
	pchan = txd->pchan_used;
	DBG_ASSERT(pchan);
	pchan->cmdRef[cmd_tag].desc = txd; //record command reference in pchan struct
	pchan->cmdRef[cmd_tag].vchan = vchan; //record command reference in pchan struct
	pVdmaCfg->CommandTag = cmd_tag;

	if(pVdmaCfg->OpCode == VDMA_DRV_OPCODE_WRITE_DIRECT) {
		VDMA_HAL_CH_ClrAndWrRefBufSingle((uint64_t)pchan->base, cmd_tag * VDMA_DRV_CMD_REFBUF_OFFSET, pVdmaCfg->WriteDirectParam.patternData);
		pVdmaCfg->WriteDirectParam.PatternLoc = cmd_tag * VDMA_DRV_CMD_REFBUF_OFFSET * pVdmaCfg->WriteDirectParam.PatternLen;
	}
	VDMA_DRV_GenerateVdmaCmd(pVdmaCfg, &VdmaCmd);

	VDMA_DRV_DumpVdmaCmdCfg(pVdmaCfg);
	VDMA_DRV_DumpVdmaCmdData(&VdmaCmd);

	spin_lock(&pchan->lock);
	VDMA_HAL_CH_WrCmdBufBurst((uint64_t)pchan->base, (uint64_t*)&(VdmaCmd.CmdDword0), 3);
	spin_unlock(&pchan->lock);
	return 0;
}

static int irq_set_en_a0(struct xdx_dma_dev *sdev, u32 stat)
{
	struct xdx_pchan *pchan;
	int i;

	for (i = 0; i < sdev->num_pchans; i++) {
		pchan = &sdev->pchans[i];
		if (stat)
			VDMA_DRV_CH_EnableIntr((uint64_t)pchan->base);
		else
			VDMA_DRV_CH_DisableIntr((uint64_t)pchan->base);
	}
	return 0;
}

static void xdx_dma_reset_a0(struct xdx_dma_dev *sdev)
{
	// TODO
	XDX_DMA_ERR(sdev->dev, " dma ctrl need reset.");
}

static void xdx_dma_interrupt_cmdcmpl_a0(struct xdx_pchan *pchan, uint64_t CmdCmplRawSts)
{
	struct xdx_vchan *vchan;
	struct xdx_desc	*txd;
	uint32_t cmd_tag;

	while (CmdCmplRawSts) {
		cmd_tag = GetLeadOnefromU64(CmdCmplRawSts);
		BIT_UNSET(CmdCmplRawSts, cmd_tag);

		txd = pchan->cmdRef[cmd_tag].desc;
		vchan = pchan->cmdRef[cmd_tag].vchan;

		spin_lock(&pchan->lock);
		xdx_dma_release_command_tag(pchan, cmd_tag);
		spin_unlock(&pchan->lock);

		if (txd && atomic_dec_and_test(&txd->cmd_cnt)) {
			spin_lock(&vchan->vc.lock);
			xdx_vchan_cookie_complete(&txd->vd);
			spin_unlock(&vchan->vc.lock);
		}
	}
}

static void xdx_dma_interrupt_pollingtimeout_a0(struct xdx_dma_dev *sdev, struct xdx_pchan *pchan,
				unsigned int i, uint64_t PollingTimeoutRawSts)
{
	struct xdx_vchan *vchan;
	struct xdx_desc *txd;
	uint32_t cmd_tag;

	while (PollingTimeoutRawSts) {
		cmd_tag = GetLeadOnefromU64(PollingTimeoutRawSts);
		BIT_UNSET(PollingTimeoutRawSts, cmd_tag);

		txd = pchan->cmdRef[cmd_tag].desc;
		vchan = pchan->cmdRef[cmd_tag].vchan;

		if (txd && atomic_dec_and_test(&txd->cmd_cnt)) {
			spin_lock(&vchan->vc.lock);
			if (txd->vd.tx_result.result != DMA_TRANS_BUSSERR)
				txd->vd.tx_result.result = DMA_TRANS_POLLINGTIMEOUT;
			xdx_vchan_cookie_complete(&txd->vd);
			spin_unlock(&vchan->vc.lock);
		}
	}
}

static void xdx_dma_interrupt_buserr_a0(struct xdx_dma_dev *sdev, struct xdx_pchan *pchan,
				unsigned int i, uint64_t BusErrRawSts)
{
	struct xdx_vchan *vchan;
	struct xdx_desc *txd;
	uint32_t cmd_tag;

	while (BusErrRawSts) {
		cmd_tag = GetLeadOnefromU64(BusErrRawSts);
		BIT_UNSET(BusErrRawSts, cmd_tag);

		txd = pchan->cmdRef[cmd_tag].desc;
		vchan = pchan->cmdRef[cmd_tag].vchan;

		spin_lock(&pchan->lock);
		xdx_dma_reset_a0(sdev);
		spin_unlock(&pchan->lock);

		if (txd && atomic_dec_and_test(&txd->cmd_cnt)) {
			spin_lock(&vchan->vc.lock);
			txd->vd.tx_result.result = DMA_TRANS_BUSSERR;
			xdx_vchan_cookie_complete(&txd->vd);
			spin_unlock(&vchan->vc.lock);
		}
	}
}

static int xdx_vdma_interrupt_a0(struct xdx_dma_dev *sdev)
{
	struct xdx_pchan *pchan;
	uint64_t CmdCmplRawSts, PollingTimeoutRawSts, BusErrRawSts;
	unsigned int i;

	for (i = 0; i < sdev->num_pchans; i++) {
		pchan = &sdev->pchans[i];
		if(pchan->submit_cmd) {
			CmdCmplRawSts = VDMA_HAL_CH_RdAndClrCmdCmplRawSts((uint64_t)pchan->base);
			if (likely(CmdCmplRawSts))
				xdx_dma_interrupt_cmdcmpl_a0(pchan, CmdCmplRawSts);
			PollingTimeoutRawSts = VDMA_HAL_CH_RdAndClrPollingTimeoutRawSts((uint64_t)sdev->base, i);
			if (unlikely(PollingTimeoutRawSts))
				xdx_dma_interrupt_pollingtimeout_a0(sdev, pchan, i, PollingTimeoutRawSts);
			BusErrRawSts = VDMA_HAL_CH_RdAndClrBusErrRawSts((uint64_t)sdev->base, i);
			if (unlikely(BusErrRawSts))
				xdx_dma_interrupt_buserr_a0(sdev, pchan, i, BusErrRawSts);
		}

		if(!atomic_read(&sdev->tasklet_shutdown) && (sdev->pendingMask)) {
			tasklet_schedule(&sdev->task);
		}
	}
	return 0;
}

static int xdx_dma_print_reg_a0(struct xdx_dma_dev *sdev)
{
	struct xdx_pchan *pchan;
	int i;

	for (i = 0; i < sdev->num_pchans; i++) {
		pchan = &sdev->pchans[i];
		XDX_DMA_ERR(sdev->dev,"Ch[%d]->CTRL				0x%08x\n", i,
			VDMA_HAL_CH_RdCtrlReg((uint64_t)sdev->pchans[i].base));
		XDX_DMA_ERR(sdev->dev,"Ch[%d]->CHANNEL_IDLE			0x%08x\n", i,
			VDMA_HAL_CH_RdChannelIdleReg((uint64_t)sdev->base, i));
		XDX_DMA_ERR(sdev->dev,"Ch[%d]->CMD_CMPL_RAW_STS		0x%08llx\n", i,
			VDMA_HAL_CH_RdCmdCmplRawSts((uint64_t)sdev->pchans[i].base));
		XDX_DMA_ERR(sdev->dev,"Ch[%d]->POLLING_TIMEOUT_RAW_STS		0x%08llx\n", i,
			VDMA_HAL_CH_RdPollingTimeoutRawSts((uint64_t)sdev->base, i));
		XDX_DMA_ERR(sdev->dev,"Ch[%d]->BUS_ERR_RAW_STS			0x%08llx\n", i,
			VDMA_HAL_CH_RdBusErrRawSts((uint64_t)sdev->base, i));
		XDX_DMA_ERR(sdev->dev,"Ch[%d]->CMD_FIFO_STS			0x%08x\n", i,
			VDMA_HAL_CH_RdCmdBufOverflowSts((uint64_t)sdev->base, i));
	}
	return 0;
}

static int xdx_dma_cmd_buffer_remain_a0(struct xdx_pchan *pchan)
{
	DBG_ASSERT(NULL != pchan);
	return VDMA_HAL_GetChCmdRemainSize((uint64_t)pchan->base);
}

struct xdx_dma_ops vdma_a0_ops = {
	.irq_set_en = irq_set_en_a0,
	.irq_handler = xdx_vdma_interrupt_a0,
	.issue_chan = xdx_dma_issue_vdma_a0,
	.package_cfg = xdx_dma_package_cfg_a0,
	.parse_sgls = xdx_dma_parse_sgls_a0,
	.print_reg = xdx_dma_print_reg_a0,
	.cmd_buffer_remain = xdx_dma_cmd_buffer_remain_a0,
};

