/*************************************************************************/ /*!
@File           dma_km.h
@Title          DMA transfer module header
@Copyright      Copyright (c) Imagination Technologies Ltd. All Rights Reserved
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef DMA_KM_H
#define DMA_KM_H

#if defined(__linux__)
#include <linux/version.h>
#else
#define KERNEL_VERSION
#endif

#include "pvrsrv_error.h"
#include "img_types.h"
#include "cache_ops.h"
#include "device.h"
#include "pmr.h"
#include "pvrsrv_sync_km.h"
#include "connection_server.h"
#include "osfunc.h"
#include <linux/scatterlist.h>
#define DMACMD_3D_NUM		(6)
#define DMACMD_TDM_NUM		(4)
#define DMACMD_COM_NUM		(4)
#define DMACMD_KICKSYNC_NUM		(4)

#define CBUF_DMA_MAX_PACKAGE_LEN (512*1024U)
#define DMA_MEMFILL_ADDR_ALIGN	 (32)
#define DMA_MEMFILL_BEST_SIZE	 (512U)
#define CBUF_DMA_MIN_PACKAGE_LEN (16)
#define DMA_TINYCOPY_MAX_LEN     (32)


typedef struct  DMA_HOST_DEV_INFO_TAG
{
	DEVMEM_MEMDESC 			   *psShadowClientCCBMemDesc;
	DEVMEM_MEMDESC 			   *psClientCCBMemDesc;
	IMG_UINT64				   Shadowphysarray[64];
	void						*pvShadowClientCCB;				/*!< CPU mapping of the CCB *///addllb0314
	IMG_DEV_PHYADDR			   DevPhyaddr;
	IMG_DEV_PHYADDR			   HostPhyaddr;
	struct sg_table 		   ppsSg;
}DMA_HOST_DEV_INFO;

typedef struct  DMA_MM_BRIDGE_INFO_TAG
{
	IMG_UINT64 *pui64Table;
	IMG_UINT32 ui32iommuisenable;
	struct sg_table *psSg;
}DMA_MM_BRIDGE_INFO;

typedef struct DMA_OP_TAG {
	union
	{
		DEVMEM_MEMDESC* psMemDesc; /* mem desc of device memory buffer*/
		IMG_HANDLE      hDevPMR;
		IMG_UINT64		ui64DevAddr;
		IMG_UINT64  	ui64DevPAddr;
	};

	union
	{
		struct
		{
			union
			{
				DEVMEM_MEMDESC* hHostMemDesc; /* mem desc of device memory buffer*/
                IMG_HANDLE      hHostPMR;
				void			*pvCPUAddr; 	/* Pointer to the CPU buffer */
				IMG_UINT64		ui64HostAddr;
			};

			IMG_UINT32 uiOffset;		/* Offset into the device buffer */
			union{
				IMG_UINT32 uiSize;			/* Size of the transfer */
				IMG_UINT32	MemOffset;      /* Rect in host offset*/
			};
		};

		union
		{
			struct
			{
				IMG_UINT32	ui32Data_L;   /*!< data need copy by ACPU to ui64DevAddr */
				IMG_UINT32	ui32Data_H;   /*!< data need copy by ACPU to ui64DevAddr */
			};
			IMG_UINT32  ui32Data[4];
		};

	};

	union{
		struct             /*Rect copy struct*/
		{
			IMG_UINT16  Width;
			IMG_UINT16  Height;
			IMG_UINT16  MemStride;
			IMG_UINT16  DevStride;
			IMG_UINT32  HOSTOffset;
			IMG_UINT32  DevOffset;
		};

		struct
		{
			IMG_UINT16  T_Width;
			IMG_UINT16  T_Height;
			IMG_UINT16  T_XStride;
			IMG_UINT16  T_Depth;
			IMG_UINT16  T_LenUnit;
			IMG_UINT16  T_HOSTOffset;
			IMG_UINT32  T_YStride;
		};

		struct{
			IMG_UINT32 value;
			IMG_UINT32 mask;
			IMG_UINT32 uiHOSTOffset;		/* Offset into the Host buffer */
		};
		IMG_UINT32  as32FillData[3];
		IMG_UINT64  as64FillData[1];
	}__packed;

	union
	{
		struct
		{
			IMG_UINT8  ui8CopySize;	/* actual copy data size, only for DMA_CMD_ACPU_COPY */
			IMG_UINT8  ui8CoreNum;	/* actual GPU core number, only for SYNC DMA_CMD_ACPU_COPY */
		};

		IMG_UINT16 ui16BufSize;
	};

	IMG_UINT16 eCmdType;	/* copy by acpu or vmcu */
} __packed DMA_OP;

#define DMA_MAX_TRANSLATION_STACK_ALLOC_H (4)
typedef struct DMADevAddrType_{
	PMR *psSrcPMR;
	IMG_UINT32 ui32AddrNums;
	IMG_DEV_PHYADDR *psDmaAddrVals;
	IMG_BOOL *pbValids;
	IMG_BOOL bIsContig;
	IMG_BOOL bIsVaddr;
	IMG_BOOL bIsHostPort;
	IMG_PBYTE asAddressArray[DMA_MAX_TRANSLATION_STACK_ALLOC_H *
		(sizeof(IMG_DMA_ADDR) + sizeof(IMG_BOOL))];
} DMADevAddrType;

PVRSRV_ERROR DmaDeviceParams(CONNECTION_DATA *psConnection,
							 PVRSRV_DEVICE_NODE *psDevNode,
							 IMG_UINT32 *ui32DmaBuffAlign,
							 IMG_UINT32 *ui32DmaTransferMult);

PVRSRV_ERROR DmaSparseMappingTable(PMR *psPMR,
								   IMG_DEVMEM_OFFSET_T uiOffset,
								   IMG_UINT32 ui32SizeInPages,
								   IMG_BOOL *pbTable);

PVRSRV_ERROR DmaTransfer(CONNECTION_DATA *psConnection,
		    PVRSRV_DEVICE_NODE *psDevNode,
			IMG_UINT32 uiNumDMAs,
			IMG_UINT32 uiNumVDMAs,
			DMA_OP *pDma_Op,
			IMG_UINT32 uiFlags,
			PVRSRV_TIMELINE iUpdateFenceTimeline);


PVRSRV_ERROR DmaMemFill(CONNECTION_DATA *psConnection,
		    PVRSRV_DEVICE_NODE *psDevNode,
			PMR* psPMR,
			IMG_DEVMEM_OFFSET_T uiOffset,
			IMG_DEVMEM_SIZE_T uiSize,
			IMG_UINT32	ui32Pattern);

IMG_EXPORT PVRSRV_ERROR DmaSparseMemFill_lma(PVRSRV_DEVICE_NODE *psDevNode,
					IMG_UINT64 *pages,
					IMG_UINT32 pageNum,
					IMG_UINT32 pageSize,
					IMG_BYTE ui32Pattern);

PVRSRV_ERROR PVRSRVInitialiseDMA(PVRSRV_DEVICE_NODE *psDeviceNode);
void PVRSRVDeInitialiseDMA(PVRSRV_DEVICE_NODE *psDeviceNode);
IMG_UINT32 KDmaTransfer(PVRSRV_DEVICE_NODE *psDevNode,
					IMG_UINT64 *puiAddress,
					IMG_UINT64 pvDevAddress,
					IMG_UINT64 pDmaAddress,
					IMG_DEVMEM_SIZE_T size,
					IMG_UINT8 dir);
IMG_UINT32 KDmaTransfer1(PVRSRV_DEVICE_NODE *psDevNode,
	IMG_UINT64 puiAddress,
	PMR* ppsPMR,
	IMG_DEVMEM_OFFSET_T puiOffset,
	IMG_DEVMEM_SIZE_T puiSize,
	IMG_UINT8 dir);

IMG_BOOL IS_Consistency(IMG_UINT64 *puiAddress,IMG_UINT64 pDmaAddress);
//IMG_BOOL IS_Consistency(IMG_UINT64 *puiAddress,IMG_UINT64 pDmaAddress);
//IMG_UINT8 Get_aligned_Byte(IMG_UINT64 Address);
IMG_DEVMEM_SIZE_T Non_Aligned_Copy(IMG_UINT64 *puiAddress,IMG_UINT64 pvDevAddress,IMG_UINT64 pDmaAddress,IMG_DEVMEM_SIZE_T size,IMG_UINT8 dir);

#ifdef GENERATE_LINKLIST
IMG_EXPORT PVRSRV_ERROR Generate_Linklist(IMG_UINT64 *puiAddress,IMG_DEVMEM_SIZE_T size,IMG_DEV_PHYADDR *Linklist_Head);
IMG_EXPORT PVRSRV_ERROR Release_Linklist(IMG_DEV_PHYADDR Linklist_Head);
IMG_EXPORT PVRSRV_ERROR Generate_Linklist2(IMG_UINT64 *puiAddress,IMG_DEVMEM_SIZE_T size,IMG_DEV_PHYADDR *Linklist_Head,IMG_BOOL *is_Linklist);
#endif

#endif /* DMA_KM_H */
