/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef __XDX_PANGU_H__
#define __XDX_PANGU_H__

#include <linux/types.h>
#include <linux/bug.h>
#include <drm/xdx_asic_type.h>

#define PANGU_SHARERAM_SIZE		 0x00040000
#define PANGU_REG_DOORBELLS_BASE         0x00100000
#define PANGU_REG_HOST_MISC_BASE         0x00200000
#define PANGU_REG_GPU_CORE0_BASE         0x00400000
#define PANGU_REG_GPU_CORE1_BASE         0x00500000
#define PANGU_REG_GPU_CORE2_BASE         0x00600000
#define PANGU_REG_GPU_CORE3_BASE         0x00700000
#define PANGU_REG_DC0_BASE               0x00800000
#define PANGU_REG_DC1_BASE               0x00600000
#define PANGU_REG_VDMA0_DOORBELL_BASE    0x00A00000
#define PANGU_REG_VDMA0_BASE             0x00B00000
#define PANGU_REG_VDMA0_SIZE             0x00100000
#define PANGU_REG_VDMA1_DOORBELL_BASE    0x00C00000
#define PANGU_REG_VDMA1_BASE             0x00D00000
#define PANGU_REG_VDMA1_SIZE             0x00100000
#define PANGU_REG_HOSTMISC_BASE          0x00200c00
#define PANGU_REG_HOSTMISC_SIZE          0x00000008

#if defined (SUPPORT_DMA_TRANSFER_B0)
#define SYS_PANGU_REG_TDMA_BASE		(0x00300000)
#define SYS_PANGU_REG_TDMA_SIZE			(0x00100000)
#endif

#define PANGU_REG_AUDIO_CONFIG_BASE      0x00E00000
#define PANGU_REG_GIC_BASE               0x01000000
#define PANGU_REG_GIC_WRAPPER_BASE       0x01400000
#define PANGU_REG_HDMI_CONTROLLER_BASE   0x01500000
#define PANGU_REG_DP_CONTROLLER_BASE     0x01700000
#define PANGU_REG_IO_ACCESS_BASE         0x01000000
#define PANGU_REG_VDEC_BASE              0x01A00000
#define PANGU_REG_VENC_BASE              0x01B00000
#define PANGU_REG_GPU_CORE_MISC_BASE     0x01F00000
#define PANGU_REG_VDMA_MMU_BASE          0x02000000
#define PANGU_REG_XDMA_MMU_BASE          0x03000000
#define PANGU_REG_XDMA_CONFIG_BASE       0x03400000
#define PANGU_REG_GDDR_INFO              0x00C005FC
#define PANGU_DC_CHANNEL_INFO_OFFSET     0x013FF0A8

#define PANGU_A0_REG_VDMA_MMU_BASE          0x02000000
#define PANGU_A0_REG_XDMA_MMU_BASE          0x03000000

#define PANGU_A0_REG_XDMA_MMU_INTERNAL_BASE          (0x20000000 + PANGU_A0_REG_XDMA_MMU_BASE)
#define PANGU_A0_REG_VDMA_MMU_INTERNAL_BASE          (0x20000000 + PANGU_A0_REG_VDMA_MMU_BASE)

#define PANGU_B0_REG_VDMA_MMU_BASE          0x02000000
#define PANGU_B0_REG_XDMA_MMU_BASE          0x03000000
#define PANGU_B0_REG_TDMA_MMU_BASE          0x09000000
#define PANGU_B0_REG_VDEC_MMU_BASE          0x08800000
#define PANGU_B0_REG_VENC_MMU_BASE          0x08C00000

#define PANGU_REG_XDMA_CONFIG_BASE       0x03400000
#define PANGU_REG_GDDR_INFO              0x00C005FC

#define PANGU_PUBLIC_FW_INFO_BASE             0x013FF000
#define PANGU_PUBLIC_FW_INFO_SIZE             0x1000

#define PANGU_REG_XDMA_MMU_SIZE           0x00100000
#define PANGU_REG_VDMA_MMU_SIZE           0x00100000

#define PANGU_B0_REG_XDMA_MMU_INTERNAL_BASE          (0x20000000 + PANGU_B0_REG_XDMA_MMU_BASE)
#define PANGU_B0_REG_VDMA_MMU_INTERNAL_BASE          (0x20000000 + PANGU_B0_REG_VDMA_MMU_BASE)
#define PANGU_B0_REG_TDMA_MMU_INTERNAL_BASE          (0x20000000 + PANGU_B0_REG_TDMA_MMU_BASE)
#define PANGU_B0_REG_VDEC_MMU_INTERNAL_BASE          (0x20000000 + PANGU_B0_REG_VDEC_MMU_BASE)
#define PANGU_B0_REG_VENC_MMU_INTERNAL_BASE          (0x20000000 + PANGU_B0_REG_VENC_MMU_BASE)

#define PANGU_IO_IND_ACCESS_ADDR_L_OUTBAR      (PANGU_REG_IO_ACCESS_BASE + 0x20)
#define PANGU_IO_IND_ACCESS_ADDR_H_OUTBAR      (PANGU_REG_IO_ACCESS_BASE + 0x24)
#define PANGU_IO_IND_ACCESS_DATA_OUTBAR        (PANGU_REG_IO_ACCESS_BASE + 0x28)


#define PANGU_PUBLIC_FW_INFO_BASE        0x013FF000
#define PANGU_PUBLIC_FW_INFO_SIZE        0x1000

#define PANGU_REG_GPU_CORE_REG_SIZE      0x00100000
#define PANGU_REG_DC0_REG_SIZE           0x00200000
#define PANGU_REG_DC0_MMU_BASE           0x00910000
#define PANGU_REG_DC0_MMU_REG_SIZE       0x100
#define PANGU_REG_DC1_REG_SIZE           0x00200000
#define PANGU_REG_DC1_MMU_BASE           0x00710000
#define PANGU_REG_DC1_MMU_REG_SIZE       0x100

#define PANGU_VRAM_DEV_PHYADDR_BASE      0x0100000000
#define PANGU_HOSTRAM_DEV_PHYSICAL_BASE  0x8000000000
/* pangu SharedSram offset in device memory map*/
#define PANGU_SSRAM_DEV_PHYSICAL_BASE	 0x90000000
#define PANGU_SSRAM_DEV_PHYSICAL_OFFSET	 0x20000

#if defined(OS_NAME_KYLIN)
	#define PANGU_VRAM_RESERVE_SIZE 1920*1080*4
#else
	#define PANGU_VRAM_RESERVE_SIZE 0
#endif

#define PANGU_GPU_CORE_MAX 4
#define MAX_OUTPUT_NUM 2

/* GPU CORE MISC */
#define mmGPU_MISC_PRIMARY_CORE_ID        0x60
#define GPU_MISC_PRIMARY_CORE_ID_MASK     0x7

#define mmPCIE_HIGH_ADDR_CNTL             0x1000
#define mmG2H_BASE_ADDR0_GRP0             0x508
#define mmG2H_BASE_ADDR1_GRP0             0x50c
#define mmG2H_TARGET_ADDR0_GRP0           0x510
#define mmG2H_TARGET_ADDR1_GRP0           0x514
#define mmMSI_MSIX_HW_ENABLE_PF0          0xC00
#define mmMSI_MSIX_HW_ENABLE_PF1          0xC04
#define mmMSI_MSIX_STATUS_PF0             0xC80
#define mmMSI_MSIX_STATUS_PF1             0xC84
#define mmMSI_MSIX_CLEAR_PF0              0xC90
#define mmMSI_MSIX_CLEAR_PF1              0xC94
#define MAX_B0_MSIX_VEC_NUM               43
#define mmMSI_HW_ENABLE_BASE  (PANGU_REG_HOST_MISC_BASE + mmMSI_MSIX_HW_ENABLE_PF0)

#define mmGEN_MSI_ADDR_L_PF0              0x6000
#define mmGEN_MSI_ADDR_H_PF0              0x6004
#define mmGEN_MSI_DATA_PF0                0x6008
#define mmGEN_MSI_ADDR_L_PF1              0x600c
#define mmGEN_MSI_ADDR_H_PF1              0x6010
#define mmGEN_MSI_DATA_PF1                0x6014
#define mmGEN_MSI_EN                      0xc00

/* In FPGA, using doorbell 126 and 127 register to save interrupt status */
#define PANGU_REG_DOORBELL_126            0x1005F8
#define PANGU_REG_DOORBELL_127            0x1005FC
#define INTERRUPT_MAX_SIZE                8
#define mmHOST_MISC_CHECKIN               0x38
#define HOST_MISC_CHECKIN_DF_HOST_NO_SNOOP_ENABLE_MASK 0x2

struct pangu_msix_vec_info {
	uint32_t clientid;
	const char *name;
};

struct pangu_interrupt_status {
	union {
		uint32_t all;
		uint8_t intr_idx[4];
	} e1;
	union {
		uint32_t all;
		uint8_t intr_idx[4];
	} e2;
};

union primary_core_id {
	struct {
		uint32_t core0 : 3;
		uint32_t core1 : 3;
		uint32_t core2 : 3;
		uint32_t core3 : 3;
		uint32_t reserved : 20;
	} bitfiles;
	uint32_t value;
};

#define MSIX_INVALID_IDX        (~0)

/* NOTE: Don't use it directly, use it with xdx_get_msix_idx()
 * to get the real msix idx.
 * e.g. vec_idx = xdx_get_msix_idx(xdev->asic_type, MSIX_DC0_INTERRUPT)
 */
enum msix_vector {
	MSIX_VD0_CHANNEL_COMB_CH0        = 0,
	MSIX_VD0_CHANNEL_COMB_CH1        = 1,
	MSIX_VD0_CHANNEL_COMB_CH2        = 2,
	MSIX_VD0_CHANNEL_COMB_CH3        = 3,
	MSIX_VD1_CHANNEL_COMB_CH0        = 4,
	MSIX_VD1_CHANNEL_COMB_CH1        = 5,
	MSIX_VD1_CHANNEL_COMB_CH2        = 6,
	MSIX_VD1_CHANNEL_COMB_CH3        = 7,
	MSIX_VD0_MCU_DOORBELL            = 8,
	MSIX_VD1_MCU_DOORBELL            = 9,
	MSIX_C_MCU_DOORBELL              = 10,
	MSIX_A_CPU_DOORBELL              = 11,
	MSIX_GPU_CORE0                   = 12,
	MSIX_GPU_CORE1                   = 13,
	MSIX_GPU_CORE2                   = 14,
	MSIX_GPU_CORE3                   = 15,
	MSIX_A_TO_HOST_RESERVE_INT       = 16,
	MSIX_VDMCU_MMU_INT               = 17,
	MSIX_VDEC_INT                    = 18,
	MSIX_VENC_INT                    = 19,
	MSIX_DC0_INTERRUPT               = 20,
	MSIX_MMU0_INTERRUPT              = 21,
	MSIX_DC0_SECURE_INTERRUPT        = 22,
	MSIX_DC1_INTERRUPT               = 23,
	MSIX_MMU1_INTERRUPT              = 24,
	MSIX_DC1_SECURE_INTERRUPT        = 25,
	MSIX_DP0_INTERRUPT_O             = 26,
	MSIX_HDMI0_OAVP_INT              = 27,
	MSIX_HDMI0_OMAIN_INT             = 28,
	MSIX_HDMI0_OCEC_INT              = 29,
	MSIX_HDMI0_HPD_INT               = 30,
	MSIX_AUDIO0_INT                  = 31,
	MSIX_DP1_INTERRUPT_O             = 32,
	MSIX_HDMI1_OAVP_INT              = 33,
	MSIX_HDMI1_OMAIN_INT             = 34,
	MSIX_HDMI1_OCEC_INT              = 35,
	MSIX_HDMI1_HPD_INT               = 36,
	MSIX_AUDIO1_INT                  = 37,
	MSIX_TDMAINTR_DMA                = 38,
	MSIX_TDMMU_COMB                  = 39,
	MSIX_PDMA_HOST_INT               = 40,
	MSIX_PDMA_HOST_ERR_INT           = 41,
	MSIX_RESV0_INT                   = 42,
	MSIX_PF1_VD0_MCU_DOORBELL        = 50,
	MSIX_PF1_VD1_MCU_DOORBELL        = 51,
	MSIX_PF1_C_MCU_DOORBELL          = 52,
	MSIX_PF1_A_CPU_DOORBELL          = 53,
	MSIX_PF1_A_TO_HOST_RESERVE_INT   = 54,

	MSIX_VECTOR_MAX                  = 55,
};

#define PROTO_HDMIQP0_TX_PCS_START      0x01500000
#define PROTO_HDMIQP0_TX_PCS_SIZE       0x17FFF
#define PROTO_HDMIQP0_TX_TILE_START     0x015F0000
#define PROTO_HDMIQP0_TX_TILE_SIZE      0xFF
#define PROTO_HDMIQP0_TX_CORE_START     0x01570000
#define PROTO_HDMIQP0_TX_CORE_SIZE      0x7FFF
#define PROTO_DP0_CTL_START             0x01770000
#define PROTO_DP0_CTL_SIZE              0x80000
#define PROTO_DP0_PCS_START             0x01700000
#define PROTO_DP0_PCS_SIZE              0x70000

#define PROTO_HDMIQP1_TX_PCS_START      0x01600000
#define PROTO_HDMIQP1_TX_PCS_SIZE       0x17FFF
#define PROTO_HDMIQP1_TX_TILE_START     0x016F0000
#define PROTO_HDMIQP1_TX_TILE_SIZE      0xFF
#define PROTO_HDMIQP1_TX_CORE_START     0x01670000
#define PROTO_HDMIQP1_TX_CORE_SIZE      0x7FFF
#define PROTO_DP1_CTL_START             0x01870000
#define PROTO_DP1_CTL_SIZE              0x80000
#define PROTO_DP1_PCS_START             0x01800000
#define PROTO_DP1_PCS_SIZE              0x70000

#define SYS_PANGU_REG_HDMIDOORBELL     0x1005d8
/* C-Mcu doorbells */
#define XGP_CMCU_DOORBELLS_BASE                     0x00100000
#define XGP_CMCU_DOORBELLS_SIZE                     0x00100000

#define CMCU_DOORBELLS_H02A_INT_REGS_OFFSET	    	(0x2b0)
#define CMCU_DOORBELLS_A2H0_INT_REGS_OFFSET	    	(0x2e0)
#define CMCU_DOORBELLS_A2H0_CLR_REGS_OFFSET	    	(0x1c0)

#define CMCU_DOORBELLS_C2H0_CLR_REGS_OFFSET	    	(0x1e0)
#define CMCU_DOORBELLS_C2H0_STATS_REGS_OFFSET	    (0x300)
#define CMCU_DOORBELLS_H02C_STATS_REGS_OFFSET	    (0x2b8)

#define CMCU_DOORBELLS_DOORBELL0_OFFSET             (0x400U)
#define CMCU_DOORBELLS_DOORBELL31_OFFSET            (0x4F8U)
#define CMCU_DOORBELLS_DOORBELL32_OFFSET            (0x500U)
#define CMCU_DOORBELLS_DOORBELL33_OFFSET            (0x508U)
#define CMCU_DOORBELLS_DOORBELL35_OFFSET            (0x518U)
#define CMCU_DOORBELLS_DOORBELL41_OFFSET            (0x548U)
#define CMCU_DOORBELLS_DOORBELL44_OFFSET            (0x560U)
#define CMCU_DOORBELLS_DOORBELL45_OFFSET            (0x568U)
#define CMCU_DOORBELLS_DOORBELL46_OFFSET            (0x570U)
#define CMCU_DOORBELLS_DOORBELL50_OFFSET	    	(0x590U)
#define CMCU_DOORBELLS_DOORBELL51_OFFSET	    	(0x598U)
#define CMCU_DOORBELLS_DOORBELL52_OFFSET	    	(0x5a0U)
#define CMCU_DOORBELLS_DOORBELL53_OFFSET	    	(0x5a8U)
#define CMCU_DOORBELLS_DOORBELL54_OFFSET	    	(0x5b0U)
#define CMCU_DOORBELLS_DOORBELL55_OFFSET	    	(0x5b8U)
#define CMCU_DOORBELLS_DOORBELL63_OFFSET            (0x5f8U)

#define CMCU_DOORBELLS_H02A_INT_STATUS		    	(XGP_CMCU_DOORBELLS_BASE + CMCU_DOORBELLS_H02A_INT_REGS_OFFSET)

#define CMCU_DOORBELLS_A2H0_INT_STATUS		    	(XGP_CMCU_DOORBELLS_BASE + CMCU_DOORBELLS_A2H0_INT_REGS_OFFSET)
#define CMCU_DOORBELLS_A2H0_CLR_STATUS			(XGP_CMCU_DOORBELLS_BASE + CMCU_DOORBELLS_A2H0_CLR_REGS_OFFSET)
#define CMCU_DOORBELLS_A2H0_VAL_STATUS			(XGP_CMCU_DOORBELLS_BASE + CMCU_DOORBELLS_DOORBELL35_OFFSET)

#define CMCU_DOORBELLS_C2H0_INT_STATUS		    	(XGP_CMCU_DOORBELLS_BASE + CMCU_DOORBELLS_C2H0_STATS_REGS_OFFSET)
#define CMCU_DOORBELLS_C2H0_CLR_STATUS		    	(XGP_CMCU_DOORBELLS_BASE + CMCU_DOORBELLS_C2H0_CLR_REGS_OFFSET)
#define CMCU_DOORBELLS_C2H0_GET_VALUE		    	(XGP_CMCU_DOORBELLS_BASE + CMCU_DOORBELLS_DOORBELL46_OFFSET)

#define CMCU_DOORBELLS_H02C_POWER		    		(XGP_CMCU_DOORBELLS_BASE + CMCU_DOORBELLS_DOORBELL45_OFFSET)
#define CMCU_DOORBELLS_H02C_INT_STATUS	  	    	(XGP_CMCU_DOORBELLS_BASE + CMCU_DOORBELLS_H02C_STATS_REGS_OFFSET)

#define CMCU_DOORBELLS_H02C_THERMAL		    		(XGP_CMCU_DOORBELLS_BASE + CMCU_DOORBELLS_DOORBELL55_OFFSET)

#define CMCU_DOORBELLS_POOL_START                   (XGP_CMCU_DOORBELLS_BASE + CMCU_DOORBELLS_DOORBELL0_OFFSET)
/*Doorbell 32 used for trigger ACPU CCB
 *Fw team saied ACCB info stored in share ram(BAR4)
 * */
#define CMCU_DOORBELLS_MTS                          (XGP_CMCU_DOORBELLS_BASE + CMCU_DOORBELLS_DOORBELL32_OFFSET)
#define CMCU_DOORBELLS_HANDSHAKE                    (XGP_CMCU_DOORBELLS_BASE + CMCU_DOORBELLS_DOORBELL31_OFFSET)

#define CMCU_DOORBELLS_INT_STATUS                   (XGP_CMCU_DOORBELLS_BASE + CMCU_DOORBELLS_DOORBELL41_OFFSET)
#define CMCU_DOORBELLS_FENCEID                      (XGP_CMCU_DOORBELLS_BASE + CMCU_DOORBELLS_DOORBELL50_OFFSET)

#endif /* __XDX_PANGU_H__ */
