/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef __PVR_DEVICE_H__
#define __PVR_DEVICE_H__

#include "xdx_device.h"
#include "physheap_config.h"

#if defined(SUPPORT_PANGU_ACPU)
#define HEAP_NUMS 6
#else
#define HEAP_NUMS 5
#endif

typedef void (*PFN_PVRSRV_DEV_RELEASE)(void *pvData);
typedef struct _PVRSRV_CONNECTION_PRIV
{
	/* pvConnectionData is used to hold Services connection data
	 * for all PVRSRV_DEVICE_INIT_MODE options.
	 */
	void *pvConnectionData;

	/* pfDeviceRelease is used to indicate the release function
	 * to be called when PVRSRV_DEVICE_INIT_MODE is PVRSRV_LINUX_DEV_INIT_ON_CONNECT,
	 * as we can then have one connections made (either for Services or Sync) per
	 * psDRMFile, and need to know which type of connection is being released
	 * (as the ioctl release call is common for both).
	 */
	PFN_PVRSRV_DEV_RELEASE pfDeviceRelease;

	/* pvSyncConnectionData is used to hold Sync connection data
	 * when PVRSRV_DEVICE_INIT_MODE is not PVRSRV_LINUX_DEV_INIT_ON_CONNECT,
	 * as we can then have two connections made (for Services and Sync) to
	 * the same psDRMFile.
	 */
	void *pvSyncConnectionData;
} PVRSRV_CONNECTION_PRIV;

void pvr_init_phys_heaps(struct xdx_device *xdev,
			     PHYS_HEAP_CONFIG *phys_heaps, uint32_t *pcount);

int drm_pvr_srvkm_init(struct drm_device *dev, void *arg, struct drm_file *psDRMFile);

#endif /* __PVR_DEVICE_H__ */
