/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#include "xdx_smmu.h"
#include "xdx_smmu_drv.h"
#include "xdx_device.h"
#include "mmu_common.h"
#include "osfunc.h"
#include "smmu_platform.h"

// static const struct xdx_smmu_tbu_cfg_s xdx_smmu_b0_vdma_tbus[] = {
// 	{XDXTBU_FLAG_BYPASS, 0},
// 	{0, 1},
// 	{0, 2},
// };

// static const struct xdx_smmu_tcu_cfg_s xdx_smmu_b0_vdma_tcu = {
// 	.flags = XDXTCU_FLAG_CONFIGHOST,
// 	.tbu_cnt = ARRAY_SIZE(xdx_smmu_b0_vdma_tbus),
// 	.tbu_cfgs = xdx_smmu_b0_vdma_tbus,
// 	.base_offset = PANGU_B0_REG_VDMA_MMU_INTERNAL_BASE,
// 	.tcu_idx = VDMA_TCU,
// 	.cb_offset = ARM_VDMA_SMMU_B0_CB_BASE,
// };

// static const struct xdx_smmu_tbu_cfg_s xdx_smmu_b0_xdma_tbus[] = {
// 	{0, 0},
// };

static const struct xdx_smmu_tbu_cfg_s xdx_smmu_b0_tdma_tbus[] = {
	{XDXTBU_FLAG_BYPASS, 0},
	{0, 1},
};
#ifdef SUPPORT_VPU_SMMU
static const struct xdx_smmu_tbu_cfg_s xdx_smmu_b0_vdec_tbus[] = {
	{0, 0},
};

static const struct xdx_smmu_tbu_cfg_s xdx_smmu_b0_venc_tbus[] = {
	{0, 0},
};
#endif
// static const struct xdx_smmu_tcu_cfg_s xdx_smmu_b0_xdma_tcu = {
// 	.flags = XDXTCU_FLAG_CONFIGHOST,
// 	.tbu_cnt = ARRAY_SIZE(xdx_smmu_b0_xdma_tbus),
// 	.tbu_cfgs = xdx_smmu_b0_xdma_tbus,
// 	.base_offset = PANGU_B0_REG_XDMA_MMU_INTERNAL_BASE,
// 	.tcu_idx = XDMA_TCU,
// };

static const struct xdx_smmu_tcu_cfg_s xdx_smmu_b0_tdma_tcu = {
	.flags = XDXTCU_FLAG_CONFIGHOST,
	.tbu_cnt = ARRAY_SIZE(xdx_smmu_b0_tdma_tbus),
	.tbu_cfgs = xdx_smmu_b0_tdma_tbus,
	.base_offset = PANGU_B0_REG_TDMA_MMU_INTERNAL_BASE,
	.tcu_idx = TDMA_TCU,
	.cb_offset = ARM_TDMA_SMMU_B0_CB_BASE,
};

#ifdef SUPPORT_VPU_SMMU
static const struct xdx_smmu_tcu_cfg_s xdx_smmu_b0_vdec_tcu = {
	.flags = XDXTCU_FLAG_CONFIGHOST,
	.tbu_cnt = ARRAY_SIZE(xdx_smmu_b0_vdec_tbus),
	.tbu_cfgs = xdx_smmu_b0_vdec_tbus,
	.base_offset = PANGU_B0_REG_VDEC_MMU_INTERNAL_BASE,
	.tcu_idx = VDEC_TCU,
	.cb_offset = ARM_VDEC_SMMU_B0_CB_BASE,
};

static const struct xdx_smmu_tcu_cfg_s xdx_smmu_b0_venc_tcu = {
	.flags = XDXTCU_FLAG_CONFIGHOST,
	.tbu_cnt = ARRAY_SIZE(xdx_smmu_b0_venc_tbus),
	.tbu_cfgs = xdx_smmu_b0_venc_tbus,
	.base_offset = PANGU_B0_REG_VENC_MMU_INTERNAL_BASE,
	.tcu_idx = VENC_TCU,
	.cb_offset = ARM_VENC_SMMU_B0_CB_BASE,
};
#endif

static const struct xdx_smmu_tcu_cfg_s *xdx_smmu_b0_tcus[] = {
	// &xdx_smmu_b0_xdma_tcu,
	// &xdx_smmu_b0_vdma_tcu,
	&xdx_smmu_b0_tdma_tcu,
#ifdef SUPPORT_VPU_SMMU
	&xdx_smmu_b0_vdec_tcu,
	&xdx_smmu_b0_venc_tcu,
#endif
};

struct xdx_smmu_cfg_s xdx_smmu_b0_cfg = {
	.tcu_cnt = ARRAY_SIZE(xdx_smmu_b0_tcus),
	.tcu_tab = xdx_smmu_b0_tcus,
};

//static const struct xdx_smmu_tbu_cfg_s xdx_smmu_a0_vdma_tbus[] = {
//	{XDXTBU_FLAG_BYPASS, 0},
//	{0, 1},
//	{0, 2},
//};
//static const struct xdx_smmu_tcu_cfg_s xdx_smmu_a0_vdma_tcu = {
//	.flags = XDXTCU_FLAG_CONFIGHOST,
//	.tbu_cnt = ARRAY_SIZE(xdx_smmu_a0_vdma_tbus),
//	.tbu_cfgs = xdx_smmu_a0_vdma_tbus,
//	.base_offset = PANGU_A0_REG_VDMA_MMU_INTERNAL_BASE,
//	.tcu_idx = VDMA_TCU,
//	.cb_offset = ARM_VDMA_SMMU_A0_CB_BASE,
//};

//static const struct xdx_smmu_tbu_cfg_s xdx_smmu_a0_xdma_tbus[] = {
//	{0, 0},
//};

//static const struct xdx_smmu_tcu_cfg_s xdx_smmu_a0_xdma_tcu = {
//	.flags = XDXTCU_FLAG_CONFIGHOST,
//	.tbu_cnt = ARRAY_SIZE(xdx_smmu_a0_xdma_tbus),
//	.tbu_cfgs = xdx_smmu_a0_xdma_tbus,
//	.base_offset = PANGU_A0_REG_XDMA_MMU_INTERNAL_BASE,
//	.tcu_idx = XDMA_TCU,
//};

static const struct xdx_smmu_tcu_cfg_s *xdx_smmu_a0_tcus[] = {
//	&xdx_smmu_a0_xdma_tcu,
//	&xdx_smmu_a0_vdma_tcu,
};

struct xdx_smmu_cfg_s xdx_smmu_a0_cfg = {
	.tcu_cnt = ARRAY_SIZE(xdx_smmu_a0_tcus),
	.tcu_tab = xdx_smmu_a0_tcus,
};

static int _xdx_smmu_dev_init(struct xdx_device *xdev, struct xdx_smmu_cfg_s *cfgs)
{
	int i;
	struct xdx_smmu_s *xsmmu = NULL;
	struct xdx_smmu *smmu = xdev_to_smmu(xdev);
	int ret;

	INIT_LIST_HEAD(&smmu->ctxs_head);
	smmu->xsmmu = NULL;
	if (!cfgs) {
		dev_info(xdev->dev, "Asic Type:%d Not config SMMU.", xdev->asic_type);
		return 0;
	}
	if (!cfgs->tcu_cnt) {
		dev_info(xdev->dev, "No TCU configured in this platform.");
		return 0;
	}
	xsmmu = devm_kcalloc(xdev->dev, 1, sizeof(struct xdx_smmu_s), GFP_KERNEL);
	if (!xsmmu)
		return -ENOMEM;
	xsmmu->xdx_dev = xdev;
	INIT_LIST_HEAD(&xsmmu->ctx_head);
	INIT_LIST_HEAD(&xsmmu->grp_head);
	ret = xdx_smmu_plat_env_init(xdev, xsmmu);
	if (ret)
		goto err0;

	for (i = 0; i < cfgs->tcu_cnt; i++) {
		dev_info(xdev->dev,
			"SMMU init tcu:%d base_offset:%llx", i, cfgs->tcu_tab[i]->base_offset);
		ret = xdx_smmu_tcu_init(xsmmu, cfgs->tcu_tab[i]);
		if (ret < 0)
			goto err1;
	}
	xsmmu->power_stat = XDX_SMMU_PWR_STAT_READY;
	smmu->xsmmu = xsmmu;
	return 0;
err1:
	xdx_smmu_tcus_deinit(xsmmu);
	xdx_smmu_plat_env_deinit(xdev, xsmmu);
err0:
	devm_kfree(xdev->dev, xsmmu);
	return ret;
}

static int xdx_smmu_dev_init(void *handle)
{
	struct xdx_device *xdev = (struct xdx_device *)handle;
	struct xdx_smmu *smmu = xdev_to_smmu(xdev);
	struct xdx_smmu_cfg_s *cfgs = NULL;
	int ret;

	switch (xdev->asic_type) {
	case CHIP_PANGU_B0:
		cfgs = &xdx_smmu_b0_cfg;
		break;
	case CHIP_PANGU_A0:
		cfgs = &xdx_smmu_a0_cfg;
		break;
	default:
		break;
	}
	ret = _xdx_smmu_dev_init(xdev, cfgs);
	if (ret != 0) {
		dev_err(xdev->dev, "SMMU dev init failed. (%d)", ret);
		return 0;
	}
	ret = xdx_smmu_plat_init(xdev);
	if (ret != 0) {
		dev_err(xdev->dev, "SMMU plat init failed. (%d)", ret);
		return 0;
	}
	smmu->stat = XDX_SMMU_STAT_INITED;
	dev_info(xdev->dev, "SMMU dev init success.");
	return 0;
}

static int xdx_smmu_dev_deinit(void *handle)
{
	struct xdx_device *xdev = (struct xdx_device *)handle;
	struct xdx_smmu_s *xsmmu = xdev_to_smmu_dev(xdev);
	struct xdx_smmu *smmu = xdev_to_smmu(xdev);

	if (!xsmmu)
		return 0;
	xdx_smmu_tcus_deinit(xsmmu);
	xdx_smmu_plat_env_deinit(xdev, xsmmu);
	devm_kfree(xdev->dev, xsmmu);
	smmu->xsmmu = NULL;
	return 0;
}

static int xdx_smmu_dev_suspend(void *handle)
{
	int ret;
	struct xdx_device *xdev = (struct xdx_device *)handle;
	struct xdx_smmu *smmu = xdev_to_smmu(xdev);

	if (!xdx_smmu_is_inited(smmu))
		return 0;

	smmu->stat |= XDX_SMMU_STAT_INSUSPEND;

	ret = xdx_smmu_suspend(xdev_to_smmu_dev(xdev));
	if (ret < 0) {
		dev_err(xdev->dev, "SMMU suspend failed.(%d)", ret);
		return ret;
	}

	return 0;
}

static int xdx_smmu_dev_resume(void *handle)
{
	int ret;
	struct xdx_device *xdev = (struct xdx_device *)handle;
	struct xdx_smmu *smmu = xdev_to_smmu(xdev);

	if (!xdx_smmu_is_inited(smmu))
		return 0;

	ret = xdx_smmu_resume(xdev_to_smmu_dev(xdev));
	if (ret < 0) {
		dev_err(xdev->dev, "SMMU resume failed.(%d)", ret);
		return ret;
	}

	ret = xdx_smmu_plat_resume(xdev);
	if (ret < 0) {
		dev_err(xdev->dev, "SMMU plat resume failed.(%d)", ret);
		return ret;
	}

	smmu->stat &= ~XDX_SMMU_STAT_INSUSPEND;
	return 0;
}

static int xdx_smmu_dev_dummy_func(void *handle)
{
	return 0;
}

const struct xdx_ip_block_version smmu_ip_block_pangu = {
	.name = "smmu",
	.type = XDX_IP_BLOCK_TYPE_SMMU,
	.major = 1,
	.minor = 0,
	.rev = 0,
	.funcs = {
		.suspend = xdx_smmu_dev_suspend,
		.resume = xdx_smmu_dev_resume,
		.early_init = xdx_smmu_dev_dummy_func,
		.early_fini = xdx_smmu_dev_dummy_func,
		.sw_init = xdx_smmu_dev_init,
		.sw_fini = xdx_smmu_dev_deinit,
		.hw_init = xdx_smmu_dev_dummy_func,
		.hw_fini = xdx_smmu_dev_dummy_func,
	},
};

