/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/version.h>
#include <linux/vmalloc.h>

#include "hantro_axife.h"
#include "hantro_device.h"

#define AXIFE_0_IO_BASE                                                       \
	0x1b03000 //the base addr of core. for PCIE, it refers to offset from bar
#define AXIFE_1_IO_BASE                                                       \
	0x1b03000 //the base addr of core. for PCIE, it refers to offset from bar
#define AXIFE_2_IO_BASE (0xc0000000 + 0x10000)
#define AXIFE_3_IO_BASE (0xd0000000 + 0x10000)
#define AXIFE_4_IO_BASE                                                       \
	0xFFFF //the base addr of core. for PCIE, it refers to offset from bar
#define AXIFE_5_IO_BASE                                                       \
	0x605000 //the base addr of core. for PCIE, it refers to offset from bar

#define AXIFE_6_IO_BASE 0x1a00200
#define AXIFE_6_IO_BASE_B0 0x1a00400

#define AXIFE_7_IO_BASE (0xd0000000 + 0x10000)

#define AXIFE_0_IO_SIZE (500 * 4) /* bytes, cache register size */
#define AXIFE_1_IO_SIZE (500 * 4) /* bytes, shaper register size */
#define AXIFE_2_IO_SIZE (500 * 4) /* bytes, cache register size */
#define AXIFE_3_IO_SIZE (500 * 4) /* bytes, shaper register size */
#define AXIFE_4_IO_SIZE (500 * 4) /* bytes, cache register size */
#define AXIFE_5_IO_SIZE (500 * 4) /* bytes, shaper register size */
#define AXIFE_6_IO_SIZE (500 * 4) /* bytes, cache register size */
#define AXIFE_7_IO_SIZE (500 * 4) /* bytes, shaper register size */

#ifndef USE_DTB_PROBE
static struct axife_core_cfg axifecores[] = {
#ifdef HAS_VC8000E
	{ AXIFE_0_IO_BASE, AXIFE_0_IO_SIZE, 0, 0x1b01000}, //enc 0
#endif
#ifdef HAS_VC8000D
	{ AXIFE_6_IO_BASE, 64*4, 0, 0x1a00800}, //dec
#endif
};

#endif

static int axifeprobed;
long AxifeReadRegs(struct axife_t *dev, struct core_desc *core)
{
	u32 i;
	long ret = 0;

	i = core->reg_id;
	/* user has to know exactly what they are asking for */

	/* read specific registers from hardware */
	for (i = core->reg_id; i < core->reg_id + core->size / 4; i++)
		dev->dec_regs[i] = ioread32((void *)dev->hwregs + i * 4);

	/* put registers to user space*/
	ret = copy_to_user(core->regs, dev->dec_regs + core->reg_id,
			   core->size);
	if (ret) {
		XDX_DEV_ERROR("copy_to_user failed, returned %li\n", ret);
		return -EFAULT;
	}
	return 0;
}

long AxifeWriteRegs(struct axife_t *dev, struct core_desc *core)
{
	u32 i;
	long ret = 0;

	i = core->reg_id;
	ret = copy_from_user(dev->dec_regs + core->reg_id,
			     core->regs, core->size);
	if (ret) {
		XDX_DEV_ERROR("copy_from_user failed, returned %li\n", ret);
		return -EFAULT;
	}
	for (i = core->reg_id; i < core->reg_id + core->size / 4; i++)
		iowrite32(dev->dec_regs[i], (void *)dev->hwregs + i * 4);

	return 0;
}

void AXIFEEnable(volatile unsigned char *hwregs)
{
	if (!hwregs)
		return;

	//AXI FE pass through
	iowrite32(0x0, (void *)(hwregs + 0x2C));
	XDX_DEV_DEBUG("AXI FE: 0x2C = 0x%x\n",
		ioread32((void *)(hwregs + 0x2C)));
	iowrite32(0x2, (void *)(hwregs + 0x28));
	XDX_DEV_DEBUG("AXI FE: 0x28 = 0x%x\n",
		ioread32((void *)(hwregs + 0x28)));
}

#if 0
int hantro_axife_probe(dtbnode *pnode, int loop, struct axife_t *axifecore)
{
	struct axife_t *paxife;

	if (loop == 0) {
#ifndef USE_DTB_PROBE
		unsigned int i;

		for (i = 0; i < ARRAY_SIZE(axifecores); i++) {
			paxife = vmalloc(sizeof(*paxife));
			if (!paxife)
				break;
			paxife->core_cfg = axifecores[i];
			add_axifenode(axifecores[i].sliceidx, paxife);
		}

#else //ndef USE_DTB_PROBE
		{
			paxife = vmalloc(sizeof(*paxife));
			if (!paxife)
				return -ENOMEM;
			paxife->core_cfg.axifecorebase = pnode->ioaddr;
			paxife->core_cfg.iosize = pnode->iosize;
			paxife->core_cfg.sliceidx = pnode->sliceidx;
			paxife->core_cfg.parentaddr = pnode->parentaddr;

			add_axifenode(pnode->sliceidx, paxife);
		}
#endif
	} else {
		if (!request_mem_region(axifecore->core_cfg.axifecorebase,
					axifecore->core_cfg.iosize,
					"hantroaxife")) {
			XDX_DEV_ERROR("axife: HW regs busy\n");
			return -ENODEV;
		}
		axifecore->hwregs =
			(u8 *)ioremap_wc(axifecore->core_cfg.axifecorebase,
				      axifecore->core_cfg.iosize);
		if (!axifecore->hwregs) {
			release_mem_region(axifecore->core_cfg.axifecorebase,
					   axifecore->core_cfg.iosize);
			XDX_DEV_ERROR("axife: failed to map HW regs\n");
			return -ENODEV;
		}
		axifecore->dec_regs =
			vmalloc(axifecore->core_cfg.iosize);
		if (!axifecore->dec_regs)
			return -ENOMEM;

		AXIFEEnable(axifecore->hwregs);
	}

	return 0;
}
#else
int hantro_axife_probe(struct xdx_device *xdev)
{
	unsigned int i;
	struct axife_t *paxife;

	for (i = 0; i < ARRAY_SIZE(axifecores); i++) {
		paxife = vmalloc(sizeof(*paxife));
		if (!paxife)
			break;

		paxife->core_cfg = axifecores[i];

		if (!request_mem_region(xdev->rmmio_base + paxife->core_cfg.axifecorebase,
						paxife->core_cfg.iosize,
						"hantroaxife")) {
			XDX_DEV_ERROR("axife: HW regs busy\n");
			return -ENODEV;
		}

		paxife->hwregs =
			(u8 *)ioremap_wc(xdev->rmmio_base + paxife->core_cfg.axifecorebase,
					paxife->core_cfg.iosize);
		if (!paxife->hwregs) {
			release_mem_region(xdev->rmmio_base + paxife->core_cfg.axifecorebase,
					paxife->core_cfg.iosize);
			XDX_DEV_ERROR("axife: failed to map HW regs\n");
			return -ENODEV;
		}

		add_axifenode(xdev, axifecores[i].sliceidx, paxife);
	}

	return 0;
}
#endif // #if 0

void hantro_axife_cleanup(struct xdx_device *xdev)
{
	int i, slicen = get_slicenumber(xdev);
	struct axife_t *dev, *pp;

	for (i = 0; i < slicen; i++) {
		dev = get_axifenodes(xdev, i, 0);
		while (dev) {
			if (dev->hwregs)
				release_mem_region(xdev->rmmio_base + dev->core_cfg.axifecorebase,
						   dev->core_cfg.iosize);
			if (dev->dec_regs)
				vfree(dev->dec_regs);
			pp = dev->next;
			vfree(dev);
			dev = pp;
		}
	}
	axifeprobed = 0;
}

int hantroaxife_init(struct xdx_device *xdev)
{
	unsigned int i;

	for (i = 0; i < ARRAY_SIZE(axifecores); i++) {
		if (axifecores[i].parentaddr == 0x1a00800) {        //dec
			if (hantro_dev.asic_type == CHIP_PANGU_B0) {
				axifecores[i].axifecorebase = AXIFE_6_IO_BASE_B0;
			}
		}
	}

	axifeprobed = 0;
	return 0;
}
