/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#ifndef __HANTRO_FPGA_PCIE__
#define __HANTRO_FPGA_PCIE__

#include "hantro_priv.h"

#define PCI_VENDOR_ID_HANTRO 0x10ee // 0x16c3
#define PCI_DEVICE_ID_HANTRO_PCI 0x8014 // 0x7011

/* Base address got control register */
#define PCI_H2_BAR 4

/* Base address DDR register */
#define PCI_DDR_BAR 0

/* hantro linear memory chunk */
struct hlinc {
	unsigned long bus_address;
	u16 chunks_reserved;
	//const struct file *filp; /* Client that allocated this chunk */
	u16 filp; /* as a flag only */
};

struct hantro_fpga_t {
	struct pci_dev *dev; /* PCI device structure. */

	/* PCI base register address of HW cores (Hardware address) */
	unsigned long reg_base;
	u32 reg_len; /* core registers address Length */

	/* PCI base register address of on board DDR (Hardware address) */
	unsigned long ddr_base;
	u32 ddr_len; /* on board DDR address Length */

	unsigned int alloc_size;
	unsigned long alloc_base;
	/* user space SW will subtract HLINA_TRANSL_OFFSET from the bus address
	 * and decoder HW will use the result as the address translated base
	 * address. The SW needs the original host memory bus address for memory
	 * mapping to virtual address.
	 */
	unsigned long addr_transl;

	/* memory part */
	spinlock_t mem_lock;
	struct hlinc *hlina_chunks;
	size_t chunks;
};

struct hantro_pci_t {
	/* PCI base register address (Hardware address) */
	unsigned long pci_base_reg_hw;
	/* PCI base register address (memalloc) */
	unsigned long pci_base_ddr_hw;
	u32 pci_base_reg_len; /* Base register address Length */
	u32 pci_base_ddr_len; /* Base register address Length */
};

/* API */

#define hantro_fpga_memalloc hantro_dev_memalloc
#define hantro_fpga_memfree hantro_dev_memfree

int hantro_fpga_memalloc(struct hantro_mem_handle *phandle, unsigned int size);
int hantro_fpga_memfree(struct hantro_mem_handle *phandle);
int hantro_fpga_release(void);
int hantro_fpga_init(unsigned long ddr_base, unsigned long offset);
int pcie_init(struct xdx_device* xdev, struct hantro_pci_t *pci_par);
u64 vpu_alloc_gddr(struct xdx_device *xdev, struct xdx_bo **bo, unsigned long size);

#endif /* __HANTRO_FPGA_PCIE__ */
