/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#include "hantro_priv.h"
#include "hantrodec.h"
#include "hx280enc.h"

static ssize_t bandwidthDecRead_show(struct device *kdev,
				     struct device_attribute *attr, char *buf)
{
	/* sys/bus/platform/drivers/hantro/xxxxx.vpu/bandwidthDecRead
	 *  used to show bandwidth info to user space
	 *  all core' bandwidth might be exported in same string.
	 *  data is just an example. Real data should be read from HW registers
	 *  this file is read only.
	 */
	u32 bandwidth;
	int sliceidx = findslice_bydev(kdev);
	struct drm_device *ddev = (struct drm_device *)kdev->driver_data;
	struct xdx_device *xdev = drm_to_xdev(ddev);

	if (sliceidx < 0)
		return 0;
	bandwidth = hantrodec_readbandwidth(xdev, sliceidx, 1);
	return snprintf(buf, PAGE_SIZE, "%u\n", bandwidth);
}

static ssize_t bandwidthDecWrite_show(struct device *kdev,
				      struct device_attribute *attr, char *buf)
{
	u32 bandwidth;
	int sliceidx = findslice_bydev(kdev);
	struct drm_device *ddev = (struct drm_device *)kdev->driver_data;
	struct xdx_device *xdev = drm_to_xdev(ddev);

	if (sliceidx < 0)
		return 0;
	bandwidth = hantrodec_readbandwidth(xdev, sliceidx, 0);
	return snprintf(buf, PAGE_SIZE, "%u\n", bandwidth);
}

static ssize_t bandwidthEncRead_show(struct device *kdev,
				     struct device_attribute *attr, char *buf)
{
	u32 bandwidth;
	int sliceidx = findslice_bydev(kdev);
	struct drm_device *ddev = (struct drm_device *)kdev->driver_data;
	struct xdx_device *xdev = drm_to_xdev(ddev);

	if (sliceidx < 0)
		return 0;
	bandwidth = hantroenc_readbandwidth(xdev, sliceidx, 1);
	return snprintf(buf, PAGE_SIZE, "%u\n", bandwidth);
}

static ssize_t bandwidthEncWrite_show(struct device *kdev,
				      struct device_attribute *attr, char *buf)
{
	u32 bandwidth;
	int sliceidx = findslice_bydev(kdev);
	struct drm_device *ddev = (struct drm_device *)kdev->driver_data;
	struct xdx_device *xdev = drm_to_xdev(ddev);

	if (sliceidx < 0)
		return 0;
	bandwidth = hantroenc_readbandwidth(xdev, sliceidx, 0);
	return snprintf(buf, PAGE_SIZE, "%u\n", bandwidth);
}

int hantro_createsysfsAPI(int sliceidx, struct device *dev)
{
	int result;
	char APIname[16] = { 0 };
	struct device_attribute APIattr = {
		.attr = {
				.mode = 0444,
			},
		.store = NULL,
	};
	memcpy(APIname, "BWDecRead", 9);
	APIname[9] = sliceidx + 0x30;
	APIattr.attr.name = APIname;
	APIattr.show = bandwidthDecRead_show;
	result = device_create_file(dev, &APIattr);
	if (result != 0)
		return result;

	memcpy(APIname, "BWEncRead", 9);
	APIname[9] = sliceidx + 0x30;
	APIattr.attr.name = APIname;
	APIattr.show = bandwidthEncRead_show;
	result = device_create_file(dev, &APIattr);
	if (result != 0)
		return result;

	memcpy(APIname, "BWDecWrite", 10);
	APIname[10] = sliceidx + 0x30;
	APIattr.attr.name = APIname;
	APIattr.show = bandwidthDecWrite_show;
	result = device_create_file(dev, &APIattr);
	if (result != 0)
		return result;

	memcpy(APIname, "BWEncWrite", 10);
	APIname[10] = sliceidx + 0x30;
	APIattr.attr.name = APIname;
	APIattr.show = bandwidthEncWrite_show;
	result = device_create_file(dev, &APIattr);

	return result;
}

int hantro_unlinksysfsAPI(struct xdx_device *xdev)
{
	int i;
	char APIname[16] = { 0 };
	struct device_attribute APIattr = {
		.attr = {
				.mode = 0444,
			},
		.store = NULL,
	};

	for (i = 0; i < get_slicenumber(xdev); i++) {
		struct slice_info *pslice = getslicenode(xdev, i);

		memcpy(APIname, "BWDecRead", 9);
		APIname[9] = i + 0x30;
		APIattr.attr.name = APIname;
		APIattr.show = bandwidthDecRead_show;
		device_remove_file(pslice->dev, &APIattr);

		memcpy(APIname, "BWEncRead", 9);
		APIname[9] = i + 0x30;
		APIattr.attr.name = APIname;
		APIattr.show = bandwidthEncRead_show;
		device_remove_file(pslice->dev, &APIattr);

		memcpy(APIname, "BWDecWrite", 10);
		APIname[10] = i + 0x30;
		APIattr.attr.name = APIname;
		APIattr.show = bandwidthDecWrite_show;
		device_remove_file(pslice->dev, &APIattr);

		memcpy(APIname, "BWEncWrite", 10);
		APIname[10] = i + 0x30;
		APIattr.attr.name = APIname;
		APIattr.show = bandwidthEncWrite_show;
		device_remove_file(pslice->dev, &APIattr);
	}
	return 0;
}
