/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#ifndef HANTRO_PRIV_H
#define HANTRO_PRIV_H
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 10, 0)
#include <drm/drm_print.h>
#endif
#include "hantro.h"
#include "hantro_device.h"
#include "drm/xdxgpu_drm.h"

#include "xdx_device.h"
#include "xdx_gem.h"

/* compile options */
#ifndef USE_HW
#define USE_HW 1
#endif
#define USE_IRQ

#define HANTRO_GEM_FLAG_IMPORT BIT(0)
#define HANTRO_GEM_FLAG_EXPORT BIT(1)
#define HANTRO_GEM_FLAG_EXPORTUSED BIT(2)
#define HANTRO_GEM_FLAG_USEVMALLOC BIT(3)

#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 0, 0) || (defined(REDHAT_83)) || (defined(XDXGPU_CENTOS85_SUPPORT)) || (defined(XDXGPU_CENTOS82_SUPPORT))
#define hantro_access_ok(a, b, c) access_ok(b, c)
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 19, 0))
#define hantro_reserve_obj_shared(a, b) dma_resv_reserve_fences(a, b)
#else
#define hantro_reserve_obj_shared(a, b) dma_resv_reserve_shared(a, b)
#endif
#define hantro_ref_drmobj drm_gem_object_get
//#define hantro_unref_drmobj drm_gem_object_put
#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 9, 0) && (!defined(XDXGPU_CENTOS85_SUPPORT))
#define hantro_unref_drmobj drm_gem_object_put_unlocked
#else
#define hantro_unref_drmobj drm_gem_object_put
#endif 
#else /*KERNEL_VERSION(5, 0, 0) <= LINUX_VERSION_CODE*/
#if (defined(KYLIN_SERVER))
#define hantro_access_ok(a, b, c) access_ok(b, c)
#else
//#define hantro_access_ok(a, b, c) access_ok(a, b, c)//UBUNTU,UOS
#define hantro_access_ok(a, b, c) access_ok(b, c)//UBUNTU,UOS
#endif
#define hantro_reserve_obj_shared(a, b) reservation_object_reserve_shared(a)
#define hantro_ref_drmobj drm_gem_object_reference
#define hantro_unref_drmobj drm_gem_object_unreference_unlocked
#endif

#define NODENAME_DECODER "decoder"
#define NODENAME_ENCODER "encoder"
#define NODENAME_CACHE "cache"
#define NODENAME_DEC400 "dec400"
#define NODENAME_AXIFE "axife"
#define NODENAME_MMU "hantrommu"
#define NODENAME_VCMD "hantrovcmd"
#define NODENAME_IM "hantroIM"
#define NODENAME_DECJPG "decjpg"
#define NODENAME_ENCJPG "encjpg"

typedef struct dtbnode {
	struct device_node *ofnode;
	int type;
	phys_addr_t ioaddr;
	phys_addr_t iosize;
	int irq[4];
	int parenttype;
	phys_addr_t parentaddr;
	int sliceidx;
	struct dtbnode *next;
} dtbnode;

struct xdxvpu_dev_info {
	u32 sub_module_type;
	u64		vcmd_reg_base;
	void	*vcmd_reg_vaddr;
	u32		vcmd_reg_size;

	u64		submodule_main_addr;
	void	*submodule_main_vaddr;
	u32		submodule_main_size;
};

struct hantro_device_handle {
	struct platform_device *platformdev; /* parent device */
	struct drm_device *drm_dev;
	u32 config;
	enum xdx_asic_type asic_type;
};

struct hantro_mem_handle {
	unsigned int sliceidx;
	unsigned int size;
	dma_addr_t paddr;
	dma_addr_t mem_base;
	void *vaddr;
};

extern struct hantro_device_handle hantro_dev;
extern const struct dma_buf_ops hantro_dmabuf_ops;
extern struct drm_driver hantro_drm_driver;
extern struct xdxvpu_dev_info xdxvpu_dev[3];
extern int dbg_mmap;
#define hantro_mmaplog(fmt, ...) {\
	if (dbg_mmap)	\
		pr_info(fmt, ##__VA_ARGS__);	\
}

#if KERNEL_VERSION(4, 13, 0) > LINUX_VERSION_CODE && (!defined(XDXGPU_CENTOS79_SUPPORT))

#define HANTRO_FENCE_FLAG_ENABLE_SIGNAL_BIT FENCE_FLAG_ENABLE_SIGNAL_BIT
#define HANTRO_FENCE_FLAG_SIGNAL_BIT FENCE_FLAG_SIGNALED_BIT

typedef struct fence hantro_fence_t;
typedef struct fence_ops hantro_fence_op_t;

static inline signed long
hantro_fence_default_wait(hantro_fence_t *fence, bool intr, signed long timeout)
{
	return fence_default_wait(fence, intr, timeout);
}

static inline void hantro_fence_init(hantro_fence_t *fence,
				     const hantro_fence_op_t *ops,
				     spinlock_t *lock, unsigned int context,
				     unsigned int seqno)
{
	return fence_init(fence, ops, lock, context, seqno);
}

static inline unsigned int hantro_fence_context_alloc(unsigned int num)
{
	return fence_context_alloc(num);
}

static inline signed long
hantro_fence_wait_timeout(hantro_fence_t *fence, bool intr, signed long timeout)
{
	return fence_wait_timeout(fence, intr, timeout);
}

static inline struct drm_gem_object *
hantro_gem_object_lookup(struct drm_device *dev, struct drm_file *filp,
			 u32 handle)
{
#if KERNEL_VERSION(4, 10, 0) > LINUX_VERSION_CODE
	return drm_gem_object_lookup(filp, handle);
#else
	return drm_gem_object_lookup(dev, filp, handle);
#endif
}

static inline void hantro_fence_put(hantro_fence_t *fence)
{
	return fence_put(fence);
}

static inline int hantro_fence_signal(hantro_fence_t *fence)
{
	return fence_signal(fence);
}

static inline void ref_page(struct page *pp)
{
#if KERNEL_VERSION(4, 10, 0) > LINUX_VERSION_CODE
	atomic_inc(&pp->_refcount);
#else
	atomic_inc(&pp->_count);
#endif
}

static inline void unref_page(struct page *pp)
{
#if KERNEL_VERSION(4, 10, 0) > LINUX_VERSION_CODE
	atomic_dec(&pp->_refcount);
#else
	atomic_dec(&pp->_count);
#endif
}

static inline bool hantro_fence_is_signaled(hantro_fence_t *fence)
{
	return fence_is_signaled(fence);
}

#else /*version higher */
#define HANTRO_FENCE_FLAG_ENABLE_SIGNAL_BIT DMA_FENCE_FLAG_ENABLE_SIGNAL_BIT
#define HANTRO_FENCE_FLAG_SIGNAL_BIT DMA_FENCE_FLAG_SIGNALED_BIT

typedef struct dma_fence hantro_fence_t;
typedef struct dma_fence_ops hantro_fence_op_t;

static inline signed long
hantro_fence_default_wait(hantro_fence_t *fence, bool intr, signed long timeout)
{
	return dma_fence_default_wait(fence, intr, timeout);
}

static inline void hantro_fence_init(hantro_fence_t *fence,
				     const hantro_fence_op_t *ops,
				     spinlock_t *lock, unsigned int context,
				     unsigned int seqno)
{
	dma_fence_init(fence, ops, lock, context, seqno);
}

static inline unsigned int hantro_fence_context_alloc(unsigned int num)
{
	return dma_fence_context_alloc(num);
}

static inline signed long
hantro_fence_wait_timeout(hantro_fence_t *fence, bool intr, signed long timeout)
{
	return dma_fence_wait_timeout(fence, intr, timeout);
}

static inline struct drm_gem_object *
hantro_gem_object_lookup(struct drm_device *dev, struct drm_file *filp,
			 u32 handle)
{
	return drm_gem_object_lookup(filp, handle);
}

static inline void hantro_fence_put(hantro_fence_t *fence)
{
	dma_fence_put(fence);
}

static inline int hantro_fence_signal(hantro_fence_t *fence)
{
	return dma_fence_signal(fence);
}

static inline void ref_page(struct page *pp)
{
#if KERNEL_VERSION(4, 10, 0) > LINUX_VERSION_CODE
        atomic_inc(&pp->_count);
#else
        atomic_inc(&pp->_refcount);
        atomic_inc(&pp->_mapcount);
#endif
}

static inline void unref_page(struct page *pp)
{
#if KERNEL_VERSION(4, 10, 0) > LINUX_VERSION_CODE
        atomic_inc(&pp->_count);
#else
        atomic_dec(&pp->_refcount);
        atomic_dec(&pp->_mapcount);
#endif
}

static inline bool hantro_fence_is_signaled(hantro_fence_t *fence)
{
	return dma_fence_is_signaled(fence);
}

#endif

static inline struct drm_gem_hantro_object *
to_drm_gem_hantro_obj(struct drm_gem_object *gem_obj)
{
	struct xdx_bo *bo;

	bo = to_xdx_gem(gem_obj);
	return bo->cma_obj;

//	return container_of(gem_obj, struct drm_gem_hantro_object, base);
}

static inline struct xdx_device *get_file_xdev(struct file *filp)
{
	struct drm_file *file_priv = filp->private_data;
	struct drm_device *ddev = file_priv->minor->dev;

	return drm_to_xdev(ddev);
}

struct drm_gem_object *
hantro_get_gem_from_dmabuf(struct dma_buf *dma_buf);

int hantro_setdomain(struct drm_device *dev, void *data,
		     struct drm_file *file_priv);
int hantro_acquirebuf(struct drm_device *dev, void *data,
		      struct drm_file *file_priv);
int hantro_testbufvalid(struct drm_device *dev, void *data,
			struct drm_file *file_priv);
int hantro_releasebuf(struct drm_device *dev, void *data,
		      struct drm_file *file_priv);
int init_hantro_resv(struct dma_resv *presv,
		     struct drm_gem_hantro_object *cma_obj);
void initFenceData(void);
void releaseFenceData(void);

int hantro_memalloc(struct hantro_mem_handle *phandle);
int hantro_memfree(struct hantro_mem_handle *phandle);

int hantro_createsysfsAPI(int sliceidx, struct device *dev);
int hantro_unlinksysfsAPI(struct xdx_device *xdev);
int hantro_get_vcmdsup(struct drm_device *dev, void *data,
		       struct drm_file *file_priv);
int hantro_map_internal_address(struct file *filp, struct vm_area_struct *vma);

#define VPU_SHARE_PAGETABLE 1

/* debug */
#define ENABLE_DEBUG
#ifdef ENABLE_DEBUG
#define DBG(...) pr_info(__VA_ARGS__)
#else
#define DBG(...)
#endif

#define VPU_DEV	(hantro_dev.drm_dev->dev)
#ifdef XDX_DRM_DEBUG
#define XDX_DEV_DEBUG(fmt,...)	DRM_DEV_DEBUG(VPU_DEV,fmt,##__VA_ARGS__)
#define XDX_DEBUG(fmt,...)		DRM_DEBUG(fmt,##__VA_ARGS__)

#define XDX_DEV_INFO(fmt,...)	DRM_DEV_INFO(VPU_DEV,fmt,##__VA_ARGS__)
#define XDX_INFO(fmt,...)		DRM_INFO(fmt,##__VA_ARGS__)

#define XDX_DEV_ERROR(fmt,...)	DRM_DEV_ERROR(VPU_DEV,fmt,##__VA_ARGS__)
#define XDX_ERROR(fmt,...)		DRM_ERROR(fmt,##__VA_ARGS__)

#else
#define XDX_DEV_DEBUG(fmt,...)	dev_dbg(VPU_DEV,"[%s]%d "fmt,__func__,__LINE__,##__VA_ARGS__)
#define XDX_DEBUG(fmt,...)		pr_dbg(fmt,##__VA_ARGS__)

#define XDX_DEV_INFO(fmt,...)	dev_info(VPU_DEV,fmt,##__VA_ARGS__)
#define XDX_INFO(fmt,...)		pr_info(fmt,##__VA_ARGS__)

#define XDX_DEV_ERROR(fmt,...)	dev_err(VPU_DEV,"[%s]%d "fmt,__func__,__LINE__,##__VA_ARGS__)
#define XDX_ERROR(fmt,...)		pr_err(fmt,##__VA_ARGS__)

#endif

#define SLICE_XDEV(a) ((struct xdx_device *)(a->xdev))

#define VPU_HW_STAT
#ifdef VPU_HW_STAT
struct _vpu_stat {
	u64 enc_stat;
	u64 dec_stat;
	u64 vcmd_stat;
	u64 vcmd_enc_stat;
	u64 vcmd_dec_stat;
};
struct _vpu_irq_stat {
	u64 irq_none;
	u64 irq_noirq;
	u64 irq_reset;
	u64 irq_abort;
	u64 irq_buserr;
	u64 irq_timeout;
	u64 irq_cmderr;
	u64 irq_endcmd;
	u64 irq_cmdid;
};
extern struct _vpu_irq_stat vpu_irq_stat;
extern struct _vpu_stat vpu_stat;
#endif

#endif /*HANTRO_PRIV_H*/
