/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#include <linux/vmalloc.h>
#include <linux/spinlock.h>
#include <linux/shmem_fs.h>
#include <linux/dma-buf.h>
#include <linux/kernel.h>
#include <linux/pfn_t.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 5, 0)) || (defined(REDHAT_83)) || (defined(XDXGPU_CENTOS85_SUPPORT))
#include <drm/drm_device.h>
#include <drm/drm_file.h>
#else
#include <drm/drmP.h>
#endif
#include <drm/drm_cache.h>
#include <drm/xdxgpu_drm.h>

#include "../gfx/include/img_types.h"
#include "../gfx/include/pvrsrv_error.h"
#include "../gfx/services/server/include/pmr.h"
#include "../gfx/services/server/devices/volcanic/rgxfwutils.h"
#include "../gfx/services/server/include/pvrsrv_device.h"
#include "../gfx/services/server/include/device.h"
#include "../platform/smmu_platform.h"
#include "pangu.h"
#include "xdx_device.h"
#include "xdx_drv.h"
#include "xdx_gem.h"
#include "xdx_drm.h"
#include "xdx_smmu.h"
#include "vpu_smmu.h"
#include "pvr_gem.h"
#include "hantro_priv.h"
#include "hantro_fpga.h"

#ifdef SUPPORT_VPU_SMMU
int vpu_smmu_map(struct xdx_device *xdev, struct xdx_bo *bo,unsigned long long* vaddr)
{
    int eError;

	eError = XdxSMMUComAllocAndMap(xdev, bo->psPMR, vaddr);
	if (PVRSRV_OK != eError)
    {
        printk("%s XdxSMMUComAllocAndMap hHostPMR Failed!", __func__);
        return eError;
	}

	return 0;
}


long vpu_smmu_ioctl(unsigned int cmd, struct file *filp, unsigned long arg)
{
	struct drm_file *file_priv = filp->private_data;
	struct drm_device *ddev = file_priv->minor->dev;
	struct vpu_smmu_addr smmu_addr;
	
	if (copy_from_user(&smmu_addr, (void *)arg, sizeof(struct vpu_smmu_addr)))
		return -EFAULT;
	
	switch (cmd) {

	case VPU_IOCS_SMMU_MEM_MAP:
		vpu_smmu_ioctl_map(ddev, &smmu_addr, file_priv);
		break;
	case VPU_IOCS_SMMU_MEM_UNMAP:
		vpu_smmu_ioctl_unmap(ddev,&smmu_addr, file_priv);
		break;
	case VPU_IOCS_SMMU_FLUSH:
		return 0;
	default:
		return -EINVAL;
	}
	if (copy_to_user((void *)arg, &smmu_addr, sizeof(struct vpu_smmu_addr))) {
		XDX_DEV_ERROR("copy_to_user failed\n");
		return -EFAULT;
	}
    
	return 0;
}

int vpu_smmu_ioctl_unmap(struct drm_device *ddev,void * data, struct drm_file *filp)
{
    return 0;
}

int vpu_smmu_ioctl_map(struct drm_device *ddev,void * data, struct drm_file *filp)
{
	struct xdx_device *xdev = drm_to_xdev(ddev);
	struct vpu_smmu_addr *args = data;
	uint32_t handle = args->handle;
    uint32_t offset = args->offset;
	struct xdx_bo *bo;
    int eError;
	unsigned long long vaddr =0;
	
	struct drm_gem_object *gobj = drm_gem_object_lookup(filp, handle);
	if (gobj == NULL) {
		XDX_DEV_ERROR("%s: wrong handle 0x%x\n", __func__, handle);
		return -ENOENT;
	}

	bo = to_xdx_gem(gobj);  
	eError = XdxSMMUComAllocAndMap(xdev, bo->psPMR, &vaddr);
    if (PVRSRV_OK != eError)
    {
        XDX_DEV_ERROR("%s XdxSMMUComAllocAndMap hHostPMR Failed!", __func__);
        return eError;
    }
    args->smmu_va = vaddr + offset;

#if LINUX_VERSION_CODE<KERNEL_VERSION(5, 9, 0)
	drm_gem_object_put_unlocked(gobj);
#else
	drm_gem_object_put(gobj);
#endif

    return 0;
}
#endif
