/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#include <drm/xdxgpu_drm.h>
#include <linux/dma-fence-array.h>
#include <linux/pm_runtime.h>
#include <drm/drm_prime.h>

#include "xdx_dmabuf.h"
#include "xdx_device.h"

/**
 * xdx_dmabuf_attach - &dma_buf_ops.attach implementation
 *
 * @dmabuf: DMA-buf where we attach to
 * @attach: attachment to add
 *
 * Add the attachment as user to the exported DMA-buf.
 */
static int xdx_dmabuf_attach(struct dma_buf *dmabuf,
			      struct dma_buf_attachment *attach)
{
	/* TODO: */
	return -1;
}

/**
 * xdx_dmabuf_detach - &dma_buf_ops.detach implementation
 *
 * @dmabuf: DMA-buf where we remove the attachment from
 * @attach: the attachment to remove
 *
 * Called when an attachment is removed from the DMA-buf.
 */
static void xdx_dmabuf_detach(struct dma_buf *dmabuf,
			       struct dma_buf_attachment *attach)
{
	/* TODO: */
}

/**
 * xdx_dmabuf_pin - &dma_buf_ops.pin implementation
 *
 * @attach: attachment to pin down
 *
 * Pin the BO which is backing the DMA-buf so that it can't move any more.
 */
static int xdx_dmabuf_pin(struct dma_buf_attachment *attach)
{
	/* TODO: */
	return -1;
}

/**
 * xdx_dmabuf_unpin - &dma_buf_ops.unpin implementation
 *
 * @attach: attachment to unpin
 *
 * Unpin a previously pinned BO to make it movable again.
 */
static void xdx_dmabuf_unpin(struct dma_buf_attachment *attach)
{
	/* TODO: */
}

/**
 * xdx_dmabuf_map - &dma_buf_ops.map_dma_buf implementation
 * @attach: DMA-buf attachment
 * @dir: DMA direction
 *
 * Makes sure that the shared DMA buffer can be accessed by the target device.
 * For now, simply pins it to the GTT domain, where it should be accessible by
 * all DMA devices.
 *
 * Returns:
 * sg_table filled with the DMA addresses to use or ERR_PRT with negative error
 * code.
 */
static struct sg_table *xdx_dmabuf_map(struct dma_buf_attachment *attach,
					enum dma_data_direction dir)
{
	/* TODO: */
	return ERR_PTR(-EBUSY);
}

/**
 * xdx_dmabuf_unmap - &dma_buf_ops.unmap_dma_buf implementation
 * @attach: DMA-buf attachment
 * @sgt: sg_table to unmap
 * @dir: DMA direction
 *
 * This is called when a shared DMA buffer no longer needs to be accessible by
 * another device. For now, simply unpins the buffer from GTT.
 */
static void xdx_dmabuf_unmap(struct dma_buf_attachment *attach,
				 struct sg_table *sgt,
				 enum dma_data_direction dir)
{
    /* TODO: */
}

/**
 * xdx_dmabuf_begin_cpu_access - &dma_buf_ops.begin_cpu_access implementation
 * @dma_buf: Shared DMA buffer
 * @direction: Direction of DMA transfer
 *
 * This is called before CPU access to the shared DMA buffer's memory. If it's
 * a read access, the buffer is moved to the GTT domain if possible, for optimal
 * CPU read performance.
 *
 * Returns:
 * 0 on success or a negative error code on failure.
 */
static int xdx_dmabuf_begin_cpu_access(struct dma_buf *dma_buf,
					   enum dma_data_direction direction)
{
    /* TODO: */
    return -1;
}

const struct dma_buf_ops xdx_dmabuf_ops = {
	.attach = xdx_dmabuf_attach,
	.detach = xdx_dmabuf_detach,
	.pin = xdx_dmabuf_pin,
	.unpin = xdx_dmabuf_unpin,
	.map_dma_buf = xdx_dmabuf_map,
	.unmap_dma_buf = xdx_dmabuf_unmap,
	.release = drm_gem_dmabuf_release,
	.begin_cpu_access = xdx_dmabuf_begin_cpu_access,
	.mmap = drm_gem_dmabuf_mmap,
	.vmap = drm_gem_dmabuf_vmap,
	.vunmap = drm_gem_dmabuf_vunmap,
};

struct drm_gem_object *xdx_drm_gem_prime_import(struct drm_device *dev,
						struct dma_buf *dma_buf)
{
	struct dma_buf_attachment *attach;
	struct drm_gem_object *obj;

	if (dma_buf->ops == &xdx_dmabuf_ops) {
		obj = dma_buf->priv;
		if (obj->dev == dev) {
			/**
			 * only increase the refcount if the dmabuf come from
			 * the same device
			 */
			drm_gem_object_get(obj);
			return obj;
		}
	}

	return xdx_dmabuf_create_obj(dev, dma_buf);
}