/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef __XDX_DRM_GEM_H__
#define __XDX_DRM_GEM_H__

#include <linux/kref.h>
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 4, 0)) || defined(REDHAT_83) || (defined(XDXGPU_CENTOS85_SUPPORT))
#include <linux/dma-resv.h>
#else
#include <linux/reservation.h>
#endif
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 5, 0)) || (defined(REDHAT_83)) || (defined(XDXGPU_CENTOS85_SUPPORT))
#include <drm/drm_device.h>
#include <drm/drm_file.h>
#else
#include <drm/drmP.h>
#endif
#include <drm/drm_vma_manager.h>
#include <drm/drm_gem.h>
#include <linux/pid.h>
#include <drm/xdxgpu_drm.h>
#include "xdx_device.h"
//#include "vpu/hantro.h"
#include "hantro_metadata.h"
#include "pvr_dma_resv.h"

#define XDX_DC_INVLIAD_VA 0xFFFFFFFF

/**
 * enum xdx_bo_type
 *
 * @xdx_bo_type_device:	These are 'normal' buffers that can
 * be mmapped by user space. Each of these bos occupy a slot in the
 * device address space, that can be used for normal vm operations.
 *
 * @xdx_bo_type_kernel: These buffers are like xdx_bo_type_device buffers,
 * but they cannot be accessed from user-space. For kernel-only use.
 *
 * @xdx_bo_type_sg: Buffer made from dmabuf sg table shared with another
 * driver.
 */
enum xdx_bo_type {
	xdx_bo_type_device,
	xdx_bo_type_kernel,
	xdx_bo_type_sg,
	xdx_bo_type_pvr,
};

struct xdx_bo_property {
	enum xdxgpu_gem_placement placement;
	enum xdxgpu_gem_gpu_cache gpu_cache;
	enum xdxgpu_gem_cpu_cache cpu_cache;
	/** the mapping flags of GPU and CPU domain */
	uint32_t mapping;
	uint32_t flags;
	uint64_t pmr_flags;
};

struct xdx_bo {
	struct drm_gem_object base;
	struct xdx_bo_property pro;
	enum xdx_bo_type type;
	uint32_t page_alignment;
	void *psPMR; /* pointer to the buffer object of PVR driver */
	struct sg_table *sgt; /* for imported dmabuf */
	void *vaddr; /* kernel virtual address */
	void *vmmap_priv; /* PVR vmmap private data */
	uint64_t *pages; /* device phy pages */
	uint64_t num_pages;
	struct drm_gem_hantro_object *cma_obj;
	struct viv_vidmem_metadata meta_data; /* for vpu */
	/* per VM structure for page tables and with virtual addresses */
	struct list_head va_list;
#if (LINUX_VERSION_CODE < KERNEL_VERSION(5, 4, 0)) && (!defined(REDHAT_83)) && (!defined(XDXGPU_CENTOS82_SUPPORT)) || (defined(XDXGPU_CENTOS85_SUPPORT)) || (defined(XDXGPU_CENTOS79_SUPPORT))
	struct dma_resv	*resv;
	struct dma_resv _resv;

#endif
};
#define to_xdx_gem(x) container_of(x, struct xdx_bo, base)

struct xdx_bo_user {
	struct xdx_bo bo; /* must be the first member */
	uint64_t tiling_flags;
	struct pid *pid;
};

struct xdx_bo_create_params {
	void *pvr_fpriv;
	uint64_t size;              /* size of bo */
	uint32_t alignment;         /* bytes alignment */
	uint32_t bo_ptr_size;       /* size of bo struture */
	struct xdx_bo_property pro;
	enum xdx_bo_type type;
};

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4, 11, 0))
vm_fault_t xdx_gem_object_vm_fault(struct vm_fault *vmf);
#else
int xdx_gem_object_vm_fault(struct vm_area_struct *vma, struct vm_fault *vmf);
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(4, 11, 0)) && (!defined(XDXGPU_CENTOS79_SUPPORT))
#define xdx_drm_helper_mode_fill_fb_struct(dev, fb, mode_cmd) drm_helper_mode_fill_fb_struct(fb, mode_cmd)
#else
#define xdx_drm_helper_mode_fill_fb_struct(dev, fb, mode_cmd) drm_helper_mode_fill_fb_struct(dev, fb, mode_cmd)
#endif

extern const struct vm_operations_struct xdx_gem_vm_ops;

int xdx_gem_mmap(struct file *filp, struct vm_area_struct *vma);
/* bo lock */
int xdx_bo_reserve(struct xdx_bo *bo, bool interruptible);
void xdx_bo_unreserve(struct xdx_bo *bo);

int xdx_bo_create(struct xdx_device *xdev, struct xdx_bo_create_params *bp,
		  struct xdx_bo **bo_ptr, uint64_t *puiPMRFlags);

int xdx_gem_object_create(struct xdx_device *xdev, void *pvr_fpriv,
			  uint64_t size, uint32_t alignment,
			  struct xdx_bo_property *pro, enum xdx_bo_type type,
			  struct drm_gem_object **obj, uint64_t *flags);

int xdx_gem_create_ioctl(struct drm_device *dev, void *data,
			 struct drm_file *filp);

int xdx_gem_info_query_ioctl(struct drm_device *ddev, void *data,
		       struct drm_file *filp);

int xdx_drm_dumb_create(struct drm_file *file_priv, struct drm_device *ddev,
			struct drm_mode_create_dumb *args);

int xdx_drm_dumb_mmap(struct drm_file *filp, struct drm_device *ddev,
		      uint32_t handle, uint64_t *offset_p);

struct drm_gem_object *xdx_gem_prime_import(struct drm_device *ddev,
					       struct dma_buf *dma_buf);

int xdx_gem_object_mmap(struct drm_gem_object *gobj,
			struct vm_area_struct *vma);

int xdx_gem_mmap_ioctl(struct drm_device *dev, void *data,
			  struct drm_file *filp);

int xdx_gem_export_to_pvr_ioctl(struct drm_device *dev, void *data,
			  struct drm_file *filp);

int xdx_gem_import_from_pvr_ioctl(struct drm_device *ddev, void *data,
			  struct drm_file *filp);

int xdx_bo_vmap(struct xdx_bo *bo);
void xdx_bo_vunmap(struct xdx_bo *bo);

int xdx_bo_create_reserved(struct xdx_device *xdev, unsigned long size,
			   int align, enum xdxgpu_gem_placement placement,
			   struct xdx_bo **bo_ptr, u64 *dev_addr,
			   void **cpu_addr);

uint64_t* xdx_bo_get_pages(struct xdx_bo *bo);

uint64_t xdx_bo_gpu_offset(struct xdx_bo *bo);

int xdx_bo_sparse_copy(struct xdx_bo *bo, void *vaddr);

void xdx_gem_object_put(struct drm_gem_object *obj);
#endif /* __XDX_DRM_GEM_H__ */
