/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */
#ifndef __XDX_IRQ_H__
#define __XDX_IRQ_H__

#include <linux/irqdomain.h>

struct xdx_device;
struct xdx_irq_src;
struct xdx_irq_entry;

enum xdx_interrupt_state {
	XDX_IRQ_STATE_DISABLE,
	XDX_IRQ_STATE_ENABLE,
};

enum xdx_interrupt_type {
	XDX_IRQ_INTX,
	XDX_IRQ_MSIX,
	XDX_IRQ_MSI,
	XDX_IRQ_UNKNOW,
};

enum xdx_irq_clientid {
	XDX_IRQ_CLIENTID_GPU = 0,
	XDX_IRQ_CLIENTID_ADMA,
	XDX_IRQ_CLIENTID_VDMA,
	XDX_IRQ_CLIENTID_AUDIO_DMA,
	XDX_IRQ_CLIENTID_DC0,
	XDX_IRQ_CLIENTID_DC1,
	XDX_IRQ_CLIENTID_VPU,
	XDX_IRQ_CLIENTID_HDMI0,
	XDX_IRQ_CLIENTID_HDMI1,
	XDX_IRQ_CLIENTID_DP0,
	XDX_IRQ_CLIENTID_DP1,
	XDX_IRQ_CLIENTID_CMCU,
	XDX_IRQ_CLIENTID_ACPU,
	XDX_IRQ_CLIENTID_TDMA,
	XDX_IRQ_CLIENTID_AUDIO,
	XDX_IRQ_CLIENTID_AUDIO1,
	XDX_IRQ_CLIENTID_MAX,
	XDX_IRQ_CLIENTID_INVALID = XDX_IRQ_CLIENTID_MAX
};

/* provided by interrupt generating IP blocks */
struct xdx_irq_src_funcs {
	int (*set)(struct xdx_device *xdev, struct xdx_irq_src *src,
		   unsigned type, enum xdx_interrupt_state state);
	int (*process)(struct xdx_device *xdev, struct xdx_irq_src *src,
		       struct xdx_irq_entry *entry);
};

struct xdx_irq_src {
	enum xdx_irq_clientid id;
	const struct xdx_irq_src_funcs *funcs;
};

struct xdx_irq_entry {
	struct xdx_device *xdev;
	uint32_t irq;
	uint32_t vec_idx; /* MSIX vector index */
	bool interested;
	bool poll_enable; /* MSI poll control */
};

struct xdx_ih_funcs {
	uint32_t (*get_client_id)(struct xdx_irq_entry *irq_entry);
	const char* (*get_irq_vector_name)(struct xdx_irq_entry *irq_entry);
	int (*intx_handler)(struct xdx_device *xdev, struct xdx_irq_entry *irq_entry);
	void (*msi_handler)(struct xdx_device *xdev, struct xdx_irq_entry *irq_entry);
	int (*set)(struct xdx_device *xdev, struct xdx_irq_entry *irq_entry, enum xdx_interrupt_state state);
};

struct xdx_irq {
	bool installed;
	struct xdx_irq_entry *irqs;
	uint32_t num_irq;
	spinlock_t lock;
	bool msi_enabled;
	enum xdx_interrupt_type irq_type;
	const struct xdx_ih_funcs *ih_funcs;
	struct xdx_irq_src *clients[XDX_IRQ_CLIENTID_MAX];
};

int xdx_irq_register_src(struct xdx_device *xdev, struct xdx_irq_src *irq_src);
int xdx_irq_unregister_src(struct xdx_device *xdev, struct xdx_irq_src *irq_src);

int xdx_irq_init(struct xdx_device *xdev);

void xdx_irq_fini(struct xdx_device *xdev);

int xdxgpu_irq_set(struct xdx_device *xdev,
		      struct xdx_irq_src *src,
		      unsigned type,
		      enum xdx_interrupt_state state);

#endif /* __XDX_IRQ_H__ */
