/*
 * @Copyright   Copyright (c) XIANGDIXIAN Computing Technology Co., Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 */

#include <linux/delay.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/slab.h>
#include <linux/wait.h>
#include <linux/module.h>
#include <linux/mutex.h>
#include <linux/firmware.h>
#include <linux/io.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/uaccess.h>
#include <linux/kernel.h>

#include <sound/info.h>
#include <sound/control.h>
#include <sound/pcm_params.h>

#include "xdxaudio_hal.h"
#include "xdx_ddcci.h"
#include "../xdx_irq.h"
#include "../gfx/services/server/include/osfunc.h"

//#define XDX_HDMI_AUDIO_DEBUG

#if defined(XDX_HDMI_AUDIO_DEBUG)
#define XDX_HDMI_AUDIO_PRINT(dev, fmt, ...)	dev_info(dev, fmt, ##__VA_ARGS__)
#else
#define XDX_HDMI_AUDIO_PRINT(dev, fmt, ...)	do { } while (0)
#endif

#define XDX_HDMI_AUDIO_SYSTEM_NAME				"xdx-hdmi-audio"

#define XDX_GDDR0_BASE_ADDR					(0x100000000)

#define B0_AUDIOIF_SUB_FRAME_IN_BYTES   	     		(1536)
#define B0_AUDIOIF_SUB_FRAME_OU_BYTES        			(2048)
#define B0_AUDIOIF_PKT_HEAD_OFFSET				(0x100)
#define XDX_AUDIOIF_PERIOD_BYTES				(6144)
#define XDX_AUDIOIF_PERIOD_BYTES_B0				(4608)
#define FRAME_CNT_PER_BLOCK					(192)
#define MIN_BLOCK_SIZE						(1536)
#define AUDIO_FIFO_SIZE						(32768)
#define AUDIO_FIFO_SIZE_B0					(32768)
#define AUDIO_FIFO_SIZE_B0_HALF					(32768/2)
#define AUDIO_FIFO_SIZE_A0					(32768)
#define AUDIO_FIFO_SIZE_A0_HALF					(32768/2)
#define AUDIO_FIFO_PERIOD_BYTES					(0x2000)

#define MAX_BUFFER_BYTES_A0					(36864)
#define MAX_BUFFER_BYTES_B0					(55296)
#define MAX_BUFFER_BYTES					(55296)
#define MIN_PERIOD_BYTES					(1536)
#define PERIOD_BYTES_A02CHNS16BIT				(1536)

#define GET_BIT(X, BP)						(((X)>>(BP))&1)

#define XDX_AUDIOIF0_INTR_STS					(0x48)
#define XDX_AUDIOIF0_INTR_STS_CLR				(0x4C)


#define XDX_PF1_IO_AUDIO_CONFIG_BASE				(0xe00000)
#define XDX_PF1_IO_AUDIO_CONFIG_SIZE				(0x100000)
#define XDX_PF1_IO_DOORBELL_CONFIG_BASE				(0x100520)
#define XDX_PF1_IO_DOORBELL_CONFIG_SIZE				(0x100000)
#define XDX_PF1_IO_ACPU_HOST_BASE				(0x100540)
#define XDX_PF1_IO_ACPU_HOST_INT_BASE				(0x1002ec)
#define XDX_PF1_IO_ACPU_HOST_CLEAR_INT_BASE			(0x1001cc)

#define XDX_CMU_CONFIG						(0xa8)
#define XDX_GLOBAL_SWDISABLE					(0x44)
#define XDX_PKTSCHED_PKT_EN					(0xaa8)
#define XDX_AUDIO_PKT						(0xbc4)
#define XDX_PKT_AUDI_CONTENTS1					(0xc24)
#define XDX_PKT_AUDI_CONTENTS2					(0xc28)
#define XDX_AUDPKT_ACR_CONTROL0					(0xe40)
#define XDX_AUDPKT_ACR_CONTROL1					(0xe44)
#define XDX_AUDIO_INTERFACE					(0x820)


#define XDX_LINK_QP_CLOCK					(0x2002)
#define XDX_GLOBAL_SWDISABLE_VALUE				(0x0)
#define XDX_PKTSCHED_PKT_EN_VALUE				(0xf00e)
#define XDX_AUDIO_PKT_VALUE					(0xc035d)
#define XDX_AUDPKB_2CHNS_S1					(0x16f)
#define XDX_AUDPKB_2CHNS_S2					(0x100)
#define XDX_AUDPKB_4CHNS_S1					(0x36a)
#define XDX_AUDPKB_4CHNS_S2					(0x101)
#define XDX_AUDPKB_6CHNS_S1					(0x560)
#define XDX_AUDPKB_6CHNS_S2					(0x10b)
#define XDX_AUDPKB_8CHNS_S1					(0xf172b)
#define XDX_AUDPKB_8CHNS_S2					(0x11f)
#define XDX_AUDIO_INTERFACE_2CHS				(0x1011)
#define XDX_AUDIO_INTERFACE_4CHS				(0x1031)
#define XDX_AUDIO_INTERFACE_6CHS				(0x1071)
#define XDX_AUDIO_INTERFACE_8CHS				(0x10f1)

#define XDX_AUDIO_DP_CONFIG_2CHS16BIT				(0x12001202)
#define XDX_AUDIO_DP_CONFIG_2CHS				(0x12001302)
#define XDX_AUDIO_DP_CONFIG_4CHS16BIT				(0x12002206)
#define XDX_AUDIO_DP_CONFIG_4CHS				(0x12002306)
#define XDX_AUDIO_DP_CONFIG_6CHS16BIT				(0x1200220e)
#define XDX_AUDIO_DP_CONFIG_6CHS				(0x1200230e)
#define XDX_AUDIO_DP_CONFIG_8CHS16BIT				(0x1200221e)
#define XDX_AUDIO_DP_CONFIG_8CHS				(0x1200231e)
#define XDX_AUDIO_DP_CFGREG					(0x80000003)
#define XDX_AUDIO_DP_CFG1REG					(0xc0000003)

#define XDX_HDMI_DEFAULT_CLOCK					(148500)
#define XDX_VDMA_TAG_CMPL_INTRSTS0				(31)

#define AUDIO_RESERVE_VRAM_LIMIT				0x1000000//16M
#define FW_RESERVE_VRAM_SIZE					0x1000000//16M
#define AUDIO_RESERVE_VRAM_SIZE					(FW_RESERVE_VRAM_SIZE + AUDIO_RESERVE_VRAM_LIMIT)//32M

#define OSReadAUHWReg8(addr, off)  ((u8)readb((u8 __iomem *)(addr) + (off)))
#define OSReadAUHWReg16(addr, off) ((u16)readw((u8 __iomem *)(addr) + (off)))
#define OSReadAUHWReg32(addr, off) ((u32)readl((u8 __iomem *)(addr) + (off)))

/* Little endian support only */
#define OSReadAUHWReg64(addr, off) \
    ({ \
     __typeof__(addr) _addr = addr; \
     __typeof__(off) _off = off; \
     (u64) \
     ( \
       ( (u64)(readl((u8 __iomem *)(_addr) + (_off) + 4)) << 32) \
       | readl((u8 __iomem *)(_addr) + (_off)) \
       ); \
     })

#define OSWriteAUHWReg8(addr, off, val)  writeb((u8)(val), (u8 __iomem *)(addr) + (off))
#define OSWriteAUHWReg16(addr, off, val) writew((u16)(val), (u8 __iomem *)(addr) + (off))
#define OSWriteAUHWReg32(addr, off, val) writel((u32)(val), (u8 __iomem *)(addr) + (off))
/* Little endian support only */
#define OSWriteAUHWReg64(addr, off, val) do \
{ \
    __typeof__(addr) _addr = addr; \
    __typeof__(off) _off = off; \
    __typeof__(val) _val = val; \
    writel((u32)((_val) & 0xffffffff), (u8 __iomem *)(_addr) + (_off)); \
    writel((u32)(((u64)(_val) >> 32) & 0xffffffff), (u8 __iomem *)(_addr) + (_off) + 4); \
} while (0)

#define __V volatile
#define jackCnts				4
#define concentCnts				3

#define xdx_audio_switch_info   xdx_hdmi_audio_switchInfo


uint64_t xdx_b0_audioif_buf_gddr_offset = 0;

typedef enum {
	cH02AAudioSubCmdForAllZeroTest = 0,
	cH02AAudioSubCmdPllConfig,
	cH02AAudioSubCmdPllDecimalConfig,
	cH02AAudioSubCmdMax
} H02AAudioSubCmdType_T;

typedef union {
	__V u32 id	: 32;
} XDX_ID_T;

typedef union {
	__V u32 all;
	struct {
		__V u32 hdmi_bypass	: 1;	//bit 0
		__V u32 dp_iis_sel	: 1;	//bit 1
		__V u32 resv0		: 1;
		__V u32 hdmi_cdiv	: 1;	//bit 3
		__V u32 precision	: 1;	//bit 4
		__V u32 chnl_num	: 2;	//bit 5
		__V u32 resv1		: 1;
		__V u32 buffer_num	: 3;	//bit 8-10
		__V u32 resv2		: 1;
		__V u32 threshold	: 2;	//bit 12-13
		__V u32 resv3		: 1;
		__V u32 resv4		: 1;
		__V u32 info_mode	: 2;	//bit 16-17
		__V u32 resv5		: 1;
		__V u32 resv6		: 1;
		__V u32 axi_qos		: 4;	//bit 20-23
		__V u32 to_thrd_sel	: 3;	//bit 24
		__V u32 resv7		: 5;
	};
} XDX_CTRL_T;


typedef union {
	__V u32 all;
	struct {
		__V u32 wptr	: 8;
		__V u32 rptr	: 8;
		__V u32 resv	: 16;
	};
} XDX_BUF_PTR_T;

typedef union {
	__V u32	all;
	struct {
		__V u32 wptr_up		: 1;
		__V u32 resv		: 31;
	};
} XDX_PTR_UP_T;

typedef union {
	__V u32 all;
	struct {
		__V u32 run	: 1;
		__V u32 resv	: 31;
	};
} XDX_TXEN_B0_T;

typedef union {
	__V u32 all;
	struct {
		__V u32 base    : 19;
		__V u32 resv	: 13;
	};
} XDX_BUF_BASE_T;

typedef union {
	__V u32 all;
	struct {
		__V u32	chnl_sts_dw0	: 32;
	};
} XDX_CHNL_STS_OVRD0_T;

typedef union {
	__V u32 all;
	struct {
		__V u32	chnl_sts_dw1	: 32;
	};
} XDX_CHNL_STS_OVRD1_T;

typedef union {
	__V u32 all;
	struct {
		__V u32	chnl_sts_dw2	: 32;
	};
} XDX_CHNL_STS_OVRD2_T;

typedef union {
	__V u32 all;
	struct {
		__V u32	chnl_sts_dw3	: 32;
	};
} XDX_CHNL_STS_OVRD3_T;

typedef union {
	__V u32 all;
	struct {
		__V u32	chnl_sts_dw4	: 32;
	};
} XDX_CHNL_STS_OVRD4_T;

typedef union {
	__V u32 all;
	struct {
		__V u32	chnl_sts_dw5	: 32;
	};
} XDX_CHNL_STS_OVRD5_T;

typedef union {
	__V u32 all;
	struct {
		__V u32 iis_idle	: 1;
		__V u32 empty		: 1;
		__V u32 aempty		: 1;
		__V u32 bus_timeout	: 1;
		__V u32 bus_err		: 1;
		__V u32 resv		: 27;
	};
} XDX_INTR_EN_T;

typedef union {
	__V u32 all;
	struct {
		__V u32 iis_idle	: 1;
		__V u32 empty		: 1;
		__V u32 aempty		: 1;
		__V u32 bus_timeout	: 1;
		__V u32 bus_err		: 1;
		__V u32 axi_rchnl_idle	: 1;
		__V u32 resv		: 25;
	};
} XDX_INTR_RAW_STS_T;

typedef union {
	__V u32 all;
	struct {
		__V u32 iis_idle	: 1;
		__V u32 empty		: 1;
		__V u32 aempty		: 1;
		__V u32 bus_timeout	: 1;
		__V u32 bus_err		: 1;
		__V u32 resv		: 27;
	};
} XDX_INTR_STS_T;

typedef union {
	__V u32 all;
	struct {
		__V u32 iis_idle	: 1;
		__V u32 empty		: 1;
		__V u32 aempty		: 1;
		__V u32 bus_timeout	: 1;
		__V u32 bus_err		: 1;
		__V u32 resv		: 27;
	};
} XDX_INTR_STS_CLR_T;

typedef union {
	 __V u32 resv		:32;
} XDX_RESV0_T;


typedef union {
	 __V u32 resv		:32;
} XDX_RESV1_T;

typedef union {
	 __V u32 resv		:32;
} XDX_RESV2_T;

typedef union {
	 __V u32 resv		:32;
} XDX_RESV3_T;

typedef struct {
	__V XDX_ID_T				ID;			//Offset 0x0
	__V XDX_CTRL_T				CTRL;			//Offset 0x04
	__V XDX_BUF_PTR_T			BUF_PTR;		//Offset 0x08
	__V XDX_PTR_UP_T			PTR_UP;			//Offset 0x0C
	__V XDX_TXEN_B0_T			TXEN;			//Offset 0x10
	__V XDX_BUF_BASE_T			BUF_BASE;		//Offset 0x14
	__V XDX_RESV0_T				RESV0;			//Offset
	__V XDX_RESV1_T				RESV1;			//Offset
	__V XDX_CHNL_STS_OVRD0_T		CHNL_STS_OVRD0;		//Offset 0x20
	__V XDX_CHNL_STS_OVRD1_T		CHNL_STS_OVRD1;		//Offset 0x24
	__V XDX_CHNL_STS_OVRD2_T		CHNL_STS_OVRD2;		//Offset 0x28
	__V XDX_CHNL_STS_OVRD3_T		CHNL_STS_OVRD3;		//Offset 0x2c
	__V XDX_CHNL_STS_OVRD4_T		CHNL_STS_OVRD4;		//Offset 0x30
	__V XDX_CHNL_STS_OVRD5_T		CHNL_STS_OVRD5;		//Offset 0x34
	__V XDX_RESV2_T				RESV2;			//Offset
	__V XDX_RESV3_T				RESV3;			//Offset
	__V XDX_INTR_EN_T			INTR_EN;		//Offset 0x40
	__V XDX_INTR_RAW_STS_T			INTR_RAW_STS;		//Offset 0x44
	__V XDX_INTR_STS_T			INTR_STS;		//offset 0x48
	__V XDX_INTR_STS_CLR_T			INTR_STS_CLR;		//offset 0x4c
} Audio_B0_Regs_T;

typedef union {
	 __V u32 all;
	struct {
		__V u32 hdmi_bypass	: 1;	//bit 0
		__V u32 resv0		: 2;	//bit 1-2
		__V u32 hdmi_cdiv	: 1;	//bit 3
		__V u32 resv1		: 1;	//bit 4
		__V u32 chnl_num	: 2;	//bit 5
		__V u32 resv2		: 1;	//bit 7
		__V u32 aempty_thrd	: 2;	//bit 8
		__V u32 dp_iis_sel	: 1;	//bit 10
		__V u32 pl_shift	: 5;	//bit 11-15
		__V u32 resv3		: 16;	//bit 16-31
	};
} CNTL_T;

typedef union {
	__V u32 all;
	struct {
		__V u32 iis_idle	: 1;
		__V u32 empty		: 1;
		__V u32 aempty		: 1;
		__V u32 resv		: 29;
	};
} IE_T;

typedef union {
	__V u32 all;
	struct {
		__V u32 iis_idle	: 1;
		__V u32 empty		: 1;
		__V u32 aempty		: 1;
		__V u32 bclk_stable	: 1;
		__V u32 resv		: 28;
	};
} RIS_T;

typedef union {
	__V u32 all;
	struct {
		__V u32 iis_idle	: 1;
		__V u32 empty		: 1;
		__V u32 aempty		: 1;
		__V u32 resv		: 29;
	};
} MIS_T;

typedef union {
	__V u32 all;
	struct {
		__V u32 hdmi_bclk_en	: 1;
		__V u32 resv		: 31;
	};
} CKEN_T;

typedef union {
	__V u32 all;
	struct {
		__V u32 iis_en		: 1;
		__V u32 resv		: 31;
	};
} TXEN_T;

typedef struct {
	__V CNTL_T CNTL;	//Offset 0x00
	__V CKEN_T CKEN;	//Offset 0x04
	__V TXEN_T TXEN;	//Offset 0x08
	__V IE_T IE;		//Offset 0x0C
	__V RIS_T RIS;		//Offset 0x10
	__V MIS_T MIS;		//Offset 0x14
} Audio_Regs_T;

/* DMA Command */
typedef union {
	uint64_t all;
	struct {
		uint64_t srcAddr_47_3:45;	// Source Address[47:3]
		uint64_t srcAddrMode:2;		// Source Address Mode:0-HOST_SVA, 1-HOST_SPA, 2-GPU_SVA, 3-GPU_GPA
		uint64_t intrEn:1;		// Interrupt Enable: 1-set intsts when cmd cmpl
		uint64_t lengthUnit:2;		// Tranfer Length Unit:0-8B, 1-64B, 2-1KB, 3-1MB
		uint64_t commandTag:6;		// Command Tag
		uint64_t opCode:8;		// Operation Code
	};
} DMACmdDw0_T;

typedef union {
	uint64_t all;
	struct {
		uint64_t dstAddr_47_3:45;	// Destination Address[47:3]
		uint64_t dstAddrMode:2;		// Destination Address Mode:0-HOST_SVA, 1-HOST_SPA, 2-GPU_SVA, 3-GPU_GPA
		uint64_t rsvd:1;
		uint64_t transferLen:16;	// Transfer Length
	};
} DMACmdDw1_T;

typedef union {
	uint64_t all;
	struct {
		uint64_t patternLen:8;		// Pattern Length (Unit Byte) (RealLength - 1)
		uint64_t patternLoc:8;		// Pattern Location (4Byte Align)
		uint64_t pcieFuncNum:1;		// PCIE Physical Function Number: PF0 or PF1
		uint64_t rsvd1:1;
		uint64_t pcieDontSnoop:1;	// PCIE TLP Snoop Attribute:1-no snoop
		uint64_t rsvd2:45;
	};
} DMACmdDw2_T;

typedef union {
	uint32_t cmd32[6];
	uint64_t cmd64[3];
	struct {
		DMACmdDw0_T dw0;
		DMACmdDw1_T dw1;
		DMACmdDw2_T dw2;
	};
} DMACmd_T;

typedef union {
	__V u64 all;
	struct
	{
		__V u32 low32;
		__V u32 high32;
	};
	struct
	{
		__V u64 done   : 1;//----
		__V u64 cmd    : 3;//    |---- Header
		__V u64 subCmd : 4;//----
		__V u64 resv   : 24;//----
		__V u64 pixclock   : 32;//-------- Payload
	};
} DBCmd_T;

typedef union {
	__V u64 all;
	struct {
		__V u32 low32;
		__V u32 high32;
	};
	struct {
		__V u64 done   : 1;//----
		__V u64 cmd    : 3;//    |---- Header
		__V u64 subCmd : 4;//----
		__V u64 data   : 56;//-------- Payload
	};
} DBCmd_a0f1_T;

typedef union {
	uint64_t all;
	struct {
		uint64_t srcAddr_47_3:45;   // 44:0 Source Address[47:3]
		uint64_t LogicOp:3;         // 47:45 Logic Operation: 0:None 1:NOT 2:XOR 3:OR 4:AND
		uint64_t rsvd:2;            // 49:48 Rsvd
		uint64_t Tag:6;             // 55:50 Command Tag
		uint64_t opCode:8;          // 63:56 Operation Code
	};
} DMAB0CmdDw0_T;

typedef union {
	uint64_t all;
	struct {
		uint64_t Xstep_SrcDescNum:16;  // 15:0 XStep(OP80/81/84) or SrcDescriptorNum(OP82/83)
		uint64_t DstAddr_47_3:45;   // 60:16 Destination Address
		uint64_t LenUnit:3;         // 63:61 Length Unit: 8B/16B/32B/64B/1KB
	};
} DMAB0CmdDw1_T;

typedef union {
	uint64_t all;
	struct {
		uint64_t PcieFuncNum:1;                 // 0
		uint64_t PcieDontSnoop:1;               // 1
		uint64_t IntrEn:1;                      // 2
		uint64_t rsvd1:1;                       // 3
		uint64_t DstrideLow_PatternLen:8;       // 11:4 DstStride[7:0](OP84) or PatternLocation
		uint64_t DstrideHigh_PatternLoc:8;      // 19:12 DstStride[15:8](OP84) or PatternLocation
		uint64_t Sstride_DWriteLenLow:16;       // 35:20 SrcStride(OP84) or DirectWriteLen[15:0]
		uint64_t Ystep_DstDescNum_DstXferLen_DWriteLenHigh:16;  // 51:36 Ystep(OP84) or DstDescNum(OP81/83)
                                                                       //       or DstXferLen(OP82) or DirectWriteLen[24:16]
		uint64_t DstAddrMode:4;     // 55:52 Destination Address Mode
		uint64_t SrcAddrMode:4;     // 59:56 Source Address Mode
		uint64_t rsvd2:4;           // 63:60 Rsvd
	};
} DMAB0CmdDw2_T;

typedef union {
	uint32_t cmd32[6];
	uint64_t cmd64[3];
	struct {
		DMAB0CmdDw0_T dw0;
		DMAB0CmdDw1_T dw1;
		DMAB0CmdDw2_T dw2;
	};
} VDMA_B0_CmdData_T;

typedef enum
{
	cPlayBackStatusNone = 0,
	cPlayBackStart,
	cPlayBackPause,
	cPlayBackStop,
	cPlayBackMax,
} PlayBackStatus_T;

typedef struct {
	uint32_t rsv:2;
	uint32_t data2:8;
	uint32_t data1:8;
	uint32_t data0:8;
	uint32_t v:1;
	uint32_t u:1;
	uint32_t c:1;
	uint32_t p:1;
	uint32_t pr:2;
} DPSubFrame_T;

typedef struct {
	uint32_t rsv:3;
	uint32_t data2:8;
	uint32_t data1:8;
	uint32_t data0:8;
	uint32_t v:1;
	uint32_t u:1;
	uint32_t c:1;
	uint32_t p:1;
	uint32_t b:1;
} HDMISubFrame_T;

typedef union {
	DPSubFrame_T DpSubFrame;
	HDMISubFrame_T HdmiSubFrame;
	uint32_t UnPacketFrame;
} ADIFSubFrame_T;

typedef struct {
	uint32_t hdmiClk;
	uint32_t nValue;
	uint32_t ctsValue;
} HDMIArcTable_T;

typedef struct {
	uint32_t dataRate;
	uint32_t tableNum;
	HDMIArcTable_T HdmiArcTbl[14];
	uint32_t otherClkNvalue;
} AdHdmiAcrCfg_T;

/*****************************************************************************************/
/*                                Static Function Statement                             */
/*****************************************************************************************/
static int xdx_hdmi_audio_volumeInfo(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_info *uinfo);
static int  xdx_hdmi_audio_volumeGet(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u);
static int xdx_hdmi_audio_volumePut(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u);
static int xdx_hdmi_audio_switchInfo(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_info *uinfo);
static int xdx_hdmi_audio_hdmi0Get(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u);
static int xdx_hdmi_audio_hdmi0Put(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u);
static int xdx_hdmi_audio_dp0Get(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u);
static int xdx_hdmi_audio_dp0Put(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u);
static int xdx_hdmi_audio_hdmi1Get(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u);
static int xdx_hdmi_audio_hdmi1Put(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u);
static int xdx_hdmi_audio_dp1Get(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u);
static int xdx_hdmi_audio_dp1Put(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u);



int xdx_audio_drv_hotPlug(struct xdx_device *xdev, PlayDrv module, PlayCon status);
static inline void xdx_hdmicore_write(
    struct hdmi_dev *hdmi, u32 offset, u32 value)
{
    hdmi->core_write(hdmi, offset, value);
}

static inline u32 xdx_hdmicore_read(
    struct hdmi_dev *hdmi, u32 offset)
{
	return hdmi->core_read(hdmi, offset);;
}

#if defined(XDX_HDMI_AUDIO_DEBUG)
static void dump_runtime(struct snd_pcm_substream *substream)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = snd_pcm_substream_chip(substream);
	struct snd_pcm_runtime *runtime = substream->runtime;
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] avail_max=0x%lx\n", __func__, __LINE__, runtime->avail_max);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] hw_ptr_base=0x%lx\n", __func__, __LINE__, runtime->hw_ptr_base);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] hw_ptr_interrupt=0x%lx\n", __func__, __LINE__, runtime->hw_ptr_interrupt);

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] access=%d\n", __func__, __LINE__, runtime->access);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] format=%d\n", __func__, __LINE__, runtime->format);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] subformat=%d\n", __func__, __LINE__, runtime->subformat);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] rate=%d\n", __func__, __LINE__, runtime->rate);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] channels=%d\n", __func__, __LINE__, runtime->channels);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] period_size=%ld\n", __func__, __LINE__, runtime->period_size);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] periods=%d\n", __func__, __LINE__, runtime->periods);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] buffer_size=%ld\n", __func__, __LINE__, runtime->buffer_size);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] period_step=%d\n", __func__, __LINE__, runtime->period_step);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] frame_bits=%d\n", __func__, __LINE__, runtime->frame_bits);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] sample_bits=%d\n", __func__, __LINE__, runtime->sample_bits);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] start_threshold=%ld\n", __func__, __LINE__, runtime->start_threshold);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] stop_threshold=%ld\n", __func__, __LINE__, runtime->stop_threshold);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] dma_bytes=0x%lx\n", __func__, __LINE__, runtime->dma_bytes);
}
#endif

static const struct snd_pcm_hardware snd_xdx_hdmi_audio_playback_hw =
{
	.info =			(SNDRV_PCM_INFO_MMAP |
				SNDRV_PCM_INFO_INTERLEAVED |
				SNDRV_PCM_INFO_BLOCK_TRANSFER),
	.formats =		(SNDRV_PCM_FMTBIT_S16_LE),
	.rates =		(SNDRV_PCM_RATE_44100 |
				SNDRV_PCM_RATE_48000),
	.rate_min =		44100,
	.rate_max =		48000,
	.channels_min =		2,				//MIN_CHANNELS,
	.channels_max =		8,				//MAX_CHANNELS,
	.buffer_bytes_max =	MAX_BUFFER_BYTES,		//MAX_BUF_SIZE,
	.period_bytes_min =	MIN_PERIOD_BYTES,		//MIN_CHANNELS * 2 * kPlayBufferFrames,
	.period_bytes_max =	XDX_AUDIOIF_PERIOD_BYTES,	//MAX_CHANNELS * 2 * kPlayBufferFrames,
	.periods_min =		1,				//MIN_PERIODS,
	.periods_max =		1024,				//MAX_PERIODS,
};

static const struct snd_kcontrol_new snd_xdx_hdmi_control[] =
{
	{
		.access =	SNDRV_CTL_ELEM_ACCESS_READ | SNDRV_CTL_ELEM_ACCESS_WRITE,
		.iface =	SNDRV_CTL_ELEM_IFACE_MIXER,
		.name =		"Monitor Left And Right Channel Volume",
		.info =		xdx_hdmi_audio_volumeInfo,
		.get =		xdx_hdmi_audio_volumeGet,
		.put =		xdx_hdmi_audio_volumePut,
	},
	{
		.access =	SNDRV_CTL_ELEM_ACCESS_READ,
		.iface =	SNDRV_CTL_ELEM_IFACE_CARD,
		.name =		"HDMI0 Jack",
		.info =		xdx_audio_switch_info,
		.get =		xdx_hdmi_audio_hdmi0Get,
	},
	{
		.access =	SNDRV_CTL_ELEM_ACCESS_READ,
		.iface =	SNDRV_CTL_ELEM_IFACE_CARD,
		.name =		"DP0 Jack",
		.info =		xdx_audio_switch_info,
		.get =		xdx_hdmi_audio_dp0Get,
	},
	{
		.access =	SNDRV_CTL_ELEM_ACCESS_READ,
		.iface =	SNDRV_CTL_ELEM_IFACE_CARD,
		.name =		"HDMI1 Jack",
		.info =		xdx_audio_switch_info,
		.get =		xdx_hdmi_audio_hdmi1Get,
	},
	{
		.access =	SNDRV_CTL_ELEM_ACCESS_READ,
		.iface =	SNDRV_CTL_ELEM_IFACE_CARD,
		.name =		"DP1 Jack",
		.info =		xdx_audio_switch_info,
		.get =		xdx_hdmi_audio_dp1Get,
	},
	{
		.access =	SNDRV_CTL_ELEM_ACCESS_READ | SNDRV_CTL_ELEM_ACCESS_WRITE,
		.iface =	SNDRV_CTL_ELEM_IFACE_MIXER,
		.name =		"HDMI0 Switch",
		.info =		xdx_audio_switch_info,
		.get =		xdx_hdmi_audio_hdmi0Get,
		.put =		xdx_hdmi_audio_hdmi0Put,
	},
	{
		.access =	SNDRV_CTL_ELEM_ACCESS_READ | SNDRV_CTL_ELEM_ACCESS_WRITE,
		.iface =	SNDRV_CTL_ELEM_IFACE_MIXER,
		.name =		"DP0 Switch",
		.info =		xdx_audio_switch_info,
		.get =		xdx_hdmi_audio_dp0Get,
		.put =		xdx_hdmi_audio_dp0Put,
	},
	{
		.access =	SNDRV_CTL_ELEM_ACCESS_READ | SNDRV_CTL_ELEM_ACCESS_WRITE,
		.iface =	SNDRV_CTL_ELEM_IFACE_MIXER,
		.name =		"HDMI1 Switch",
		.info =		xdx_audio_switch_info,
		.get =		xdx_hdmi_audio_hdmi1Get,
		.put =		xdx_hdmi_audio_hdmi1Put,
	},
	{
		.access =	SNDRV_CTL_ELEM_ACCESS_READ | SNDRV_CTL_ELEM_ACCESS_WRITE,
		.iface =	SNDRV_CTL_ELEM_IFACE_MIXER,
		.name =		"DP1 Switch",
		.info =		xdx_audio_switch_info,
		.get =		xdx_hdmi_audio_dp1Get,
		.put =		xdx_hdmi_audio_dp1Put,
	},
};

typedef enum {
	VIDEO_TYPE_HDMI,
	VIDEO_TYPE_DP,
	VIDEO_TYPE_MAX,
} VIDEO_TYPE_MODE;

typedef enum {
	PCM_BWIDTH_16,
	PCM_BWIDTH_24,
	PCM_BWIDTH_32,
	PCM_BWIDTH_MAX,
} PCM_BWIDTH_MODE;

typedef enum {
	PCM_CHNUM_20,
	PCM_CHNUM_21,
	PCM_CHNUM_51,
	PCM_CHNUM_71,
	PCM_CHNUM_MAX,
} PCM_CHNUM_MODE;

static void snd_xdx_hdmi_audio_free_pcm(struct snd_pcm *pcm)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = pcm->private_data;

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] snd_xdx_hdmi_audio_free_pcm.\n", __func__, __LINE__);

	xdx_hdmi_audio_ptr->pcm = NULL;
}

static uint32_t period_bytes_a0[] = {1536, 3072, 4608, 6144};
static struct snd_pcm_hw_constraint_list constraints_period_bytes_a0 = {
	.count = ARRAY_SIZE(period_bytes_a0),
	.list = period_bytes_a0,
	.mask = 0,
};

static uint32_t period_bytes_b0[] = {3072, 4608};
static struct snd_pcm_hw_constraint_list constraints_period_bytes_b0 = {
	.count = ARRAY_SIZE(period_bytes_b0),
	.list = period_bytes_b0,
	.mask = 0,
};

static int snd_xdx_hdmi_audio_playback_open(struct snd_pcm_substream *substream)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = snd_pcm_substream_chip(substream);
	struct snd_pcm_runtime *runtime = substream->runtime;
	struct xdx_device *xdev = xdx_hdmi_audio_ptr->xdev;

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);

	runtime->hw = snd_xdx_hdmi_audio_playback_hw;
	xdx_hdmi_audio_ptr->playback_substream = substream;
	xdx_hdmi_audio_ptr->pos = 0;
	xdx_hdmi_audio_ptr->fifo_bytes = 0;
	xdx_hdmi_audio_ptr->fifo_empty = 1;
	xdx_hdmi_audio_ptr->dma_pending_cnt = 0;
	xdx_hdmi_audio_ptr->transfer_flag = 0;
	xdx_hdmi_audio_ptr->dmaTag = 0;
	xdx_hdmi_audio_ptr->ptr = 0;
	xdx_hdmi_audio_ptr->buf_ptr = 0;
	xdx_hdmi_audio_ptr->ptr_bytes = 0;

	if (xdev->asic_type == CHIP_PANGU_A0) {
		snd_pcm_hw_constraint_single(runtime, SNDRV_PCM_HW_PARAM_BUFFER_BYTES, MAX_BUFFER_BYTES_A0);
		snd_pcm_hw_constraint_list(runtime, 0, SNDRV_PCM_HW_PARAM_PERIOD_BYTES, &constraints_period_bytes_a0);
		runtime->hw.formats |= SNDRV_PCM_FMTBIT_S24_LE;
	} else if (xdev->asic_type == CHIP_PANGU_B0) {
		snd_pcm_hw_constraint_single(runtime, SNDRV_PCM_HW_PARAM_BUFFER_BYTES, MAX_BUFFER_BYTES_B0);
		snd_pcm_hw_constraint_list(runtime, 0, SNDRV_PCM_HW_PARAM_PERIOD_BYTES, &constraints_period_bytes_b0);
		runtime->hw.formats |= SNDRV_PCM_FMTBIT_S24_3LE;
	}

	return 0;
}

static int snd_xdx_hdmi_audio_playback_close(struct snd_pcm_substream *substream)
{
#if defined(XDX_HDMI_AUDIO_DEBUG)
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = snd_pcm_substream_chip(substream);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);
#endif
	return 0;
}

static void ADIFOutputSelConfig(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr)
{
	uint32_t value = 0;
	volatile uint32_t* pDpAudioCfgReg;
	volatile uint32_t* pDpAudioSdpCfg0Reg;
	volatile uint32_t* pDpAudioSdpCfg1Reg;

	if (2 == xdx_hdmi_audio_ptr->channels || 0 == xdx_hdmi_audio_ptr->channels) {
		if (16 == xdx_hdmi_audio_ptr->bit_width)
			value = XDX_AUDIO_DP_CONFIG_2CHS16BIT;
		else
			value = XDX_AUDIO_DP_CONFIG_2CHS;
	} else if (4 == xdx_hdmi_audio_ptr->channels) {
		if (16 == xdx_hdmi_audio_ptr->bit_width)
			value = XDX_AUDIO_DP_CONFIG_4CHS16BIT;
		else
			value = XDX_AUDIO_DP_CONFIG_4CHS;
	} else if (6 == xdx_hdmi_audio_ptr->channels) {
		if (16 == xdx_hdmi_audio_ptr->bit_width)
			value = XDX_AUDIO_DP_CONFIG_6CHS16BIT;
		else
			value = XDX_AUDIO_DP_CONFIG_6CHS;
	} else if (8 == xdx_hdmi_audio_ptr->channels) {
		if (16 == xdx_hdmi_audio_ptr->bit_width)
			value = XDX_AUDIO_DP_CONFIG_8CHS16BIT;
		else
			value = XDX_AUDIO_DP_CONFIG_8CHS;
	} else {
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "Channel number error!\n");
		return;
	}

	/// Setup the DP register
	if (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP) {
		pDpAudioCfgReg = (volatile uint32_t*)(xdx_hdmi_audio_ptr->iobase_dp0 + 0x400);
		*pDpAudioCfgReg = value;

		pDpAudioSdpCfg0Reg =  (volatile uint32_t*)(xdx_hdmi_audio_ptr->iobase_dp0 + 0x504);
		*pDpAudioSdpCfg0Reg = XDX_AUDIO_DP_CFGREG;

		pDpAudioSdpCfg1Reg =  (volatile uint32_t*)(xdx_hdmi_audio_ptr->iobase_dp0 +  0x500);
		*pDpAudioSdpCfg1Reg = XDX_AUDIO_DP_CFG1REG;
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);
	} else if (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP1) {
		pDpAudioCfgReg = (volatile uint32_t*)(xdx_hdmi_audio_ptr->iobase_dp1 + 0x400);
		*pDpAudioCfgReg = value;

		pDpAudioSdpCfg0Reg =  (volatile uint32_t*)(xdx_hdmi_audio_ptr->iobase_dp1 + 0x504);
		*pDpAudioSdpCfg0Reg = XDX_AUDIO_DP_CFGREG;

		pDpAudioSdpCfg1Reg =  (volatile uint32_t*)(xdx_hdmi_audio_ptr->iobase_dp1 +  0x500);
		*pDpAudioSdpCfg1Reg = XDX_AUDIO_DP_CFG1REG;
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);
	}
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);

}

//{audio_rate, the number of table, {hdmi_clock, n, cts}, {the value of other hdmi_clock to n}}
const AdHdmiAcrCfg_T audioRateCfg[]=
{
	{32000, 14,
	{{25175,4576,28125},{25200,4096,25200},{27000,4096,27000},{27027,4096,27027},{54000,4096,54000},{54054,4096,54054},
	{74176,11648,210937},{742500,4096,74250},{148352,11648,421875},{148500,4096,148500},{296703,5824,421875},
	{297000,3072,222750},{593407,5824,843750},{594000,3072,445500}},
	4096},

	{64000, 14,
	{{25175,9152,28125},{25200,8192,25200},{27000,8192,27000},{27027,8192,27027},{54000,8192,54000},{54054,8192,54054},
	{74176,23296,210937},{742500,8192,74250},{148352,23296,421875},{148500,8192,148500},{296703,11648,421875},
	{297000,8192,297000},{593407,11648,84375},{594000,8192,594000}},
	8192},

	{128000, 14,
	{{25175,18304,28125},{25200,16384,25200},{27000,16384,27000},{27027,16384,27027},{54000,16384,54000},{54054,16384,54054},
	{74176,46592,210937},{742500,16384,74250},{148352,46592,421875},{148500,16384,148500},{296703,23296,421875},
	{297000,16384,297000},{593407,23296,843750},{594000,16384,594000}},
	16384},

	{256000, 14,
	{{25175,36608,28125},{25200,327668,25200},{27000,32768,27000},{27027,32768,27027},{54000,32768,54000},{54054,32768,54054},
	{74176,93184,210937},{742500,32768,74250},{148352,93184,421875},{148500,32768,148500},{296703,46592,421875},
	{297000,32768,297000},{593407,46592,843750},{594000,32768,594000}},
	32768},

	{44100, 14,
	{{25175,7007,31250},{25200,6272,28000},{27000,6272,30000},{27027,6272,30030},{54000,6272,60000},{54054,6272,60060},
	{74176,17836,234375},{742500,6272,82500},{148352,8918,234375},{148500,6272,165000},{296703,4459,234375},
	{297000,4704,247500},{593407,8918,937500},{594000,9408,990000}},
	6272},

	{88200, 14,
	{{25175,14014,31250},{25200,12544,28000},{27000,12544,30000},{27027,12544,30030},{54000,12544,60000},{54054,12544,60060},
	{74176,35672,234375},{742500,12544,82500},{148352,17836,234375},{148500,12544,165000},{296703,8918,234375},
	{297000,9408,247500},{593407,17836,937500},{594000,18816,990000}},
	12544},

	{176400, 14,
	{{25175,28028,31250},{25200,25088,28000},{27000,25088,30000},{27027,25088,30030},{54000,25088,60000},{54054,25088,60060},
	{74176,71344,234375},{742500,25088,82500},{148352,35672,234375},{148500,25088,165000},{296703,17836,234375},
	{297000,18816,247500},{593407,35672,937500},{594000,37632,990000}},
	25088},

	{352800, 14,
	{{25175,56056,31250},{25200,50176,28000},{27000,50176,30000},{27027,50176,30030},{54000,50176,60000},{54054,50176,60060},
	{74176,142688,234375},{742500,50176,82500},{148352,71344,234375},{148500,50176,165000},{296703,35672,234375},
	{297000,37632,247500},{593407,71344,937500},{594000,75264,990000}},
	50176},

	{48000, 14,
	{{25175,6864,28125},{25200,6144,25200},{27000,6144,27000},{27027,6144,27027},{54000,6144,54000},{54054,6144,54054},
	{74176,11648,140625},{742500,6144,74250},{148352,5824,140625},{148500,6144,148500},{296703,5824,281250},
	{297000,5120,247500},{593407,5824,562500},{594000,6144,594000}},
	6144},

	{96000, 14,
	{{25175,13728,28125},{25200,12288,25200},{27000,12288,27000},{27027,12288,27027},{54000,12288,54000},{54054,12288,54054},
	{74176,23296,140625},{742500,12288,74250},{148352,11648,140625},{148500,12288,148500},{296703,11648,281250},
	{297000,10240,247500},{593407,11648,562500},{594000,12288,594000}},
	12288},

	{192000, 14,
	{{25175,27456,28125},{25200,24576,25200},{27000,24576,27000},{27027,24576,27027},{54000,24576,54000},{54054,24576,54054},
	{74176,46592,140625},{742500,24576,74250},{148352,23296,140625},{148500,24576,148500},{296703,23296,281250},
	{297000,20480,247500},{593407,23296,562500},{594000,24576,594000}},
	24576},

	{384000, 14,
	{{25175,54912,28125},{25200,49152,25200},{27000,49152,27000},{27027,49152,27027},{54000,49152,54000},{54054,49152,54054},
	{74176,93184,140625},{742500,49152,74250},{148352,46592,140625},{148500,49152,148500},{296703,46592,281250},
	{297000,40960,247500},{593407,46592,562500},{594000,49152,594000}},
	49152},
};
int xdx_hdmi_audio_setRate(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr)
{
	uint32_t audio_rate = xdx_hdmi_audio_ptr->rate;
	uint32_t hdmi_clock = 0;
	uint32_t s1 = 0, s2 = 0, i = 0, j = 0, n = 0, cts = 0, cfg = 0;
	struct hdmi_dev *hdmi = NULL;

	if (audio_rate == 0) {
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "Audio Rate is 0!\n");
		return 0;
	}

	if (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI)
		hdmi = xdx_hdmi_audio_ptr->hdmi;
	else if (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI1)
		hdmi = xdx_hdmi_audio_ptr->hdmi1;

	if (!hdmi->hdmi_mode.clock)
		hdmi_clock = XDX_HDMI_DEFAULT_CLOCK;
	else
		hdmi_clock = hdmi->hdmi_mode.clock;

	if (2 == xdx_hdmi_audio_ptr->channels || 0 == xdx_hdmi_audio_ptr->channels) {
		s1 = XDX_AUDPKB_2CHNS_S1;
		s2 = XDX_AUDPKB_2CHNS_S2;
		cfg = XDX_AUDIO_INTERFACE_2CHS;
	} else if (4 == xdx_hdmi_audio_ptr->channels) {
		s1 = XDX_AUDPKB_4CHNS_S1;
		s2 = XDX_AUDPKB_4CHNS_S2;
		cfg = XDX_AUDIO_INTERFACE_4CHS;
	} else if (6 == xdx_hdmi_audio_ptr->channels) {
		s1 = XDX_AUDPKB_6CHNS_S1;
		s2 = XDX_AUDPKB_6CHNS_S2;
		cfg = XDX_AUDIO_INTERFACE_6CHS;
	} else if (8 == xdx_hdmi_audio_ptr->channels) {
		s1 = XDX_AUDPKB_8CHNS_S1;
		s2 = XDX_AUDPKB_8CHNS_S2;
		cfg = XDX_AUDIO_INTERFACE_8CHS;
	} else {
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "Channel number error!\n");
		return -1;
	}

	for (i = 0; i < sizeof(audioRateCfg)/sizeof(AdHdmiAcrCfg_T); i++) {
		if (audio_rate == audioRateCfg[i].dataRate) {
			for (j = 0; j < audioRateCfg[i].tableNum; j++) {
				if (hdmi_clock == audioRateCfg[i].HdmiArcTbl[j].hdmiClk) {
					n = audioRateCfg[i].HdmiArcTbl[j].nValue;
					cts = audioRateCfg[i].HdmiArcTbl[j].ctsValue;
					XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] n=%d, cts=%d, hdmi_clock=%d, audio_rate=%d\n", __func__, __LINE__, n, cts, hdmi_clock, audio_rate);
					break;
				}
			}
			if (j == audioRateCfg[i].tableNum) {
				n = audioRateCfg[i].otherClkNvalue;
				cts = n*hdmi_clock/128/audio_rate*1000;
				XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] n=%d, cts=%d, hdmi_clock=%d, audio_rate=%d\n", __func__, __LINE__, n, cts, hdmi_clock, audio_rate);
			}
			if (j > audioRateCfg[i].tableNum) {
				XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] Input pixel error!\n", __func__, __LINE__);
				return -1;
			}
			break;
		}
	}
	if (i >= sizeof(audioRateCfg)/sizeof(AdHdmiAcrCfg_T)) {
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] Input rate error!\n", __func__, __LINE__);
		return -1;
	}

	xdx_hdmicore_write(hdmi, XDX_CMU_CONFIG, XDX_LINK_QP_CLOCK);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] Read XDX_CMU_CONFIG: 0x%x\n", __func__, __LINE__, xdx_hdmicore_read(hdmi, XDX_CMU_CONFIG));
	xdx_hdmicore_write(hdmi, XDX_GLOBAL_SWDISABLE, XDX_GLOBAL_SWDISABLE_VALUE);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] Read XDX_GLOBAL_SWDISABLE: 0x%x\n", __func__, __LINE__, xdx_hdmicore_read(hdmi, XDX_GLOBAL_SWDISABLE));
	xdx_hdmicore_write(hdmi, XDX_PKTSCHED_PKT_EN, XDX_PKTSCHED_PKT_EN_VALUE);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] Read XDX_PKTSCHED_PKT_EN: 0x%x\n", __func__, __LINE__, xdx_hdmicore_read(hdmi, XDX_PKTSCHED_PKT_EN));
	xdx_hdmicore_write(hdmi, XDX_AUDIO_PKT, XDX_AUDIO_PKT_VALUE);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] Read XDX_AUDIO_PKT: 0x%x\n", __func__, __LINE__, xdx_hdmicore_read(hdmi, XDX_AUDIO_PKT));

	xdx_hdmicore_write(hdmi, XDX_PKT_AUDI_CONTENTS1, s1);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] Read XDX_PKT_AUDI_CONTENTS1: 0x%x\n", __func__, __LINE__, xdx_hdmicore_read(hdmi, XDX_PKT_AUDI_CONTENTS1));
	xdx_hdmicore_write(hdmi, XDX_PKT_AUDI_CONTENTS2, s2);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] Read XDX_PKT_AUDI_CONTENTS2: 0x%x\n", __func__, __LINE__, xdx_hdmicore_read(hdmi, XDX_PKT_AUDI_CONTENTS2));
	xdx_hdmicore_write(hdmi, XDX_AUDPKT_ACR_CONTROL0, n);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] Read XDX_AUDPK_ACR_CONTROL0: 0x%x\n", __func__, __LINE__, xdx_hdmicore_read(hdmi, XDX_AUDPKT_ACR_CONTROL0));
	xdx_hdmicore_write(hdmi, XDX_AUDPKT_ACR_CONTROL1, cts<<4);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] Read XDX_ADPKT_ACR_CONTROL1: 0x%x\n", __func__, __LINE__, xdx_hdmicore_read(hdmi, XDX_AUDPKT_ACR_CONTROL1));
	xdx_hdmicore_write(hdmi, XDX_AUDIO_INTERFACE, cfg);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] Read XDX_AUDIO_INTERFACE: 0x%x\n", __func__, __LINE__, xdx_hdmicore_read(hdmi, XDX_AUDIO_INTERFACE));

	return 0;
}

static int xdx_hdmi_audio_blkSize(int channels, struct xdx_device *xdev)
{
	uint32_t blkSize = 0;
	if (channels == 2)
		blkSize = 1536;
	else if (channels == 4)
		blkSize = 3072;
	else if	((channels == 6) && (xdev->asic_type == CHIP_PANGU_A0))
		blkSize = 4608;
	else if	((channels == 6) && (xdev->asic_type == CHIP_PANGU_B0))
		blkSize = 6144;
	else if	(channels == 8)
		blkSize = 6144;
	else
		blkSize = 1536;
	return blkSize;
}

static int snd_xdx_hdmi_audio_hw_params(struct snd_pcm_substream *substream,
                                 struct snd_pcm_hw_params *params)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = NULL;
	DBCmd_T dbCmd;
	DBCmd_a0f1_T dbCmd_a0f1;
	uint32_t TimeOut = 0;
	uint32_t dbvalue = 0;
	struct xdx_device *xdev = NULL;

	xdx_hdmi_audio_ptr = snd_pcm_substream_chip(substream);
	xdev = xdx_hdmi_audio_ptr->xdev;
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] access=%d\n", __func__, __LINE__, params_access(params));
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] format=%d\n", __func__, __LINE__, params_format(params));
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] subformat=%d\n", __func__, __LINE__, params_subformat(params));
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] rate=%d\n", __func__, __LINE__, params_rate(params));
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] channels=%d\n", __func__, __LINE__, params_channels(params));
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] width=%d\n", __func__, __LINE__, params_width(params));
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] phsical_width=%d\n", __func__, __LINE__, params_physical_width(params));
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] period_bytes=%d\n", __func__, __LINE__, params_period_bytes(params));
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] buffer_bytes=%d\n", __func__, __LINE__, params_buffer_bytes(params));

	xdx_hdmi_audio_ptr->bit_width = params_width(params);
	xdx_hdmi_audio_ptr->channels = params_channels(params);
	xdx_hdmi_audio_ptr->rate = params_rate(params);
	xdx_hdmi_audio_ptr->blkSize = xdx_hdmi_audio_blkSize(xdx_hdmi_audio_ptr->channels, xdev);
	xdx_hdmi_audio_ptr->last_bytes = 0;

	if (xdev->shareram_size == XDX_AUDIO_BAR4_16M && xdev->pdev->device != 0x15A4) {
		switch (params_rate(params)) {
			case 44100:
				dbCmd_a0f1.data = 44100;
				break;
			case 48000:
				dbCmd_a0f1.data = 48000;
				break;
			default:
				break;
		}
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] dbCmd_a0f1.all=0x%llx\n", __func__, __LINE__, dbCmd_a0f1.all);
		OSWriteAUHWReg64(xdx_hdmi_audio_ptr->iobase, 0x100588, dbCmd_a0f1.all);
	} else if (xdev->shareram_size == XDX_AUDIO_BAR4_1M) {
		dbCmd.pixclock = params_rate(params);
		dbCmd.subCmd = cH02AAudioSubCmdPllDecimalConfig;
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] dbCmd.all=0x%llx\n", __func__, __LINE__, dbCmd.all);
		OSWriteAUHWReg64(xdx_hdmi_audio_ptr->iobase_acpu, 0x100588, dbCmd.all);
	}

	TimeOut = 10000;
	while(1) {
		if (xdev->shareram_size == XDX_AUDIO_BAR4_16M && xdev->pdev->device != 0x15A4)
			dbvalue = OSReadAUHWReg32(xdx_hdmi_audio_ptr->iobase, 0x100588);
		else if (xdev->shareram_size == XDX_AUDIO_BAR4_1M)
			dbvalue = OSReadAUHWReg32(xdx_hdmi_audio_ptr->iobase_acpu, 0x100588);
		if ((dbvalue & 1) == 1)
			break;

		TimeOut--;
		if (TimeOut == 0) {
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] dbCmd fail\n", __func__, __LINE__);
			return -1;
		}
		udelay(1000);
	}

	if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP) || (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP1))
		ADIFOutputSelConfig(xdx_hdmi_audio_ptr);
	else {
		///set audio configure
		if ((xdx_hdmi_audio_ptr->hdmi) || (xdx_hdmi_audio_ptr->hdmi1))
			xdx_hdmi_audio_setRate(xdx_hdmi_audio_ptr);
	}

	return snd_pcm_lib_malloc_pages(substream, params_buffer_bytes(params));
}

static void xdx_hdmi_audio_a0_Enable(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr, PlayDrv audioif_video_connect, bool enable)
{
	Audio_Regs_T *pAudioReg = (Audio_Regs_T *)(xdx_hdmi_audio_ptr->iobase_audio0);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);

	mutex_lock(xdx_hdmi_audio_ptr->hw_mutex);
	if (enable == 0) {
		pAudioReg->CNTL.all = 0;
		pAudioReg->CKEN.all = 0;
		pAudioReg->TXEN.all = 0;
		pAudioReg->IE.all = 0;
	} else {
		if (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI) {
			pAudioReg->CNTL.hdmi_cdiv = 1;
			pAudioReg->CNTL.dp_iis_sel = 0;
			pAudioReg->CNTL.aempty_thrd = 1;
			pAudioReg->CKEN.hdmi_bclk_en = 1;
			pAudioReg->IE.all = 0x6;
			pAudioReg->TXEN.iis_en = 1;
		} else if (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP) {
			pAudioReg->CNTL.hdmi_cdiv = 0;
			pAudioReg->CNTL.dp_iis_sel = 1;
			pAudioReg->CNTL.aempty_thrd = 1;
			pAudioReg->CKEN.hdmi_bclk_en = 1;
			pAudioReg->IE.all = 0x6;
			pAudioReg->TXEN.iis_en = 1;
		} else {
			pAudioReg->CNTL.hdmi_cdiv = 1;
			pAudioReg->CNTL.dp_iis_sel = 0;
			pAudioReg->CNTL.aempty_thrd = 1;
			pAudioReg->CKEN.hdmi_bclk_en = 1;
			pAudioReg->IE.all = 0x6;
			pAudioReg->TXEN.iis_en = 1;
		}
		switch(xdx_hdmi_audio_ptr->channels) {
			case 2:
				pAudioReg->CNTL.chnl_num = 0;
				break;
			case 4:
				pAudioReg->CNTL.chnl_num = 1;
				break;
			case 6:
				pAudioReg->CNTL.chnl_num = 2;
				break;
			case 8:
				pAudioReg->CNTL.chnl_num = 3;
				break;
			default:
				pAudioReg->CNTL.chnl_num = 0;
				break;
		}
	}
	mutex_unlock(xdx_hdmi_audio_ptr->hw_mutex);

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);
}

static void xdx_hdmi_audio_b0_Enable(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr, PlayDrv audioif_video_connect, bool enable)
{
	Audio_B0_Regs_T *pAudioB0Reg = (Audio_B0_Regs_T *)(xdx_hdmi_audio_ptr->iobase_audio0);
	Audio_B0_Regs_T *pAudio1B0Reg = (Audio_B0_Regs_T *)(xdx_hdmi_audio_ptr->iobase_audio1);
	struct xdx_device *xdev = xdx_hdmi_audio_ptr->xdev;
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);

	mutex_lock(xdx_hdmi_audio_ptr->hw_mutex);

	if (enable == 0) {
		if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI) || (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP))
			pAudioB0Reg->TXEN.all = 0;
		else if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI1) || (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP1))
			pAudio1B0Reg->TXEN.all = 0;
		else
			pAudioB0Reg->TXEN.all = 0;
	} else  if (enable == 1) {
		if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI) || (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP)) {
			//we reserved 64M for ACPU, FW use high 16M, audio use at most 16M, the space is enough
			pAudioB0Reg->BUF_BASE.all = (xdev->vram_size - AUDIO_RESERVE_VRAM_SIZE + XDX_GDDR0_BASE_ADDR)>>21;
			pAudioB0Reg->INTR_EN.all = 0x1f;
			pAudioB0Reg->CHNL_STS_OVRD0.all = 0x108000;
			pAudioB0Reg->CHNL_STS_OVRD1.all = 0xb;
			pAudioB0Reg->CHNL_STS_OVRD2.all = 0x0;
			pAudioB0Reg->CHNL_STS_OVRD3.all = 0x0;
			pAudioB0Reg->CHNL_STS_OVRD4.all = 0x0;
			pAudioB0Reg->CHNL_STS_OVRD5.all = 0x0;

			if (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI) {
				if (xdx_hdmi_audio_ptr->bit_width == 16) {
					switch(xdx_hdmi_audio_ptr->channels) {
						case 2:
							pAudioB0Reg->CTRL.all = 0x4021308;
							break;
						case 4:
							pAudioB0Reg->CTRL.all = 0x4021228;
							break;
						case 6:
							pAudioB0Reg->CTRL.all = 0x4021148;
							break;
						case 8:
							pAudioB0Reg->CTRL.all = 0x4021168;
							break;
					}
				} else if (xdx_hdmi_audio_ptr->bit_width == 24) {
					switch(xdx_hdmi_audio_ptr->channels) {
						case 2:
							pAudioB0Reg->CTRL.all = 0x4021318;
							break;
						case 4:
							pAudioB0Reg->CTRL.all = 0x4021238;
							break;
						case 6:
							pAudioB0Reg->CTRL.all = 0x4021158;
							break;
						case 8:
							pAudioB0Reg->CTRL.all = 0x4021178;
							break;
					}
				}
				XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]HDMI\n", __func__, __LINE__);

			} else if (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP) {
				if (xdx_hdmi_audio_ptr->bit_width == 16) {
					switch(xdx_hdmi_audio_ptr->channels) {
						case 2:
							pAudioB0Reg->CTRL.all = 0x402130a;
							break;
						case 4:
							pAudioB0Reg->CTRL.all = 0x402122a;
							break;
						case 6:
							pAudioB0Reg->CTRL.all = 0x402114a;
							break;
						case 8:
							pAudioB0Reg->CTRL.all = 0x402116a;
							break;
					}
				} else if (xdx_hdmi_audio_ptr->bit_width == 24) {
					switch(xdx_hdmi_audio_ptr->channels) {
						case 2:
							pAudioB0Reg->CTRL.all = 0x402131a;
							break;
						case 4:
							pAudioB0Reg->CTRL.all = 0x402123a;
							break;
						case 6:
							pAudioB0Reg->CTRL.all = 0x402115a;
							break;
						case 8:
							pAudioB0Reg->CTRL.all = 0x402117a;
							break;
					}
				}
				XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]DP\n", __func__, __LINE__);
			}
		} else if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI1) || (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP1)) {
			pAudio1B0Reg->BUF_BASE.all = (xdev->vram_size - AUDIO_RESERVE_VRAM_SIZE + XDX_GDDR0_BASE_ADDR)>>21;
			pAudio1B0Reg->INTR_EN.all = 0x1f;
			pAudio1B0Reg->CHNL_STS_OVRD0.all = 0x108000;
			pAudio1B0Reg->CHNL_STS_OVRD1.all = 0xb;
			pAudio1B0Reg->CHNL_STS_OVRD2.all = 0x0;
			pAudio1B0Reg->CHNL_STS_OVRD3.all = 0x0;
			pAudio1B0Reg->CHNL_STS_OVRD4.all = 0x0;
			pAudio1B0Reg->CHNL_STS_OVRD5.all = 0x0;

			if (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI1) {
				if (xdx_hdmi_audio_ptr->bit_width == 16) {
					switch(xdx_hdmi_audio_ptr->channels) {
						case 2:
							pAudio1B0Reg->CTRL.all = 0x4021308;
							break;
						case 4:
							pAudio1B0Reg->CTRL.all = 0x4021228;
							break;
						case 6:
							pAudio1B0Reg->CTRL.all = 0x4021148;
							break;
						case 8:
							pAudio1B0Reg->CTRL.all = 0x4021168;
							break;
					}
				} else if (xdx_hdmi_audio_ptr->bit_width == 24) {
					switch(xdx_hdmi_audio_ptr->channels) {
						case 2:
							pAudio1B0Reg->CTRL.all = 0x4021318;
							break;
						case 4:
							pAudio1B0Reg->CTRL.all = 0x4021238;
							break;
						case 6:
							pAudio1B0Reg->CTRL.all = 0x4021158;
							break;
						case 8:
							pAudio1B0Reg->CTRL.all = 0x4021178;
							break;
					}
				}
				XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]HDMI1\n", __func__, __LINE__);

			} else if (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP1) {
				if (xdx_hdmi_audio_ptr->bit_width == 16) {
					switch(xdx_hdmi_audio_ptr->channels) {
						case 2:
							pAudio1B0Reg->CTRL.all = 0x402130a;
							break;
						case 4:
							pAudio1B0Reg->CTRL.all = 0x402122a;
							break;
						case 6:
							pAudio1B0Reg->CTRL.all = 0x402114a;
							break;
						case 8:
							pAudio1B0Reg->CTRL.all = 0x402116a;
							break;
					}
				} else if (xdx_hdmi_audio_ptr->bit_width == 24) {
					switch(xdx_hdmi_audio_ptr->channels) {
						case 2:
							pAudio1B0Reg->CTRL.all = 0x402131a;
							break;
						case 4:
							pAudio1B0Reg->CTRL.all = 0x402123a;
							break;
						case 6:
							pAudio1B0Reg->CTRL.all = 0x402115a;
							break;
						case 8:
							pAudio1B0Reg->CTRL.all = 0x402117a;
							break;
					}
				}
				XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]DP1\n", __func__, __LINE__);
			}

		} else {
			pAudioB0Reg->BUF_BASE.all = (xdev->vram_size - AUDIO_RESERVE_VRAM_SIZE + XDX_GDDR0_BASE_ADDR)>>21;
			pAudioB0Reg->INTR_EN.all = 0x1f;
			pAudioB0Reg->CHNL_STS_OVRD0.all = 0x108000;
			pAudioB0Reg->CHNL_STS_OVRD1.all = 0xb;
			pAudioB0Reg->CHNL_STS_OVRD2.all = 0x0;
			pAudioB0Reg->CHNL_STS_OVRD3.all = 0x0;
			pAudioB0Reg->CHNL_STS_OVRD4.all = 0x0;
			pAudioB0Reg->CHNL_STS_OVRD5.all = 0x0;
			if (xdx_hdmi_audio_ptr->bit_width == 16) {
				switch(xdx_hdmi_audio_ptr->channels) {
					case 2:
						pAudioB0Reg->CTRL.all = 0x4021308;
						break;
					case 4:
						pAudioB0Reg->CTRL.all = 0x4021228;
						break;
					case 6:
						pAudioB0Reg->CTRL.all = 0x4021148;
						break;
					case 8:
						pAudioB0Reg->CTRL.all = 0x4021168;
						break;
				}
			} else if (xdx_hdmi_audio_ptr->bit_width == 24) {
				switch(xdx_hdmi_audio_ptr->channels) {
					case 2:
						pAudioB0Reg->CTRL.all = 0x4021318;
						break;
					case 4:
						pAudioB0Reg->CTRL.all = 0x4021238;
						break;
					case 6:
						pAudioB0Reg->CTRL.all = 0x4021158;
						break;
					case 8:
						pAudioB0Reg->CTRL.all = 0x4021178;
						break;
				}
			}
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]Default\n", __func__, __LINE__);

		}

	}
	mutex_unlock(xdx_hdmi_audio_ptr->hw_mutex);

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);
}

static void xdx_hdmi_audio_waitempty(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr)
{
	uint64_t time_close = 0, timeout_close = 0, time_now = 0;
	struct xdx_device *xdev = xdx_hdmi_audio_ptr->xdev;

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);

	timeout_close = msecs_to_jiffies(300);
	if (xdev->asic_type == CHIP_PANGU_B0)
		xdx_hdmi_audio_b0_Enable(xdx_hdmi_audio_ptr, xdx_hdmi_audio_ptr->audioif_video_connect, 0);
	time_close = get_jiffies_64();
	while (xdx_hdmi_audio_ptr->transfer_flag == 1) {
		udelay(1000);
		time_now = get_jiffies_64();
		if (time_now > time_close + timeout_close) {
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]Wait transfer done timeout!\n", __func__, __LINE__);
			break;
		}
	}

	time_close = get_jiffies_64();
	while (!xdx_hdmi_audio_ptr->fifo_empty || xdx_hdmi_audio_ptr->dma_pending_cnt) {
		udelay(1000);
		time_now = get_jiffies_64();
		if (time_now > time_close + timeout_close) {
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]Wait fifo_empty timeout!\n", __func__, __LINE__);
			break;
		}
	}

	if (xdev->asic_type == CHIP_PANGU_A0)
		xdx_hdmi_audio_a0_Enable(xdx_hdmi_audio_ptr, xdx_hdmi_audio_ptr->audioif_video_connect, 0);

}

static int snd_xdx_hdmi_audio_hw_free(struct snd_pcm_substream *substream)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = snd_pcm_substream_chip(substream);

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);

	if (xdx_hdmi_audio_ptr->status != cPlayBackStop)
		xdx_hdmi_audio_ptr->status = cPlayBackStop;

	xdx_hdmi_audio_waitempty(xdx_hdmi_audio_ptr);

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);

	return snd_pcm_lib_free_pages(substream);
}

static int snd_xdx_hdmi_audio_prepare(struct snd_pcm_substream *substream)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = snd_pcm_substream_chip(substream);
	struct xdx_device *xdev = xdx_hdmi_audio_ptr->xdev;
	Audio_B0_Regs_T *pAudioB0Reg = (Audio_B0_Regs_T *)(xdx_hdmi_audio_ptr->iobase_audio0);
	Audio_B0_Regs_T *pAudio1B0Reg = (Audio_B0_Regs_T *)(xdx_hdmi_audio_ptr->iobase_audio1);
	uint64_t time_close = 0, timeout_close = 0, time_now = 0;
#if defined(XDX_HDMI_AUDIO_DEBUG)
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);
#endif

#if defined(XDX_HDMI_AUDIO_DEBUG)
	dump_runtime(substream);
#endif

	if ((xdev->asic_type == CHIP_PANGU_B0) && ((OSReadAUHWReg32(xdx_hdmi_audio_ptr->iobase_audio1, 0x10) & 0x1) || (OSReadAUHWReg32(xdx_hdmi_audio_ptr->iobase_audio0, 0x10) & 0x1))) {
		if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI1) || (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP1))
			pAudio1B0Reg->TXEN.all = 0;
		else if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI) || (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP))
			pAudioB0Reg->TXEN.all = 0;
		timeout_close = msecs_to_jiffies(300);
		time_close = get_jiffies_64();
		while (!xdx_hdmi_audio_ptr->fifo_empty || xdx_hdmi_audio_ptr->dma_pending_cnt) {
			udelay(1000);
			time_now = get_jiffies_64();
			if (time_now > time_close + timeout_close) {
				XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]Wait fifo_empty timeout!\n", __func__, __LINE__);
				break;
			}
		}
	}

	if (xdev->asic_type == CHIP_PANGU_A0)
		xdx_hdmi_audio_a0_Enable(xdx_hdmi_audio_ptr, xdx_hdmi_audio_ptr->audioif_video_connect, 1);
	else if (xdev->asic_type == CHIP_PANGU_B0)
		xdx_hdmi_audio_b0_Enable(xdx_hdmi_audio_ptr, xdx_hdmi_audio_ptr->audioif_video_connect, 1);
	return 0;
}

static void ADIFGetStatusData(uint8_t ucChnlIndex, uint32_t ucSmplRate, uint32_t *uStsBuffer)
{
	uint32_t value = 0x8000;
	uint8_t code = 0;
	uint32_t *pDst = uStsBuffer;


	if (uStsBuffer == NULL)
		return;

	memset(pDst, 0x0, 24);//192bit

	if (0 == (ucChnlIndex%2))//left channel
		value |= 0x100000;
	else
		value |= 0x200000;

	switch (ucSmplRate) {
		case PCM_SAMPLEBRATE_32K:
			code = 0x3;
			break;
		case PCM_SAMPLEBRATE_64K://TODO
			code = 0x3;
			break;
		case PCM_SAMPLEBRATE_128K://TODO
			code = 0x3;
			break;
		case PCM_SAMPLEBRATE_441K:
			code = 0x0;
			break;
		case PCM_SAMPLEBRATE_882K:
			code = 0x8;
			break;
		case PCM_SAMPLEBRATE_1764K:
			code = 0xC;
			break;
		case PCM_SAMPLEBRATE_48K:
			code = 0x2;
			break;
		case PCM_SAMPLEBRATE_96K:
			code = 0xA;
			break;
		case PCM_SAMPLEBRATE_192K:
			code = 0xE;
			break;
		default:
			break;
	}

	value |= (code << 24);

	*pDst++ = value;

	*pDst = 0xB;//24bit

	return;
}

static uint8_t AdifEvenCheck(uint32_t data, uint32_t len)
{
	uint32_t i = 0;
	uint32_t check = 0;

	for (i = 0; i < len; i++) {
		check += GET_BIT(data, i);
	}

	if (0 == (check % 2))
		return 0;
	else
		return 1;
}

static int xdx_hdmi_audio_prepareDataForDp(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr, uint64_t src, uint64_t dst, uint32_t *size)
{
	uint32_t i = 0, frameCnt = 0, cl = 0, count = 0, blkSize = 0;
	uint32_t uAudioStatus[8][8];
	uint8_t *srcPtr = (uint8_t *)src;
	ADIFSubFrame_T *dstData = (ADIFSubFrame_T *)dst;

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "Src = 0x%llx, Dst = 0x%llx, Count = %d\n", src, dst, *size);

	if (*size % MIN_BLOCK_SIZE != 0 || *size <= 0) {
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "Audio data size is not 1536 bytes aligned or greater than 0.\n");
		return -1;
	}

	//init channel status data
	for (cl = 0; cl < xdx_hdmi_audio_ptr->channels; cl++) {
		//channel number (2,4,8)
		ADIFGetStatusData(cl, xdx_hdmi_audio_ptr->rate, &uAudioStatus[cl][0]);
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "Audio Status %d=[%x, %x, %x, %x,%x, %x]\n", cl, uAudioStatus[cl][0], uAudioStatus[cl][1],\
			uAudioStatus[cl][2], uAudioStatus[cl][3], uAudioStatus[cl][4], uAudioStatus[cl][5]);
	}

	blkSize = xdx_hdmi_audio_ptr->blkSize;
	count = *size / blkSize;

	for (i = 0; i < count; i++) {
		//packet data for XDMA transmit
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "Audio DP Data Block (%d)\n", i);
		for (frameCnt = 0; frameCnt < FRAME_CNT_PER_BLOCK; frameCnt++) {
			//IEC60958 audio block data
			for (cl = 0; cl < xdx_hdmi_audio_ptr->channels; cl++) {
				//channel number (2,4,8)
				dstData->UnPacketFrame = 0;
				dstData->DpSubFrame.c = GET_BIT(uAudioStatus[cl][frameCnt / 32], frameCnt % 32);
				if (xdx_hdmi_audio_ptr->bit_width == 24)
					dstData->DpSubFrame.data2 = *srcPtr++;
				dstData->DpSubFrame.data1 = *srcPtr++;
				dstData->DpSubFrame.data0 = *srcPtr++;
				if (xdx_hdmi_audio_ptr->bit_width == 24)
					srcPtr++;   /// Pass 8 bits of dummy data
				else if (xdx_hdmi_audio_ptr->bit_width == 16) {
					/// Pass 16 bits of dummy data
					srcPtr++;
					srcPtr++;
				}

				dstData->DpSubFrame.p = AdifEvenCheck(dstData->UnPacketFrame, 29);
				if ((cl & 0x1) == 0) {
					if (frameCnt == 0)
						dstData->DpSubFrame.pr = 0;
					else
						dstData->DpSubFrame.pr = 1;
				} else
					dstData->DpSubFrame.pr = 2;

				dstData++;
			}
			if (xdx_hdmi_audio_ptr->channels == 6) {
				dstData->UnPacketFrame = 0;
				dstData++;
				dstData->UnPacketFrame = 0;
				dstData++;
			}
		}
	}
	if (xdx_hdmi_audio_ptr->channels == 6)
		*size += *size/3;

	return 0;
}


static int xdx_hdmi_audio_prepareDataForHdmi(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr, uint64_t src, uint64_t dst, uint32_t *size)
{
	uint32_t i = 0, frameCnt = 0, cl = 0, count = 0, blkSize = 0;
	uint32_t uAudioStatus[8][8];
	uint8_t *srcPtr = (uint8_t *)src;
	ADIFSubFrame_T *dstData = (ADIFSubFrame_T *)dst;

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "Src = 0x%llx, Dst = 0x%llx, Count = %d\n", src, dst, *size);

	if (*size % MIN_BLOCK_SIZE != 0 || *size <= 0) {
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "Audio data size is not 1536 bytes aligned or greater than 0.\n");
		return -1;
	}

	//init channel status data
	for (cl = 0; cl < xdx_hdmi_audio_ptr->channels; cl++) {//channel number (2,4,8)
		ADIFGetStatusData(cl, xdx_hdmi_audio_ptr->rate, &uAudioStatus[cl][0]);
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "Audio Status %d=[%x, %x, %x, %x,%x, %x]\n", cl, uAudioStatus[cl][0], uAudioStatus[cl][1],\
			uAudioStatus[cl][2], uAudioStatus[cl][3], uAudioStatus[cl][4], uAudioStatus[cl][5]);
	}


	blkSize = xdx_hdmi_audio_ptr->blkSize;
	count = *size / blkSize;

	for (i = 0; i < count; i++) {
		//packet data for XDMA transmit
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "Audio HDMI Data Block (%d)\n", i);
		for (frameCnt = 0; frameCnt < FRAME_CNT_PER_BLOCK; frameCnt++) {
			for (cl = 0; cl < xdx_hdmi_audio_ptr->channels; cl++) {
				//channel number (2,4,8)
				dstData->UnPacketFrame = 0;
				dstData->HdmiSubFrame.c = GET_BIT(uAudioStatus[cl][frameCnt / 32], frameCnt % 32);
				if (xdx_hdmi_audio_ptr->bit_width == 24)
					dstData->HdmiSubFrame.data2 = *srcPtr++;
				dstData->HdmiSubFrame.data1 = *srcPtr++;
				dstData->HdmiSubFrame.data0 = *srcPtr++;
				if (xdx_hdmi_audio_ptr->bit_width == 24)
					srcPtr++;   /// Pass 8 bits of dummy data
				else if (xdx_hdmi_audio_ptr->bit_width == 16) {
					/// Pass 16 bits of dummy data
					srcPtr++;
					srcPtr++;
				}

				dstData->HdmiSubFrame.p = AdifEvenCheck(dstData->UnPacketFrame, 30);
				if ((frameCnt == 0) && (cl & 0x1) == 0)
					dstData->HdmiSubFrame.b = 1;
				dstData++;
			}
			if (xdx_hdmi_audio_ptr->channels == 6) {
				dstData->UnPacketFrame = 0;
				dstData++;
				dstData->UnPacketFrame = 0;
				dstData++;
			}
		}
	}
	if (xdx_hdmi_audio_ptr->channels == 6)
		*size += *size/3;

	return 0;
}

static int xdx_hdmi_audio_prepareData(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr, uint64_t src, uint64_t dst)
{
	uint8_t *srcPtr = (uint8_t *)src;
	uint8_t *dstData = (uint8_t *)dst;
	uint32_t i = 0;

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "Src = 0x%llx, Dst = 0x%llx\n", src, dst);

	for (i = 0; i < xdx_hdmi_audio_ptr->blkSize; i++) {
		*(dstData++) = *(srcPtr++);
	}

	return 0;
}

static uint32_t xdx_hdmi_audio_pcmToAudioData(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr, uint64_t src, uint64_t dst, uint32_t *size)
{
	uint16_t *u16_src = (uint16_t *)src;
	uint32_t *u32_src = (uint32_t *)src;
	uint32_t *u32_dst = (uint32_t *)dst;
	uint8_t *u8_src = (uint8_t *)src;
	uint8_t *u8_dst = (uint8_t *)dst;
	uint32_t i = 0, sampleNum = 0;
	struct xdx_device *xdev = xdx_hdmi_audio_ptr->xdev;

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);

	if (xdx_hdmi_audio_ptr->bit_width == 16) {
		sampleNum = *size / 2;
		for (i = 0; i < sampleNum; i++) {
			*(u32_dst + i) = *(u16_src + i);
		}
		*size =  (*size) * 2;
	} else if ((xdx_hdmi_audio_ptr->bit_width == 24) && (xdev->asic_type == CHIP_PANGU_A0)) {
		sampleNum = *size / 2;
		for (i = 0; i < sampleNum; i++) {
			*(u32_dst + i) = *(u32_src + i);
		}
	} else if ((xdx_hdmi_audio_ptr->bit_width == 24) && (xdev->asic_type == CHIP_PANGU_B0)) {
		sampleNum = *size / 3;
		for (i = 0; i < sampleNum; i++) {
			*(u8_dst + i * 4) = *(u8_src + i * 3);
			*(u8_dst + i * 4 + 1) = *(u8_src + i * 3 + 1);
			*(u8_dst + i * 4 + 2) = *(u8_src + i * 3 + 2);
			*(u8_dst + i * 4 + 3) = 0x0;
		}
		*size += *size/3;
	}

	return 0;
}
static uint32_t xdx_hdmi_audio_chnl6data(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr, uint64_t src, uint64_t dst, uint32_t *size)
{
	uint32_t *u32_src = (uint32_t *)src;
	uint32_t *u32_dst = (uint32_t *)dst;
	uint32_t frames = 0, i = 0;
	frames = *size/24;
	for (i = 0; i < frames; i++) {
		*(u32_dst + i * 8) = *(u32_src + i * 6);
		*(u32_dst + i * 8 + 1) = *(u32_src + i * 6 + 1);
		*(u32_dst + i * 8 + 2) = *(u32_src + i * 6 + 2);
		*(u32_dst + i * 8 + 3) = *(u32_src + i * 6 + 3);
		*(u32_dst + i * 8 + 4) = *(u32_src + i * 6 + 4);
		*(u32_dst + i * 8 + 5) = *(u32_src + i * 6 + 5);
		*(u32_dst + i * 8 + 6) = 0x0;
		*(u32_dst + i * 8 + 7) = 0x0;
	}
	*size += *size/3;
	return 0;
}
static void xdx_hdmi_audio_dma_one_period_a0(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr)
{
	struct snd_pcm_substream *substream = xdx_hdmi_audio_ptr->playback_substream;
	struct snd_pcm_runtime *runtime = substream->runtime;
	DMACmd_T dmaCmd;
	uint32_t period_bytes = 0;
	uint32_t hw_avail = 0;
	uint32_t tag = xdx_hdmi_audio_ptr->dmaTag;
	int i = 0, pkt_num = 0;
	uint32_t fifo_avail = 0;
	uint32_t dma_bytes = 0, audio_bytes = 0;
	uint32_t buffer_bytes = 0;
	uint32_t pos = 0;
	uint64_t srcVir = 0;
	unsigned long flags = 0;
	struct xdx_device *xdev = xdx_hdmi_audio_ptr->xdev;

	mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);

	if (xdx_hdmi_audio_ptr->dma_pending_cnt > 0 || (xdx_hdmi_audio_ptr->status != cPlayBackStart) || (xdx_hdmi_audio_ptr->transfer_flag != 1)) {
		xdx_hdmi_audio_ptr->transfer_flag = 0;
		mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);
		return;
	}

	if ((xdx_hdmi_audio_ptr->channels == 2) && (xdx_hdmi_audio_ptr->bit_width == 16))
		period_bytes = PERIOD_BYTES_A02CHNS16BIT;
	else
		period_bytes = snd_pcm_lib_period_bytes(substream);
	hw_avail = frames_to_bytes(runtime, snd_pcm_playback_hw_avail(runtime));
	fifo_avail = snd_pcm_lib_buffer_bytes(substream) - xdx_hdmi_audio_ptr->fifo_bytes;
	dma_bytes = min(period_bytes, hw_avail);
	dma_bytes = min(dma_bytes, fifo_avail);
	buffer_bytes = snd_pcm_lib_buffer_bytes(xdx_hdmi_audio_ptr->playback_substream);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "%s fifo_bytes=0x%x pos=0x%x dma_bytes=0x%x\n", __func__, xdx_hdmi_audio_ptr->fifo_bytes, xdx_hdmi_audio_ptr->pos, dma_bytes);

	if ((dma_bytes + xdx_hdmi_audio_ptr->fifo_bytes) > snd_pcm_lib_buffer_bytes(substream)) {
		xdx_hdmi_audio_ptr->transfer_flag = 0;
		mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s -%d] Transfer dma_bytes and fifo_bytes are overflow.\n", __func__, __LINE__);
		return;
	}

	if (dma_bytes == 0) {
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] fifo_bytes=%d pos=%d hw_avail=%d dma_bytes=%d \n", __func__, __LINE__, xdx_hdmi_audio_ptr->fifo_bytes, xdx_hdmi_audio_ptr->pos, hw_avail, dma_bytes);
		xdx_hdmi_audio_ptr->transfer_flag = 0;
		mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

		return;
	}

	memset(&dmaCmd, 0, 24);
	pos = xdx_hdmi_audio_ptr->pos;
	pkt_num = (xdx_hdmi_audio_ptr->last_bytes + dma_bytes)/xdx_hdmi_audio_ptr->blkSize;
	if (xdx_hdmi_audio_ptr->pos + dma_bytes > MAX_BUFFER_BYTES_A0) {
		dma_bytes = MAX_BUFFER_BYTES_A0 - xdx_hdmi_audio_ptr->pos;
		if ((xdx_hdmi_audio_ptr->last_bytes > 0) && ((dma_bytes + xdx_hdmi_audio_ptr->last_bytes) >= xdx_hdmi_audio_ptr->blkSize)) {
			pkt_num = (xdx_hdmi_audio_ptr->last_bytes + dma_bytes)/xdx_hdmi_audio_ptr->blkSize;
			dma_bytes = pkt_num * xdx_hdmi_audio_ptr->blkSize - xdx_hdmi_audio_ptr->last_bytes;
			memcpy((void*)((uint64_t)xdx_hdmi_audio_ptr->tempBufPtr + xdx_hdmi_audio_ptr->last_bytes), (void*)((uint64_t)runtime->dma_area + xdx_hdmi_audio_ptr->pos), dma_bytes);
			xdx_hdmi_audio_ptr->last_bytes = pkt_num * xdx_hdmi_audio_ptr->blkSize;
		} else if ((dma_bytes + xdx_hdmi_audio_ptr->last_bytes) < xdx_hdmi_audio_ptr->blkSize) {
			memcpy((void*)((uint64_t)xdx_hdmi_audio_ptr->tempBufPtr + xdx_hdmi_audio_ptr->last_bytes), (void*)((uint64_t)runtime->dma_area + xdx_hdmi_audio_ptr->pos), dma_bytes);
			xdx_hdmi_audio_ptr->last_bytes += dma_bytes;
			pos += dma_bytes;
			if (pos >= buffer_bytes)
				pos -= buffer_bytes;
			xdx_hdmi_audio_ptr->pos = pos;

			xdx_hdmi_audio_ptr->transfer_flag = 0;
			mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

			return;
		}
	} else if (xdx_hdmi_audio_ptr->last_bytes > 0) {
		if (dma_bytes >= (xdx_hdmi_audio_ptr->blkSize - xdx_hdmi_audio_ptr->last_bytes)) {
			dma_bytes = pkt_num * xdx_hdmi_audio_ptr->blkSize - xdx_hdmi_audio_ptr->last_bytes;
			memcpy((void*)((uint64_t)xdx_hdmi_audio_ptr->tempBufPtr + xdx_hdmi_audio_ptr->last_bytes), (void*)((uint64_t)runtime->dma_area + xdx_hdmi_audio_ptr->pos), dma_bytes);
			xdx_hdmi_audio_ptr->last_bytes = pkt_num * xdx_hdmi_audio_ptr->blkSize;
		} else if ((dma_bytes + xdx_hdmi_audio_ptr->last_bytes) < xdx_hdmi_audio_ptr->blkSize) {
			memcpy((void*)((uint64_t)xdx_hdmi_audio_ptr->tempBufPtr + xdx_hdmi_audio_ptr->last_bytes), (void*)((uint64_t)runtime->dma_area + xdx_hdmi_audio_ptr->pos), dma_bytes);
			xdx_hdmi_audio_ptr->last_bytes += dma_bytes;
			pos += dma_bytes;
			if (pos >= buffer_bytes)
				pos -= buffer_bytes;
			xdx_hdmi_audio_ptr->pos = pos;

			xdx_hdmi_audio_ptr->transfer_flag = 0;
			mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

			return;
		}
	}

	if ((dma_bytes > xdx_hdmi_audio_ptr->blkSize) && (dma_bytes % xdx_hdmi_audio_ptr->blkSize != 0) && (xdx_hdmi_audio_ptr->last_bytes == 0))
		dma_bytes = pkt_num * xdx_hdmi_audio_ptr->blkSize;
	else if ((dma_bytes < xdx_hdmi_audio_ptr->blkSize) && (xdx_hdmi_audio_ptr->last_bytes == 0)) {
		memcpy((void*)((uint64_t)xdx_hdmi_audio_ptr->tempBufPtr + xdx_hdmi_audio_ptr->last_bytes), (void*)((uint64_t)runtime->dma_area + xdx_hdmi_audio_ptr->pos), dma_bytes);
		xdx_hdmi_audio_ptr->last_bytes += dma_bytes;
		pos += dma_bytes;
		if (pos >= buffer_bytes)
			pos -= buffer_bytes;
		xdx_hdmi_audio_ptr->pos = pos;

		xdx_hdmi_audio_ptr->transfer_flag = 0;
		mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

		return;
	}

	audio_bytes = dma_bytes;
	if (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP) {
		if (xdx_hdmi_audio_ptr->last_bytes == (pkt_num * xdx_hdmi_audio_ptr->blkSize)) {
			srcVir = (uint64_t)(xdx_hdmi_audio_ptr->tempBufPtr);

			xdx_hdmi_audio_pcmToAudioData(xdx_hdmi_audio_ptr, srcVir, (uint64_t)(xdx_hdmi_audio_ptr->bitBufPtr), &xdx_hdmi_audio_ptr->last_bytes);
			if (xdx_hdmi_audio_prepareDataForDp(xdx_hdmi_audio_ptr,  (uint64_t)(xdx_hdmi_audio_ptr->bitBufPtr), (uint64_t)(xdx_hdmi_audio_ptr->periodBufPtr), &xdx_hdmi_audio_ptr->last_bytes) < 0) {
				xdx_hdmi_audio_ptr->transfer_flag = 0;
				mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);
				return;
			}
			dmaCmd.dw0.srcAddr_47_3   = xdx_hdmi_audio_ptr->periodBufPhy >> 3;
		} else if (xdx_hdmi_audio_ptr->last_bytes == 0) {
			srcVir = (uint64_t)(runtime->dma_area + xdx_hdmi_audio_ptr->pos);

			xdx_hdmi_audio_pcmToAudioData(xdx_hdmi_audio_ptr, srcVir, (uint64_t)(xdx_hdmi_audio_ptr->bitBufPtr), &dma_bytes);
			if (xdx_hdmi_audio_prepareDataForDp(xdx_hdmi_audio_ptr, (uint64_t)(xdx_hdmi_audio_ptr->bitBufPtr), (uint64_t)(xdx_hdmi_audio_ptr->periodBufPtr), &dma_bytes) < 0) {
				xdx_hdmi_audio_ptr->transfer_flag = 0;
				mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);
				return;
			}
			dmaCmd.dw0.srcAddr_47_3   = xdx_hdmi_audio_ptr->periodBufPhy >> 3;
		} else {
			xdx_hdmi_audio_ptr->pos = pos;
			xdx_hdmi_audio_ptr->transfer_flag = 0;
			mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);
			snd_pcm_period_elapsed(xdx_hdmi_audio_ptr->playback_substream);
			return;
		}
	} else {
		if (xdx_hdmi_audio_ptr->last_bytes == (pkt_num * xdx_hdmi_audio_ptr->blkSize)) {
			srcVir = (uint64_t)(xdx_hdmi_audio_ptr->tempBufPtr);

			xdx_hdmi_audio_pcmToAudioData(xdx_hdmi_audio_ptr, srcVir, (uint64_t)(xdx_hdmi_audio_ptr->bitBufPtr), &xdx_hdmi_audio_ptr->last_bytes);
			if (xdx_hdmi_audio_prepareDataForHdmi(xdx_hdmi_audio_ptr,  (uint64_t)(xdx_hdmi_audio_ptr->bitBufPtr), (uint64_t)(xdx_hdmi_audio_ptr->periodBufPtr), &xdx_hdmi_audio_ptr->last_bytes) < 0) {
				xdx_hdmi_audio_ptr->transfer_flag = 0;
				mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);
				return;
			}
			dmaCmd.dw0.srcAddr_47_3   = xdx_hdmi_audio_ptr->periodBufPhy >> 3;
		} else if (xdx_hdmi_audio_ptr->last_bytes == 0) {
			srcVir = (uint64_t)(runtime->dma_area + xdx_hdmi_audio_ptr->pos);

			xdx_hdmi_audio_pcmToAudioData(xdx_hdmi_audio_ptr, srcVir, (uint64_t)(xdx_hdmi_audio_ptr->bitBufPtr), &dma_bytes);
			if (xdx_hdmi_audio_prepareDataForHdmi(xdx_hdmi_audio_ptr, (uint64_t)(xdx_hdmi_audio_ptr->bitBufPtr), (uint64_t)(xdx_hdmi_audio_ptr->periodBufPtr), &dma_bytes) < 0) {
			        xdx_hdmi_audio_ptr->transfer_flag = 0;
				mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);
				return;
			}
			dmaCmd.dw0.srcAddr_47_3   = xdx_hdmi_audio_ptr->periodBufPhy >> 3;
		} else {
			xdx_hdmi_audio_ptr->pos = pos;
			xdx_hdmi_audio_ptr->transfer_flag = 0;
			mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);
			snd_pcm_period_elapsed(xdx_hdmi_audio_ptr->playback_substream);
			return;
		}
	}

	dmaCmd.dw0.srcAddrMode  = 1;   // Source Address Mode:0-HOST_SVA, 1-HOST_SPA, 2-GPU_SVA, 3-GPU_GPA
	dmaCmd.dw0.intrEn = 1;         // Interrupt Enable: 1-set intsts when cmd cmpl
	dmaCmd.dw0.commandTag = tag;       // Command Tag
	dmaCmd.dw0.opCode = 0x80;           // Operation Code

	dmaCmd.dw1.dstAddr_47_3 = 0x92200000>>3;       // Destination Address[47:3]
	dmaCmd.dw1.dstAddrMode  = 3;   // Source Address Mode:0-HOST_SVA, 1-HOST_SPA, 2-GPU_SVA, 3-GPU_GPA
	if (xdx_hdmi_audio_ptr->last_bytes != 0)
		dmaCmd.dw1.transferLen = (xdx_hdmi_audio_ptr->last_bytes>>3) - 1;
	else
		dmaCmd.dw1.transferLen = (dma_bytes>>3) - 1;        // Transfer Length

	dmaCmd.dw2.pcieFuncNum = 1;

	++tag;
	tag &= XDX_VDMA_TAG_CMPL_INTRSTS0;	//Tag31-0
	xdx_hdmi_audio_ptr->dmaTag = tag;
	xdx_hdmi_audio_ptr->dma_bytes = audio_bytes;
	spin_lock_irqsave(&xdx_hdmi_audio_ptr->lock, flags);
	if (xdx_hdmi_audio_ptr->last_bytes != 0) {
		xdx_hdmi_audio_ptr->fifo_bytes += xdx_hdmi_audio_ptr->last_bytes;
		xdx_hdmi_audio_ptr->last_bytes = 0;
	} else
		xdx_hdmi_audio_ptr->fifo_bytes += dma_bytes;
	xdx_hdmi_audio_ptr->fifo_empty = 0;
	spin_unlock_irqrestore(&xdx_hdmi_audio_ptr->lock, flags);

	xdx_hdmi_audio_ptr->dma_pending_cnt = 1;
	xdx_hdmi_audio_ptr->transfer_flag = 0;
	mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

	if (xdev->shareram_size == XDX_AUDIO_BAR4_16M) {
		for (i = 0; i < 6; ++i) {
			iowrite32(dmaCmd.cmd32[i], xdx_hdmi_audio_ptr->iobase + 0xD00000 + 0x2800 + 0x200 + i*4);
		}
		OSWriteAUHWReg64(xdx_hdmi_audio_ptr->iobase, 0xc00428, 0x1);
	} else if (xdev->shareram_size == XDX_AUDIO_BAR4_1M) {
		for (i = 0; i < 6; ++i) {
			iowrite32(dmaCmd.cmd32[i], xdx_hdmi_audio_ptr->iobase_vdma1 + 0x2800 + 0x200 + i*4);
		}
	}
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s -%d] Write VDMA register.\n", __func__, __LINE__);
}

static void xdx_hdmi_audio_dma_one_period_b0(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr)
{
	struct snd_pcm_substream *substream = xdx_hdmi_audio_ptr->playback_substream;
	struct snd_pcm_runtime *runtime = substream->runtime;
	VDMA_B0_CmdData_T dmaCmd;
	uint32_t period_bytes = snd_pcm_lib_period_bytes(substream);
	uint32_t hw_avail = 0;
	uint32_t tag = xdx_hdmi_audio_ptr->dmaTag;
	int i = 0, n = 0;
	uint32_t fifo_avail = 0;
	uint32_t dma_bytes = 0, audio_bytes = 0, temp_bytes = 0, chnl_head = 0, buf_num = 0;
	uint32_t buffer_bytes = 0;
	uint32_t pos = 0;
	uint64_t srcVir = 0;
	unsigned long flags = 0;

	mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);

	if (xdx_hdmi_audio_ptr->dma_pending_cnt > 0 || (xdx_hdmi_audio_ptr->status != cPlayBackStart) || (xdx_hdmi_audio_ptr->transfer_flag != 1)) {
		xdx_hdmi_audio_ptr->transfer_flag = 0;
		mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);
		return;
	}

	hw_avail = frames_to_bytes(runtime, snd_pcm_playback_hw_avail(runtime));
	fifo_avail = AUDIO_FIFO_SIZE_B0 - xdx_hdmi_audio_ptr->fifo_bytes;
	if (xdx_hdmi_audio_ptr->bit_width  == 16) {
		switch (xdx_hdmi_audio_ptr->channels) {
			case 2:
				temp_bytes = DMA_BYTES_16BIT2CHS;
				chnl_head = CHNL_HEAD_2CHS;
				buf_num = BUF_NUM_2CHS;
				break;
			case 4:
				temp_bytes = DMA_BYTES_16BIT4CHS;
				chnl_head = CHNL_HEAD_4CHS;
				buf_num = BUF_NUM_4CHS;
				break;
			case 6:
				temp_bytes = DMA_BYTES_16BIT6CHS;
				chnl_head = CHNL_HEAD_6CHS;
				buf_num = BUF_NUM_6CHS;
				break;
			case 8:
				temp_bytes = DMA_BYTES_16BIT8CHS;
				chnl_head = CHNL_HEAD_8CHS;
				buf_num = BUF_NUM_8CHS;
				break;

		}
	} else if (xdx_hdmi_audio_ptr->bit_width  == 24) {
		switch (xdx_hdmi_audio_ptr->channels) {
			case 2:
				temp_bytes = DMA_BYTES_24BIT2CHS;
				chnl_head = CHNL_HEAD_2CHS;
				buf_num = BUF_NUM_2CHS;
				break;
			case 4:
				temp_bytes = DMA_BYTES_24BIT4CHS;
				chnl_head = CHNL_HEAD_4CHS;
				buf_num = BUF_NUM_4CHS;
				break;
			case 6:
				temp_bytes = DMA_BYTES_24BIT6CHS;
				chnl_head = CHNL_HEAD_6CHS;
				buf_num = BUF_NUM_6CHS;
				break;
			case 8:
				temp_bytes = DMA_BYTES_24BIT8CHS;
				chnl_head = CHNL_HEAD_8CHS;
				buf_num = BUF_NUM_8CHS;
				break;

		}
	}
	dma_bytes = min(period_bytes, hw_avail);
	dma_bytes = min(fifo_avail, dma_bytes);
	dma_bytes = min(temp_bytes, dma_bytes);

	buffer_bytes = snd_pcm_lib_buffer_bytes(xdx_hdmi_audio_ptr->playback_substream);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "%s fifo_bytes=0x%x pos=0x%x dma_bytes=0x%x\n", __func__, xdx_hdmi_audio_ptr->fifo_bytes, xdx_hdmi_audio_ptr->pos, dma_bytes);

	if ((dma_bytes + AUDIO_FIFO_PERIOD_BYTES) > AUDIO_FIFO_SIZE_B0) {
		xdx_hdmi_audio_ptr->transfer_flag = 0;
		mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s -%d] Transfer dma_bytes and fifo_bytes are overflow.\n", __func__, __LINE__);
		return;
	}

	if (dma_bytes == 0) {
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] fifo_bytes=%d pos=%d hw_avail=%d dma_bytes=%d \n", __func__, __LINE__, xdx_hdmi_audio_ptr->fifo_bytes, xdx_hdmi_audio_ptr->pos, hw_avail, dma_bytes);
		xdx_hdmi_audio_ptr->transfer_flag = 0;
        	mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

 		return;
	}

	memset(&dmaCmd, 0, 24);
	pos = xdx_hdmi_audio_ptr->pos;
	if (xdx_hdmi_audio_ptr->pos + dma_bytes >= MAX_BUFFER_BYTES_B0) {
		dma_bytes = MAX_BUFFER_BYTES_B0 - xdx_hdmi_audio_ptr->pos;
		if ((xdx_hdmi_audio_ptr->last_bytes > 0) && ((dma_bytes + xdx_hdmi_audio_ptr->last_bytes) >= temp_bytes)) {
			dma_bytes = temp_bytes - xdx_hdmi_audio_ptr->last_bytes;
			memcpy((void*)((uint64_t)xdx_hdmi_audio_ptr->tempBufPtr + xdx_hdmi_audio_ptr->last_bytes), (void*)((uint64_t)runtime->dma_area + xdx_hdmi_audio_ptr->pos), dma_bytes);
			xdx_hdmi_audio_ptr->last_bytes = temp_bytes;
		} else if ((dma_bytes + xdx_hdmi_audio_ptr->last_bytes) < temp_bytes) {
			memcpy((void*)((uint64_t)xdx_hdmi_audio_ptr->tempBufPtr + xdx_hdmi_audio_ptr->last_bytes), (void*)((uint64_t)runtime->dma_area + xdx_hdmi_audio_ptr->pos), dma_bytes);
			xdx_hdmi_audio_ptr->last_bytes += dma_bytes;
			pos += dma_bytes;
			if (pos >= buffer_bytes)
				pos -= buffer_bytes;
			xdx_hdmi_audio_ptr->pos = pos;

			xdx_hdmi_audio_ptr->transfer_flag = 0;
			mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

			return;
		}
	} else if (xdx_hdmi_audio_ptr->last_bytes > 0) {
		if (dma_bytes >= (temp_bytes - xdx_hdmi_audio_ptr->last_bytes)) {
			dma_bytes = temp_bytes - xdx_hdmi_audio_ptr->last_bytes;
			memcpy((void*)((uint64_t)xdx_hdmi_audio_ptr->tempBufPtr + xdx_hdmi_audio_ptr->last_bytes), (void*)((uint64_t)runtime->dma_area + xdx_hdmi_audio_ptr->pos), dma_bytes);
			xdx_hdmi_audio_ptr->last_bytes = temp_bytes;
		} else if ((dma_bytes + xdx_hdmi_audio_ptr->last_bytes) < temp_bytes) {
			memcpy((void*)((uint64_t)xdx_hdmi_audio_ptr->tempBufPtr + xdx_hdmi_audio_ptr->last_bytes), (void*)((uint64_t)runtime->dma_area + xdx_hdmi_audio_ptr->pos), dma_bytes);
			xdx_hdmi_audio_ptr->last_bytes += dma_bytes;
			pos += dma_bytes;
			if (pos >= buffer_bytes)
				pos -= buffer_bytes;
			xdx_hdmi_audio_ptr->pos = pos;

			xdx_hdmi_audio_ptr->transfer_flag = 0;
			mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

			return;
		}
	}

	if ((dma_bytes > temp_bytes) && (dma_bytes % temp_bytes != 0) && (xdx_hdmi_audio_ptr->last_bytes == 0))
		dma_bytes = temp_bytes;
	else if ((dma_bytes < temp_bytes) && (xdx_hdmi_audio_ptr->last_bytes == 0)) {
		memcpy((void*)((uint64_t)xdx_hdmi_audio_ptr->tempBufPtr + xdx_hdmi_audio_ptr->last_bytes), (void*)((uint64_t)runtime->dma_area + xdx_hdmi_audio_ptr->pos), dma_bytes);
		xdx_hdmi_audio_ptr->last_bytes += dma_bytes;
		pos += dma_bytes;
		if (pos >= buffer_bytes)
			pos -= buffer_bytes;
		xdx_hdmi_audio_ptr->pos = pos;

		xdx_hdmi_audio_ptr->transfer_flag = 0;
		mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

		return;
	}

	audio_bytes = dma_bytes;
	if (xdx_hdmi_audio_ptr->last_bytes == temp_bytes) {
		srcVir = (uint64_t)(xdx_hdmi_audio_ptr->tempBufPtr);

		xdx_hdmi_audio_pcmToAudioData(xdx_hdmi_audio_ptr, srcVir, (uint64_t)(xdx_hdmi_audio_ptr->bitBufPtr), &xdx_hdmi_audio_ptr->last_bytes);

		if (xdx_hdmi_audio_ptr->channels == 6)
			xdx_hdmi_audio_chnl6data(xdx_hdmi_audio_ptr, (uint64_t)(xdx_hdmi_audio_ptr->bitBufPtr), (uint64_t)(xdx_hdmi_audio_ptr->chnl6BufPtr), &dma_bytes);

		for (i = 0; i < xdx_hdmi_audio_ptr->last_bytes/xdx_hdmi_audio_ptr->blkSize; i++) {
			if (xdx_hdmi_audio_ptr->channels == 6)
				xdx_hdmi_audio_prepareData(xdx_hdmi_audio_ptr, (uint64_t)(xdx_hdmi_audio_ptr->chnl6BufPtr) + i * chnl_head * B0_AUDIOIF_SUB_FRAME_IN_BYTES,
						(uint64_t)(xdx_hdmi_audio_ptr->periodBufPtr) + i * chnl_head * B0_AUDIOIF_SUB_FRAME_OU_BYTES + chnl_head * B0_AUDIOIF_PKT_HEAD_OFFSET);
			else
				xdx_hdmi_audio_prepareData(xdx_hdmi_audio_ptr, (uint64_t)(xdx_hdmi_audio_ptr->bitBufPtr) + i * chnl_head * B0_AUDIOIF_SUB_FRAME_IN_BYTES,
						(uint64_t)(xdx_hdmi_audio_ptr->periodBufPtr) + i * chnl_head * B0_AUDIOIF_SUB_FRAME_OU_BYTES + chnl_head * B0_AUDIOIF_PKT_HEAD_OFFSET);
		}
		dmaCmd.dw0.srcAddr_47_3   = xdx_hdmi_audio_ptr->periodBufPhy >> 3;
	} else if (xdx_hdmi_audio_ptr->last_bytes == 0){
		srcVir = (uint64_t)(runtime->dma_area + xdx_hdmi_audio_ptr->pos);

		xdx_hdmi_audio_pcmToAudioData(xdx_hdmi_audio_ptr, srcVir, (uint64_t)(xdx_hdmi_audio_ptr->bitBufPtr), &dma_bytes);

		if (xdx_hdmi_audio_ptr->channels == 6)
			xdx_hdmi_audio_chnl6data(xdx_hdmi_audio_ptr, (uint64_t)(xdx_hdmi_audio_ptr->bitBufPtr), (uint64_t)(xdx_hdmi_audio_ptr->chnl6BufPtr), &dma_bytes);

		for (i = 0; i < dma_bytes/xdx_hdmi_audio_ptr->blkSize; i++) {
			if (xdx_hdmi_audio_ptr->channels == 6)
				xdx_hdmi_audio_prepareData(xdx_hdmi_audio_ptr, (uint64_t)(xdx_hdmi_audio_ptr->chnl6BufPtr) + i * chnl_head * B0_AUDIOIF_SUB_FRAME_IN_BYTES,
						(uint64_t)(xdx_hdmi_audio_ptr->periodBufPtr) + i * chnl_head * B0_AUDIOIF_SUB_FRAME_OU_BYTES + chnl_head * B0_AUDIOIF_PKT_HEAD_OFFSET);
			else
				xdx_hdmi_audio_prepareData(xdx_hdmi_audio_ptr, (uint64_t)(xdx_hdmi_audio_ptr->bitBufPtr) + i * chnl_head * B0_AUDIOIF_SUB_FRAME_IN_BYTES,
						(uint64_t)(xdx_hdmi_audio_ptr->periodBufPtr) + i * chnl_head * B0_AUDIOIF_SUB_FRAME_OU_BYTES + chnl_head * B0_AUDIOIF_PKT_HEAD_OFFSET);
		}
		dmaCmd.dw0.srcAddr_47_3   = xdx_hdmi_audio_ptr->periodBufPhy >> 3;
	} else {
		xdx_hdmi_audio_ptr->pos = pos;
		xdx_hdmi_audio_ptr->transfer_flag = 0;
		mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);
		return;
	}

	dmaCmd.dw0.opCode = 0x80;           // Operation Code
	dmaCmd.dw0.LogicOp = 0;
	dmaCmd.dw0.Tag = tag;	// Command Tag
	dmaCmd.dw1.LenUnit = 0;
	dmaCmd.dw2.PcieFuncNum = 0;
	dmaCmd.dw2.PcieDontSnoop = 0;
	dmaCmd.dw2.IntrEn = 0x1;

	//before transfer, judge wheteher audio data lenght exceed the limit
	if (dma_bytes > AUDIO_RESERVE_VRAM_LIMIT) {
		dev_err(xdx_hdmi_audio_ptr->dev, "dma_bytes exceed AUDIO_RESERVE_VRAM_LIMIT!");
		return;
	}

	if (xdx_hdmi_audio_ptr->last_bytes != 0)
                n = xdx_hdmi_audio_ptr->last_bytes/(chnl_head * B0_AUDIOIF_SUB_FRAME_IN_BYTES);
        else
                n = dma_bytes/(chnl_head * B0_AUDIOIF_SUB_FRAME_IN_BYTES);

	if (xdx_hdmi_audio_ptr->ptr == 0) {
                xdx_hdmi_audio_ptr->buf_ptr = -1;
                xdx_hdmi_audio_ptr->buf_ptr = (xdx_hdmi_audio_ptr->buf_ptr + n) % buf_num;
                xdx_hdmi_audio_ptr->ptr++;
        } else
                xdx_hdmi_audio_ptr->buf_ptr = (xdx_hdmi_audio_ptr->buf_ptr + n) % buf_num;

	dmaCmd.dw1.DstAddr_47_3 = (XDX_GDDR0_BASE_ADDR + xdx_b0_audioif_buf_gddr_offset + xdx_hdmi_audio_ptr->ptr_bytes)>>3;

	dmaCmd.dw2.DstAddrMode = 0xf;
	dmaCmd.dw2.SrcAddrMode = 0x0;
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev,"[%s +%d] (uint64_t)(dmaCmd.dw0.SrcAddr_47_3)=0x%llx, (uint64_t)(dmaCmd.dw1.DstAddr_47_3)=0x%llx\n", __func__, __LINE__, (uint64_t)(dmaCmd.dw0.srcAddr_47_3), (uint64_t)(dmaCmd.dw1.DstAddr_47_3));

	if (xdx_hdmi_audio_ptr->last_bytes != 0)
		dmaCmd.dw1.Xstep_SrcDescNum = (xdx_hdmi_audio_ptr->last_bytes * B0_AUDIOIF_SUB_FRAME_OU_BYTES/B0_AUDIOIF_SUB_FRAME_IN_BYTES/8) - 1;
        else
		dmaCmd.dw1.Xstep_SrcDescNum = (dma_bytes * B0_AUDIOIF_SUB_FRAME_OU_BYTES/B0_AUDIOIF_SUB_FRAME_IN_BYTES/8) - 1;

	dmaCmd.dw2.Ystep_DstDescNum_DstXferLen_DWriteLenHigh = 0;
	dmaCmd.dw2.DstrideHigh_PatternLoc = 0;
	dmaCmd.dw2.DstrideLow_PatternLen = 0;
	dmaCmd.dw2.Sstride_DWriteLenLow = 0;

	++tag;
	tag &= XDX_VDMA_TAG_CMPL_INTRSTS0;      //Tag31-0
	xdx_hdmi_audio_ptr->dmaTag = tag;
	xdx_hdmi_audio_ptr->dma_bytes = audio_bytes;

	spin_lock_irqsave(&xdx_hdmi_audio_ptr->lock, flags);
	if (xdx_hdmi_audio_ptr->last_bytes != 0) {
		xdx_hdmi_audio_ptr->fifo_bytes += xdx_hdmi_audio_ptr->last_bytes + n * chnl_head * B0_AUDIOIF_PKT_HEAD_OFFSET * 2;
		xdx_hdmi_audio_ptr->last_bytes = 0;
	} else
		xdx_hdmi_audio_ptr->fifo_bytes += dma_bytes + n * chnl_head * B0_AUDIOIF_PKT_HEAD_OFFSET * 2;
	xdx_hdmi_audio_ptr->fifo_empty = 0;
	xdx_hdmi_audio_ptr->dma_pending_cnt = 1;
	spin_unlock_irqrestore(&xdx_hdmi_audio_ptr->lock, flags);

	xdx_hdmi_audio_ptr->ptr_bytes = (xdx_hdmi_audio_ptr->buf_ptr + 1) * chnl_head * B0_AUDIOIF_SUB_FRAME_OU_BYTES;
	if (xdx_hdmi_audio_ptr->ptr_bytes >= AUDIO_FIFO_SIZE_B0)
                xdx_hdmi_audio_ptr->ptr_bytes = AUDIO_FIFO_SIZE_B0 - xdx_hdmi_audio_ptr->ptr_bytes;

	xdx_hdmi_audio_ptr->transfer_flag = 0;
	mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

	for (i = 0; i < 6; ++i) {
		iowrite32(dmaCmd.cmd32[i], xdx_hdmi_audio_ptr->iobase_vdma1 + 0x2800 + 0x400 + i*4);
	}
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s -%d] Write VDMA register.\n", __func__, __LINE__);
}

static int snd_xdx_hdmi_audio_trigger(struct snd_pcm_substream *substream,
                               int cmd)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = snd_pcm_substream_chip(substream);
	uint64_t time_close  = 0, timeout_close = 0, time_now = 0;
	struct xdx_device *xdev = xdx_hdmi_audio_ptr->xdev;
	Audio_B0_Regs_T *pAudioB0Reg = (Audio_B0_Regs_T *)(xdx_hdmi_audio_ptr->iobase_audio0);
	Audio_B0_Regs_T *pAudio1B0Reg = (Audio_B0_Regs_T *)(xdx_hdmi_audio_ptr->iobase_audio1);

	switch (cmd) {
		case SNDRV_PCM_TRIGGER_START:
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] start\n", __func__, __LINE__);

			mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
			xdx_hdmi_audio_ptr->status = cPlayBackStart;
			mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

			if (xdx_hdmi_audio_ptr->last_bytes == 0)
				xdx_hdmi_audio_ptr->pos = 0;
			xdx_hdmi_audio_ptr->fifo_bytes = 0;
			xdx_hdmi_audio_ptr->fifo_empty = 1;
			xdx_hdmi_audio_ptr->dma_pending_cnt = 0;

			xdx_hdmi_audio_ptr->transfer_flag = 1;
			(void) queue_work(xdx_hdmi_audio_ptr->pWorkQueue, &xdx_hdmi_audio_ptr->audioIntWork);
			break;

		case SNDRV_PCM_TRIGGER_STOP:
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] stop\n", __func__, __LINE__);

			mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
			xdx_hdmi_audio_ptr->status = cPlayBackStop;
			mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

			if (xdev->asic_type == CHIP_PANGU_A0) {
				mutex_lock(xdx_hdmi_audio_ptr->hw_mutex);
				OSWriteAUHWReg32(xdx_hdmi_audio_ptr->iobase_audio0, 0x8, 0);
				mutex_unlock(xdx_hdmi_audio_ptr->hw_mutex);
			}

			break;

		case SNDRV_PCM_TRIGGER_SUSPEND:
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] suspend\n", __func__, __LINE__);

			mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
			xdx_hdmi_audio_ptr->status = cPlayBackStop;
			mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);
			if (xdev->asic_type == CHIP_PANGU_B0) {
				if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI1) || (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP1))
					pAudio1B0Reg->TXEN.all = 0;
				else
					pAudioB0Reg->TXEN.all = 0;
			}
			timeout_close = msecs_to_jiffies(300);
			time_close = get_jiffies_64();
			while (!(xdx_hdmi_audio_ptr->fifo_empty)) {
				udelay(1000);
				time_now = get_jiffies_64();
				if (time_now > time_close + timeout_close) {
					XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]Wait fifo_empty timeout!\n", __func__, __LINE__);
					break;
				}
			}
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);
			break;

		default:
			return -EINVAL;
	}

	return 0;
}

static snd_pcm_uframes_t snd_xdx_hdmi_audio_playback_pointer(struct snd_pcm_substream *substream)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = snd_pcm_substream_chip(substream);
	struct snd_pcm_runtime *runtime = substream->runtime;
	snd_pcm_uframes_t pos = 0;

	pos = bytes_to_frames(runtime, xdx_hdmi_audio_ptr->pos);

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "%s %d pos bytes=0x%x frames=0x%lx\n", __func__, __LINE__, xdx_hdmi_audio_ptr->pos, pos);

	if ((xdx_hdmi_audio_ptr->fifo_empty == 1) && (xdx_hdmi_audio_ptr->dma_pending_cnt == 0) && (xdx_hdmi_audio_ptr->transfer_flag == 0) && (xdx_hdmi_audio_ptr->status == cPlayBackStart)) {

		xdx_hdmi_audio_ptr->transfer_flag = 1;
		(void) queue_work(xdx_hdmi_audio_ptr->pWorkQueue, &xdx_hdmi_audio_ptr->audioIntWork);
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "%s %d Continuous transmission of data streams\n", __func__, __LINE__);
	}
	return pos;
}

static const struct snd_pcm_ops snd_xdx_hdmi_audio_playback_ops = {
	.open      =	snd_xdx_hdmi_audio_playback_open,
	.close     =	snd_xdx_hdmi_audio_playback_close,
	.ioctl     =	snd_pcm_lib_ioctl,
	.hw_params =	snd_xdx_hdmi_audio_hw_params,
	.hw_free   =	snd_xdx_hdmi_audio_hw_free,
	.prepare   =	snd_xdx_hdmi_audio_prepare,
	.trigger   =	snd_xdx_hdmi_audio_trigger,
	.pointer   =	snd_xdx_hdmi_audio_playback_pointer,
};

static void pangu_disable_irq(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr)
{
	int i = 0;
	int irqvec = 0;
	for (i = 0; i < xdx_hdmi_audio_ptr->irq; i++) {
		irqvec = pci_irq_vector(xdx_hdmi_audio_ptr->pci, i);
		free_irq(irqvec, xdx_hdmi_audio_ptr);
	}
	pci_free_irq_vectors(xdx_hdmi_audio_ptr->pci);
}

static int snd_xdx_hdmi_audio_free(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr)
{
	struct xdx_device *xdev = xdx_hdmi_audio_ptr->xdev;
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);

	destroy_workqueue(xdx_hdmi_audio_ptr->pWorkQueue);

	if (xdx_hdmi_audio_ptr->irq >= 0) {
		pangu_disable_irq(xdx_hdmi_audio_ptr);
		xdx_hdmi_audio_ptr->irq = -1;
	}

	snd_device_free(xdx_hdmi_audio_ptr->card, xdx_hdmi_audio_ptr);

	if (xdx_hdmi_audio_ptr->iobase != NULL) {
		iounmap(xdx_hdmi_audio_ptr->iobase);
		xdx_hdmi_audio_ptr->iobase = NULL;
	}

	if (xdx_hdmi_audio_ptr->dma_period.area) {
		snd_dma_free_pages(&xdx_hdmi_audio_ptr->dma_period);
		xdx_hdmi_audio_ptr->dma_period.area = NULL;
	}

	if (xdx_hdmi_audio_ptr->transfer_mutex != NULL)
		kfree(xdx_hdmi_audio_ptr->transfer_mutex);

	if (xdx_hdmi_audio_ptr->hw_mutex != NULL)
		kfree(xdx_hdmi_audio_ptr->hw_mutex);

	if (xdx_hdmi_audio_ptr->tempBuf.area) {
		snd_dma_free_pages(&xdx_hdmi_audio_ptr->tempBuf);
		xdx_hdmi_audio_ptr->tempBuf.area = NULL;
	}

	if (xdx_hdmi_audio_ptr->bitBuf.area) {
		snd_dma_free_pages(&xdx_hdmi_audio_ptr->bitBuf);
		xdx_hdmi_audio_ptr->bitBuf.area = NULL;
	}

	if (xdx_hdmi_audio_ptr->chnl6Buf.area) {
		snd_dma_free_pages(&xdx_hdmi_audio_ptr->chnl6Buf);
		xdx_hdmi_audio_ptr->chnl6Buf.area = NULL;
	}

	if (xdx_hdmi_audio_ptr->iobase_audio0 != NULL) {
		iounmap(xdx_hdmi_audio_ptr->iobase_audio0);
		xdx_hdmi_audio_ptr->iobase_audio0 = NULL;
	}

	if (xdx_hdmi_audio_ptr->iobase_audio1 != NULL) {
		iounmap(xdx_hdmi_audio_ptr->iobase_audio1);
		xdx_hdmi_audio_ptr->iobase_audio1 = NULL;
	}

	if (xdx_hdmi_audio_ptr->iobase_vdma1 != NULL) {
		iounmap(xdx_hdmi_audio_ptr->iobase_vdma1);
		xdx_hdmi_audio_ptr->iobase_vdma1 = NULL;
	}

	if (xdx_hdmi_audio_ptr->iobase_acpu != NULL) {
		iounmap(xdx_hdmi_audio_ptr->iobase_acpu);
		xdx_hdmi_audio_ptr->iobase_acpu = NULL;
	}

	if (xdx_hdmi_audio_ptr->iobase_dp0 != NULL) {
		iounmap(xdx_hdmi_audio_ptr->iobase_dp0);
		xdx_hdmi_audio_ptr->iobase_dp0 = NULL;
	}

	if (xdx_hdmi_audio_ptr->iobase_dp1 != NULL) {
		iounmap(xdx_hdmi_audio_ptr->iobase_dp1);
		xdx_hdmi_audio_ptr->iobase_dp1 = NULL;
	}

	if (xdev->shareram_size == XDX_AUDIO_BAR4_16M) {
		pci_release_regions(xdx_hdmi_audio_ptr->pci);
		pci_disable_device(xdx_hdmi_audio_ptr->pci);
	}
	kfree(xdx_hdmi_audio_ptr);

	return 0;
}

static int snd_xdx_hdmi_audio_dev_free(struct snd_device *device)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = device->device_data;
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] Freeing device\n", __func__, __LINE__);

	return snd_xdx_hdmi_audio_free(xdx_hdmi_audio_ptr);
}

static void xdx_hdmi_audio_int_work(struct work_struct *data)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = container_of(data, struct snd_xdx_hdmi_audio, audioIntWork);
	struct xdx_device *xdev = xdx_hdmi_audio_ptr->xdev;

	if (xdev->asic_type == CHIP_PANGU_A0)
		xdx_hdmi_audio_dma_one_period_a0(xdx_hdmi_audio_ptr);
	else if (xdev->asic_type == CHIP_PANGU_B0)
	        xdx_hdmi_audio_dma_one_period_b0(xdx_hdmi_audio_ptr);
}

static irqreturn_t pangu_a0_audiof1_irq_handler(int irq, void *data)
{
	irqreturn_t ret = IRQ_NONE;
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = (struct snd_xdx_hdmi_audio *)data;
	uint64_t intStatus = 0;
	uint32_t period_bytes = 0;
	uint32_t buffer_bytes = 0;
	DBCmd_a0f1_T dbCmd_a0f1;

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] irq %d index %d\n", __func__, __LINE__, irq, pci_irq_vector(xdx_hdmi_audio_ptr->pci, PANGU_INTERRUPT_H1_START));

	if (!xdx_hdmi_audio_ptr->playback_substream->runtime) {
		/* workqueue to transfer data is destroyed during system poweroff */
		return IRQ_HANDLED;
	}

	period_bytes = snd_pcm_lib_period_bytes(xdx_hdmi_audio_ptr->playback_substream);
	buffer_bytes = snd_pcm_lib_buffer_bytes(xdx_hdmi_audio_ptr->playback_substream);

	if (irq == pci_irq_vector(xdx_hdmi_audio_ptr->pci, PANGU_INTERRUPT_ACPU)) {
		intStatus = OSReadAUHWReg64(xdx_hdmi_audio_ptr->iobase, 0x1002e8);
		dbCmd_a0f1.all = OSReadAUHWReg64(xdx_hdmi_audio_ptr->iobase, 0x100540);

		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "%s %d dbCmd_a0f1.cmd=%d\n", __func__, __LINE__, dbCmd_a0f1.cmd);

		if (dbCmd_a0f1.cmd == 2)
			xdx_hdmi_audio_ptr->fifo_bytes = buffer_bytes/2;
		else if (dbCmd_a0f1.cmd == 3) {
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "empty %s %d period_bytes=0x%x buffer_bytes=0x%x fifo_bytes=0x%x pos=0x%x status=%d peding_cnt=%d\n", __func__, __LINE__,
					period_bytes, buffer_bytes, xdx_hdmi_audio_ptr->fifo_bytes, xdx_hdmi_audio_ptr->pos, xdx_hdmi_audio_ptr->status, xdx_hdmi_audio_ptr->dma_pending_cnt);
			xdx_hdmi_audio_ptr->fifo_bytes = 0;
			xdx_hdmi_audio_ptr->fifo_empty = 1;
		} else if (dbCmd_a0f1.cmd == 4)
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s -%d] iis idle\n", __func__, __LINE__);
		else {
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s -%d] dbCmd_a0f1.cmd is error!\n", __func__, __LINE__);
			return IRQ_HANDLED;
		}

		OSWriteAUHWReg64(xdx_hdmi_audio_ptr->iobase, 0x1001c8, intStatus);
		dbCmd_a0f1.low32 |= 1;

		OSWriteAUHWReg32(xdx_hdmi_audio_ptr->iobase, 0x100540, dbCmd_a0f1.low32);
	}

	if (irq == pci_irq_vector(xdx_hdmi_audio_ptr->pci, PANGU_INTERRUPT_VD1MCU)) {
		intStatus = OSReadAUHWReg64(xdx_hdmi_audio_ptr->iobase, 0xc00328);
		dbCmd_a0f1.all = OSReadAUHWReg64(xdx_hdmi_audio_ptr->iobase, 0xc00430);

		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "%s %d dbCmd_a0f0.cmd=%d\n", __func__, __LINE__, dbCmd_a0f1.cmd);

		if (dbCmd_a0f1.cmd==1) {
			xdx_hdmi_audio_ptr->pos += xdx_hdmi_audio_ptr->dma_bytes;
			if(xdx_hdmi_audio_ptr->pos >= buffer_bytes)
				xdx_hdmi_audio_ptr->pos -= buffer_bytes;

			snd_pcm_period_elapsed(xdx_hdmi_audio_ptr->playback_substream);
			if (xdx_hdmi_audio_ptr->dma_pending_cnt != 1)
				XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s -%d] Receive invalid interrupt.\n", __func__, __LINE__);
			else
				xdx_hdmi_audio_ptr->dma_pending_cnt = 0;
		} else
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s -%d] Receive DMA interrupt is not 1.\n", __func__, __LINE__);

		OSWriteAUHWReg64(xdx_hdmi_audio_ptr->iobase, 0xc00208, intStatus);
		dbCmd_a0f1.low32 |= 1;

		OSWriteAUHWReg32(xdx_hdmi_audio_ptr->iobase, 0xc00430, dbCmd_a0f1.low32);
	}

	if (((xdx_hdmi_audio_ptr->fifo_bytes + period_bytes) <= buffer_bytes) && (xdx_hdmi_audio_ptr->status == cPlayBackStart) && (xdx_hdmi_audio_ptr->dma_pending_cnt == 0)) {
		xdx_hdmi_audio_ptr->transfer_flag = 1;
		(void) queue_work(xdx_hdmi_audio_ptr->pWorkQueue, &xdx_hdmi_audio_ptr->audioIntWork);
	} else
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "%s %d irq %d period_bytes=0x%x buffer_bytes=0x%x fifo_bytes=0x%x pos=0x%x status=%d dma_pending_cnt=%d\n", __func__, __LINE__,
				irq, period_bytes, buffer_bytes, xdx_hdmi_audio_ptr->fifo_bytes, xdx_hdmi_audio_ptr->pos, xdx_hdmi_audio_ptr->status, xdx_hdmi_audio_ptr->dma_pending_cnt);

	ret = IRQ_HANDLED;

	return ret;
}

void pangu_a0_audiof0_db_interrupt(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr)
{
	uint64_t intStatus = 0;
	uint32_t period_bytes = 0;
	uint32_t buffer_bytes = 0;
	DBCmd_T dbCmd;

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]  index %d\n", __func__, __LINE__,  pci_irq_vector(xdx_hdmi_audio_ptr->pci, PANGU_INTERRUPT_H1_START));

	if (!xdx_hdmi_audio_ptr->playback_substream->runtime) {
		/* workqueue to transfer data is destroyed during system poweroff */
		return;
	}

	period_bytes = snd_pcm_lib_period_bytes(xdx_hdmi_audio_ptr->playback_substream);
	buffer_bytes = snd_pcm_lib_buffer_bytes(xdx_hdmi_audio_ptr->playback_substream);
	if (xdx_hdmi_audio_ptr->bit_width == 16)
		period_bytes = period_bytes*2;

	intStatus = OSReadAUHWReg64(xdx_hdmi_audio_ptr->iobase_acpu, 0x1002e0);
	dbCmd.all = OSReadAUHWReg64(xdx_hdmi_audio_ptr->iobase_acpu, 0x100540);

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "%s %d dbCmd.cmd=%d\n", __func__, __LINE__, dbCmd.cmd);

	if (xdx_hdmi_audio_ptr->status == cPlayBackStart)
		snd_pcm_period_elapsed(xdx_hdmi_audio_ptr->playback_substream);

	if (dbCmd.cmd == 2) {
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "aempty %s %d period_bytes=0x%x buffer_bytes=0x%x fifo_bytes=0x%x pos=0x%x status=%d peding_cnt=%d\n", __func__, __LINE__,
				period_bytes, buffer_bytes, xdx_hdmi_audio_ptr->fifo_bytes, xdx_hdmi_audio_ptr->pos, xdx_hdmi_audio_ptr->status, xdx_hdmi_audio_ptr->dma_pending_cnt);
		if (xdx_hdmi_audio_ptr->fifo_bytes >= AUDIO_FIFO_SIZE_A0/4)
			xdx_hdmi_audio_ptr->fifo_bytes = AUDIO_FIFO_SIZE_A0/4;
		else
			xdx_hdmi_audio_ptr->fifo_bytes = 0;
	} else if (dbCmd.cmd == 3) {
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "empty %s %d period_bytes=0x%x buffer_bytes=0x%x fifo_bytes=0x%x pos=0x%x status=%d peding_cnt=%d\n", __func__, __LINE__,
				period_bytes, buffer_bytes, xdx_hdmi_audio_ptr->fifo_bytes, xdx_hdmi_audio_ptr->pos, xdx_hdmi_audio_ptr->status, xdx_hdmi_audio_ptr->dma_pending_cnt);
		xdx_hdmi_audio_ptr->fifo_bytes = 0;
		xdx_hdmi_audio_ptr->fifo_empty = 1;
	} else if (dbCmd.cmd == 4)
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s -%d] iis idle\n", __func__, __LINE__);
	else {
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s -%d] dbCmd.cmd is error!\n", __func__, __LINE__);
		return;
	}

	OSWriteAUHWReg64(xdx_hdmi_audio_ptr->iobase_acpu, 0x1001c0, intStatus);
	dbCmd.low32 |= 1;

	OSWriteAUHWReg32(xdx_hdmi_audio_ptr->iobase_acpu, 0x100540, dbCmd.low32);

	if (((xdx_hdmi_audio_ptr->fifo_bytes + period_bytes) <= AUDIO_FIFO_SIZE_A0) && (xdx_hdmi_audio_ptr->status == cPlayBackStart) && (xdx_hdmi_audio_ptr->dma_pending_cnt == 0)) {
		xdx_hdmi_audio_ptr->transfer_flag = 1;
		(void) queue_work(xdx_hdmi_audio_ptr->pWorkQueue, &xdx_hdmi_audio_ptr->audioIntWork);
	} else
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "%s %d period_bytes=0x%x buffer_bytes=0x%x fifo_bytes=0x%x pos=0x%x status=%d dma_pending_cnt=%d\n", __func__, __LINE__,
				period_bytes, buffer_bytes, xdx_hdmi_audio_ptr->fifo_bytes, xdx_hdmi_audio_ptr->pos, xdx_hdmi_audio_ptr->status, xdx_hdmi_audio_ptr->dma_pending_cnt);

}

static void pangu_b0_audio1f0_irq_handler(void *data)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = (struct snd_xdx_hdmi_audio *)data;
	uint32_t period_bytes = 0;
	uint32_t buffer_bytes = 0;
	uint32_t pReg_value = 0;
	Audio_B0_Regs_T *pAudio1B0Reg = (Audio_B0_Regs_T *)(xdx_hdmi_audio_ptr->iobase_audio1);

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev,"[%s %d]\n", __func__, __LINE__);

	period_bytes = snd_pcm_lib_period_bytes(xdx_hdmi_audio_ptr->playback_substream);
	buffer_bytes = snd_pcm_lib_buffer_bytes(xdx_hdmi_audio_ptr->playback_substream);
	if (xdx_hdmi_audio_ptr->bit_width == 16)
		period_bytes = period_bytes*2;

	pReg_value = OSReadAUHWReg64(xdx_hdmi_audio_ptr->iobase_audio1, XDX_AUDIOIF0_INTR_STS);

	if (pReg_value != 0) {
		OSWriteAUHWReg64(xdx_hdmi_audio_ptr->iobase_audio1, XDX_AUDIOIF0_INTR_STS_CLR, pReg_value);
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]  Clear audioif status\n", __func__, __LINE__);
	}

	if ((OSReadAUHWReg32(xdx_hdmi_audio_ptr->iobase_audio1, 0x10) & 0x1) && (xdx_hdmi_audio_ptr->status == cPlayBackStop))
		pAudio1B0Reg->TXEN.all = 0;

	if (xdx_hdmi_audio_ptr->status == cPlayBackStart)
		snd_pcm_period_elapsed(xdx_hdmi_audio_ptr->playback_substream);

	if (pReg_value == 4) {
		if (xdx_hdmi_audio_ptr->dma_pending_cnt == 1)
			xdx_hdmi_audio_ptr->dma_pending_cnt = 0;
		if ((xdx_hdmi_audio_ptr->fifo_bytes >= (AUDIO_FIFO_SIZE_B0/4)))
			xdx_hdmi_audio_ptr->fifo_bytes = (AUDIO_FIFO_SIZE_B0/4);
		else
			xdx_hdmi_audio_ptr->fifo_bytes  = 0;

		if (pReg_value == 6)
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "aempty and empty %s %d period_bytes=0x%x buffer_bytes=0x%x fifo_bytes=0x%x pos=0x%x status=%d peding_cnt=%d\n", __func__, __LINE__,
					period_bytes, buffer_bytes, xdx_hdmi_audio_ptr->fifo_bytes, xdx_hdmi_audio_ptr->pos, xdx_hdmi_audio_ptr->status, xdx_hdmi_audio_ptr->dma_pending_cnt);
		else
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "aempty %s %d period_bytes=0x%x buffer_bytes=0x%x fifo_bytes=0x%x pos=0x%x status=%d peding_cnt=%d\n", __func__, __LINE__,
					period_bytes, buffer_bytes, xdx_hdmi_audio_ptr->fifo_bytes, xdx_hdmi_audio_ptr->pos, xdx_hdmi_audio_ptr->status, xdx_hdmi_audio_ptr->dma_pending_cnt);

	} else {
		if (((OSReadAUHWReg32(xdx_hdmi_audio_ptr->iobase_audio1, 0x10) & 0x1)) && (xdx_hdmi_audio_ptr->status == cPlayBackStart))
			pAudio1B0Reg->TXEN.all = 0;
		xdx_hdmi_audio_ptr->fifo_bytes = 0;
		xdx_hdmi_audio_ptr->fifo_empty = 1;
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s +%d] iis idle\n", __func__, __LINE__);
	}

	if (((xdx_hdmi_audio_ptr->fifo_bytes + AUDIO_FIFO_PERIOD_BYTES) <= AUDIO_FIFO_SIZE_B0) && (xdx_hdmi_audio_ptr->status == cPlayBackStart) && (xdx_hdmi_audio_ptr->dma_pending_cnt == 0)) {
		xdx_hdmi_audio_ptr->transfer_flag = 1;
	        (void) queue_work(xdx_hdmi_audio_ptr->pWorkQueue, &xdx_hdmi_audio_ptr->audioIntWork);
	} else
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "%s %d  period_bytes=0x%x buffer_bytes=0x%x fifo_bytes=0x%x pos=0x%x status=%d dma_pending_cnt=%d\n", __func__, __LINE__,
				period_bytes, buffer_bytes, xdx_hdmi_audio_ptr->fifo_bytes, xdx_hdmi_audio_ptr->pos, xdx_hdmi_audio_ptr->status, xdx_hdmi_audio_ptr->dma_pending_cnt);

}

static void pangu_b0_audiof0_irq_handler(void *data)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = (struct snd_xdx_hdmi_audio *)data;
	uint32_t period_bytes = 0;
	uint32_t buffer_bytes = 0;
	uint32_t pReg_value = 0;
	Audio_B0_Regs_T *pAudioB0Reg = (Audio_B0_Regs_T *)(xdx_hdmi_audio_ptr->iobase_audio0);

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev,"[%s %d]\n", __func__, __LINE__);

	period_bytes = snd_pcm_lib_period_bytes(xdx_hdmi_audio_ptr->playback_substream);
	buffer_bytes = snd_pcm_lib_buffer_bytes(xdx_hdmi_audio_ptr->playback_substream);
	if (xdx_hdmi_audio_ptr->bit_width == 16)
		period_bytes = period_bytes*2;

	pReg_value = OSReadAUHWReg64(xdx_hdmi_audio_ptr->iobase_audio0, XDX_AUDIOIF0_INTR_STS);
	if (pReg_value != 0) {
		OSWriteAUHWReg64(xdx_hdmi_audio_ptr->iobase_audio0, XDX_AUDIOIF0_INTR_STS_CLR, pReg_value);
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]  Clear HDMI or DP audioif status\n", __func__, __LINE__);
	}

	if ((OSReadAUHWReg32(xdx_hdmi_audio_ptr->iobase_audio0, 0x10) & 0x1) && (xdx_hdmi_audio_ptr->status == cPlayBackStop))
		pAudioB0Reg->TXEN.all = 0;

	if (xdx_hdmi_audio_ptr->status == cPlayBackStart)
		snd_pcm_period_elapsed(xdx_hdmi_audio_ptr->playback_substream);

	if (pReg_value == 4) {
		if (xdx_hdmi_audio_ptr->dma_pending_cnt == 1)
			xdx_hdmi_audio_ptr->dma_pending_cnt = 0;
		if ((xdx_hdmi_audio_ptr->fifo_bytes >= (AUDIO_FIFO_SIZE_B0/4)))
			xdx_hdmi_audio_ptr->fifo_bytes = (AUDIO_FIFO_SIZE_B0/4);
		else
			xdx_hdmi_audio_ptr->fifo_bytes  = 0;

		if (pReg_value == 6)
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "aempty and empty %s %d period_bytes=0x%x buffer_bytes=0x%x fifo_bytes=0x%x pos=0x%x status=%d peding_cnt=%d\n", __func__, __LINE__,
					period_bytes, buffer_bytes, xdx_hdmi_audio_ptr->fifo_bytes, xdx_hdmi_audio_ptr->pos, xdx_hdmi_audio_ptr->status, xdx_hdmi_audio_ptr->dma_pending_cnt);
		else
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "aempty %s %d period_bytes=0x%x buffer_bytes=0x%x fifo_bytes=0x%x pos=0x%x status=%d peding_cnt=%d\n", __func__, __LINE__,
					period_bytes, buffer_bytes, xdx_hdmi_audio_ptr->fifo_bytes, xdx_hdmi_audio_ptr->pos, xdx_hdmi_audio_ptr->status, xdx_hdmi_audio_ptr->dma_pending_cnt);
	} else {
		if ((OSReadAUHWReg32(xdx_hdmi_audio_ptr->iobase_audio0, 0x10) & 0x1) && (xdx_hdmi_audio_ptr->status == cPlayBackStart))
			pAudioB0Reg->TXEN.all = 0;
		xdx_hdmi_audio_ptr->fifo_bytes = 0;
		xdx_hdmi_audio_ptr->fifo_empty = 1;
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s +%d] iis idle\n", __func__, __LINE__);
	}

	if (((xdx_hdmi_audio_ptr->fifo_bytes +  AUDIO_FIFO_PERIOD_BYTES) <= AUDIO_FIFO_SIZE_B0) && (xdx_hdmi_audio_ptr->status == cPlayBackStart) && (xdx_hdmi_audio_ptr->dma_pending_cnt == 0)) {
		xdx_hdmi_audio_ptr->transfer_flag = 1;
	        (void) queue_work(xdx_hdmi_audio_ptr->pWorkQueue, &xdx_hdmi_audio_ptr->audioIntWork);
	} else
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "%s %d  period_bytes=0x%x buffer_bytes=0x%x fifo_bytes=0x%x pos=0x%x status=%d dma_pending_cnt=%d\n", __func__, __LINE__,
				period_bytes, buffer_bytes, xdx_hdmi_audio_ptr->fifo_bytes, xdx_hdmi_audio_ptr->pos, xdx_hdmi_audio_ptr->status, xdx_hdmi_audio_ptr->dma_pending_cnt);

}

static void pangu_vdma1ch3_irq_handler(void *data)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = (struct snd_xdx_hdmi_audio *)data;
	uint32_t period_bytes = 0;
	uint32_t buffer_bytes = 0;
	uint64_t intStatus = 0;
	struct xdx_device *xdev = xdx_hdmi_audio_ptr->xdev;
	Audio_B0_Regs_T *pAudioB0Reg = (Audio_B0_Regs_T *)(xdx_hdmi_audio_ptr->iobase_audio0);
	Audio_B0_Regs_T *pAudio1B0Reg = (Audio_B0_Regs_T *)(xdx_hdmi_audio_ptr->iobase_audio1);
	uint32_t pos = 0;

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev,"[%s %d]\n", __func__, __LINE__);
	if ((xdx_hdmi_audio_ptr->playback_substream == NULL) || (!xdx_hdmi_audio_ptr->playback_substream->runtime)) {
		/* workqueue to transfer data is destroyed during system poweroff */
		return;
	}

	period_bytes = snd_pcm_lib_period_bytes(xdx_hdmi_audio_ptr->playback_substream);
	buffer_bytes = snd_pcm_lib_buffer_bytes(xdx_hdmi_audio_ptr->playback_substream);
	intStatus = OSReadAUHWReg64(xdx_hdmi_audio_ptr->iobase_vdma1, 0x2800 + 0x10);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "%s %d intStatus=0x%llx\n", __func__, __LINE__, intStatus);

	// then check waitfor
	if (0 != intStatus) {
		OSWriteAUHWReg64(xdx_hdmi_audio_ptr->iobase_vdma1, 0x2800 + 0x40, intStatus);
		if (xdx_hdmi_audio_ptr->dma_pending_cnt == 1) {
			pos = xdx_hdmi_audio_ptr->pos;
			pos += xdx_hdmi_audio_ptr->dma_bytes;
			if (pos >= buffer_bytes)
				pos -= buffer_bytes;
			xdx_hdmi_audio_ptr->pos = pos;
			xdx_hdmi_audio_ptr->dma_pending_cnt = 0;

			if (xdev->asic_type == CHIP_PANGU_B0) {
				XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s +%d] xdx_hdmi_audio_ptr->buf_ptr=0x%x\n", __func__, __LINE__, xdx_hdmi_audio_ptr->buf_ptr);
				if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI1) || (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP1)) {
					OSWriteAUHWReg32(xdx_hdmi_audio_ptr->iobase_audio1, 0x8, xdx_hdmi_audio_ptr->buf_ptr);
					pAudio1B0Reg->PTR_UP.all = 0x1;
				} else {
					OSWriteAUHWReg32(xdx_hdmi_audio_ptr->iobase_audio0, 0x8, xdx_hdmi_audio_ptr->buf_ptr);
					pAudioB0Reg->PTR_UP.all = 0x1; //poweron no screen
				}
			}

		} else
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s +%d] Recieved double isr\n", __func__, __LINE__);

	}

	if ((!(OSReadAUHWReg32(xdx_hdmi_audio_ptr->iobase_audio1, 0x10) & 0x1)) && (xdx_hdmi_audio_ptr->status == cPlayBackStart) && (xdev->asic_type == CHIP_PANGU_B0)
			&& ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI1) || (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP1)))
		pAudio1B0Reg->TXEN.all = 0x1;
	else if ((!(OSReadAUHWReg32(xdx_hdmi_audio_ptr->iobase_audio0, 0x10) & 0x1)) && (xdx_hdmi_audio_ptr->status == cPlayBackStart) && (xdev->asic_type == CHIP_PANGU_B0)
			&& ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI) || (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP)))
		pAudioB0Reg->TXEN.all = 0x1;
	else if ((!(OSReadAUHWReg32(xdx_hdmi_audio_ptr->iobase_audio0, 0x10) & 0x1)) && (xdx_hdmi_audio_ptr->status == cPlayBackStart) && (xdev->asic_type == CHIP_PANGU_B0)
			&& (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_NULL))
		pAudioB0Reg->TXEN.all = 0x1;  //poweron no screen

	if (xdx_hdmi_audio_ptr->bit_width == 16)
		period_bytes = period_bytes*2;

	if ((xdev->asic_type == CHIP_PANGU_B0) && ((xdx_hdmi_audio_ptr->fifo_bytes + AUDIO_FIFO_PERIOD_BYTES) <= AUDIO_FIFO_SIZE_B0_HALF) && (xdx_hdmi_audio_ptr->status == cPlayBackStart) && (xdx_hdmi_audio_ptr->dma_pending_cnt == 0)) {
		snd_pcm_period_elapsed(xdx_hdmi_audio_ptr->playback_substream);
		xdx_hdmi_audio_ptr->transfer_flag = 1;
		(void) queue_work(xdx_hdmi_audio_ptr->pWorkQueue, &xdx_hdmi_audio_ptr->audioIntWork);
	} else if ((xdev->asic_type == CHIP_PANGU_A0) && ((xdx_hdmi_audio_ptr->fifo_bytes + period_bytes) <= AUDIO_FIFO_SIZE_A0_HALF) && (xdx_hdmi_audio_ptr->status == cPlayBackStart) && (xdx_hdmi_audio_ptr->dma_pending_cnt == 0)) {
		snd_pcm_period_elapsed(xdx_hdmi_audio_ptr->playback_substream);
		xdx_hdmi_audio_ptr->transfer_flag = 1;
		(void) queue_work(xdx_hdmi_audio_ptr->pWorkQueue, &xdx_hdmi_audio_ptr->audioIntWork);
	} else
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "%s %d  period_bytes=0x%x buffer_bytes=0x%x fifo_bytes=0x%x pos=0x%x status=%d dma_pending_cnt=%d\n", __func__, __LINE__,
				period_bytes, buffer_bytes, xdx_hdmi_audio_ptr->fifo_bytes, xdx_hdmi_audio_ptr->pos, xdx_hdmi_audio_ptr->status, xdx_hdmi_audio_ptr->dma_pending_cnt);

}

static int xdx_hdmi_audio_createVolumeControl(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr)
{
	int i = 0, err = 0;

	for (i = 0; i < ARRAY_SIZE(snd_xdx_hdmi_control); i++) {
		err = snd_ctl_add(xdx_hdmi_audio_ptr->card, snd_ctl_new1(&snd_xdx_hdmi_control[i], xdx_hdmi_audio_ptr));
		if (err < 0)
			return err;
	}

	return 1;
}

int snd_xdx_hdmi_audio_create(struct snd_card *card, struct xdx_device *xdev, struct snd_xdx_hdmi_audio **rchip)

{
	int err = 0;
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = NULL;
	static struct snd_device_ops ops =
	{
		.dev_free = snd_xdx_hdmi_audio_dev_free,
	};
	void *pointer = NULL;
	void *hw_pointer = NULL;
	unsigned iomem_size;
	int i = 0,irq_num = 0,irqvec = 0;
	struct pci_dev *pci = NULL;

	* rchip = NULL;

	xdx_hdmi_audio_ptr = kzalloc(sizeof(struct snd_xdx_hdmi_audio), GFP_KERNEL);
	if (xdx_hdmi_audio_ptr == NULL) {
		err = -ENOMEM;
		goto ERROR_XDX_HDMI_AUDIO_CREATE_1;
	}

	if (xdev->shareram_size == XDX_AUDIO_BAR4_16M && xdev->pdev->device != 0x15A4) {
		pci = xdev->audio_pdev;
		err = pci_enable_device(pci);
	        if (err < 0)
		        return err;
		pci_set_master(pci);
		err = pci_request_regions(pci, "xdx_hdmi_audio");
		if (err < 0)
			return err;

	} else if (xdev->shareram_size == XDX_AUDIO_BAR4_1M || xdev->pdev->device == 0x15A4)
		pci = xdev->pdev;

	xdx_hdmi_audio_ptr->dev = &pci->dev;
	xdx_hdmi_audio_ptr->card = card;
	xdx_hdmi_audio_ptr->pci = pci;
	xdx_hdmi_audio_ptr->xdev = xdev;

	xdx_hdmi_audio_ptr->periodBufSize = AUDIO_FIFO_SIZE;
	if (snd_dma_alloc_pages(SNDRV_DMA_TYPE_DEV, &pci->dev, AUDIO_FIFO_SIZE, &xdx_hdmi_audio_ptr->dma_period) < 0) {
		err = -ENOMEM;
		goto ERROR_XDX_HDMI_AUDIO_CREATE_2;
	}
	xdx_hdmi_audio_ptr->periodBufPtr = (uint8_t*)(xdx_hdmi_audio_ptr->dma_period.area);
	xdx_hdmi_audio_ptr->periodBufPhy = xdx_hdmi_audio_ptr->dma_period.addr;

	if (snd_dma_alloc_pages(SNDRV_DMA_TYPE_DEV, &pci->dev, XDX_AUDIOIF_PERIOD_BYTES, &xdx_hdmi_audio_ptr->tempBuf) < 0) {
		err = -ENOMEM;
		goto ERROR_XDX_HDMI_AUDIO_CREATE_3;
	}
	xdx_hdmi_audio_ptr->tempBufPtr = (uint8_t*)(xdx_hdmi_audio_ptr->tempBuf.area);

	if (snd_dma_alloc_pages(SNDRV_DMA_TYPE_DEV, &pci->dev, AUDIO_FIFO_SIZE, &xdx_hdmi_audio_ptr->bitBuf) < 0) {
		err = -ENOMEM;
		goto ERROR_XDX_HDMI_AUDIO_CREATE_4;
	}
	xdx_hdmi_audio_ptr->bitBufPtr = (uint8_t*)(xdx_hdmi_audio_ptr->bitBuf.area);
	if (snd_dma_alloc_pages(SNDRV_DMA_TYPE_DEV, &pci->dev, AUDIO_FIFO_SIZE, &xdx_hdmi_audio_ptr->chnl6Buf) < 0) {
		err = -ENOMEM;
		goto ERROR_XDX_HDMI_AUDIO_CREATE_5;
	}
	xdx_hdmi_audio_ptr->chnl6BufPtr = (uint8_t*)(xdx_hdmi_audio_ptr->chnl6Buf.area);

	xdx_hdmi_audio_ptr->pWorkQueue = create_singlethread_workqueue("xdx_hdmi_audio_misr");
	INIT_WORK(&(xdx_hdmi_audio_ptr->audioIntWork), xdx_hdmi_audio_int_work);

	err = snd_device_new(card, SNDRV_DEV_LOWLEVEL, xdx_hdmi_audio_ptr, &ops);
	if (err < 0) {
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s +%d] snd_device_new err=%d\n", __func__, __LINE__, err);
		goto ERROR_XDX_HDMI_AUDIO_CREATE_6;
	}

	err = snd_pcm_new(xdx_hdmi_audio_ptr->card, "xdx hdmi pcm", 0, 1, 0, &xdx_hdmi_audio_ptr->pcm);
	if (err < 0) {
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s +%d] snd_pcm_new err=%d\n", __func__, __LINE__, err);
		return err;
	}

	pointer = kzalloc(sizeof(struct mutex), GFP_KERNEL);
	if (!pointer) {
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s +%d] Allocate memory failed.\n", __func__, __LINE__);
		goto ERROR_XDX_HDMI_AUDIO_CREATE_7;
	}
	xdx_hdmi_audio_ptr->transfer_mutex = pointer;
	mutex_init((struct mutex *)xdx_hdmi_audio_ptr->transfer_mutex);

	hw_pointer = kzalloc(sizeof(struct mutex), GFP_KERNEL);
	if (!hw_pointer) {
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s +%d] Allocate memory failed.\n", __func__, __LINE__);
		goto ERROR_XDX_HDMI_AUDIO_CREATE_8;
	}
	xdx_hdmi_audio_ptr->hw_mutex = hw_pointer;
	mutex_init((struct mutex *)xdx_hdmi_audio_ptr->hw_mutex);

	xdx_hdmi_audio_ptr->pcm->private_data = xdx_hdmi_audio_ptr;
	xdx_hdmi_audio_ptr->pcm->private_free = snd_xdx_hdmi_audio_free_pcm;
	strcpy(xdx_hdmi_audio_ptr->pcm->name, "xdx_hdmi_pcm");

	snd_pcm_set_ops(xdx_hdmi_audio_ptr->pcm, SNDRV_PCM_STREAM_PLAYBACK, &snd_xdx_hdmi_audio_playback_ops);

	snd_pcm_lib_preallocate_pages_for_all(xdx_hdmi_audio_ptr->pcm, SNDRV_DMA_TYPE_DEV, &((xdx_hdmi_audio_ptr->pci)->dev), 64*1024, 64*1024);

	if (xdx_hdmi_audio_createVolumeControl(xdx_hdmi_audio_ptr))
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s +%d] Create control done.\n", __func__, __LINE__);
	else
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s +%d] Create control fail.\n", __func__, __LINE__);

	xdx_hdmi_audio_ptr->audioif_video_connect = XDX_AUDIO_MODULE_NULL;

	//we reserved 64M for ACPU, FW use high 16M, audio use at most 16M, the space is enough
	xdx_b0_audioif_buf_gddr_offset = xdev->vram_size - AUDIO_RESERVE_VRAM_SIZE;

	xdx_hdmi_audio_ptr->flag = 0;
	for (i = 0; i <= concentCnts; i++) {
		xdx_hdmi_audio_ptr->current_connect[i] = 0;
	}

	if (xdev->pdev->device != 0x15A4) {     // not X1900, A0 for xdxsmi
		xdx_hdmi_audio_ptr->iomem = pci_resource_start(xdev->audio_pdev, 0);
		iomem_size = pci_resource_len(xdev->audio_pdev, 0);
		pr_info("[%s +%d] resources: iomem = 0x%lx size=0x%x\n", __func__, __LINE__, xdx_hdmi_audio_ptr->iomem, iomem_size);

		xdx_hdmi_audio_ptr->iobase = ioremap(xdx_hdmi_audio_ptr->iomem, iomem_size);
		if (!xdx_hdmi_audio_ptr->iobase) {
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s +%d] ioremap failed\n", __func__, __LINE__);
			goto ERROR_XDX_HDMI_AUDIO_CREATE_9;
		}
	}

	if (xdev->shareram_size == XDX_AUDIO_BAR4_16M) {
		irq_num = pci_alloc_irq_vectors(pci, PANGU_INTERRUPT_MAX, PANGU_INTERRUPT_MAX, PCI_IRQ_MSIX);
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s +%d] irq_num=%d\n", __func__, __LINE__, irq_num);
		if (irq_num < 0) {
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s +%d] Fail to enable MSI/MSIX interrupt ret=%d. PANGU_INTERRUPT_MAX=%d\n", __func__, __LINE__, xdx_hdmi_audio_ptr->irq, PANGU_INTERRUPT_MAX);
			err = irq_num;
			return err;
		}
		for (i = 0; i < irq_num; ++i) {
			irqvec = pci_irq_vector(pci, i);

			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "%s pci_irq_vector irq=%d, irq_nr=%d\n",__func__, i, irqvec);

			err = request_irq(irqvec, pangu_a0_audiof1_irq_handler, IRQF_SHARED, XDX_HDMI_AUDIO_SYSTEM_NAME"-msix", xdx_hdmi_audio_ptr);
			if (err < 0) {
				XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s +%d] Fail to request_irq(%d) ret=%#x\n", __func__, __LINE__, i, err);
				return err;
			}
		}

		xdx_hdmi_audio_ptr->irq = irq_num;
	}

	* rchip = xdx_hdmi_audio_ptr;
	return 0;

ERROR_XDX_HDMI_AUDIO_CREATE_9:
	kfree(xdx_hdmi_audio_ptr->hw_mutex);
ERROR_XDX_HDMI_AUDIO_CREATE_8:
	kfree(xdx_hdmi_audio_ptr->transfer_mutex);
ERROR_XDX_HDMI_AUDIO_CREATE_7:
	snd_device_free(card, xdx_hdmi_audio_ptr);
ERROR_XDX_HDMI_AUDIO_CREATE_6:
	snd_dma_free_pages(&xdx_hdmi_audio_ptr->chnl6Buf);
ERROR_XDX_HDMI_AUDIO_CREATE_5:
	snd_dma_free_pages(&xdx_hdmi_audio_ptr->bitBuf);
ERROR_XDX_HDMI_AUDIO_CREATE_4:
	snd_dma_free_pages(&xdx_hdmi_audio_ptr->tempBuf);
ERROR_XDX_HDMI_AUDIO_CREATE_3:
	snd_dma_free_pages(&xdx_hdmi_audio_ptr->dma_period);
ERROR_XDX_HDMI_AUDIO_CREATE_2:
	kfree(xdx_hdmi_audio_ptr);
ERROR_XDX_HDMI_AUDIO_CREATE_1:
	return err;

}

static int xdx_hdmi_audio_switchInfo(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_info *uinfo)
{
	uinfo->type = SNDRV_CTL_ELEM_TYPE_BOOLEAN;
	uinfo->count = 1;
	uinfo->value.integer.min = 0;
	uinfo->value.integer.max = 1;
	return 0;
}

static int xdx_hdmi_audio_hdmi0Get(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = snd_kcontrol_chip(kcontrol);

	mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
	u->value.integer.value[0] = xdx_hdmi_audio_ptr->current_connect[AUDIOIF_HDMI] ? 1 : 0;
	mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);

	return 0;
}
static int xdx_hdmi_audio_hdmi0Put(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = snd_kcontrol_chip(kcontrol);
	uint32_t val = 0, temp_status = 0;
	struct xdx_device *xdev = xdx_hdmi_audio_ptr->xdev;

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);
	val = u->value.integer.value[0];
	if (val == On) {
		if (xdx_hdmi_audio_ptr->current_connect[AUDIOIF_HDMI]) {
			if((xdx_hdmi_audio_ptr->audioif_video_connect != XDX_AUDIO_MODULE_HDMI)) {
				mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
				temp_status = xdx_hdmi_audio_ptr->status;
				xdx_hdmi_audio_ptr->status = cPlayBackStop;
				mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

				xdx_hdmi_audio_waitempty(xdx_hdmi_audio_ptr);
				xdx_hdmi_audio_ptr->audioif_video_connect = XDX_AUDIO_MODULE_HDMI;
				if (xdev->asic_type == CHIP_PANGU_B0)
					xdx_hdmi_audio_b0_Enable(xdx_hdmi_audio_ptr,  XDX_AUDIO_MODULE_HDMI, 1);
				else if (xdev->asic_type == CHIP_PANGU_A0)
					xdx_hdmi_audio_a0_Enable(xdx_hdmi_audio_ptr,  XDX_AUDIO_MODULE_HDMI, 1);

				mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
				xdx_hdmi_audio_ptr->status = temp_status;
				mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

				if (xdx_hdmi_audio_ptr->status == cPlayBackStart) {
					if(xdx_hdmi_audio_ptr->hdmi)
						xdx_hdmi_audio_setRate(xdx_hdmi_audio_ptr);
					xdx_hdmi_audio_ptr->transfer_flag = 1;
					(void) queue_work(xdx_hdmi_audio_ptr->pWorkQueue, &xdx_hdmi_audio_ptr->audioIntWork);
				}
			}
		} else {
			return -1;
		}
	}

	return 1;
}
static int xdx_hdmi_audio_dp0Get(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = snd_kcontrol_chip(kcontrol);

	mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
	u->value.integer.value[0] = xdx_hdmi_audio_ptr->current_connect[AUDIOIF_DP] ? 1 : 0;
	mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);

	return 0;
}
static int xdx_hdmi_audio_dp0Put(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = snd_kcontrol_chip(kcontrol);
	uint32_t val = 0, temp_status;
	struct xdx_device *xdev = xdx_hdmi_audio_ptr->xdev;

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);
	val = u->value.integer.value[0];
	if (val == On) {
		if ((xdx_hdmi_audio_ptr->current_connect[AUDIOIF_DP])) {
			if (xdx_hdmi_audio_ptr->audioif_video_connect != XDX_AUDIO_MODULE_DP) {

				mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
				temp_status = xdx_hdmi_audio_ptr->status;
				xdx_hdmi_audio_ptr->status = cPlayBackStop;
				mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

				xdx_hdmi_audio_waitempty(xdx_hdmi_audio_ptr);
				xdx_hdmi_audio_ptr->audioif_video_connect = XDX_AUDIO_MODULE_DP;
				if (xdev->asic_type == CHIP_PANGU_B0)
					xdx_hdmi_audio_b0_Enable(xdx_hdmi_audio_ptr,  XDX_AUDIO_MODULE_DP, 1);
				else if (xdev->asic_type == CHIP_PANGU_A0)
					xdx_hdmi_audio_a0_Enable(xdx_hdmi_audio_ptr,  XDX_AUDIO_MODULE_DP, 1);

				mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
				xdx_hdmi_audio_ptr->status = temp_status;
				mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

				if (xdx_hdmi_audio_ptr->status == cPlayBackStart) {
					ADIFOutputSelConfig(xdx_hdmi_audio_ptr);
					xdx_hdmi_audio_ptr->transfer_flag = 1;
					(void) queue_work(xdx_hdmi_audio_ptr->pWorkQueue, &xdx_hdmi_audio_ptr->audioIntWork);
				}
			}
		} else {
			return -1;
		}
	}

	return 1;
}

static int xdx_hdmi_audio_hdmi1Get(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = snd_kcontrol_chip(kcontrol);

	mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
	u->value.integer.value[0] = xdx_hdmi_audio_ptr->current_connect[AUDIOIF_HDMI1] ? 1 : 0;
	mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);

	return 0;
}

static int xdx_hdmi_audio_hdmi1Put(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = snd_kcontrol_chip(kcontrol);
	u32 val = 0;
	uint32_t temp_status = 0;

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);
	val = u->value.integer.value[0];
	if (val == On) {
		if (xdx_hdmi_audio_ptr->current_connect[AUDIOIF_HDMI1]) {
			if ((xdx_hdmi_audio_ptr->audioif_video_connect != XDX_AUDIO_MODULE_HDMI1)) {

				mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
				temp_status = xdx_hdmi_audio_ptr->status;
				xdx_hdmi_audio_ptr->status = cPlayBackStop;
				mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

				xdx_hdmi_audio_waitempty(xdx_hdmi_audio_ptr);
				xdx_hdmi_audio_ptr->audioif_video_connect = XDX_AUDIO_MODULE_HDMI1;
				xdx_hdmi_audio_b0_Enable(xdx_hdmi_audio_ptr,  XDX_AUDIO_MODULE_HDMI1, 1);

				mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
				xdx_hdmi_audio_ptr->status = temp_status;
				mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

				if (xdx_hdmi_audio_ptr->status == cPlayBackStart) {
					if(xdx_hdmi_audio_ptr->hdmi1)
						xdx_hdmi_audio_setRate(xdx_hdmi_audio_ptr);
					xdx_hdmi_audio_ptr->transfer_flag = 1;
					(void) queue_work(xdx_hdmi_audio_ptr->pWorkQueue, &xdx_hdmi_audio_ptr->audioIntWork);
				}
			}
		} else {
			return -1;
		}
	}

	return 1;
}

static int xdx_hdmi_audio_dp1Get(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = snd_kcontrol_chip(kcontrol);

	mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
	u->value.integer.value[0] = xdx_hdmi_audio_ptr->current_connect[AUDIOIF_DP1] ? 1 : 0;
	mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);

	return 0;
}

static int xdx_hdmi_audio_dp1Put(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = snd_kcontrol_chip(kcontrol);
	u32 val = 0;
	u32 temp_status;

	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);
	val = u->value.integer.value[0];
	if (val == On) {
		if ((xdx_hdmi_audio_ptr->current_connect[AUDIOIF_DP1])) {
			if (xdx_hdmi_audio_ptr->audioif_video_connect != XDX_AUDIO_MODULE_DP1) {
				mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
				temp_status = xdx_hdmi_audio_ptr->status;
				xdx_hdmi_audio_ptr->status = cPlayBackStop;
				mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

				xdx_hdmi_audio_waitempty(xdx_hdmi_audio_ptr);
				xdx_hdmi_audio_ptr->audioif_video_connect = XDX_AUDIO_MODULE_DP1;

				xdx_hdmi_audio_b0_Enable(xdx_hdmi_audio_ptr,  XDX_AUDIO_MODULE_DP1, 1);

				mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
				xdx_hdmi_audio_ptr->status = temp_status;
				mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

				if (xdx_hdmi_audio_ptr->status == cPlayBackStart) {
					ADIFOutputSelConfig(xdx_hdmi_audio_ptr);
					xdx_hdmi_audio_ptr->transfer_flag = 1;
					(void) queue_work(xdx_hdmi_audio_ptr->pWorkQueue, &xdx_hdmi_audio_ptr->audioIntWork);
				}
			}
		} else {
			return -1;
		}
	}

	return 1;
}

/*Audio volume info*/
static int xdx_hdmi_audio_volumeInfo(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_info *uinfo)
{
	uinfo->value.integer.step = 1;
	uinfo->type = SNDRV_CTL_ELEM_TYPE_INTEGER;
	uinfo->count = 1;

	uinfo->value.integer.max = DEFAULT_MAX_VOLUME;
	uinfo->value.integer.min = DEFAULT_MIN_VOLUME;

	return 0;
}

static int  xdx_hdmi_audio_volumeGet(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = snd_kcontrol_chip(kcontrol);
	uint8_t hwVolume = 0;

	if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI) && (xdx_hdmi_audio_ptr->hdmi != NULL)) {
		if (xdx_hdmi_ddcci_dumpctrl(xdx_hdmi_audio_ptr->hdmi, DEFAULT_DDCCI_ADDR, SPEAKER_VOLUME_ADJUST_OFFSET, &hwVolume, NULL) != 0) {
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]HDMI volume get fail[VOLUME_ADJUST].\n", __func__, __LINE__);
			u->value.integer.value[0] = DEFAULT_MIN_VOLUME;

			return 0;
		}
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]HDMI volume get value: %d \n", __func__, __LINE__, hwVolume);
	} else if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP) && (xdx_hdmi_audio_ptr->dp != NULL)) {
		if (xdx_dp_ddcci_dumpctrl(xdx_hdmi_audio_ptr->dp, DEFAULT_DDCCI_ADDR, SPEAKER_VOLUME_ADJUST_OFFSET, &hwVolume, NULL) != 0) {
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]DP volume get fail[VOLUME_ADJUST].\n", __func__, __LINE__);
			u->value.integer.value[0] = DEFAULT_MIN_VOLUME;
			return 0;
		}
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]DP Volume get value: %d \n", __func__, __LINE__, hwVolume);
	} else if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI1) && (xdx_hdmi_audio_ptr->hdmi1 != NULL)) {
		if (xdx_hdmi_ddcci_dumpctrl(xdx_hdmi_audio_ptr->hdmi1, DEFAULT_DDCCI_ADDR, SPEAKER_VOLUME_ADJUST_OFFSET, &hwVolume, NULL) != 0) {
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]HDMI1 volume get fail[VOLUME_ADJUST].\n", __func__, __LINE__);
			u->value.integer.value[0] = DEFAULT_MIN_VOLUME;

			return 0;
		}
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]HDMI1 volume get value: %d \n", __func__, __LINE__, hwVolume);
	} else if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP1) && (xdx_hdmi_audio_ptr->dp1 != NULL)) {
		if (xdx_dp_ddcci_dumpctrl(xdx_hdmi_audio_ptr->dp1, DEFAULT_DDCCI_ADDR, SPEAKER_VOLUME_ADJUST_OFFSET, &hwVolume, NULL) != 0) {
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]DP1 volume get fail[VOLUME_ADJUST].\n", __func__, __LINE__);
			u->value.integer.value[0] = DEFAULT_MIN_VOLUME;

			return 0;
		}
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]DP1 volume get value: %d \n", __func__, __LINE__, hwVolume);
	} else
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]The current connect video is error!\n", __func__, __LINE__);

	u->value.integer.value[0] = hwVolume;

	return 0;
}
static int xdx_hdmi_audio_volumePut(struct snd_kcontrol *kcontrol, struct snd_ctl_elem_value *u)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = snd_kcontrol_chip(kcontrol);
	int change = 0;
	int val = 0;

	if (strcmp(OSGetCurrentClientProcessNameKM(), "amixer") != 0) {
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] VolumePut process name is not amixer: %s \n", __func__, __LINE__, OSGetCurrentClientProcessNameKM());
		change = 1;
		return change;
	}
	val = u->value.integer.value[0];
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] Volume change value: %d \n", __func__, __LINE__, val);
	/// Start to adjust
	if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI) && (xdx_hdmi_audio_ptr->hdmi != NULL)) {
		if (!(xdx_hdmi_ddcci_writectrl(xdx_hdmi_audio_ptr->hdmi, DEFAULT_DDCCI_ADDR, SPEAKER_VOLUME_ADJUST_OFFSET, val))) {
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] Volume change fail[VOLUME_ADJUST].\n", __func__, __LINE__);

			return 0;
		}
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]HDMI volume change success: %d \n", __func__, __LINE__, val);
	} else if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP) && (xdx_hdmi_audio_ptr->dp != NULL)) {
		if (!(xdx_dp_ddcci_writectrl(xdx_hdmi_audio_ptr->dp, DEFAULT_DDCCI_ADDR, SPEAKER_VOLUME_ADJUST_OFFSET, val)))
		{
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]DP volume change fail[VOLUME_ADJUST].\n", __func__, __LINE__);

			return 0;
		}
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]DP volume change success: %d \n", __func__, __LINE__, val);
	} else if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI1) && (xdx_hdmi_audio_ptr->hdmi1 != NULL)) {
		if (!(xdx_hdmi_ddcci_writectrl(xdx_hdmi_audio_ptr->hdmi1, DEFAULT_DDCCI_ADDR, SPEAKER_VOLUME_ADJUST_OFFSET, val))) {
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] Volume change fail[VOLUME_ADJUST].\n", __func__, __LINE__);

			return 0;
		}
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]HDMI1 volume change success: %d \n", __func__, __LINE__, val);
	} else if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP1 && (xdx_hdmi_audio_ptr->dp1 != NULL))) {
		if (!(xdx_dp_ddcci_writectrl(xdx_hdmi_audio_ptr->dp1, DEFAULT_DDCCI_ADDR, SPEAKER_VOLUME_ADJUST_OFFSET, val)))
		{
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]DP volume change fail[VOLUME_ADJUST].\n", __func__, __LINE__);

			return 0;
		}
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]DP1 volume change success: %d \n", __func__, __LINE__, val);
	} else
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]The current connect video is error!\n", __func__, __LINE__);

	change = 1;

	return change;
}

static void xdx_audio_control_update(struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr)
{
	struct snd_ctl_elem_id elem_id;
	struct snd_kcontrol *kctl;
	int i = 0;

	for (i = 1; i <= jackCnts; i++) {
		memset(&elem_id, 0, sizeof(elem_id));
		elem_id.iface = SNDRV_CTL_ELEM_IFACE_CARD;
		strscpy(elem_id.name, snd_xdx_hdmi_control[i].name, sizeof(elem_id.name));
		kctl = snd_ctl_find_id(xdx_hdmi_audio_ptr->card, &elem_id);
		if(kctl)
			snd_ctl_notify(xdx_hdmi_audio_ptr->card, SNDRV_CTL_EVENT_MASK_VALUE, &kctl->id);
	}

}

int xdx_audio_drv_hotPlug(struct xdx_device *xdev, PlayDrv module, PlayCon status)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = xdev->xdx_hdmi_audio_hotplug_ptr;
	struct xdx_dm *dm = xdev_to_dm(xdev);
	struct xdx_output *output = dm_to_output(dm);
	struct hdmi_dev *hdmi = output->hdmi[0];
	struct hdmi_dev *hdmi1 = output->hdmi[1];
	struct dp_dev *dp = output->dp[0];
	struct dp_dev *dp1 = output->dp[1];
	int change = 0, i = 0;   //0 do nothing 1-dp/hdmi 2-default hdmi0 3-set hdmi/dp reg
	uint64_t time_close = 0, timeout_close = 0, time_now = 0;
	uint32_t temp_status = 0;
	Audio_Regs_T *pAudioReg = NULL;
	Audio_B0_Regs_T *pAudioB0Reg = NULL;
	Audio_B0_Regs_T *pAudio1B0Reg =NULL;
	if (!xdx_hdmi_audio_ptr)
		return xdev->audio_pdev ? -1 : 0;
	pAudioReg = (Audio_Regs_T *)(xdx_hdmi_audio_ptr->iobase_audio0);
	pAudioB0Reg = (Audio_B0_Regs_T *)(xdx_hdmi_audio_ptr->iobase_audio0);
	pAudio1B0Reg = (Audio_B0_Regs_T *)(xdx_hdmi_audio_ptr->iobase_audio1);

	///hdmi0 > dp0 > hdmi1 > dp1
	if (XDX_AUDIO_MODULE_DISCONNECT == status) {//unplug
		if (module == XDX_AUDIO_MODULE_HDMI) {
			xdx_hdmi_audio_ptr->current_connect[AUDIOIF_HDMI] = 0;
			xdx_hdmi_audio_ptr->audioif_video_disconnect = XDX_AUDIO_MODULE_HDMI;
			change = 0; //do nothing
			for (i = 0; i <= 3; i++) {
				if ((xdx_hdmi_audio_ptr->current_connect[i] > 0) && (i > AUDIOIF_HDMI)) {
					xdx_hdmi_audio_ptr->audioif_video_connect = xdx_hdmi_audio_ptr->current_connect[i];
					change = 1; //hdmi0->other
					break;
				}
			}
		} else if (module == XDX_AUDIO_MODULE_DP) {
			xdx_hdmi_audio_ptr->current_connect[AUDIOIF_DP]= 0;
			xdx_hdmi_audio_ptr->audioif_video_disconnect = XDX_AUDIO_MODULE_DP;
			change = 2; //to default-hdmi0
			for (i = 0; i <= 3; i++) {
				if ((xdx_hdmi_audio_ptr->current_connect[i] > 0) && (i > AUDIOIF_DP)) {
					xdx_hdmi_audio_ptr->audioif_video_connect = xdx_hdmi_audio_ptr->current_connect[i];
					change = 1;//dp0 to other
					break;
				} else if ((xdx_hdmi_audio_ptr->current_connect[i] > 0) && (i < AUDIOIF_DP)) {
					change = 0; //dp0 do nothing
					break;
				}
			}
		} else if (module == XDX_AUDIO_MODULE_HDMI1) {
			xdx_hdmi_audio_ptr->current_connect[AUDIOIF_HDMI1] = 0;
			xdx_hdmi_audio_ptr->audioif_video_disconnect = XDX_AUDIO_MODULE_HDMI1;
			change = 2;
			for (i = 0; i <= 3; i++) {
				if ((xdx_hdmi_audio_ptr->current_connect[i] > 0) && (i > AUDIOIF_HDMI1)) {
					xdx_hdmi_audio_ptr->audioif_video_connect = xdx_hdmi_audio_ptr->current_connect[i];
					change = 1; //hdmi1 to other
					break;
				} else if ((xdx_hdmi_audio_ptr->current_connect[i] > 0) && (i < AUDIOIF_HDMI1)) {
					change = 0;  //hdmi1 do nothing
					break;
				}
			}
		} else if (module == XDX_AUDIO_MODULE_DP1) {
			xdx_hdmi_audio_ptr->current_connect[AUDIOIF_DP1] =0;
			xdx_hdmi_audio_ptr->audioif_video_disconnect = XDX_AUDIO_MODULE_DP1;
			change = 2;
			for (i = 0; i <= 3; i++) {
				if ((xdx_hdmi_audio_ptr->current_connect[i] > 0) && (i > AUDIOIF_DP1)) {
					xdx_hdmi_audio_ptr->audioif_video_connect = XDX_AUDIO_MODULE_DP1;
					change = 1;
					break;
				} else if ((xdx_hdmi_audio_ptr->current_connect[i] > 0) && (i < AUDIOIF_DP1)) {
					change = 0;  //> dp1 do nothing
					break;
				}
			}
		}
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]The display status is unplug! \n", __func__, __LINE__);
	} else if (XDX_AUDIO_MODULE_CONNECT == status) {
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]The display status is plug! \n", __func__, __LINE__);
		if (module == XDX_AUDIO_MODULE_HDMI) {
			change = 3;  //hdmi0->hdmi0
			if ((((xdx_hdmi_audio_ptr->audioif_video_disconnect) || (xdx_hdmi_audio_ptr->audioif_hdmi)) && (xdx_hdmi_audio_ptr->audioif_video_connect != XDX_AUDIO_MODULE_HDMI))
					&& (xdx_hdmi_audio_ptr->audioif_video_disconnect != XDX_AUDIO_MODULE_HDMI)) {
				change = 1;//other to hdmi
			}
			xdx_hdmi_audio_ptr->audioif_video_connect = XDX_AUDIO_MODULE_HDMI;
			for (i = 0; i <= 3; i++){
				if ((xdx_hdmi_audio_ptr->current_connect[i] > 0) && (i < AUDIOIF_HDMI)) {
					xdx_hdmi_audio_ptr->audioif_video_connect = xdx_hdmi_audio_ptr->current_connect[i];
					change = 0;
					break;
				} else if ((xdx_hdmi_audio_ptr->current_connect[i] > 0) && (i > AUDIOIF_HDMI) && (xdx_hdmi_audio_ptr->current_connect[AUDIOIF_HDMI] == 0)) {
					change = 1;//other->hdmi
					xdx_hdmi_audio_ptr->last_connect = xdx_hdmi_audio_ptr->current_connect[i];
					break;
				}
			}
			xdx_hdmi_audio_ptr->current_connect[AUDIOIF_HDMI] = XDX_AUDIO_MODULE_HDMI;
		} else if (module == XDX_AUDIO_MODULE_DP) {
			change = 3;
			if (((xdx_hdmi_audio_ptr->audioif_video_disconnect) && (xdx_hdmi_audio_ptr->audioif_video_disconnect != XDX_AUDIO_MODULE_DP) && (xdx_hdmi_audio_ptr->audioif_video_connect != XDX_AUDIO_MODULE_DP))
					|| (xdx_hdmi_audio_ptr->audioif_hdmi) || (xdx_hdmi_audio_ptr->flag == 0)) {
				//dp connect twice
				change = 1;//other to dp
			}
			xdx_hdmi_audio_ptr->audioif_video_connect = XDX_AUDIO_MODULE_DP;
			for (i = 0; i <= 3; i++) {
				if ((xdx_hdmi_audio_ptr->current_connect[i] > 0) && (i < AUDIOIF_DP)) {
					xdx_hdmi_audio_ptr->audioif_video_connect = xdx_hdmi_audio_ptr->current_connect[i];
					change = 0;
					break;
				} else if ((xdx_hdmi_audio_ptr->current_connect[i] > 0) && (i > AUDIOIF_DP) && (xdx_hdmi_audio_ptr->current_connect[AUDIOIF_DP] == 0)) {
					change = 1;//other->dp
					xdx_hdmi_audio_ptr->last_connect = xdx_hdmi_audio_ptr->current_connect[i];
					break;
				}
			}

			xdx_hdmi_audio_ptr->current_connect[AUDIOIF_DP] = XDX_AUDIO_MODULE_DP;
		} else if (module == XDX_AUDIO_MODULE_HDMI1) {
			change = 3;
			if (((xdx_hdmi_audio_ptr->audioif_video_disconnect) && (xdx_hdmi_audio_ptr->audioif_video_disconnect != XDX_AUDIO_MODULE_HDMI1) && (xdx_hdmi_audio_ptr->audioif_video_connect != XDX_AUDIO_MODULE_HDMI1))
					|| (xdx_hdmi_audio_ptr->audioif_hdmi) || (xdx_hdmi_audio_ptr->flag == 0)) {
				change = 1;//other to hdmi1
			}
			xdx_hdmi_audio_ptr->audioif_video_connect = XDX_AUDIO_MODULE_HDMI1;
			for (i = 0; i <= 3; i++) {
				if ((xdx_hdmi_audio_ptr->current_connect[i] > 0)  && (i < AUDIOIF_HDMI1)) {
					xdx_hdmi_audio_ptr->audioif_video_connect = xdx_hdmi_audio_ptr->current_connect[i];
					change = 0;
					break;
				} else if ((xdx_hdmi_audio_ptr->current_connect[i] > 0) && (i > AUDIOIF_HDMI1) && (xdx_hdmi_audio_ptr->current_connect[AUDIOIF_HDMI1] == 0)) {
					change = 1;//other->hdmi1
					xdx_hdmi_audio_ptr->last_connect = xdx_hdmi_audio_ptr->current_connect[i];
					break;
				}
			}
			xdx_hdmi_audio_ptr->current_connect[AUDIOIF_HDMI1] = XDX_AUDIO_MODULE_HDMI1;
		} else if (module == XDX_AUDIO_MODULE_DP1) {
			change = 3;
			if (((xdx_hdmi_audio_ptr->audioif_video_disconnect) && (xdx_hdmi_audio_ptr->audioif_video_disconnect != XDX_AUDIO_MODULE_DP1) && (xdx_hdmi_audio_ptr->audioif_video_connect != XDX_AUDIO_MODULE_DP1))
					|| (xdx_hdmi_audio_ptr->audioif_hdmi) || (xdx_hdmi_audio_ptr->flag == 0)) {
				change = 1;//other to dp1
			}
			xdx_hdmi_audio_ptr->audioif_video_connect = XDX_AUDIO_MODULE_DP1;
			for (i = 0; i <= 3; i++) {
				if ((xdx_hdmi_audio_ptr->current_connect[i] > 0)  && (i < AUDIOIF_DP1)) {
					xdx_hdmi_audio_ptr->audioif_video_connect = xdx_hdmi_audio_ptr->current_connect[i];
					change = 0;
					break;
				} else if ((xdx_hdmi_audio_ptr->current_connect[i] > 0) && (i > AUDIOIF_DP1) && (xdx_hdmi_audio_ptr->current_connect[AUDIOIF_DP1] == 0)) {
					change = 1;//other->dp1
					xdx_hdmi_audio_ptr->last_connect = xdx_hdmi_audio_ptr->current_connect[i];
					break;
				}
			}
			xdx_hdmi_audio_ptr->current_connect[AUDIOIF_DP1] = XDX_AUDIO_MODULE_DP1;
		}
	}

	if (change == 1) {
		mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
		temp_status = xdx_hdmi_audio_ptr->status;
		xdx_hdmi_audio_ptr->status = cPlayBackStop;
		mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

		if (xdev->asic_type == CHIP_PANGU_B0) {
			if ((xdx_hdmi_audio_ptr->audioif_video_disconnect ==XDX_AUDIO_MODULE_HDMI ) || (xdx_hdmi_audio_ptr->audioif_video_disconnect ==XDX_AUDIO_MODULE_DP)
					|| (xdx_hdmi_audio_ptr->last_connect == XDX_AUDIO_MODULE_HDMI) || (xdx_hdmi_audio_ptr->last_connect == XDX_AUDIO_MODULE_DP)|| (xdx_hdmi_audio_ptr->audioif_hdmi) || (xdx_hdmi_audio_ptr->flag == 0)) {
				mutex_lock(xdx_hdmi_audio_ptr->hw_mutex);
				pAudioB0Reg->TXEN.all = 0;
				mutex_unlock(xdx_hdmi_audio_ptr->hw_mutex);

				xdx_hdmi_audio_ptr->audioif_hdmi = XDX_AUDIO_MODULE_NULL;
				xdx_hdmi_audio_ptr->audioif_video_disconnect = XDX_AUDIO_MODULE_NULL;
				xdx_hdmi_audio_ptr->last_connect = XDX_AUDIO_MODULE_NULL;
			} else {
				mutex_lock(xdx_hdmi_audio_ptr->hw_mutex);
				pAudio1B0Reg->TXEN.all = 0;
				mutex_unlock(xdx_hdmi_audio_ptr->hw_mutex);
			}
		}

		timeout_close = msecs_to_jiffies(200);
		time_close = get_jiffies_64();
		while (!xdx_hdmi_audio_ptr->fifo_empty || xdx_hdmi_audio_ptr->dma_pending_cnt) {
			udelay(1000);
			time_now = get_jiffies_64();
			if (time_now > time_close + timeout_close) {
				XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]Wait fifo_empty timeout!\n", __func__, __LINE__);
				xdx_hdmi_audio_ptr->fifo_bytes = 0;
				xdx_hdmi_audio_ptr->fifo_empty = 1;
				break;
			}
		}
		if (xdev->asic_type == CHIP_PANGU_A0) {
			mutex_lock(xdx_hdmi_audio_ptr->hw_mutex);
			pAudioReg->TXEN.iis_en = 0;
			pAudioReg->CKEN.hdmi_bclk_en = 0;
			mutex_unlock(xdx_hdmi_audio_ptr->hw_mutex);
		}

		if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI) || (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP)) {
			if (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI) {
				if (xdev->asic_type == CHIP_PANGU_A0) {
					xdx_hdmi_audio_a0_Enable(xdx_hdmi_audio_ptr, XDX_AUDIO_MODULE_HDMI, 1);
				} else if (xdev->asic_type == CHIP_PANGU_B0) {
					xdx_hdmi_audio_b0_Enable(xdx_hdmi_audio_ptr, XDX_AUDIO_MODULE_HDMI, 1);
				}
				if (hdmi) {
					xdx_hdmi_audio_ptr->hdmi = hdmi;
					xdx_hdmi_audio_setRate(xdx_hdmi_audio_ptr);
				}
			}
			else if (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP){
				if (xdev->asic_type == CHIP_PANGU_A0) {
					xdx_hdmi_audio_a0_Enable(xdx_hdmi_audio_ptr, xdx_hdmi_audio_ptr->audioif_video_connect, 1);
				} else if (xdev->asic_type == CHIP_PANGU_B0) {
					xdx_hdmi_audio_b0_Enable(xdx_hdmi_audio_ptr, xdx_hdmi_audio_ptr->audioif_video_connect, 1);
				}
				ADIFOutputSelConfig(xdx_hdmi_audio_ptr);
			}
		} else if ((xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI1) || (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP1)) {
			if (xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_HDMI1) {
				xdx_hdmi_audio_b0_Enable(xdx_hdmi_audio_ptr, xdx_hdmi_audio_ptr->audioif_video_connect, 1);
				if (hdmi1) {
					xdx_hdmi_audio_ptr->hdmi1 = hdmi1;
					xdx_hdmi_audio_setRate(xdx_hdmi_audio_ptr);
				}
			} else if(xdx_hdmi_audio_ptr->audioif_video_connect == XDX_AUDIO_MODULE_DP1) {
				xdx_hdmi_audio_b0_Enable(xdx_hdmi_audio_ptr, xdx_hdmi_audio_ptr->audioif_video_connect, 1);
				ADIFOutputSelConfig(xdx_hdmi_audio_ptr);
			}
		}
		mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
		xdx_hdmi_audio_ptr->status = temp_status;
		mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);

	}
	if (change == 2) {
		//to default hdmi
		mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
		temp_status = xdx_hdmi_audio_ptr->status;
		xdx_hdmi_audio_ptr->status = cPlayBackStop;
		mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);
		//change other
		if (xdev->asic_type == CHIP_PANGU_B0) {
			if ((xdx_hdmi_audio_ptr->audioif_video_disconnect == XDX_AUDIO_MODULE_HDMI ) || (xdx_hdmi_audio_ptr->audioif_video_disconnect == XDX_AUDIO_MODULE_DP)) {
				mutex_lock(xdx_hdmi_audio_ptr->hw_mutex);
				pAudioB0Reg->TXEN.all = 0;
				mutex_unlock(xdx_hdmi_audio_ptr->hw_mutex);
			} else {
				mutex_lock(xdx_hdmi_audio_ptr->hw_mutex);
				pAudio1B0Reg->TXEN.all = 0;
				mutex_unlock(xdx_hdmi_audio_ptr->hw_mutex);
			}
		}

		timeout_close = msecs_to_jiffies(200);
		time_close = get_jiffies_64();
		while (!xdx_hdmi_audio_ptr->fifo_empty || xdx_hdmi_audio_ptr->dma_pending_cnt) {
			udelay(1000);
			time_now = get_jiffies_64();
			if (time_now > time_close + timeout_close) {
				XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]Wait fifo_empty timeout!\n", __func__, __LINE__);
				break;
			}
		}
		if (xdev->asic_type == CHIP_PANGU_A0) {
			mutex_lock(xdx_hdmi_audio_ptr->hw_mutex);
			pAudioReg->TXEN.iis_en = 0;
			pAudioReg->CKEN.hdmi_bclk_en = 0;
			mutex_unlock(xdx_hdmi_audio_ptr->hw_mutex);
		}

		xdx_hdmi_audio_ptr->audioif_video_connect = XDX_AUDIO_MODULE_HDMI;
		xdx_hdmi_audio_ptr->audioif_hdmi = XDX_AUDIO_MODULE_HDMI;
		if (xdev->asic_type == CHIP_PANGU_A0) {
			xdx_hdmi_audio_a0_Enable(xdx_hdmi_audio_ptr, xdx_hdmi_audio_ptr->audioif_video_connect, 1);
		} else if (xdev->asic_type == CHIP_PANGU_B0) {
			xdx_hdmi_audio_b0_Enable(xdx_hdmi_audio_ptr, xdx_hdmi_audio_ptr->audioif_video_connect, 1);
		}
		mutex_lock(xdx_hdmi_audio_ptr->transfer_mutex);
		xdx_hdmi_audio_ptr->status = temp_status;
		mutex_unlock(xdx_hdmi_audio_ptr->transfer_mutex);
	}
	if (change ==3) {
		if ((xdx_hdmi_audio_ptr->audioif_video_connect ==XDX_AUDIO_MODULE_HDMI ) && hdmi) {
			xdx_hdmi_audio_ptr->hdmi = hdmi;
			xdx_hdmi_audio_setRate(xdx_hdmi_audio_ptr);
		} else if ((xdx_hdmi_audio_ptr->audioif_video_connect ==XDX_AUDIO_MODULE_HDMI1 ) && hdmi1) {
			xdx_hdmi_audio_ptr->hdmi1 = hdmi1;
			xdx_hdmi_audio_setRate(xdx_hdmi_audio_ptr);
		} else if ((xdx_hdmi_audio_ptr->audioif_video_connect ==XDX_AUDIO_MODULE_DP ) && dp) {
			xdx_hdmi_audio_ptr->dp = dp;
			ADIFOutputSelConfig(xdx_hdmi_audio_ptr);
		} else if((xdx_hdmi_audio_ptr->audioif_video_connect ==XDX_AUDIO_MODULE_DP1 ) && dp1) {
			xdx_hdmi_audio_ptr->dp1 = dp1;
			ADIFOutputSelConfig(xdx_hdmi_audio_ptr);
		}
	}

	if ((xdx_hdmi_audio_ptr->audioif_video_connect==XDX_AUDIO_MODULE_HDMI) && hdmi)
		xdx_hdmi_audio_ptr->hdmi = hdmi;
	else if ((xdx_hdmi_audio_ptr->audioif_video_connect==XDX_AUDIO_MODULE_DP) && dp)
		xdx_hdmi_audio_ptr->dp = dp;
	else if ((xdx_hdmi_audio_ptr->audioif_video_connect==XDX_AUDIO_MODULE_HDMI1) && hdmi1)
		xdx_hdmi_audio_ptr->hdmi1 = hdmi1;
	else if ((xdx_hdmi_audio_ptr->audioif_video_connect==XDX_AUDIO_MODULE_DP1) && dp1)
		xdx_hdmi_audio_ptr->dp1 = dp1;

	xdx_hdmi_audio_ptr->flag = 1;
	xdx_audio_control_update(xdx_hdmi_audio_ptr);

	return 0;
}

int xdx_audio_resume(struct xdx_device *xdev)
{
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = xdev->xdx_hdmi_audio_hotplug_ptr;
	struct snd_pcm_runtime *runtime = NULL;
	DBCmd_T dbCmd;
	DBCmd_a0f1_T dbCmd_a0f1;
	uint32_t TimeOut = 0;
	uint32_t dbvalue = 0;

	if (xdx_hdmi_audio_ptr->rate == 0) {
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]Audio rate is 0\n", __func__, __LINE__);
		return 0;
	}
	if (xdev->shareram_size == XDX_AUDIO_BAR4_16M) {
		switch (xdx_hdmi_audio_ptr->rate) {
			case 44100:
				dbCmd_a0f1.data = 44100;
				break;
			case 48000:
				dbCmd_a0f1.data = 48000;
				break;
			default:
				break;
		}
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] dbCmd_a0f1.all=0x%llx\n", __func__, __LINE__, dbCmd_a0f1.all);
		OSWriteAUHWReg64(xdx_hdmi_audio_ptr->iobase, 0x100588, dbCmd_a0f1.all);
	} else if (xdev->shareram_size == XDX_AUDIO_BAR4_1M) {
		dbCmd.pixclock = xdx_hdmi_audio_ptr->rate;
		dbCmd.subCmd = cH02AAudioSubCmdPllDecimalConfig;
		XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] dbCmd.all=0x%llx\n", __func__, __LINE__, dbCmd.all);
		OSWriteAUHWReg64(xdx_hdmi_audio_ptr->iobase_acpu, 0x100588, dbCmd.all);
	}
	TimeOut = 10000;
	while(1) {
		if (xdev->shareram_size == XDX_AUDIO_BAR4_16M)
			dbvalue = OSReadAUHWReg32(xdx_hdmi_audio_ptr->iobase, 0x100588);
		else if (xdev->shareram_size == XDX_AUDIO_BAR4_1M)
			dbvalue = OSReadAUHWReg32(xdx_hdmi_audio_ptr->iobase_acpu, 0x100588);
		if ((dbvalue & 1) == 1)
			break;

		TimeOut--;
		if (TimeOut == 0) {
			XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d] dbCmd fail\n", __func__, __LINE__);
			return -1;
		}
		udelay(1000);
	}

	if ((xdx_hdmi_audio_ptr->playback_substream != NULL) && (xdx_hdmi_audio_ptr->playback_substream->runtime != NULL)) {
		runtime = xdx_hdmi_audio_ptr->playback_substream->runtime;
		if (runtime->status->state != runtime->status->suspended_state)
			runtime->status->state = SNDRV_PCM_STATE_SETUP;
	}
	XDX_HDMI_AUDIO_PRINT(xdx_hdmi_audio_ptr->dev, "[%s %d]\n", __func__, __LINE__);
	return 0;
}

static int xdxaudio_irq_process(struct xdx_device *xdev,
			       struct xdx_irq_src *src,
			       struct xdx_irq_entry *entry)
{
	pangu_b0_audiof0_irq_handler(xdev->xdx_hdmi_audio_hotplug_ptr);
	return 0;
}

static int xdxaudio1_irq_process(struct xdx_device *xdev,
			       struct xdx_irq_src *src,
			       struct xdx_irq_entry *entry)
{
	pangu_b0_audio1f0_irq_handler(xdev->xdx_hdmi_audio_hotplug_ptr);
	return 0;
}

static struct xdx_irq_src_funcs xdxaudio_src_funcs = {
	.process = xdxaudio_irq_process,
};

static struct xdx_irq_src_funcs xdxaudio1_src_funcs = {
	.process = xdxaudio1_irq_process,
};

static int xdxvdma1ch3_irq_process(struct xdx_device *xdev,
                               struct xdx_irq_src *src,
                               struct xdx_irq_entry *entry)
{
	/* directly return when int status is not raised for MSI poll case */
	if (entry->poll_enable && (NULL != xdev->xdx_hdmi_audio_hotplug_ptr) &&
	(!OSReadHWReg64(xdev->xdx_hdmi_audio_hotplug_ptr->iobase_vdma1, 0x2800 + 0x10))) {
		return 0;
	}

	pangu_vdma1ch3_irq_handler(xdev->xdx_hdmi_audio_hotplug_ptr);
	return 0;
}


static struct xdx_irq_src_funcs xdxvdma1ch3_src_funcs = {
	.process = xdxvdma1ch3_irq_process,
};

int xdxaudio_isr_install(struct xdx_device *xdev)
{
	struct xdx_irq_src *irq_src_audio;
	struct xdx_irq_src *irq_src_audio1;
	struct xdx_irq_src *irq_src_vdma1;

	if (xdev->asic_type == CHIP_PANGU_B0) {
		irq_src_audio = kmalloc(sizeof(*irq_src_audio), GFP_KERNEL);
		if (!irq_src_audio)
			return -ENOMEM;

		irq_src_audio->id = XDX_IRQ_CLIENTID_AUDIO;
		irq_src_audio->funcs = &xdxaudio_src_funcs;

		xdx_irq_register_src(xdev, irq_src_audio);
		xdev->xdx_hdmi_audio_hotplug_ptr->irq_src_audio = irq_src_audio;

		irq_src_audio1 = kmalloc(sizeof(*irq_src_audio1), GFP_KERNEL);
		if (!irq_src_audio1)
			return -ENOMEM;

		irq_src_audio1->id = XDX_IRQ_CLIENTID_AUDIO1;
		irq_src_audio1->funcs = &xdxaudio1_src_funcs;

		xdx_irq_register_src(xdev, irq_src_audio1);
		xdev->xdx_hdmi_audio_hotplug_ptr->irq_src_audio1 = irq_src_audio1;
	}

	irq_src_vdma1 = kmalloc(sizeof(*irq_src_vdma1), GFP_KERNEL);
	if (!irq_src_vdma1)
		return -ENOMEM;

	irq_src_vdma1->id = XDX_IRQ_CLIENTID_AUDIO_DMA;
	irq_src_vdma1->funcs = &xdxvdma1ch3_src_funcs;

	xdx_irq_register_src(xdev, irq_src_vdma1);
	xdev->xdx_hdmi_audio_hotplug_ptr->irq_src_vdma1 = irq_src_vdma1;
	return 0;
}

int xdxaudio_isr_uninstall(struct xdx_device *xdev)
{
	int ret;
	struct snd_xdx_hdmi_audio *xdx_hdmi_audio_ptr = xdev->xdx_hdmi_audio_hotplug_ptr;
	if (xdev->asic_type == CHIP_PANGU_B0) {
		ret = xdx_irq_unregister_src(xdev, xdx_hdmi_audio_ptr->irq_src_audio);
		if (ret) {
			dev_err(xdev->dev, "failed to unregister irq source\n");
			return ret;
		}

		kfree(xdx_hdmi_audio_ptr->irq_src_audio);
		xdx_hdmi_audio_ptr->irq_src_audio = NULL;

		ret = xdx_irq_unregister_src(xdev, xdx_hdmi_audio_ptr->irq_src_audio1);
		if (ret) {
			dev_err(xdev->dev, "failed to unregister irq source\n");
			return ret;
		}

		kfree(xdx_hdmi_audio_ptr->irq_src_audio1);
		xdx_hdmi_audio_ptr->irq_src_audio1 = NULL;
	}

	ret = xdx_irq_unregister_src(xdev, xdx_hdmi_audio_ptr->irq_src_vdma1);
	if (ret) {
		dev_err(xdev->dev, "failed to unregister irq source\n");
		return ret;
	}

	kfree(xdx_hdmi_audio_ptr->irq_src_vdma1);
	xdx_hdmi_audio_ptr->irq_src_vdma1 = NULL;

	return 0;
}
